unit SurrogatePairs;

{$I+}

interface

uses
  Windows, SysUtils, ErrorText;

type AnsiCharArray = array[0..4095] of AnsiChar;

type
  SurrogatePairList = class
  private
    a: AnsiCharArray;
    A_SIZE: Integer;
    A_SIZE_1: Integer;
    B_110110_S: Integer;
    B_110111_S: Integer;
    CLEAR_ARRAY: AnsiCharArray;
    combo: array of AnsiChar;
    combo_size: Integer;
    comboLength: Integer;
    COMMA_C: AnsiChar;
    E: array[0..255] of Integer;
    file_name: String;
    H: array[0..7] of Integer;
    H_0A_C: AnsiChar;
    H_0D_C: AnsiChar;
    H_FE_C: AnsiChar;
    H_FF_C: AnsiChar;
    higher_part_of_kanji: Integer;
    lower_part_of_kanji: Integer;
    LOWER_PART_MASK: Integer;
    new_name: String;
    x: ErrorTexts;
    function bufferingWriting(comboLength: Integer): Integer;
    function copying_string_to_part_of_array(s: String; m: Integer): Integer;
    function E_table: Integer;
    function file_size(name: String): Integer;
    function hexadecimalCharacter(c: Char): Boolean;
    function newLine(m: Integer): Integer;
    function setBufferSize: Integer;
  public
    destructor Destroy; override;
    function hexadecimal_sequence_to_integer(s: PChar; s_length: Integer): Integer;
    function reading: Integer;
    procedure setting(fn: String; nn: String);
  end;

const
  B_110110 = 54;
  B_110111 = 55;
  C_ERR = 'err';
  COMBO_ERROR = -3;
  COMMA_BYTE = 44; { 2Ch, ',' }
  EMPTY = '';
  EXPANSION_FACTOR = 11;
  FILE_ERROR = -2;
  FIELD_SEPARATOR = ',';
  FINAL_INTEGER = 2147483647; { 7FFFFFFFh }
  H_00010000 = 65536;
  H_00FF = 255;
  H_0A = 10; { LF }
  H_0D = 13; { CR }
  H_30 = 48; { '0' }
  H_31 = 49; { '1' }
  H_39 = 57; { '9' }
  H_3F = 63;
  H_41 = 65; { 'A' }
  H_42 = 66; { 'B' }
  H_46 = 70; { 'F' }
  H_D840 = 55360;
  H_DC02 = 56322;
  H_FE = 254;
  H_FF = 255;
  HEXADECIMAL_CHARACTERS = '0123456789ABCDEF';
  KANJI_SIZE = 2;
  NOT_CONVERTED = -1;
  SIZE_OF_ANSICHAR = 1;
  SIZE_OF_WIDECHAR = 2;
  TEXT_FILE = '.txt';

implementation

destructor SurrogatePairList.Destroy;
begin
  combo := nil;

  inherited Destroy;
end;

function SurrogatePairList.bufferingWriting(comboLength: Integer): Integer;
var
  block_has_been_written: Integer;
  exit_code: Integer;
  k: Integer;
  n: Integer;
  newFile: File;
  s: String;
begin
  exit_code := 0;
  n := 0;

{ Writing }
  try
    AssignFile(newFile, new_name);
    Rewrite(newFile, SIZE_OF_ANSICHAR);

    repeat
      a := CLEAR_ARRAY;

      for k := 0 to A_SIZE_1 do
      begin
        a[k] := combo[n];
        Inc(n);
        if n >= comboLength then
          begin
            A_SIZE := k + 1;
            break;
          end;

      end;

      BlockWrite(newFile, a, A_SIZE, block_has_been_written);
    until n >= comboLength;

    CloseFile(newFile);
    A_SIZE := SizeOf(a);
  except
    s := 'Rewrite() error-98: ' + new_name;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    exit_code := FILE_ERROR;
  end;

  Result := exit_code;
end;

function SurrogatePairList.copying_string_to_part_of_array(s: String; m: Integer): Integer;
var
  first_byte: Integer;
  i: Longint;
  k: Integer;
  n: Integer;
  s_length: Integer;
  second_byte: Integer;
begin
  s_length := Length(s);

  for k := 1 to s_length do
  begin
    i := Ord(s[k]);
    if i > H_FF then
      begin
        first_byte := i and H_00FF;
        combo[m] := Chr(first_byte); { little-endian architecture }
        n := i shr 8;
        second_byte := n and H_00FF;
        combo[ m + 1 ] := Chr(second_byte);
        Inc(m, SIZE_OF_WIDECHAR);
      end
    else
      begin
{ ASCII }
        combo[m] := Chr(i);
        combo[ m + 1 ] := #0;
        Inc(m, SIZE_OF_WIDECHAR);
      end;

  end;

  Result := m;
end;

function SurrogatePairList.E_table: Integer;
var
  beginning_of_E: Integer;
  end_1: Integer;
  h: Integer;
  i: Integer;
  k: Integer;
  m: Integer;
begin
  for i := 0 to H_FF do
  begin
    E[i] := NOT_CONVERTED;
  end;

{ 0 to 9 }
  beginning_of_E := H_30;

  end_1 := H_39 - 1;
  E[H_30] := 0;
  h := E[H_30];
  m := H_31;

  for k := beginning_of_E to end_1 do
  begin
    Inc(h);
    E[m] := h;
    Inc(m);
  end;

{ A to F }
  beginning_of_E := H_41;

  end_1 := H_46 - 1;
  E[H_41] := 10; { 'A' }
  h := E[H_41];
  m := H_42;

  for k := beginning_of_E to end_1 do
  begin
    Inc(h);
    E[m] := h;
    Inc(m);
  end;

  Result := h; { 'F' }
end;

function SurrogatePairList.file_size(name: String): Integer;
var
  f: Integer;
  size: Integer;
begin
  f := FileOpen(name, fmOpenRead or fmShareDenyNone);
  size := FileSeek(f, 0, 2); { EOF }
  FileClose(f);
  Result := size;
end;

function SurrogatePairList.hexadecimalCharacter(c: Char): Boolean;
var
  q: Integer;
  r: Boolean;
begin
  q := AnsiPos(c, HEXADECIMAL_CHARACTERS);
  if q > 0 then
    r := True
  else
    r := False;

  Result := r;
end;

function SurrogatePairList.hexadecimal_sequence_to_integer(s: PChar; s_length: Integer): Integer;
var
  c: Char;
  g: String;
  g_length: Integer;
  i: Longint;
  k: Integer;
  m: Integer;
  messages: String;
  n: Integer;
  s_length_1: Integer;
  term: Integer;
begin
  g := EMPTY;
  i := 0;
  m := 0;
  n := 0;
  s_length_1 := s_length - 1;

  for k := 0 to s_length_1 do
  begin
    c := s[k];
    if hexadecimalCharacter(c) then
      begin
        g := g + c;
      end
    else
      begin
        break;
      end;

  end;

  g_length := Length(g);
  try
    for k := g_length downto 1 do
    begin
      i := Ord(g[k]);
      term := E[i] * H[m];
      if term < 0 then
        raise Exception.Create('hexadecimal_sequence_to_integer() error');

      n := n + term;
      Inc(m);
    end;

  except
    messages := 'hexadecimal_sequence_to_integer() error-99: i = ' + IntToStr(i);
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, messages);
    x.Free;
    n := NOT_CONVERTED;
    Halt(1);
  end;

  Result := n;
end;

function SurrogatePairList.newLine(m: Integer): Integer;
begin
{ new line }
  combo[m] := H_0D_C;
  combo[ m + 1 ] := #0;
  Inc(m, SIZE_OF_WIDECHAR);

  combo[m] := H_0A_C;
  combo[ m + 1 ] := #0;
  Inc(m, SIZE_OF_WIDECHAR);

  Result := m;
end;

function SurrogatePairList.reading: Integer;
var
  basic_code: Integer;
  exit_code: Integer;
  f: String;
  k: Integer;
  code_point: Integer;
  m: Integer;
  s: String;
  s_length: Integer;
  testFile: TextFile;
begin
  exit_code := 0;
  m := 0;
  combo[m] := H_FF_C;
  combo[ m + 1 ] := H_FE_C;
  Inc(m, KANJI_SIZE);

  try
    AssignFile(testFile, file_name);
    Reset(testFile);

    while not Eof(testFile) do
    begin
      ReadLn(testFile, s);
      s_length := Length(s);
      if s_length = 0 then
        break;

      code_point := hexadecimal_sequence_to_integer( PChar(s), Length(s) );

{ Four bytes of UTF-16 }
      basic_code := code_point - H_00010000;
      k := basic_code shr 10;
      higher_part_of_kanji := k + B_110110_S;

      k := higher_part_of_kanji and H_00FF;
      combo[m] := Chr(k); { little-endian architecture }

      k := higher_part_of_kanji shr 8;
      k := k and H_00FF;
      combo[ m + 1 ] := Chr(k);
      Inc(m, KANJI_SIZE);

      basic_code := code_point - H_00010000;
      k := basic_code and LOWER_PART_MASK;
      lower_part_of_kanji := k + B_110111_S;

      k := lower_part_of_kanji and H_00FF;
      combo[m] := Chr(k); { little-endian architecture }

      k := lower_part_of_kanji shr 8;
      k := k and H_00FF;
      combo[ m + 1 ] := Chr(k);
      Inc(m, KANJI_SIZE);

      combo[m] := COMMA_C;
      combo[ m + 1 ] := #0;
      Inc(m, SIZE_OF_WIDECHAR);

{ higher-part and lower-part of kanji }
      f := Format('%4x', [higher_part_of_kanji]);
      m := copying_string_to_part_of_array(f, m);
      m := copying_string_to_part_of_array(FIELD_SEPARATOR, m);

      f := Format('%4x', [lower_part_of_kanji]);
      m := copying_string_to_part_of_array(f, m);
      m := copying_string_to_part_of_array(FIELD_SEPARATOR, m);

{ Unicode }
      f := Format('%5x', [code_point]);
      m := copying_string_to_part_of_array(f, m);
      m := copying_string_to_part_of_array(FIELD_SEPARATOR, m);

{ Higher-part of kanji will be useful for arrays if software subtracts H_D840 from the kanji code. }
      higher_part_of_kanji := higher_part_of_kanji - H_D840;

{ Lower-part of kanji will be useful for arrays if software subtracts H_DC02 from the kanji code. }
      lower_part_of_kanji := lower_part_of_kanji - H_DC02;
      f := Format('%d', [higher_part_of_kanji]);
      m := copying_string_to_part_of_array(f, m);
      m := copying_string_to_part_of_array(FIELD_SEPARATOR, m);

      f := Format('%d', [lower_part_of_kanji]);
      m := copying_string_to_part_of_array(f, m);
      m := copying_string_to_part_of_array(FIELD_SEPARATOR, m);

{ Underline of 0xFF3F }
      combo[m] := Chr(H_3F); { little-endian architecture }
      combo[ m + 1 ] := H_FF_C;
      Inc(m, KANJI_SIZE);

      m := newLine(m);
    end;

    CloseFile(testFile);
    comboLength := m;
    bufferingWriting(comboLength);
  except
    s := 'Reset() error-100: ' + file_name;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    exit_code := FILE_ERROR;
  end;

  Result := exit_code;
end;

function SurrogatePairList.setBufferSize: Integer;
var
  r: Integer;
  s: String;
begin
  r := combo_size;

  try
    SetLength(combo, combo_size);
  except
    s := 'SetLength() error-101: ' + IntToStr(combo_size);
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    r := COMBO_ERROR;
  end;

  Result := r;
end;

procedure SurrogatePairList.setting(fn: String; nn: String);
var
  buffer_size: Integer;
  i: Integer;
  s: String;
  testFile: File;
begin
  file_name := fn;
  new_name := nn;

  try
    AssignFile(testFile, file_name);
    Reset(testFile, SIZE_OF_ANSICHAR);
    CloseFile(testFile);

    buffer_size := file_size(file_name);
    combo_size := EXPANSION_FACTOR * buffer_size;
  except
    s := 'Reset() error-102: ' + file_name;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
  end;

  A_SIZE := SizeOf(a);
  A_SIZE_1 := A_SIZE - 1;
  B_110110_S := B_110110 shl 10;
  B_110111_S := B_110111 shl 10;

  E_table;
  H_FF_C := Chr(H_FF);
  COMMA_C := Chr(COMMA_BYTE);
  H_0A_C := Chr(H_0A);
  H_0D_C := Chr(H_0D);
  H_FE_C := Chr(H_FE);

  H[0] := 1;
  H[1] := 16;
  H[2] := 256;
  H[3] := 4096;
  H[4] := 65536;
  H[5] := 1048576;
  H[6] := 16777216;
  H[7] := 268435456;

  LOWER_PART_MASK := 1 + 2 + 4 + 8 + 16 + 32 + 64 + 128 + 256 + 512;

  for i := 0 to A_SIZE_1 do
  begin
    CLEAR_ARRAY[i] := #0;
  end;

  setBufferSize;
end;

end.
