unit FinalEffect;

interface

uses
  SysUtils;

type
  FinalEffects = class
  private
    carriage_return: Boolean;
    extract_decimal: Boolean;
    extract_integer: Boolean;
    field_count: Integer;
    field_to_be_selected: Integer;
    file_name: String;
    focal_point: Integer;
    identical_time: Boolean;
    joint_byte: Byte;
    label_byte: Byte;
    omit_H_and_I_from_ASCII_group: Boolean;
    new_name: String;
    no_removing: Boolean;
    no_searching: Boolean;
    path_name: String;
    quote_byte: Byte;
    remove_0Dh: Boolean;
    remove_controllers: Boolean;
    remove_dakuten: Boolean;
    remove_separators: Boolean;
    replace_kana: Boolean;
    selected_directory_has_priority: Boolean;
    separator_byte: Byte;
    surrogate_pair_to_underline: Boolean;
    temporary_binary: Boolean;
    which_table: Boolean;
  public
    function atPoint: Integer;
    function carriageReturn: Boolean;
    function extractDecimal: Boolean;
    function extractInteger: Boolean;
    function fieldCount: Integer;
    function fieldSelected: Integer;
    function fileName: String;
    function identicalTime: Boolean;
    function jointByte: Byte;
    function labelByte: Byte;
    function newName: String;
    function noRemoving: Boolean;
    function noSearching: Boolean;
    function omit_H_and_I: Boolean;
    function pathName: String;
    function priority: Boolean;
    function produceTemporaryBinary: Boolean;
    function quoteByte: Byte;
    function remove0Dh: Boolean;
    function removeControllers: Boolean;
    function removeDakuten: Boolean;
    function removeSeparators: Boolean;
    function replaceKana: Boolean;
    function separatorByte: Byte;
    function surrogatePairToUnderline: Boolean;
    function whichTable: Boolean;
    procedure setting(fn: String; nn: String); overload;
    procedure setting(fn: String; nn: String; pn: String; 
      fc: Integer; ft: Integer; aa: Integer; 
      jb: Byte; LB: Byte; qb: Byte; sb: Byte; 
      cr: Boolean; ed: Boolean; ei: Boolean; it: Boolean; nr: Boolean; ns: Boolean; oh: Boolean; r0: Boolean; rc: Boolean; rd: Boolean; rs: Boolean; rk: Boolean; sd: Boolean; sp: Boolean; tb: Boolean; wt: Boolean); overload;

  end;

implementation

function FinalEffects.atPoint: Integer;
begin
  Result := focal_point;
end;

function FinalEffects.carriageReturn: Boolean;
begin
  Result := carriage_return;
end;

function FinalEffects.extractDecimal: Boolean;
begin
  Result := extract_decimal;
end;

function FinalEffects.extractInteger: Boolean;
begin
  Result := extract_integer;
end;

function FinalEffects.fieldCount: Integer;
begin
  Result := field_count;
end;

function FinalEffects.fieldSelected: Integer;
begin
  Result := field_to_be_selected;
end;

function FinalEffects.fileName: String;
begin
  Result := file_name;
end;

function FinalEffects.identicalTime: Boolean;
begin
  Result := identical_time;
end;

function FinalEffects.jointByte: Byte;
begin
  Result := joint_byte;
end;

function FinalEffects.labelByte: Byte;
begin
  Result := label_byte;
end;

function FinalEffects.newName: String;
begin
  Result := new_name;
end;

function FinalEffects.noRemoving: Boolean;
begin
  Result := no_removing;
end;

function FinalEffects.noSearching: Boolean;
begin
  Result := no_searching;
end;

function FinalEffects.omit_H_and_I: Boolean;
begin
  Result := omit_H_and_I_from_ASCII_group;
end;

function FinalEffects.pathName: String;
begin
  Result := path_name;
end;

function FinalEffects.priority: Boolean;
begin
  Result := selected_directory_has_priority;
end;

function FinalEffects.produceTemporaryBinary: Boolean;
begin
  Result := temporary_binary;
end;

function FinalEffects.quoteByte: Byte;
begin
  Result := quote_byte;
end;

function FinalEffects.remove0Dh: Boolean;
begin
  Result := remove_0Dh;
end;

function FinalEffects.removeControllers: Boolean;
begin
  Result := remove_controllers;
end;

function FinalEffects.removeDakuten: Boolean;
begin
  Result := remove_dakuten;
end;

function FinalEffects.removeSeparators: Boolean;
begin
  Result := remove_separators;
end;

function FinalEffects.replaceKana: Boolean;
begin
  Result := replace_kana;
end;

function FinalEffects.separatorByte: Byte;
begin
  Result := separator_byte;
end;

function FinalEffects.surrogatePairToUnderline: Boolean;
begin
  Result := surrogate_pair_to_underline;
end;

function FinalEffects.whichTable: Boolean;
begin
  Result := which_table;
end;

procedure FinalEffects.setting(fn: String; nn: String);
begin
{ String }
  file_name := fn;
  new_name := nn;
  path_name := '';

{ Integer }
  field_count := 255;
  field_to_be_selected := 1;
  focal_point := 1;

{ Byte }
  joint_byte := 48; { 30Fh, '0' }
  label_byte := 95; { 5Fh, '_' }
  quote_byte := 34; { 22h, '"' }
  separator_byte := 44; { 2Ch, ',' }

{ Boolean }
  carriage_return := True;
  extract_decimal := False;
  extract_integer := False;
  identical_time := False;
  no_removing := False;
  no_searching := False;
  omit_H_and_I_from_ASCII_group := False;
  remove_0Dh := False;
  remove_controllers := False;
  remove_dakuten := False;
  remove_separators := False;
  replace_kana := False;
  selected_directory_has_priority := False;
  surrogate_pair_to_underline := True;
  temporary_binary := False;
  which_table := False;
end;

procedure FinalEffects.setting(fn: String; nn: String; pn: String; 
  fc: Integer; ft: Integer; aa: Integer; 
  jb: Byte; LB: Byte; qb: Byte; sb: Byte; 
  cr: Boolean; ed: Boolean; ei: Boolean; it: Boolean; nr: Boolean; ns: Boolean; oh: Boolean; r0: Boolean; rc: Boolean; rd: Boolean; rs: Boolean; rk: Boolean; sd: Boolean; sp: Boolean; tb: Boolean; wt: Boolean);

begin
{ String }
  file_name := fn;
  new_name := nn;
  path_name := pn;

{ Integer }
  field_count := fc;
  field_to_be_selected := ft;
  focal_point := aa; { at and around the point }

{ Byte }
  joint_byte := jb;
  label_byte := LB;
  quote_byte := qb;
  separator_byte := sb;

{ Boolean }
  carriage_return := cr;
  extract_decimal := ed;
  extract_integer := ei;
  identical_time := it;
  no_removing := nr;
  no_searching := ns;
  omit_H_and_I_from_ASCII_group := oh;
  remove_0Dh := r0;
  remove_controllers := rc;
  remove_dakuten := rd;
  remove_separators := rs;
  replace_kana := rk;
  selected_directory_has_priority := sd;
  surrogate_pair_to_underline := sp;
  temporary_binary := tb;
  which_table := wt;
end;

end.
