program Unarfij;
{$APPTYPE CONSOLE}

{*
Unarfij Version 1.04
Selecting a field from the UTF-16 file to produce a Shift_JIS text
Copyright 2020 Sogaya

Free Pascal Compiler 3.0.4

*}

uses
  SysUtils,
  ErrorText in 'ErrorText.pas',
  FinalEffect in 'FinalEffect.pas',
  Guide in 'Guide.pas',
  HyphenTerm in 'HyphenTerm.pas',
  MessagesText in 'MessagesText.pas',
  Numbered in 'Numbered.pas',
  TimeName in 'TimeName.pas',
  AnsiQueue in 'AnsiQueue.pas',
  DivideSJIS in 'DivideSJIS.pas',
  FieldsArray in 'FieldsArray.pas',
  PathEXE in 'PathEXE.pas',
  SearchBinary in 'SearchBinary.pas',
  SurvivorUTF16 in 'SurvivorUTF16.pas',
  Ucombo in 'Ucombo.pas',
  DeriveBinary in 'DeriveBinary.pas',
  SelectUTF16 in 'SelectUTF16.pas';

{$R *.RES}

const
  BUFFER_ERROR = -1;
  DATA_TEXT = 'data.txt';
  REMOVE_ANCIENT_FILE = false;
  SURRO_CSV = 'surro.csv';
  SURRO_SJIS_CSV = 'SurroSJIS.csv';
  TEMPORARY_BINARY = 'binary.tmp';
  UNDERLINES_REPLACE_SURROGATE_PAIRS = True;
  VERSION_NUMBER = '1.04';
  VERSION_WITH_COMMENTS = True;
  VERSION_WITHOUT_COMMENTS = False;

var
  c: Selecting;
  carriage_return: Boolean;
  d: DeriveTable;
  dataFile: TextFile;
  extract_decimal: Boolean;
  extract_integer: Boolean;
  f: FinalEffects;
  field_to_be_selected: Integer;
  file_name_length: Integer;
  g: Guiding;
  h: HyphenTerms;
  maximumFieldCount: Integer;
  new_name: String;
  new_name_length: Integer;
  remove_controllers: Boolean;
  remove_dakuten: Boolean;
  replace_kana: Boolean;
  standard_ASCII: Boolean;
  s: String;

begin
  s := ParamStr(1);
  if Length(s) = 0 then
    begin
      d := DeriveTable.Create;
      d.setting;
      Halt(0);
    end;

{ command parameters }
  h := HyphenTerms.Create;
  h.presetting;
  h.setting;
  maximumFieldCount := h.upperLimit;

  if h.guideFile then
    begin
      g := Guiding.Create;
      g.versionGuide(VERSION_NUMBER, VERSION_WITH_COMMENTS);
      g.Free;

      h.Free;
      Halt(1);
    end
  else
    begin
      if h.versionNumber then
        begin
          g := Guiding.Create;
          g.versionGuide(VERSION_NUMBER, VERSION_WITHOUT_COMMENTS);
          g.Free;

          h.Free;
          Halt(1);
        end
      else
        begin
          h.joints; { joint_byte }
          h.labels; { label_byte }
          h.quotes; { quote_byte }
          h.separators; { separator_byte }

          extract_decimal := h.extractDecimal;
          extract_integer := h.extractInteger;
          remove_controllers := h.removeControllers;
          remove_dakuten := h.removeDakuten;
          replace_kana := h.replaceKana;

          if h.extractDecimalFromField < maximumFieldCount then
            begin
              extract_decimal := True;
              field_to_be_selected := h.extractDecimalFromField;
            end
          else
            begin
              if h.extractIntegerFromField < maximumFieldCount then
                begin
                  extract_integer := True;
                  field_to_be_selected := h.extractIntegerFromField;
                end
              else
                begin
                  if h.removeDakutenFromField < maximumFieldCount then
                    begin
                      remove_dakuten := True;
                      field_to_be_selected := h.removeDakutenFromField;
                    end
                  else
                    begin
                      if h.replaceKanaWithLabel < maximumFieldCount then
                        begin
                          replace_kana := True;
                          field_to_be_selected := h.replaceKanaWithLabel;
                        end
                      else
                        begin
                          if h.removeControllersFromField < maximumFieldCount then
                            begin
                              remove_controllers := True;
                              field_to_be_selected := h.removeControllersFromField;
                            end
                          else
                            begin
                              if h.fieldCount < maximumFieldCount then
                                begin
                                  if h.fieldSelected < maximumFieldCount then
                                    begin
                                      field_to_be_selected := h.fieldSelected;
                                    end
                                  else
                                    begin
{ The user can use -c6 as -j6 to select Field-6, for example. }
                                      field_to_be_selected := h.fieldCount;
                                    end;

                                end
                              else
                                begin
                                  field_to_be_selected := h.fieldSelected;
                                end;

                            end;

                        end;

                    end;

                end;

            end;

          new_name := h.listName(1);
          new_name_length := Length(new_name);
          if new_name_length = 0 then
            begin
              new_name := h.newName;
            end;

          if h.remove0Dh then
            begin
              if h.carriageReturn then
                begin
                  carriage_return := True;
                end
              else
                begin
                  carriage_return := False;
                end;

            end
          else
            begin
              carriage_return := True;
            end;

          f := FinalEffects.Create;
          f.setting(h.fileName, new_name, h.pathName, 
            h.fieldCount, field_to_be_selected, h.atPoint, 
            h.jointByte, h.labelByte, h.quoteByte, h.separatorByte, 
            carriage_return, extract_decimal, extract_integer, h.identicalTime, h.noRemoving, h.noSearching, h.omit_H_and_I, h.remove0Dh, remove_controllers, remove_dakuten, h.removeSeparators, replace_kana, h.priority, UNDERLINES_REPLACE_SURROGATE_PAIRS, h.produceTemporaryBinary, h.whichTable);

{ main function }
  if f.whichTable then
    begin
      try
        Assignfile(dataFile, DATA_TEXT);
        Reset(dataFile);
        CloseFile(dataFile);

        Assignfile(dataFile, DATA_TEXT);
        Erase(dataFile);
      except
        ;
      end;

    end;

  file_name_length := Length(f.fileName);
  if file_name_length = 0 then
    begin
      g := Guiding.Create;
      g.versionGuide(VERSION_NUMBER, VERSION_WITHOUT_COMMENTS);

      g.Free; f.Free; h.Free;
      Halt(1);
    end;

  c := Selecting.Create;
  c.presetting(f);
  c.setting(f, f.fileName);
  c.ifErrorOccurs(BUFFER_ERROR);

  if f.removeControllers or  f.omit_H_and_I or f.extractDecimal or f.extractInteger then
    begin
      standard_ASCII := False;
    end
  else
    begin
      standard_ASCII := True;
    end;

  new_name_length := Length(f.newName);
  if new_name_length = 0 then
    begin
      if f.extractDecimal then
        begin
          c.standard_output_of_decimals;
        end
      else
        begin
          if f.extractInteger then
            begin
              c.standard_output_of_integers;
            end
          else
            begin
              if standard_ASCII then
                begin
                  c.standard_output;
                end
              else
                begin
                  c.standard_output_of_non_controllers;
                end;

            end;

        end;

      c.Free; f.Free; h.Free;
      Halt(0);
    end;

  if f.identicalTime then
    begin
      c.produceBinaryFile(f.fileName, REMOVE_ANCIENT_FILE);
      c.renameBinaryFile(f.fileName, f.noRemoving);
    end;

  if f.carriageReturn then
    begin
      if f.produceTemporaryBinary then
        begin
          if f.extractDecimal then
            begin
              c.extracting_decimal_to_produce_MS932_CRLF(TEMPORARY_BINARY);
            end
          else
            begin
              if f.extractInteger then
                begin
                  c.extracting_integer_to_produce_MS932_CRLF(TEMPORARY_BINARY);
                end
              else
                begin
                  if standard_ASCII then
                    begin
                      c.reading_to_produce_MS932_CRLF(TEMPORARY_BINARY);
                    end
                  else
                    begin
                      c.skipping_control_to_produce_MS932_CRLF(TEMPORARY_BINARY);
                    end;

                end;

            end;

          c.renameBinaryFile(f.newName, f.noRemoving);
        end
      else
        begin
          if f.extractDecimal then
            begin
              c.extracting_decimal_to_produce_MS932_CRLF(f.newName);
            end
          else
            begin
              if f.extractInteger then
                begin
                  c.extracting_integer_to_produce_MS932_CRLF(f.newName);
                end
              else
                begin
                  if standard_ASCII then
                    begin
                      c.reading_to_produce_MS932_CRLF(f.newName);
                    end
                  else
                    begin
                      c.skipping_control_to_produce_MS932_CRLF(f.newName);
                    end;

                end;

            end;

        end;

    end
  else
    begin
      if f.remove0Dh then
        begin
          if f.produceTemporaryBinary then
            begin
              if f.extractDecimal then
                begin
                  c.extracting_decimal_to_produce_MS932_LF(TEMPORARY_BINARY);
                end
              else
                begin
                  if f.extractInteger then
                    begin
                      c.extracting_integer_to_produce_MS932_LF(TEMPORARY_BINARY);
                    end
                  else
                    begin
                      if standard_ASCII then
                        begin
                          c.reading_to_produce_MS932_LF(TEMPORARY_BINARY);
                        end
                      else
                        begin
                          c.skipping_control_to_produce_MS932_LF(TEMPORARY_BINARY);
                        end;

                    end;

                end;

              c.renameBinaryFile(f.newName, f.noRemoving);
            end
          else
            begin
              if f.extractDecimal then
                begin
                  c.extracting_decimal_to_produce_MS932_LF(f.newName);
                end
              else
                begin
                  if f.extractInteger then
                    begin
                      c.extracting_integer_to_produce_MS932_LF(f.newName);
                    end
                  else
                    begin
                      if standard_ASCII then
                        begin
                          c.reading_to_produce_MS932_LF(f.newName);
                        end
                      else
                        begin
                          c.skipping_control_to_produce_MS932_LF(f.newName);
                        end;

                    end;

                end;

            end;

        end
      else
        begin
          if f.produceTemporaryBinary then
            begin
              if f.extractDecimal then
                begin
                  c.extracting_decimal_to_produce_MS932_text(TEMPORARY_BINARY);
                end
              else
                begin
                  if f.extractInteger then
                    begin
                      c.extracting_integer_to_produce_MS932_text(TEMPORARY_BINARY);
                    end
                  else
                    begin
                      if standard_ASCII then
                        begin
                          c.reading_to_produce_MS932_text(TEMPORARY_BINARY);
                        end
                      else
                        begin
                          c.skipping_control_to_produce_MS932_text(TEMPORARY_BINARY);
                        end;

                    end;

                end;

              c.renameBinaryFile(f.newName, f.noRemoving);
            end
          else
            begin
              if f.extractDecimal then
                begin
                  c.extracting_decimal_to_produce_MS932_text(TEMPORARY_BINARY);
                end
              else
                begin
                  if f.extractInteger then
                    begin
                      c.extracting_integer_to_produce_MS932_text(TEMPORARY_BINARY);
                    end
                  else
                    begin
                      if standard_ASCII then
                        begin
                          c.reading_to_produce_MS932_text(f.newName);
                        end
                      else
                        begin
                          c.skipping_control_to_produce_MS932_text(f.newName);
                        end;

                    end;

                end;

            end;

        end;

    end;

  if f.whichTable then
    begin
      c.displayDataText;
    end;

  c.Free;

{ end of the main function }
          f.Free; h.Free;
        end; { if h.versionNumber then, else }

    end; { if h.guideFile then, else }

end.
