unit ConvertNarrow;

{$I+}

{*
ASCII_C is the underline character (5Fh, '_') for example.
ASCII_C is used as a substitute for hankaku katakana.
ASCII_C is used as a substitute for kanji.
Control characters are removed.
*}

interface

uses
  Windows, SysUtils, ErrorText;

type AnsiCharArray = array[0..4095] of AnsiChar;
type ByteArray = array[0..223] of Byte;
type WideCharArray = array[0..4095] of WideChar;

type
  ConvertUTF16intoASCII = class
  private
    ascii_code_byte: Byte;
    ASCII_C: AnsiChar;
    BELL_ASCII_KATAKANA: ByteArray;
    beginning: Integer;
    buffer: array of WideChar;
    buffer_size: Integer;
    carriage_return_appears: Boolean;
    CLEAR_ARRAY: WideCharArray;
    CLEAR_V_ARRAY: AnsiCharArray;
    combo: array of AnsiChar;
    combo_size: Integer;
    comboLength: Integer;
    count: Integer;
    dough: array[0..8191] of WideChar;
    doughLength: Integer;
    EMPTY_ANSI: AnsiString;
    file_name: String;
    H_0A_ANSI: AnsiChar;
    H_0A_C: WideChar;
    H_0D_ANSI: AnsiChar;
    H_0D_C: WideChar;
    H_FF_ANSI: AnsiChar;
    NEW_LINE: String;
    NEW_LINE_LENGTH: Integer;
    new_name: String;
    v: AnsiCharArray;
    V_SIZE: Integer;
    V_SIZE_1: Integer;
    x: ErrorTexts;
    y: WideCharArray;
    Y_SIZE: Integer;
    Y_SIZE_1: Integer;
    function bufferingWriting(comboLength: Integer): Integer;
    function file_size(name: String): Integer;
  public
    function displayAsciiText: Boolean;
    function non_kanji_array(s: array of WideChar; s_length: Integer; m: Integer): Integer;
    function reading_to_produce_text(nn: String): Integer;
    function setBufferSize: Integer;
    procedure ifErrorOccurs(exit_code: Integer);
    procedure setting(fn: String; a: Byte);
  end;

const
  ASCII_TEXT = 'ascii.txt';
  BELL = 7;
  BOM_COUNT = 1;
  BUFFER_ERROR = -1;
  C_ERR = 'err';
  COMBO_ERROR = -3;
  EMPTY = '';
  EXPANSION_FACTOR = 1;
  FILE_ERROR = -2;
  H_0A = 10; { LF }
  H_0D = 13; { CR }
  H_FE = 254;
  H_FF = 255;
  H_80 = 128;
  H_FF66 = 65382;
  H_FF9F = 65439;
  SIZE_OF_ANSICHAR = 1;
  SIZE_OF_WIDECHAR = 2;
  TEXT_FILE = '.txt';

implementation

function ConvertUTF16intoASCII.bufferingWriting(comboLength: Integer): Integer;
var
  block_has_been_written: Integer;
  exit_code: Integer;
  k: Integer;
  n: Integer;
  newFile: File;
  s: String;
begin
  exit_code := 0;
  n := 0;

{ Writing }
  try
    AssignFile(newFile, new_name);
    Rewrite(newFile, SIZE_OF_ANSICHAR);

    repeat
      v := CLEAR_V_ARRAY;

      for k := 0 to V_SIZE_1 do
      begin
        v[k] := combo[n];
        Inc(n);
        if n >= comboLength then
          begin
            V_SIZE := k + 1;
            V_SIZE_1 := V_SIZE - 1;
            break;
          end;

      end;

      BlockWrite(newFile, v, V_SIZE, block_has_been_written);
    until n >= comboLength;

    CloseFile(newFile);
    V_SIZE := SizeOf(v);
    V_SIZE_1 := V_SIZE - 1;
  except
    s := 'Rewrite() error-117: ' + new_name;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    exit_code := FILE_ERROR;
  end;

  Result := exit_code;
end;

function ConvertUTF16intoASCII.displayAsciiText: Boolean;
var
  b: Boolean;
  s: String;
  testFile: TextFile;
begin
  b := True;

  try
    AssignFile(testFile, ASCII_TEXT);
    Reset(testFile);

    while not Eof(testFile) do
    begin
      ReadLn(testFile, s);
      WriteLn(s);
    end;

    CloseFile(testFile);
  except
    s := 'Reset() error-118: ' + ASCII_TEXT;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    b := False;
  end;

  Result := b;
end;

function ConvertUTF16intoASCII.file_size(name: String): Integer;
var
  f: Integer;
  size: Integer;
begin
  f := FileOpen(name, fmOpenRead or fmShareDenyNone);
  size := FileSeek(f, 0, 2); { EOF }
  FileClose(f);
  Result := size;
end;

function ConvertUTF16intoASCII.non_kanji_array(s: array of WideChar; s_length: Integer; m: Integer): Integer;
var
  b: Byte;
  c: WideChar;
  k: Integer;
  n: Longint;
  s_length_1: Integer;
  u: Integer;
begin
  s_length_1 := s_length - 1;

{ Removing control characters and hankaku katakanas from each field }
  for k := 0 to s_length_1 do
  begin
    c := s[k];
    n := Ord(c);

    if (H_FF66 <= n) and (n <= H_FF9F) then
      begin
        u := n - H_FF66 + 166; { H_FF9F - H_FF66 + 166 = 223 }
        b := BELL_ASCII_KATAKANA[u];
        if b = BELL then
          begin
            combo[m] := ASCII_C; { ASCII_C is used as a substitute for hankaku katakana. }
            Inc(m);
            continue;
          end
        else
          begin
            combo[m] := Chr(n);
            Inc(m);
            continue;
          end;

      end
    else
      begin
        if H_80 <= n then
          begin
            combo[m] := ASCII_C; { ASCII_C is used as a substitute for kanji. }
            Inc(m);
            continue;
          end;

      end;

{ ASCII }
    b := BELL_ASCII_KATAKANA[n];
    if b = BELL then
      begin
        continue; { skipping control characters }
      end
    else
      begin
        combo[m] := Chr(n);
        Inc(m);
      end;

  end;

  comboLength := m;
  Result := m;
end;

function ConvertUTF16intoASCII.reading_to_produce_text(nn: String): Integer;
var
  b: Integer;
  buffer_size_1: Integer;
  block_has_been_read: Integer;
  c: WideChar;
  exit_code: Integer;
  i: Longint;
  m: Integer;
  n: Integer;
  s: String;
  testFile: File;
  trace: Integer;
begin
  b := 0;
  buffer_size_1 := buffer_size - 1;
  carriage_return_appears := False;
  count := 0;
  exit_code := 0;
  new_name := nn;

  for n := 0 to buffer_size_1 do
  begin
    buffer[n] := #0;
  end;

  try
{ Reading a file }
    AssignFile(testFile, file_name);
    Reset(testFile, SIZE_OF_WIDECHAR);

    m := 0;
    repeat
      y := CLEAR_ARRAY;
      BlockRead(testFile, y, Y_SIZE, block_has_been_read);

      for n:= 0 to Y_SIZE_1 do
      begin
        buffer[m] := y[n];
        Inc(m);
        if m > buffer_size_1 then
          break;

      end;
    until m > buffer_size_1;

    CloseFile(testFile);
  except
    s := 'Reset() error-119: ' + file_name;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    Halt(1);
  end;

{ Finding the first NEW_LINE }
  m := 0;
  for n := beginning to buffer_size_1 do
  begin
    c := buffer[n];
    i := Ord(c);

    if i = H_0A then
      begin
        count := 1;

        dough[b] := H_0D_C;
        Inc(b);

        dough[b] := H_0A_C;
        Inc(b);

{ Reading a line }
        doughLength := b;
        b := 0;
        m := non_kanji_array(dough, doughLength, m);

        trace := n;
        beginning := trace + 1;
        break;
      end
    else
      begin
        if i = H_0D then
          begin
            carriage_return_appears := True;
            NEW_LINE_LENGTH := 2; { 0Dh and 0Ah }
            NEW_LINE := EMPTY + H_0D_ANSI;
            NEW_LINE := NEW_LINE + H_0A_ANSI;
          end
        else
          begin
            dough[b] := buffer[n];
            Inc(b);
          end;

      end;

  end; { for n := beginning to buffer_size_1 do }

{ Counting lines }
  for n := beginning to buffer_size_1 do
  begin
    if buffer[n] = H_0A_C then
      begin
        Inc(count);

        dough[b] := H_0D_C;
        Inc(b);

        dough[b] := H_0A_C;
        Inc(b);

{ Reading a line }
        doughLength := b;
        b := 0;
        m := non_kanji_array(dough, doughLength, m);
      end
    else
      begin
        if buffer[n] = H_0D_C then
          begin
            continue;
          end
        else
          begin
            dough[b] := buffer[n];
            Inc(b);
          end;

      end;

  end; { for n := beginning to buffer_size_1 do }

  comboLength := m;
  bufferingWriting(m);
  Result := exit_code;
end;

function ConvertUTF16intoASCII.setBufferSize: Integer;
var
  r: Integer;
  s: String;
begin
  r := buffer_size;

  try
    SetLength(buffer, buffer_size);
  except
    s := 'SetLength() error-120: ' + IntToStr(buffer_size);
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    r := BUFFER_ERROR;
  end;

  try
    SetLength(combo, combo_size);
  except
    s := 'SetLength() error-121: ' + IntToStr(combo_size);
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    r := COMBO_ERROR;
  end;

  Result := r;
end;

procedure ConvertUTF16intoASCII.ifErrorOccurs(exit_code: Integer);
begin
  if buffer_size < 0 then
    Halt(exit_code);

end;

procedure ConvertUTF16intoASCII.setting(fn: String; a: Byte);
var
  b: array[0..13] of WideChar;
  block_has_been_read: Integer;
  i: Integer;
  q: array[0..1] of Byte;
  s: String;
  testFile: File;
  w: WideChar;
begin
  ascii_code_byte := a;
  file_name := fn;

  V_SIZE := Sizeof(v);
  V_SIZE_1 := V_SIZE - 1;
  Y_SIZE := SizeOf(y) div SIZE_OF_WIDECHAR;
  Y_SIZE_1 := Y_SIZE - 1;

  for i := 0 to Y_SIZE_1 do
  begin
    CLEAR_ARRAY[i] := #0;
  end;

  for i := 0 to V_SIZE_1 do
  begin
    CLEAR_V_ARRAY[i] := #0;
  end;

  try
    AssignFile(testFile, file_name);
    Reset(testFile, SIZE_OF_ANSICHAR);
    BlockRead(testFile, v, V_SIZE, block_has_been_read);
    q[0] := Ord(v[0]);
    q[1] := Ord(v[1]);
    if (q[0] = H_FF) and (q[1] = H_FE) then
      beginning := BOM_COUNT
    else
      beginning := 0;

    CloseFile(testFile);

    buffer_size := file_size(file_name) div SIZE_OF_WIDECHAR;
    combo_size := EXPANSION_FACTOR * buffer_size;
  except
    s := 'Reset() error-122: ' + file_name;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
  end;

  BELL_ASCII_KATAKANA[0] := 7; BELL_ASCII_KATAKANA[1] := 7; BELL_ASCII_KATAKANA[2] := 7; BELL_ASCII_KATAKANA[3] := 7; BELL_ASCII_KATAKANA[4] := 7; BELL_ASCII_KATAKANA[5] := 7;
  BELL_ASCII_KATAKANA[6] := 7; BELL_ASCII_KATAKANA[7] := 7; BELL_ASCII_KATAKANA[8] := 7; BELL_ASCII_KATAKANA[9] := 7; BELL_ASCII_KATAKANA[10] := H_0A; BELL_ASCII_KATAKANA[11] := 7;
  BELL_ASCII_KATAKANA[12] := 7; BELL_ASCII_KATAKANA[13] := H_0D; BELL_ASCII_KATAKANA[14] := 7; BELL_ASCII_KATAKANA[15] := 7; BELL_ASCII_KATAKANA[16] := 7; BELL_ASCII_KATAKANA[17] := 7;
  BELL_ASCII_KATAKANA[18] := 7; BELL_ASCII_KATAKANA[19] := 7; BELL_ASCII_KATAKANA[20] := 7; BELL_ASCII_KATAKANA[21] := 7; BELL_ASCII_KATAKANA[22] := 7; BELL_ASCII_KATAKANA[23] := 7;
  BELL_ASCII_KATAKANA[24] := 7; BELL_ASCII_KATAKANA[25] := 7; BELL_ASCII_KATAKANA[26] := 7; BELL_ASCII_KATAKANA[27] := 7; BELL_ASCII_KATAKANA[28] := 7; BELL_ASCII_KATAKANA[29] := 7;
  BELL_ASCII_KATAKANA[30] := 7; BELL_ASCII_KATAKANA[31] := 7; BELL_ASCII_KATAKANA[32] := 32; BELL_ASCII_KATAKANA[33] := 33; BELL_ASCII_KATAKANA[34] := 34; BELL_ASCII_KATAKANA[35] := 35;
  BELL_ASCII_KATAKANA[36] := 36; BELL_ASCII_KATAKANA[37] := 37; BELL_ASCII_KATAKANA[38] := 38; BELL_ASCII_KATAKANA[39] := 39; BELL_ASCII_KATAKANA[40] := 40; BELL_ASCII_KATAKANA[41] := 41;
  BELL_ASCII_KATAKANA[42] := 42; BELL_ASCII_KATAKANA[43] := 43; BELL_ASCII_KATAKANA[44] := 44; BELL_ASCII_KATAKANA[45] := 45; BELL_ASCII_KATAKANA[46] := 46; BELL_ASCII_KATAKANA[47] := 47;
  BELL_ASCII_KATAKANA[48] := 48; BELL_ASCII_KATAKANA[49] := 49; BELL_ASCII_KATAKANA[50] := 50; BELL_ASCII_KATAKANA[51] := 51; BELL_ASCII_KATAKANA[52] := 52; BELL_ASCII_KATAKANA[53] := 53;
  BELL_ASCII_KATAKANA[54] := 54; BELL_ASCII_KATAKANA[55] := 55; BELL_ASCII_KATAKANA[56] := 56; BELL_ASCII_KATAKANA[57] := 57; BELL_ASCII_KATAKANA[58] := 58; BELL_ASCII_KATAKANA[59] := 59;
  BELL_ASCII_KATAKANA[60] := 60; BELL_ASCII_KATAKANA[61] := 61; BELL_ASCII_KATAKANA[62] := 62; BELL_ASCII_KATAKANA[63] := 63; BELL_ASCII_KATAKANA[64] := 64; BELL_ASCII_KATAKANA[65] := 65;
  BELL_ASCII_KATAKANA[66] := 66; BELL_ASCII_KATAKANA[67] := 67; BELL_ASCII_KATAKANA[68] := 68; BELL_ASCII_KATAKANA[69] := 69; BELL_ASCII_KATAKANA[70] := 70; BELL_ASCII_KATAKANA[71] := 71;
  BELL_ASCII_KATAKANA[72] := 72; BELL_ASCII_KATAKANA[73] := 73; BELL_ASCII_KATAKANA[74] := 74; BELL_ASCII_KATAKANA[75] := 75; BELL_ASCII_KATAKANA[76] := 76; BELL_ASCII_KATAKANA[77] := 77;
  BELL_ASCII_KATAKANA[78] := 78; BELL_ASCII_KATAKANA[79] := 79; BELL_ASCII_KATAKANA[80] := 80; BELL_ASCII_KATAKANA[81] := 81; BELL_ASCII_KATAKANA[82] := 82; BELL_ASCII_KATAKANA[83] := 83;
  BELL_ASCII_KATAKANA[84] := 84; BELL_ASCII_KATAKANA[85] := 85; BELL_ASCII_KATAKANA[86] := 86; BELL_ASCII_KATAKANA[87] := 87; BELL_ASCII_KATAKANA[88] := 88; BELL_ASCII_KATAKANA[89] := 89;
  BELL_ASCII_KATAKANA[90] := 90; BELL_ASCII_KATAKANA[91] := 91; BELL_ASCII_KATAKANA[92] := 92; BELL_ASCII_KATAKANA[93] := 93; BELL_ASCII_KATAKANA[94] := 94; BELL_ASCII_KATAKANA[95] := 95;
  BELL_ASCII_KATAKANA[96] := 96; BELL_ASCII_KATAKANA[97] := 97; BELL_ASCII_KATAKANA[98] := 98; BELL_ASCII_KATAKANA[99] := 99; BELL_ASCII_KATAKANA[100] := 100; BELL_ASCII_KATAKANA[101] := 101;
  BELL_ASCII_KATAKANA[102] := 102; BELL_ASCII_KATAKANA[103] := 103; BELL_ASCII_KATAKANA[104] := 104; BELL_ASCII_KATAKANA[105] := 105; BELL_ASCII_KATAKANA[106] := 106; BELL_ASCII_KATAKANA[107] := 107;
  BELL_ASCII_KATAKANA[108] := 108; BELL_ASCII_KATAKANA[109] := 109; BELL_ASCII_KATAKANA[110] := 110; BELL_ASCII_KATAKANA[111] := 111; BELL_ASCII_KATAKANA[112] := 112; BELL_ASCII_KATAKANA[113] := 113;
  BELL_ASCII_KATAKANA[114] := 114; BELL_ASCII_KATAKANA[115] := 115; BELL_ASCII_KATAKANA[116] := 116; BELL_ASCII_KATAKANA[117] := 117; BELL_ASCII_KATAKANA[118] := 118; BELL_ASCII_KATAKANA[119] := 119;
  BELL_ASCII_KATAKANA[120] := 120; BELL_ASCII_KATAKANA[121] := 121; BELL_ASCII_KATAKANA[122] := 122; BELL_ASCII_KATAKANA[123] := 123; BELL_ASCII_KATAKANA[124] := 124; BELL_ASCII_KATAKANA[125] := 125;
  BELL_ASCII_KATAKANA[126] := 126; BELL_ASCII_KATAKANA[127] := 127; BELL_ASCII_KATAKANA[128] := 128; BELL_ASCII_KATAKANA[129] := 129; BELL_ASCII_KATAKANA[130] := 130; BELL_ASCII_KATAKANA[131] := 131;
  BELL_ASCII_KATAKANA[132] := 132; BELL_ASCII_KATAKANA[133] := 133; BELL_ASCII_KATAKANA[134] := 134; BELL_ASCII_KATAKANA[135] := 135; BELL_ASCII_KATAKANA[136] := 136; BELL_ASCII_KATAKANA[137] := 137;
  BELL_ASCII_KATAKANA[138] := 138; BELL_ASCII_KATAKANA[139] := 139; BELL_ASCII_KATAKANA[140] := 140; BELL_ASCII_KATAKANA[141] := 141; BELL_ASCII_KATAKANA[142] := 142; BELL_ASCII_KATAKANA[143] := 143;
  BELL_ASCII_KATAKANA[144] := 144; BELL_ASCII_KATAKANA[145] := 145; BELL_ASCII_KATAKANA[146] := 146; BELL_ASCII_KATAKANA[147] := 147; BELL_ASCII_KATAKANA[148] := 148; BELL_ASCII_KATAKANA[149] := 149;
  BELL_ASCII_KATAKANA[150] := 150; BELL_ASCII_KATAKANA[151] := 151; BELL_ASCII_KATAKANA[152] := 152; BELL_ASCII_KATAKANA[153] := 153; BELL_ASCII_KATAKANA[154] := 154; BELL_ASCII_KATAKANA[155] := 155;
  BELL_ASCII_KATAKANA[156] := 156; BELL_ASCII_KATAKANA[157] := 157; BELL_ASCII_KATAKANA[158] := 158; BELL_ASCII_KATAKANA[159] := 159; BELL_ASCII_KATAKANA[160] := 160; BELL_ASCII_KATAKANA[161] := 161;
  BELL_ASCII_KATAKANA[162] := 162; BELL_ASCII_KATAKANA[163] := 163; BELL_ASCII_KATAKANA[164] := 164; BELL_ASCII_KATAKANA[165] := 165; BELL_ASCII_KATAKANA[166] := 7; BELL_ASCII_KATAKANA[167] := 7;
  BELL_ASCII_KATAKANA[168] := 7; BELL_ASCII_KATAKANA[169] := 7; BELL_ASCII_KATAKANA[170] := 7; BELL_ASCII_KATAKANA[171] := 7; BELL_ASCII_KATAKANA[172] := 7; BELL_ASCII_KATAKANA[173] := 7;
  BELL_ASCII_KATAKANA[174] := 7; BELL_ASCII_KATAKANA[175] := 7; BELL_ASCII_KATAKANA[176] := 7; BELL_ASCII_KATAKANA[177] := 7; BELL_ASCII_KATAKANA[178] := 7; BELL_ASCII_KATAKANA[179] := 7;
  BELL_ASCII_KATAKANA[180] := 7; BELL_ASCII_KATAKANA[181] := 7; BELL_ASCII_KATAKANA[182] := 7; BELL_ASCII_KATAKANA[183] := 7; BELL_ASCII_KATAKANA[184] := 7; BELL_ASCII_KATAKANA[185] := 7;
  BELL_ASCII_KATAKANA[186] := 7; BELL_ASCII_KATAKANA[187] := 7; BELL_ASCII_KATAKANA[188] := 7; BELL_ASCII_KATAKANA[189] := 7; BELL_ASCII_KATAKANA[190] := 7; BELL_ASCII_KATAKANA[191] := 7;
  BELL_ASCII_KATAKANA[192] := 7; BELL_ASCII_KATAKANA[193] := 7; BELL_ASCII_KATAKANA[194] := 7; BELL_ASCII_KATAKANA[195] := 7; BELL_ASCII_KATAKANA[196] := 7; BELL_ASCII_KATAKANA[197] := 7;
  BELL_ASCII_KATAKANA[198] := 7; BELL_ASCII_KATAKANA[199] := 7; BELL_ASCII_KATAKANA[200] := 7; BELL_ASCII_KATAKANA[201] := 7; BELL_ASCII_KATAKANA[202] := 7; BELL_ASCII_KATAKANA[203] := 7;
  BELL_ASCII_KATAKANA[204] := 7; BELL_ASCII_KATAKANA[205] := 7; BELL_ASCII_KATAKANA[206] := 7; BELL_ASCII_KATAKANA[207] := 7; BELL_ASCII_KATAKANA[208] := 7; BELL_ASCII_KATAKANA[209] := 7;
  BELL_ASCII_KATAKANA[210] := 7; BELL_ASCII_KATAKANA[211] := 7; BELL_ASCII_KATAKANA[212] := 7; BELL_ASCII_KATAKANA[213] := 7; BELL_ASCII_KATAKANA[214] := 7; BELL_ASCII_KATAKANA[215] := 7;
  BELL_ASCII_KATAKANA[216] := 7; BELL_ASCII_KATAKANA[217] := 7; BELL_ASCII_KATAKANA[218] := 7; BELL_ASCII_KATAKANA[219] := 7; BELL_ASCII_KATAKANA[220] := 7; BELL_ASCII_KATAKANA[221] := 7;
  BELL_ASCII_KATAKANA[222] := 7; BELL_ASCII_KATAKANA[223] := 7;
  EMPTY_ANSI := '';

  H_FF_ANSI := Chr(H_FF);
  ASCII_C := Chr(ascii_code_byte);

{ Characters of WIdeChar type }
  w := WideChar(#0);
  b[0] := w;
  for i := 1 to H_0D do
  begin
    Inc(w);
    b[i] := w;
  end;

  H_0A_C := b[H_0A];
  H_0D_C := b[H_0D];
  H_0A_ANSI := Chr(H_0A);
  H_0D_ANSI := Chr(H_0D);

  NEW_LINE := EMPTY + H_0D_ANSI;
  NEW_LINE := NEW_LINE + H_0A_ANSI;
  NEW_LINE_LENGTH := 2;

  setBufferSize;
end;

end.
