unit SurvivorUTF16;

interface

uses
  Windows, SysUtils, ErrorText, PathEXE, MessagesText, FinalEffect, SearchBinary, DivideSJIS, FieldsArray;

type OpenCloseFunction = function: String;

type AnsiCharArray = array[0..127] of AnsiChar;
type CharArray = array[0..4095] of Char;

type
  SurvivorsUTF16 = class
  private
    a: AnsiCharArray;
    A_SIZE: Integer;
    A_SIZE_1: Integer;
    ASCII_C: AnsiChar;
    B_110110_S: Integer;
    B_110111_S: Integer;
    beginning: Integer;
    buffer: array of Char;
    buffer_length: Integer;
    carriage_return_appears: Boolean;
    CLEAR_ARRAY: AnsiCharArray;
    CLEAR_Y_ARRAY: CharArray;
    count: Integer;
    d: DividingSJIS;
    dough: array[0..128] of Char;
    doughLength: Integer;
    E: array[0..255] of Integer;
    f: FieldsList;
    field_count: Integer;
    first_byte_of_Shift_JIS_kanji: array[0..127, 0..1023] of AnsiChar;
    H: array[0..7] of Integer;
    H_0A_C: Char;
    H_0D_C: Char;
    higher_part_of_kanji: Integer;
    lower_part_of_kanji: Integer;
    KANJI_C: AnsiChar;
    KATAKANA_C: AnsiChar;
    LOWER_PART_MASK: Integer;
    m: MessagesTexts;
    NEW_LINE: String;
    NEW_LINE_SIZE: Integer;
    NON_KANJI_C: AnsiChar;
    open_close: array[1..4] of OpenCloseFunction;
    SET_BIT_C: Char;
    SURROGATE_PAIR_C: AnsiChar;
    SURRO_PATTERN: array[1..7] of Char;
    second_byte_of_Shift_JIS_kanji: array[0..127, 0..1023] of AnsiChar;
    y: CharArray;
    Y_COUNT: Integer;
    Y_COUNT_1: Integer;
    z: array[1..262040] of AnsiChar;
    ZERO_CHAR: AnsiChar;
    function E_table: Integer;
    function file_size(name: String): Integer;
    function hexadecimalCharacter(c: Char): Boolean;
    function setBufferSize: Integer;
{*

UTF16 encoding: 65509 (FFE5h)
( 65509 + 1 ) * ( FLAG_SIZE + UTF16_KANJI_SIZE + 1 ) = 262040
*}
  public
    function hexadecimal_sequence_to_integer(s: PChar; s_length: Integer): Integer;
    function newLine: String;
    function sequence_to_integer(s: String): Integer;
    function surrogatePairSetting: Integer;
    function surrogatePairUnderline: Integer;
    function working(index: Integer; place: Integer): AnsiChar;
    function workingPair(high_part_of_kanji: Integer; low_part_of_kanji: Integer; place: Integer): AnsiChar;
    procedure clear;
    procedure populated(index: Integer; place: Integer; c: AnsiChar);
    procedure setting(BINARY_TABLE: String); overload;
    procedure setting(e: SearchTable); overload;
    procedure setting(f: FinalEffects); overload;
  end;

const
  B_110110 = 54;
  B_110111 = 55;
  BUFFER_ERROR = -1;
  C_ERR = 'err';
  C_UTF16_MS932_BIN = 'C:\Sjisutf16\UTF16~MS932.bin';
  DATA_TEXT = 'data.txt';
  DO_NOT_REMOVE_SEPARATORS = False;
  EMPTY = '';
  FILE_ERROR = -2;
  FIRST_PART_OF_KANJI = 1; { Field-2 }
  FLAG_SIZE = 1;
  H_00010000 = 65536;
  H_30 = 48; { '0' }
  H_31 = 49; { '1' }
  H_39 = 57; { '9' }
  H_41 = 65; { 'A' }
  H_42 = 66; { 'B' }
  H_46 = 70; { 'F' }
  H_D840 = 55360;
  H_DC02 = 56322;
  H_FE = 254;
  H_FF = 255;
  HEXADECIMAL_CHARACTERS = '0123456789ABCDEF';
  NOT_CONVERTED = -1;
  RECORD_SIZE = 5;
  SEARCH_COUNT = 4;
  SET_BIT = 49; { '1' }
  SETTING_TEXT = 'set_utf16sjis.txt';
  SIZE_OF_ANSICHAR = 1;
  SUBSTITUTE_FOR_SURROGATE_PAIR = 6; { Field-7 }
  SURRO_COUNT = 7;
  SURRO_SJIS_CSV = 'SurroSJIS.csv';
  SURRO_QUOTE = 34; { 22h, '"' }
  SURRO_SEPARATOR = 44; { 2Ch, ',' }
  SURRO_TEXT = 'surro.txt';
  SUBSTITUTE_FOR_SURROGATE_PAIR_DATA = 'Q'; { Shift_JIS underline }
  TABLE_ARRAY_INDEX = 5; { Field-6 }
  TABLE_NUMBER = 4; { Field-5 }
  TEXT_FILE = '.txt';
  UTF16_KANJI_SIZE = 2;
  UTF16_KANJI_SIZE_2 = 4; { FLAG_SIZE + UTF16_KANJI_SIZE + 1 }
  UTF16_MS932_BIN = 'UTF16~MS932.bin';

var
  x: ErrorTexts;

implementation

function binary_file_in_current_directory: String;
var
  s: String;
  testFile: File;
begin
  s := EMPTY;

  try
    Assign(testFile, UTF16_MS932_BIN);
    Reset(testFile);
    CloseFile(testFile);
    s := UTF16_MS932_BIN;
  except
    s := 'Reset() error-241: ' + UTF16_MS932_BIN;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    s := EMPTY;
  end;

  Result := s;
end;

function binary_file_in_installation_directory: String;
var
  s: String;
  testFile: File;
begin
  s := EMPTY;

  try
    Assign(testFile, C_UTF16_MS932_BIN);
    Reset(testFile);
    CloseFile(testFile);
    s := C_UTF16_MS932_BIN;
  except
    s := 'Reset() error-242: ' + C_UTF16_MS932_BIN;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    s := EMPTY;
  end;

  Result := s;
end;

function binary_file_in_path_directory: String;
{ directory included in Path }
var
  p: CommandPath;
  s: String;
begin
  p := CommandPath.Create;
  p.setting(True);
  s := p.pathName(UTF16_MS932_BIN);
  p.Free;

  Result := s;
end;

function binary_file_in_selected_directory: String;
{ directory that may have priority }
var
  messages: String;
  s: String;
  settingFile: TextFile;
  testFile: File;
begin
  s := EMPTY;

  try
    Assign(settingFile, SETTING_TEXT);
    Reset(settingFile);
    ReadLn(settingFile, s); { Line-1 }
    CloseFile(settingFile);
  except
    messages := 'Reset() error-243: ' + SETTING_TEXT;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, messages);
    x.Free;
    s := EMPTY;
  end;

  if Length(s) > 0 then
    begin
      try
        Assign(testFile, s);
        Reset(testFile);
        CloseFile(testFile);
      except
        messages := 'Reset() error-244: ' + s;
        x := ErrorTexts.Create;
        x.setting(C_ERR, TEXT_FILE, messages);
        x.Free;
        s := EMPTY;
      end;

    end;

  Result := s;
end;

function SurvivorsUTF16.E_table: Integer;
var
  beginning_of_E: Integer;
  end_1: Integer;
  h: Integer;
  i: Integer;
  k: Integer;
  m: Integer;
begin
  for i := 0 to H_FF do
  begin
    E[i] := NOT_CONVERTED;
  end;

{ 0 to 9 }
  beginning_of_E := H_30;

  end_1 := H_39 - 1;
  E[H_30] := 0;
  h := E[H_30];
  m := H_31;

  for k := beginning_of_E to end_1 do
  begin
    Inc(h);
    E[m] := h;
    Inc(m);
  end;

{ A to F }
  beginning_of_E := H_41;

  end_1 := H_46 - 1;
  E[H_41] := 10; { 'A' }
  h := E[H_41];
  m := H_42;

  for k := beginning_of_E to end_1 do
  begin
    Inc(h);
    E[m] := h;
    Inc(m);
  end;

  Result := h; { 'F' }
end;

function SurvivorsUTF16.file_size(name: String): Integer;
var
  f: Integer;
  size: Integer;
begin
  f := FileOpen(name, fmOpenRead or fmShareDenyNone);
  size := FileSeek(f, 0, 2); { EOF }
  FileClose(f);
  Result := size;
end;

function SurvivorsUTF16.setBufferSize: Integer;
var
  r: Integer;
  s: String;
begin
  r := buffer_length;

  try
    SetLength(buffer, buffer_length);
  except
    s := 'SetLength() error-245: ' + IntToStr(buffer_length);
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    r := BUFFER_ERROR;
  end;

  Result := r;
end;

function SurvivorsUTF16.hexadecimal_sequence_to_integer(s: PChar; s_length: Integer): Integer;
var
  c: Char;
  g: String;
  g_length: Integer;
  i: Longint;
  k: Integer;
  m: Integer;
  messages: String;
  n: Integer;
  s_length_1: Integer;
  term: Integer;
begin
  g := EMPTY;
  i := 0;
  m := 0;
  n := 0;
  s_length_1 := s_length - 1;

  for k := 0 to s_length_1 do
  begin
    c := s[k];
    if hexadecimalCharacter(c) then
      begin
        g := g + c;
      end
    else
      begin
        break;
      end;

  end;

  g_length := Length(g);
  try
    for k := g_length downto 1 do
    begin
      i := Ord(g[k]);
      term := E[i] * H[m];
      if term < 0 then
        raise Exception.Create('hexadecimal_sequence_to_integer() error');

      n := n + term;
      Inc(m);
    end;

  except
    messages := 'hexadecimal_sequence_to_integer() error-246: i = ' + IntToStr(i);
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, messages);
    x.Free;
    n := NOT_CONVERTED;
    Halt(1);
  end;

  Result := n;
end;

function SurvivorsUTF16.hexadecimalCharacter(c: Char): Boolean;
var
  q: Integer;
  r: Boolean;
begin
  q := AnsiPos(c, HEXADECIMAL_CHARACTERS);
  if q > 0 then
    r := True
  else
    r := False;

  Result := r;
end;

function SurvivorsUTF16.newLine: String;
begin
  Result := NEW_LINE;
end;

function SurvivorsUTF16.sequence_to_integer(s: String): Integer;
var
  k: Integer;
  messages: String;
begin
  k := NOT_CONVERTED;

  try
    k := StrToInt(s);
  except
{ EConvertError }
    x := ErrorTexts.Create;
    messages := 'sequence_to_integer() error-247: ' + s;
    x.setting(C_ERR, TEXT_FILE, messages);
    x.Free;
  end;

  Result := k;
end;

function SurvivorsUTF16.surrogatePairSetting: Integer;
var
  b: Integer;
  block_has_been_read: Integer;
  buffer_length_1: Integer;
  exit_code: Integer;
  g: String;
  i: Integer;
  k: Integer;
  m: Integer;
  n: Integer;
  s: String;
  testFile: File;
  trace: Integer;
  u: Integer;
begin
  H_0A_C := Chr(H_0A);
  H_0D_C := Chr(H_0D);
  Y_COUNT := SizeOf(y);
  Y_COUNT_1 := Y_COUNT - 1;

  for n := 1 to SURRO_COUNT do
  begin
    SURRO_PATTERN[n] := SET_BIT_C;
  end;

  try
    AssignFile(testFile, SURRO_SJIS_CSV);
    Reset(testFile, SIZE_OF_ANSICHAR);
    beginning := 0;
    CloseFile(testFile);

    buffer_length := file_size(SURRO_SJIS_CSV);
  except
    s := 'Reset() error-248: ' + SURRO_SJIS_CSV;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    Halt(1);
  end;

  for m := 0 to Y_COUNT_1 do
  begin
    CLEAR_Y_ARRAY[m] := #0;
  end;

  b := 0;
  buffer_length_1 := buffer_length - 1;
  carriage_return_appears := False;
  count := 0;
  exit_code := 0;
  m := 0;

  setBufferSize;
  for n := 0 to buffer_length_1 do
  begin
    buffer[n] := #0;
  end;

  try
{ Reading a file }
    AssignFile(testFile, SURRO_SJIS_CSV);
    Reset(testFile, SIZE_OF_ANSICHAR);

    repeat
      y := CLEAR_Y_ARRAY;
      BlockRead(testFile, y, Y_COUNT, block_has_been_read);

      for n:= 0 to Y_COUNT_1 do
      begin
        buffer[m] := y[n];
        Inc(m);
        if m > buffer_length_1 then
          break;

      end;
    until m > buffer_length_1;

    CloseFile(testFile);
  except
    s := 'Reset() error-249: ' + SURRO_SJIS_CSV;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    Halt(1);
  end;

{ Finding the first NEW_LINE }
  for n := beginning to buffer_length_1 do
  begin
    if buffer[n] = H_0A_C then
      begin
        count := 1;

{ Reading a line }
        d.Free;
        f.Free;
        f := FieldsList.Create;
        d := DividingSJIS.Create;
        d.presetting(SURRO_QUOTE, SURRO_SEPARATOR, DO_NOT_REMOVE_SEPARATORS);
        d.setting(f, SURRO_COUNT, SURRO_PATTERN);

        doughLength := b;
        b := 0;
        field_count := d.mapping_fields_list(dough, doughLength);
        i := sequence_to_integer( f.item(TABLE_NUMBER) );
        k := sequence_to_integer( f.item(TABLE_ARRAY_INDEX) );
        g := f.item(SUBSTITUTE_FOR_SURROGATE_PAIR);
        first_byte_of_Shift_JIS_kanji[i, k] := g[1];
        second_byte_of_Shift_JIS_kanji[i, k] := g[2];

        s := f.item(FIRST_PART_OF_KANJI);
        u := hexadecimal_sequence_to_integer( PChar(s), Length(s) );
        populated(u, 0, SURROGATE_PAIR_C);
        populated(u, 1, Chr(i) );
        populated(u, 2, NON_KANJI_C);

        trace := n;
        beginning := trace + 1;
        break;
      end
    else
      begin
        if buffer[n] = H_0D_C then
          begin
            carriage_return_appears := True;
            NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
            NEW_LINE := EMPTY + H_0D_C;
            NEW_LINE := NEW_LINE + H_0A_C;
          end
        else
          begin
            dough[b] := buffer[n];
            Inc(b);
          end; { if buffer[n] = H_0D_C then, else }

      end; { if buffer[n] = H_0A_C then, else }

  end; { for n := beginning to buffer_length_1 do }

{ Counting lines }
  for n := beginning to buffer_length_1 do
  begin
    if buffer[n] = H_0A_C then
      begin
        Inc(count);

{ Reading a line }
        d.Free;
        f.Free;
        f := FieldsList.Create;
        d := DividingSJIS.Create;
        d.presetting(SURRO_QUOTE, SURRO_SEPARATOR, DO_NOT_REMOVE_SEPARATORS);
        d.setting(f, SURRO_COUNT, SURRO_PATTERN);

        doughLength := b;
        b := 0;
        field_count := d.mapping_fields_list(dough, doughLength);
        i := sequence_to_integer( f.item(TABLE_NUMBER) );
        k := sequence_to_integer( f.item(TABLE_ARRAY_INDEX) );
        g := f.item(SUBSTITUTE_FOR_SURROGATE_PAIR);
        first_byte_of_Shift_JIS_kanji[i, k] := g[1];
        second_byte_of_Shift_JIS_kanji[i, k] := g[2];

        s := f.item(FIRST_PART_OF_KANJI);
        u := hexadecimal_sequence_to_integer( PChar(s), Length(s) );
        populated(u, 0, SURROGATE_PAIR_C);
        populated(u, 1, Chr(i) );
        populated(u, 2, NON_KANJI_C);
      end
    else
      begin
        if buffer[n] = H_0D_C then
          begin
            continue;
          end
        else
          begin
            dough[b] := buffer[n];
            Inc(b);
          end; { if buffer[n] = H_0D_C then, else }

      end; { if buffer[n] = H_0A_C then, else }

  end; { for n := beginning to buffer_length_1 do }

  d.Free; f.Free;
  Result := exit_code;
end;

function SurvivorsUTF16.surrogatePairUnderline: Integer;
var
  basic_code: Integer;
  exit_code: Integer;
  g: String;
  i: Integer;
  k: Integer;
  code_point: Integer;
  s: String;
  s_length: Integer;
  testFile: TextFile;
begin
  exit_code := 0;
  LOWER_PART_MASK := 1 + 2 + 4 + 8 + 16 + 32 + 64 + 128 + 256 + 512;

  try
    AssignFile(testFile, SURRO_TEXT);
    Reset(testFile);

    while not Eof(testFile) do
    begin
      ReadLn(testFile, s);
      s_length := Length(s);
      if s_length = 0 then
        break;

      code_point := hexadecimal_sequence_to_integer( PChar(s), Length(s) );

{ Four bytes of UTF-16 }
      basic_code := code_point - H_00010000;
      k := basic_code shr 10;
      higher_part_of_kanji := k + B_110110_S;

      k := basic_code and LOWER_PART_MASK;
      lower_part_of_kanji := k + B_110111_S;

{ Higher-part of kanji will be useful for arrays if software subtracts H_D840 from the kanji code. }
      i := higher_part_of_kanji - H_D840;

{ Lower-part of kanji will be useful for arrays if software subtracts H_DC02 from the kanji code. }
      k := lower_part_of_kanji - H_DC02;

      populated(higher_part_of_kanji, 0, SURROGATE_PAIR_C);
      populated(higher_part_of_kanji, 1, Chr(i) );
      populated(higher_part_of_kanji, 2, NON_KANJI_C);

      g := SUBSTITUTE_FOR_SURROGATE_PAIR_DATA;
      first_byte_of_Shift_JIS_kanji[i, k] := g[1];
      second_byte_of_Shift_JIS_kanji[i, k] := g[2];
    end;

    CloseFile(testFile);
  except
    s := 'Reset() error-191: ' + SURRO_TEXT;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    exit_code := FILE_ERROR;
  end;

  Result := exit_code;
end;
function SurvivorsUTF16.working(index: Integer; place: Integer): AnsiChar;
var
  k: Integer;
begin
{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
  k := UTF16_KANJI_SIZE_2 * index + place;
  Result := z[k];
end;

function SurvivorsUTF16.workingPair(high_part_of_kanji: Integer; low_part_of_kanji: Integer; place: Integer): AnsiChar;
var
  k: Integer;
  r: AnsiChar;
  tableNumber: Integer;
begin
{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
  tableNumber := high_part_of_kanji - H_D840;
  k := low_part_of_kanji - H_DC02;

  case place of
    1: r := first_byte_of_Shift_JIS_kanji[tableNumber, k];
    2: r := second_byte_of_Shift_JIS_kanji[tableNumber, k];
  else
    r := ZERO_CHAR;
  end; { case i of, else }

  Result := r;
end;

procedure SurvivorsUTF16.populated(index: Integer; place: Integer; c: AnsiChar);
var
  k: Integer;
begin
{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
  k := UTF16_KANJI_SIZE_2 * index + place;
  z[k] := c;
end;

procedure SurvivorsUTF16.clear;
var
  k: Integer;
begin
  for k := 1 to 262040 do
  begin
    z[k] := #0;
  end;

end;

procedure SurvivorsUTF16.setting(BINARY_TABLE: String);
var
  b: Longint;
  block_has_been_read: Integer;
  i: Integer;
  k: Integer;
  messages: String;
  tableFile: File;
begin
  A_SIZE := SizeOf(a);
  A_SIZE_1 := A_SIZE - 1;
  ASCII_C := #0;
  KANJI_C := #1;
  KATAKANA_C := #2;
  NON_KANJI_C := #0;
  SET_BIT_C := Chr(SET_BIT);
  SURROGATE_PAIR_C := #3;
  ZERO_CHAR := #0;

  H[0] := 1;
  H[1] := 16;
  H[2] := 256;
  H[3] := 4096;
  H[4] := 65536;
  H[5] := 1048576;
  H[6] := 16777216;
  H[7] := 268435456;

  B_110110_S := B_110110 shl 10;
  B_110111_S := B_110111 shl 10;
  E_table;

  for i := 0 to A_SIZE_1 do
  begin
    CLEAR_ARRAY[i] := #0;
  end;

{ Reading BINARY_TABLE to produce a table in the main memory }
  try
    AssignFile(tableFile, BINARY_TABLE);
    Reset(tableFile, RECORD_SIZE);

    repeat
      a := CLEAR_ARRAY;
      BlockRead(tableFile, a, 1, block_has_been_read);

      if a[2] = ASCII_C then
        begin
          k := Ord(a[0]);
          populated(k, 0, ASCII_C);
          populated(k, 1, a[3]);
          populated(k, 2, NON_KANJI_C);
        end
      else
        begin
          if a[2] = KATAKANA_C then
            begin
              b := Ord(a[1]);
              k := b shl 8;

              b := Ord(a[0]); { little-endian architecture }
              k := k + b;

              populated(k, 0, KATAKANA_C);
              populated(k, 1, a[3]);
              populated(k, 2, NON_KANJI_C);
            end
          else
            begin
{ kanji }
              b := Ord(a[1]);
              k := b shl 8;

              b := Ord(a[0]); { little-endian architecture }
              k := k + b;

              populated(k, 0, KANJI_C);
              populated(k, 1, a[3]);
              populated(k, 2, a[4]);
            end;

        end;

    until block_has_been_read = 0;

    CloseFile(tableFile);
  except
    messages := 'Reset() error-250: ' + BINARY_TABLE;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, messages);
    x.Free;
  end;

end;

procedure SurvivorsUTF16.setting(e: SearchTable);
var
  b: Longint;
  beginning: Integer;
  binaryTable: String;
  block_has_been_read: Integer;
  i: Integer;
  k: Integer;
  messages: String;
  tableFile: File;
begin
  A_SIZE := SizeOf(a);
  A_SIZE_1 := A_SIZE - 1;
  ASCII_C := #0;
  KANJI_C := #1;
  KATAKANA_C := #2;
  NON_KANJI_C := #0;
  SET_BIT_C := Chr(SET_BIT);
  SURROGATE_PAIR_C := #3;
  ZERO_CHAR := #0;

  H[0] := 1;
  H[1] := 16;
  H[2] := 256;
  H[3] := 4096;
  H[4] := 65536;
  H[5] := 1048576;
  H[6] := 16777216;
  H[7] := 268435456;

  B_110110_S := B_110110 shl 10;
  B_110111_S := B_110111 shl 10;
  E_table;

  for i := 0 to A_SIZE_1 do
  begin
    CLEAR_ARRAY[i] := #0;
  end;

{ Finding UTF16_MS932_BIN }
  open_close[1] := binary_file_in_installation_directory;
  open_close[2] := binary_file_in_current_directory;
  open_close[3] := binary_file_in_path_directory;
  open_close[4] := binary_file_in_selected_directory;

  if e.priority then
    begin
      beginning := SEARCH_COUNT;
    end
  else
    begin
      beginning := 1;
    end;

  for i := beginning to SEARCH_COUNT do
  begin
    binaryTable := open_close[i];

    if Length(binaryTable) > 0 then
      begin
        if e.whichTable then
          begin
            m := MessagesTexts.Create;
            messages := binaryTable + ' is primarily accessible to Utf16sjis.';
            m.setting(DATA_TEXT, messages);
            m.Free;
          end;

        break;
      end
    else
      begin
        case i of
          1: { beginning }
            begin
              if e.noSearching then
                begin
                  messages := 'Reset() error-251: ' + C_UTF16_MS932_BIN;
                  x := ErrorTexts.Create;
                  x.setting(C_ERR, TEXT_FILE, messages);
                  x.Free;
                  Halt(1);
                end;

            end;

          4: { SEARCH_COUNT }
            begin
              messages := 'Reset() error-252: i = ' + IntToStr(SEARCH_COUNT);
              x := ErrorTexts.Create;
              x.setting(C_ERR, TEXT_FILE, messages);
              x.Free;
              Halt(1);
            end;
        else
          continue;
        end; { case i of, else }

      end; { if Length(binaryTable) > 0 then, else }

  end; { for i := beginning to SEARCH_COUNT do }

  if Length(binaryTable) > 0 then
    begin
{ Reading UTF16_MS932_BIN to produce a table in the main memory }
      AssignFile(tableFile, binaryTable);
      Reset(tableFile, RECORD_SIZE);

      repeat
        a := CLEAR_ARRAY;
        BlockRead(tableFile, a, 1, block_has_been_read);

        if a[2] = ASCII_C then
          begin
            k := Ord(a[0]);
            populated(k, 0, ASCII_C);
            populated(k, 1, a[3]);
            populated(k, 2, NON_KANJI_C);
          end
        else
          begin
            if a[2] = KATAKANA_C then
              begin
                b := Ord(a[1]);
                k := b shl 8;

                b := Ord(a[0]); { little-endian architecture }
                k := k + b;

                populated(k, 0, KATAKANA_C);
                populated(k, 1, a[3]);
                populated(k, 2, NON_KANJI_C);
              end
            else
              begin
{ kanji }
                b := Ord(a[1]);
                k := b shl 8;

                b := Ord(a[0]); { little-endian architecture }
                k := k + b;

                populated(k, 0, KANJI_C);
                populated(k, 1, a[3]);
                populated(k, 2, a[4]);
              end;

          end;

      until block_has_been_read = 0;

      CloseFile(tableFile);
    end
  else
    begin
      messages := 'Length() error-253: Length(binaryTable) = 0';
      x := ErrorTexts.Create;
      x.setting(C_ERR, TEXT_FILE, messages);
      x.Free;
      Halt(1);
    end;

end;

procedure SurvivorsUTF16.setting(f: FinalEffects);
var
  b: Longint;
  beginning: Integer;
  binaryTable: String;
  block_has_been_read: Integer;
  i: Integer;
  k: Integer;
  messages: String;
  tableFile: File;
begin
  A_SIZE := SizeOf(a);
  A_SIZE_1 := A_SIZE - 1;
  ASCII_C := #0;
  KANJI_C := #1;
  KATAKANA_C := #2;
  NON_KANJI_C := #0;
  SET_BIT_C := Chr(SET_BIT);
  SURROGATE_PAIR_C := #3;
  ZERO_CHAR := #0;

  H[0] := 1;
  H[1] := 16;
  H[2] := 256;
  H[3] := 4096;
  H[4] := 65536;
  H[5] := 1048576;
  H[6] := 16777216;
  H[7] := 268435456;

  B_110110_S := B_110110 shl 10;
  B_110111_S := B_110111 shl 10;
  E_table;

  for i := 0 to A_SIZE_1 do
  begin
    CLEAR_ARRAY[i] := #0;
  end;

{ Finding UTF16_MS932_BIN }
  open_close[1] := binary_file_in_installation_directory;
  open_close[2] := binary_file_in_current_directory;
  open_close[3] := binary_file_in_path_directory;
  open_close[4] := binary_file_in_selected_directory;

  if f.priority then
    begin
      beginning := SEARCH_COUNT;
    end
  else
    begin
      beginning := 1;
    end;

  for i := beginning to SEARCH_COUNT do
  begin
    binaryTable := open_close[i];

    if Length(binaryTable) > 0 then
      begin
        if f.whichTable then
          begin
            m := MessagesTexts.Create;
            messages := binaryTable + ' is primarily accessible to Utf16sjis.';
            m.setting(DATA_TEXT, messages);
            m.Free;
          end;

        break;
      end
    else
      begin
        case i of
          1: { beginning }
            begin
              if f.noSearching then
                begin
                  messages := 'Reset() error-254: ' + C_UTF16_MS932_BIN;
                  x := ErrorTexts.Create;
                  x.setting(C_ERR, TEXT_FILE, messages);
                  x.Free;
                  Halt(1);
                end;

            end;

          4: { SEARCH_COUNT }
            begin
              messages := 'Reset() error-255: i = ' + IntToStr(SEARCH_COUNT);
              x := ErrorTexts.Create;
              x.setting(C_ERR, TEXT_FILE, messages);
              x.Free;
              Halt(1);
            end;
        else
          continue;
        end; { case i of, else }

      end; { if Length(binaryTable) > 0 then, else }

  end; { for i := beginning to SEARCH_COUNT do }

  if Length(binaryTable) > 0 then
    begin
{ Reading UTF16_MS932_BIN to produce a table in the main memory }
      AssignFile(tableFile, binaryTable);
      Reset(tableFile, RECORD_SIZE);

      repeat
        a := CLEAR_ARRAY;
        BlockRead(tableFile, a, 1, block_has_been_read);

        if a[2] = ASCII_C then
          begin
            k := Ord(a[0]);
            populated(k, 0, ASCII_C);
            populated(k, 1, a[3]);
            populated(k, 2, NON_KANJI_C);
          end
        else
          begin
            if a[2] = KATAKANA_C then
              begin
                b := Ord(a[1]);
                k := b shl 8;

                b := Ord(a[0]); { little-endian architecture }
                k := k + b;

                populated(k, 0, KATAKANA_C);
                populated(k, 1, a[3]);
                populated(k, 2, NON_KANJI_C);
              end
            else
              begin
{ kanji }
                b := Ord(a[1]);
                k := b shl 8;

                b := Ord(a[0]); { little-endian architecture }
                k := k + b;

                populated(k, 0, KANJI_C);
                populated(k, 1, a[3]);
                populated(k, 2, a[4]);
              end;

          end;

      until block_has_been_read = 0;

      CloseFile(tableFile);
    end
  else
    begin
      messages := 'Length() error-256: Length(binaryTable) = 0';
      x := ErrorTexts.Create;
      x.setting(C_ERR, TEXT_FILE, messages);
      x.Free;
      Halt(1);
    end;

end;

end.
