program Utf16sjis;
{$APPTYPE CONSOLE}

{*
Utf16sjis FPC Version 1.05
Converting the text from UTF-16 to MS932
Copyright 2020 Sogaya

Free Pascal Compiler 3.0.4

*}

uses
  SysUtils,
  AnsiQueue in 'AnsiQueue.pas',
  ConvertNarrow in 'ConvertNarrow.pas',
  ConvertUTF16 in 'ConvertUTF16.pas',
  DeriveBinary in 'DeriveBinary.pas',
  DivideSJIS in 'DivideSJIS.pas',
  ErrorText in 'ErrorText.pas',
  FieldsArray in 'FieldsArray.pas',
  FinalEffect in 'FinalEffect.pas',
  Guide in 'Guide.pas',
  HyphenTerm in 'HyphenTerm.pas',
  MessagesText in 'MessagesText.pas',
  Numbered in 'Numbered.pas',
  PathEXE in 'PathEXE.pas',
  PutCSV in 'PutCSV.pas',
  SearchBinary in 'SearchBinary.pas',
  SurvivorUTF16 in 'SurvivorUTF16.pas',
  TimeName in 'TimeName.pas',
  Ucombo in 'Ucombo.pas',
  SelectField in 'SelectField.pas';

{$R *.RES}

const
  ASCII_TEXT = 'ascii.txt';
  BUFFER_ERROR = -1;
  DATA_TEXT = 'data.txt';
  FIELD_4 = 4;
  REMOVE_ANCIENT_FILE = false;
  SURRO_CSV = 'surro.csv';
  SURRO_SJIS_CSV = 'SurroSJIS.csv';
  SURRO_TEXT = 'surro.txt';
  TEMPORARY_BINARY = 'binary.tmp';
  UNDERLINES_REPLACE_SURROGATE_PAIRS = True;
  VERSION_NUMBER = '1.05';
  VERSION_WITH_COMMENTS = True;
  VERSION_WITHOUT_COMMENTS = False;

var
  a: ConvertUTF16intoASCII;
  c: Converting;
  carriage_return: Boolean;
  d: DeriveTable;
  dataFile: TextFile;
  f: FinalEffects;
  field_to_be_selected: Integer;
  file_name_length: Integer;
  g: Guiding;
  h: HyphenTerms;
  maximumFieldCount: Integer;
  new_name: String;
  new_name_length: Integer;
  p: PutSurroCSV;
  s: String;
  t: FinalEffects;
  testFile: TextFile;
  u: Selecting;

begin
  s := ParamStr(1);
  if Length(s) = 0 then
    begin
      d := DeriveTable.Create;
      d.setting;
      Halt(0);
    end;

{ Checking if software can open SURRO_CSV in the current directory. }
  try
    AssignFile(testFile, SURRO_CSV);
    Reset(testFile);
    CloseFile(testFile);
  except
{ Producing SURRO_CSV }
    p := PutSurroCSV.Create;
    p.setting(SURRO_CSV);
  end;

{ Checking if software can open SURRO_TEXT in the current directory. }
  try
    AssignFile(testFile, SURRO_TEXT);
    Reset(testFile);
    CloseFile(testFile);
  except
{ Producing SURRO_TEXT from SURRO_CSV }
    t := FinalEffects.Create;
    t.setting(SURRO_CSV, SURRO_TEXT, FIELD_4, UNDERLINES_REPLACE_SURROGATE_PAIRS);
    u := Selecting.Create;
    u.presetting(t);
    u.setting(t,SURRO_CSV);
    u.ifErrorOccurs(BUFFER_ERROR);

    u.reading_to_produce_MS932_CRLF(SURRO_TEXT);
    u.Free; t.Free;
  end;

{ Checking if software can open SURRO_SJIS_CSV in the current directory. }
  try
    AssignFile(testFile, SURRO_SJIS_CSV);
    Reset(testFile);
    CloseFile(testFile);
  except
{ Producing SURRO_SJIS_CSV in the current directory. }
    t := FinalEffects.Create;
    t.setting(SURRO_CSV, SURRO_SJIS_CSV, FIELD_4, UNDERLINES_REPLACE_SURROGATE_PAIRS);

    c := Converting.Create;
    c.setting(t, SURRO_CSV);
    c.ifErrorOccurs(BUFFER_ERROR);
    c.reading_to_produce_MS932_CRLF(SURRO_SJIS_CSV);
    c.Free; t.Free;
  end;

{ command parameters }
  h := HyphenTerms.Create;
  h.presetting;
  h.setting;
  maximumFieldCount := h.upperLimit;

  if h.guideFile then
    begin
      g := Guiding.Create;
      g.versionGuide(VERSION_NUMBER, VERSION_WITH_COMMENTS);
      g.Free;

      h.Free;
      Halt(1);
    end
  else
    begin
      if h.versionNumber then
        begin
          g := Guiding.Create;
          g.versionGuide(VERSION_NUMBER, VERSION_WITHOUT_COMMENTS);
          g.Free;

          h.Free;
          Halt(1);
        end
      else
        begin
          if h.fieldSelected < maximumFieldCount then
            begin
              field_to_be_selected := h.fieldSelected;
            end
          else
            begin
              field_to_be_selected := maximumFieldCount;
            end;

          new_name := h.listName(1);
          new_name_length := Length(new_name);
          if new_name_length = 0 then
            begin
              new_name := h.newName;
            end;

          if h.remove0Dh then
            begin
              if h.carriageReturn then
                begin
                  carriage_return := True;
                end
              else
                begin
                  carriage_return := False;
                end;

            end
          else
            begin
              carriage_return := True;
            end;

          h.joints; { joint_byte }
          h.quotes; { quote_byte }

          f := FinalEffects.Create;
          f.setting(h.fileName, new_name, h.pathName, 
            h.atPoint, field_to_be_selected, 
            h.jointByte, 
            h.asciiText, carriage_return, h.identicalTime, h.noRemoving, h.noSearching, h.remove0Dh, h.removeControllers, h.priority, h.surrogatePairToUnderline, h.produceTemporaryBinary, h.whichTable);

{ main function }
  if f.asciiText then
    begin
      a := ConvertUTF16intoASCII.Create;
      a.setting(f.fileName, f.jointByte);
      a.ifErrorOccurs(BUFFER_ERROR);

      new_name_length := Length(f.newName);
      if new_name_length = 0 then
        begin
          a.reading_to_produce_text(ASCII_TEXT);
          a.displayAsciiText;

          if not f.noRemoving then
            begin
              Assignfile(dataFile, ASCII_TEXT);
              Erase(dataFile);
            end;

        end
      else
        begin
          a.reading_to_produce_text(f.newName);
        end;

      a.Free; f.Free; h.Free;
      Halt(0);
    end;

  if f.whichTable then
    begin
      try
        Assignfile(dataFile, DATA_TEXT);
        Reset(dataFile);
        CloseFile(dataFile);

        Assignfile(dataFile, DATA_TEXT);
        Erase(dataFile);
      except
        ;
      end;

    end;

    if f.fieldSelected < maximumFieldCount then
      begin
{ Selecting a field from the UTF-16 file to produce a Shift_JIS text }
        u := Selecting.Create;
        u.presetting(f);
        u.setting(f, f.fileName);
        u.ifErrorOccurs(BUFFER_ERROR);

        u.reading_to_produce_MS932_CRLF(f.newName);
        u.Free; f.Free;
        Halt(0);
      end;
            
  file_name_length := Length(f.fileName);
  if file_name_length = 0 then
    begin
      g := Guiding.Create;
      g.versionGuide(VERSION_NUMBER, VERSION_WITHOUT_COMMENTS);

      g.Free; f.Free; h.Free;
      Halt(1);
    end;

  c := Converting.Create;
  c.setting(f, f.fileName);
  c.ifErrorOccurs(BUFFER_ERROR);

  new_name_length := Length(f.newName);
  if new_name_length = 0 then
    begin
      if f.removeControllers then
        begin
          c.presetting;
          c.standard_output_of_non_controllers;
        end
      else
        begin
          c.standard_output;
        end;

      c.Free; f.Free; h.Free;
      Halt(0);
    end;

  if f.identicalTime then
    begin
      c.produceBinaryFile(f.fileName, REMOVE_ANCIENT_FILE);
      c.renameBinaryFile(f.fileName, f.noRemoving);
    end;

  if f.carriageReturn then
    begin
      if f.produceTemporaryBinary then
        begin
          if f.removeControllers then
            begin
              c.presetting;
              c.skipping_control_to_produce_MS932_CRLF(TEMPORARY_BINARY);
            end
          else
            begin
              c.reading_to_produce_MS932_CRLF(TEMPORARY_BINARY);
            end;

          c.renameBinaryFile(f.newName, f.noRemoving);
        end
      else
        begin
          if f.removeControllers then
            begin
              c.presetting;
              c.skipping_control_to_produce_MS932_CRLF(f.newName);
            end
          else
            begin
              c.reading_to_produce_MS932_CRLF(f.newName);
            end;

        end;

    end
  else
    begin
      if f.remove0Dh then
        begin
          if f.produceTemporaryBinary then
            begin
              if f.removeControllers then
                begin
                  c.presetting;
                  c.skipping_control_to_produce_MS932_LF(TEMPORARY_BINARY);
                end
              else
                begin
                  c.reading_to_produce_MS932_LF(TEMPORARY_BINARY);
                end;

              c.renameBinaryFile(f.newName, f.noRemoving);
            end
          else
            begin
              if f.removeControllers then
                begin
                  c.presetting;
                  c.skipping_control_to_produce_MS932_LF(f.newName);
                end
              else
                begin
                  c.reading_to_produce_MS932_LF(f.newName);
                end;

            end;

        end
      else
        begin
          if f.produceTemporaryBinary then
            begin
              if f.removeControllers then
                begin
                  c.presetting;
                  c.skipping_control_to_produce_MS932_text(TEMPORARY_BINARY);
                end
              else
                begin
                  c.reading_to_produce_MS932_text(TEMPORARY_BINARY);
                end;

              c.renameBinaryFile(f.newName, f.noRemoving);
            end
          else
            begin
              if f.removeControllers then
                begin
                  c.presetting;
                  c.skipping_control_to_produce_MS932_text(f.newName);
                end
              else
                begin
                  c.reading_to_produce_MS932_text(f.newName);
                end;

            end;

        end;

    end;

  if f.whichTable then
    begin
      c.displayDataText;
    end;

  c.Free;

{ end of the main function }
          f.Free; h.Free;
        end; { if h.versionNumber then, else }

    end; { if h.guideFile then, else }

end.
