unit ProduceUTF16;

{*
Utf8to16 FPC Version 1.01
Producing a UTF-16 text from the UTF-8 text
Copyright 2019 Sogaya

Delphi 4
Free Pascal Compiler 3.0.4

*}

{$I+}

interface

uses
  Windows, SysUtils, ErrorText;

type AnsiCharArray = array[0..4095] of AnsiChar;
type PatternArray = array[0..20] of Integer;

type
  CodePoint = class
  private
    a: AnsiCharArray;
    A_SIZE: Integer;
    B: array[0..20] of Integer;
    B_110110_S: Integer;
    B_110111_S: Integer;
    B_LENGTH: Integer;
    B_LENGTH_1: Integer;
    beginning: Integer;
    buffer: array of AnsiChar;
    buffer_size: Integer;
    carriage_return_appears: Boolean;
    CLEAR_ARRAY: AnsiCharArray;
    CLEAR_BIT_C: AnsiChar;
    CLEAR_PATTERN: PatternArray;
    combo: array of AnsiChar;
    combo_size: Integer;
    count: Integer;
    E: array[0..255] of Integer;
    EMPTY_ANSI: AnsiString;
    file_name: String;
    group_part_mode: Integer;
    H: array[0..7] of Integer;
    H_0A_C: AnsiChar;
    H_0D_C: AnsiChar;
    H_FE_C: AnsiChar;
    H_FF_C: AnsiChar;
    higher_part_of_kanji: Integer;
    lower_part_of_kanji: Integer;
    LOWER_PART_MASK: Integer;
    NEW_LINE: AnsiString;
    NEW_LINE_SIZE: Integer;
    new_name: String;
    pattern: PatternArray;
    PATTERN_LENGTH: Integer;
    PATTERN_LENGTH_1: Integer;
    SET_BIT_C: AnsiChar;
    unknown_mode: Boolean;
    v: AnsiCharArray;
    V_SIZE: Integer;
    V_SIZE_1: Integer;
    x: ErrorTexts;
    y: AnsiCharArray;
    Y_SIZE: Integer;
    Y_SIZE_1: Integer;
    function checkGroupPartMode(k: Integer): Integer;
    function combo_KANJI_PART_1(k: Integer): Integer;
    function combo_KANJI_PART_2(k: Integer): Integer;
    function combo_KANJI_PART_3(k: Integer; m: Integer): Integer;
    function combo_SURROGATE_PAIR_PART_1(k: Integer): Integer;
    function combo_SURROGATE_PAIR_PART_2(k: Integer): Integer;
    function combo_SURROGATE_PAIR_PART_3(k: Integer): Integer;
    function combo_SURROGATE_PAIR_PART_4(k: Integer; m: Integer): Integer;
    function combo_TWO_BYTE_CHARACTER_PART_1(k: Integer): Integer;
    function combo_TWO_BYTE_CHARACTER_PART_2(k: Integer; m: Integer): Integer;
    function combo_WHOLE_ASCII(k: Integer; m: Integer): Integer;
    function E_table: Integer;
    function file_size(name: String): Integer;
    function setBufferSize: Integer;
  public
    destructor Destroy; override;
    function bufferingWriting(comboLength: Integer): Integer;
    function lineCount: Integer;
    function newLine: AnsiString;
    function produceBinaryFile(name: String; no_removing: Boolean): Integer;
    function reading_to_produce_UTF16_CRLF(nn: String): Integer;
    function reading_to_produce_UTF16_LF(nn: String): Integer;
    function reading_to_produce_UTF16_text(nn: String): Integer;
    function renameBinaryFile(name: String; no_removing: Boolean): Integer;
    procedure ifErrorOccurs(exit_code: Integer);
    procedure setting(fn: String);
  end;

const
  B_110110 = 54;
  B_110111 = 55;
  BOM_SIZE = 3; { EFh, BBh, BFh }
  BUFFER_ERROR = -1;
  C_ERR = 'err';
  CLEAR_BIT = 48; { 30h, '0' }
  COMBO_ERROR = -3;
  EMPTY = '';
  EXPANSION_FACTOR = 3;
  FILE_ERROR = -2;
  H_00000001 = 1;
  H_000000FF = 255;
  H_00010000 = 65536;
  H_00FF = 255;
  H_0A = 10; { LF }
  H_0D = 13; { CR }
  H_10000000 = 128; { 80h }
  H_11000000 = 192; { C0h }
  H_11100000 = 224; { E0h }
  H_11110000 = 240; { F0h }
  H_30 = 48; { '0' }
  H_31 = 49; { '1' }
  H_39 = 57; { '9' }
  H_3E = 62; { '>' }
  H_3F = 63;
  H_41 = 65; { 'A' }
  H_42 = 66; { 'B' }
  H_46 = 70; { 'F' }
  H_5F = 95; { '_' }
  H_7E = 126; { '~' }
  H_80 = 128;
  H_BB = 187;
  H_BF = 191;
  H_D840 = 55360;
  H_DC02 = 56322;
  H_EF = 239;
  H_FE = 254;
  H_FF = 255;
  KANJI_PART_1 = 3;
  KANJI_PART_2 = 4;
  KANJI_PART_3 = 5;
  KANJI_SIZE = 2;
  NOT_CONVERTED = -1;
  SET_BIT = 49; { 31h, '1'}
  SIZE_OF_ANSICHAR = 1;
  SIZE_OF_CHAR = 2;
  SIZE_OF_SURROGATE_PAIR = 4;
  SURROGATE_PAIR_PART_1 = 6;
  SURROGATE_PAIR_PART_2 = 7;
  SURROGATE_PAIR_PART_3 = 8;
  SURROGATE_PAIR_PART_4 = 9;
  TEXT_FILE = '.txt';
  TEMPORARY_BINARY = 'binary.tmp';
  TWO_BYTE_CHARACTER_PART_1 = 1;
  TWO_BYTE_CHARACTER_PART_2 = 2;
  WHOLE_ASCII = 0;

implementation

destructor CodePoint.Destroy;
begin
  buffer := nil;
  combo := nil;

  inherited Destroy;
end;

function CodePoint.bufferingWriting(comboLength: Integer): Integer;
var
  block_has_been_written: Integer;
  exit_code: Integer;
  k: Integer;
  n: Integer;
  newFile: File;
  s: String;
begin
  exit_code := 0;
  n := 0;

{ Writing }
  try
    AssignFile(newFile, new_name);
    Rewrite(newFile, SIZE_OF_ANSICHAR);

    repeat
      v := CLEAR_ARRAY;

      for k := 0 to V_SIZE_1 do
      begin
        v[k] := combo[n];
        Inc(n);
        if n >= comboLength then
          begin
            V_SIZE := k + 1;
            V_SIZE_1 := V_SIZE - 1;
            break;
          end;

      end;

      BlockWrite(newFile, v, V_SIZE, block_has_been_written);
    until n >= comboLength;

    CloseFile(newFile);
    V_SIZE := SizeOf(v);
    V_SIZE_1 := V_SIZE - 1;
  except
    s := 'Rewrite() error-179: ' + new_name;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    exit_code := FILE_ERROR;
  end;

  Result := exit_code;
end;

function CodePoint.checkGroupPartMode(k: Integer): Integer;
var
  n: Integer;
begin
  n := WHOLE_ASCII;

  if k < H_80 then
    begin
      n := WHOLE_ASCII;
      unknown_mode := False;
    end
  else
    begin
      if k >= H_11110000 then
        begin
          n := SURROGATE_PAIR_PART_1;
          unknown_mode := False;
        end
      else
        begin
          if k >= H_11100000 then
            begin
              n := KANJI_PART_1;
              unknown_mode := False;
            end
          else
            begin
              if k >= H_11000000 then
                begin
                  n := TWO_BYTE_CHARACTER_PART_1;
                  unknown_mode := False;
                end
              else
                begin
{*
  WHOLE_ASCII = 0;
  TWO_BYTE_CHARACTER_PART_1 = 1;
  TWO_BYTE_CHARACTER_PART_2 = 2;
  KANJI_PART_1 = 3;
  KANJI_PART_2 = 4;
  KANJI_PART_3 = 5;
  SURROGATE_PAIR_PART_1 = 6;
  SURROGATE_PAIR_PART_2 = 7;
  SURROGATE_PAIR_PART_3 = 8;
  SURROGATE_PAIR_PART_4 = 9;
*}
                  case group_part_mode of
                    1: n := TWO_BYTE_CHARACTER_PART_2;
                    3: n := KANJI_PART_2;
                    4: n := KANJI_PART_3;
                    6: n := SURROGATE_PAIR_PART_2;
                    7: n := SURROGATE_PAIR_PART_3;
                    8: n := SURROGATE_PAIR_PART_4;
                  else
                    ;
                  end;

                  unknown_mode := False;
                end;

            end;

         end;

    end;

  Result := n;
end;

function CodePoint.combo_KANJI_PART_1(k: Integer): Integer;
var
  n: Integer;
begin
  pattern := CLEAR_PATTERN;
  n := k - H_11100000;
  pattern[3] := n and H_00000001;

  n := n shr 1;
  pattern[2] := n and H_00000001;

  n := n shr 1;
  pattern[1] := n and H_00000001;

  n := n shr 1;
  pattern[0] := n and H_00000001;

  group_part_mode := KANJI_PART_2;
  Result := KANJI_PART_2;
end;

function CodePoint.combo_KANJI_PART_2(k: Integer): Integer;
var
  n: Integer;
begin
  n := k - H_10000000;
  pattern[9] := n and H_00000001;

  n := n shr 1;
  pattern[8] := n and H_00000001;

  n := n shr 1;
  pattern[7] := n and H_00000001;

  n := n shr 1;
  pattern[6] := n and H_00000001;

  n := n shr 1;
  pattern[5] := n and H_00000001;

  n := n shr 1;
  pattern[4] := n and H_00000001;

  group_part_mode := KANJI_PART_3;
  Result := KANJI_PART_3;
end;

function CodePoint.combo_KANJI_PART_3(k: Integer; m: Integer): Integer;
var
  i: Integer;
  n: Integer;
  p: Integer;
  q: Integer;
  r: Integer;
begin
  n := k - H_10000000;
  pattern[15] := n and H_00000001;

  n := n shr 1;
  pattern[14] := n and H_00000001;

  n := n shr 1;
  pattern[13] := n and H_00000001;

  n := n shr 1;
  pattern[12] := n and H_00000001;

  n := n shr 1;
  pattern[11] := n and H_00000001;

  n := n shr 1;
  pattern[10] := n and H_00000001;

{ second byte }
  p := 0;
  q := 0;
  for i := 15 downto 8 do
  begin
    if pattern[i] = H_00000001 then
      q := q + B[p];

    Inc(p);
  end;

  combo[m] := Chr(q); { little-endian architecture }

{ first byte }
  p := 0;
  r := 0;
  for i := 7 downto 0 do
  begin
    if pattern[i] = H_00000001 then
      r := r + B[p];

    Inc(p);
  end;

  combo[ m + 1 ] := Chr(r);
  Inc(m, SIZE_OF_CHAR);

  unknown_mode := True;
  Result := m;
end;

function CodePoint.combo_SURROGATE_PAIR_PART_1(k: Integer): Integer;
var
  n: Integer;
begin
  pattern := CLEAR_PATTERN;
  n := k - H_11110000;
  pattern[2] := n and H_00000001;

  n := n shr 1;
  pattern[1] := n and H_00000001;

  n := n shr 1;
  pattern[0] := n and H_00000001;

  group_part_mode := SURROGATE_PAIR_PART_2;
  Result := SURROGATE_PAIR_PART_2;
end;

function CodePoint.combo_SURROGATE_PAIR_PART_2(k: Integer): Integer;
var
  n: Integer;
begin
  n := k - H_10000000;
  pattern[8] := n and H_00000001;

  n := n shr 1;
  pattern[7] := n and H_00000001;

  n := n shr 1;
  pattern[6] := n and H_00000001;

  n := n shr 1;
  pattern[5] := n and H_00000001;

  n := n shr 1;
  pattern[4] := n and H_00000001;

  n := n shr 1;
  pattern[3] := n and H_00000001;

  group_part_mode := SURROGATE_PAIR_PART_3;
  Result := SURROGATE_PAIR_PART_3;
end;

function CodePoint.combo_SURROGATE_PAIR_PART_3(k: Integer): Integer;
var
  n: Integer;
begin
  n := k - H_10000000;
  pattern[14] := n and H_00000001;

  n := n shr 1;
  pattern[13] := n and H_00000001;

  n := n shr 1;
  pattern[12] := n and H_00000001;

  n := n shr 1;
  pattern[11] := n and H_00000001;

  n := n shr 1;
  pattern[10] := n and H_00000001;

  n := n shr 1;
  pattern[9] := n and H_00000001;

  group_part_mode := SURROGATE_PAIR_PART_4;
  Result := SURROGATE_PAIR_PART_4;
end;

function CodePoint.combo_SURROGATE_PAIR_PART_4(k: Integer; m: Integer): Integer;
var
  basic_code: Integer;
  code_point: Integer;
  i: Integer;
  n: Integer;
  p: Integer;
  q: Integer;
begin
  n := k - H_10000000;
  pattern[20] := n and H_00000001;

  n := n shr 1;
  pattern[19] := n and H_00000001;

  n := n shr 1;
  pattern[18] := n and H_00000001;

  n := n shr 1;
  pattern[17] := n and H_00000001;

  n := n shr 1;
  pattern[16] := n and H_00000001;

  n := n shr 1;
  pattern[15] := n and H_00000001;

  p := 0;
  q := 0;
  for i := 20 downto 0 do
  begin
    if pattern[i] = H_00000001 then
      begin
        q := q + B[p];
      end;

    Inc(p);
  end;

  code_point := q;

{ Four bytes of UTF-16 }
  basic_code := code_point - H_00010000;
  k := basic_code shr 10;
  higher_part_of_kanji := k + B_110110_S;

  k := higher_part_of_kanji and H_00FF;
  combo[m] := Chr(k); { little-endian architecture }

  k := higher_part_of_kanji shr 8;
  k := k and H_00FF;
  combo[ m + 1 ] := Chr(k);
  Inc(m, KANJI_SIZE);

  basic_code := code_point - H_00010000;
  k := basic_code and LOWER_PART_MASK;
  lower_part_of_kanji := k + B_110111_S;

  k := lower_part_of_kanji and H_00FF;
  combo[m] := Chr(k); { little-endian architecture }

  k := lower_part_of_kanji shr 8;
  k := k and H_00FF;
  combo[ m + 1 ] := Chr(k);
  Inc(m, KANJI_SIZE);

  unknown_mode := True;
  Result := m;
end;

function CodePoint.combo_TWO_BYTE_CHARACTER_PART_1(k: Integer): Integer;
var
  n: Integer;
begin
  pattern := CLEAR_PATTERN;
  n := k - H_11000000;
  pattern[4] := n and H_00000001;

  n := n shr 1;
  pattern[3] := n and H_00000001;

  n := n shr 1;
  pattern[2] := n and H_00000001;

  n := n shr 1;
  pattern[1] := n and H_00000001;

  n := n shr 1;
  pattern[0] := n and H_00000001;

  group_part_mode := TWO_BYTE_CHARACTER_PART_2;
  Result := TWO_BYTE_CHARACTER_PART_2;
end;

function CodePoint.combo_TWO_BYTE_CHARACTER_PART_2(k: Integer; m: Integer): Integer;
var
  i: Integer;
  n: Integer;
  p: Integer;
  q: Integer;
  r: Integer;
begin
  n := k - H_10000000;
  pattern[10] := n and H_00000001;

  n := n shr 1;
  pattern[9] := n and H_00000001;

  n := n shr 1;
  pattern[8] := n and H_00000001;

  n := n shr 1;
  pattern[7] := n and H_00000001;

  n := n shr 1;
  pattern[6] := n and H_00000001;

  n := n shr 1;
  pattern[5] := n and H_00000001;

{ second byte }
  p := 0;
  q := 0;
  for i := 10 downto 3 do
  begin
    if pattern[i] = H_00000001 then
      q := q + B[p];

    Inc(p);
  end;

  combo[m] := Chr(q); { little-endian architecture }

{ first byte }
  p := 0;
  r := 0;
  for i := 2 downto 0 do
  begin
    if pattern[i] = H_00000001 then
      r := r + B[p];

    Inc(p);
  end;

  combo[ m + 1 ] := Chr(r);
  Inc(m, SIZE_OF_CHAR);

  unknown_mode := True;
  Result := m;
end;

function CodePoint.combo_WHOLE_ASCII(k: Integer; m: Integer): Integer;
begin
  combo[m] := Chr(k);
  combo[ m + 1 ] := #0;
  Inc(m, SIZE_OF_CHAR);

  unknown_mode := True;
  Result := m;
end;

function CodePoint.E_table: Integer;
var
  beginning_of_E: Integer;
  end_1: Integer;
  h: Integer;
  i: Integer;
  k: Integer;
  m: Integer;
begin
  for i := 0 to H_FF do
  begin
    E[i] := NOT_CONVERTED;
  end;

{ 0 to 9 }
  beginning_of_E := H_30;

  end_1 := H_39 - 1;
  E[H_30] := 0;
  h := E[H_30];
  m := H_31;

  for k := beginning_of_E to end_1 do
  begin
    Inc(h);
    E[m] := h;
    Inc(m);
  end;

{ A to F }
  beginning_of_E := H_41;

  end_1 := H_46 - 1;
  E[H_41] := 10; { 'A' }
  h := E[H_41];
  m := H_42;

  for k := beginning_of_E to end_1 do
  begin
    Inc(h);
    E[m] := h;
    Inc(m);
  end;

  Result := h; { 'F' }
end;

function CodePoint.file_size(name: String): Integer;
var
  f: Integer;
  size: Integer;
begin
  f := FileOpen(name, fmOpenRead or fmShareDenyNone);
  size := FileSeek(f, 0, 2); { EOF }
  FileClose(f);
  Result := size;
end;

function CodePoint.lineCount: Integer;
begin
  Result := count;
end;

function CodePoint.newLine: AnsiString;
begin
  Result := NEW_LINE;
end;

function CodePoint.produceBinaryFile(name: String; no_removing: Boolean): Integer;
var
  block_has_been_read: Integer;
  block_has_been_written: Integer;
  newFile: File;
  s: String;
  sum: Integer;
  testFile: File;
begin
  sum := 0;

  try
    AssignFile(testFile, name);
    Reset(testFile, SIZE_OF_ANSICHAR);

    AssignFile(newFile, TEMPORARY_BINARY);
    Rewrite(newFile, SIZE_OF_ANSICHAR);

    repeat
      a := CLEAR_ARRAY;
      BlockRead(testFile, a, A_SIZE, block_has_been_read);
      BlockWrite(newFile, a, block_has_been_read, block_has_been_written);
      sum := sum + block_has_been_written;
    until (block_has_been_read = 0) or (block_has_been_read <> block_has_been_written);

    CloseFile(newFile);
    CloseFile(testFile);

    if not no_removing then
      begin
        AssignFile(testFile, name);
        Erase(testFile);
      end;

  except
    s := 'Reset() error-180: ' + name;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    Halt(1);
  end;

  Result := sum;
end;

function CodePoint.reading_to_produce_UTF16_CRLF(nn: String): Integer;
var
  block_has_been_read: Integer;
  buffer_size_1: Integer;
  k: Integer;
  m: Integer;
  n: Integer;
  s: String;
  testFile: File;
begin
  buffer_size_1 := buffer_size - 1;
  carriage_return_appears := False;
  count := 0;
  new_name := nn;
  unknown_mode := True;

  for n := 0 to buffer_size_1 do
  begin
    buffer[n] := #0;
  end;

  try
{ Reaeding a file }
    AssignFile(testFile, file_name);
    Reset(testFile, SIZE_OF_ANSICHAR);

    m := 0;
    repeat
      y := CLEAR_ARRAY;
      BlockRead(testFile, y, Y_SIZE, block_has_been_read);

      for n:= 0 to Y_SIZE_1 do
      begin
        buffer[m] := y[n];
        Inc(m);
        if m > buffer_size_1 then
          break;
        end;

    until m > buffer_size_1;

    CloseFile(testFile);
  except
    s := 'Reset() error-181: ' + file_name;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    Halt(1);
  end;

{ Finding the first NEW_LINE }
  m := 0;
  combo[m] := H_FF_C;
  combo[ m + 1 ] := H_FE_C;
  Inc(m, KANJI_SIZE);
  unknown_mode := True;
  for n := beginning to buffer_size_1 do
  begin
    beginning := n + 1;
    k := Ord(buffer[n]);

    if buffer[n] = H_0A_C then
      begin
        count := 1;

        combo[m] := H_0D_C;
        combo[ m + 1 ] := #0;
        Inc(m, SIZE_OF_CHAR);

        combo[m] := H_0A_C;
        combo[ m + 1 ] := #0;
        Inc(m, SIZE_OF_CHAR);

        break;
      end
    else
      begin
        if buffer[n] = H_0D_C then
          begin
            carriage_return_appears := True;
            NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
            NEW_LINE := EMPTY_ANSI + H_0D_C;
            NEW_LINE := NEW_LINE + H_0A_C;
          end
        else
          begin
            if unknown_mode then
              begin
                group_part_mode := checkGroupPartMode(k);
              end;

            case group_part_mode of
              0: m := combo_WHOLE_ASCII(k, m);
              1:      combo_TWO_BYTE_CHARACTER_PART_1(k);
              2: m := combo_TWO_BYTE_CHARACTER_PART_2(k, m);
              3:      combo_KANJI_PART_1(k);
              4:      combo_KANJI_PART_2(k);
              5: m := combo_KANJI_PART_3(k, m);
              6:      combo_SURROGATE_PAIR_PART_1(k);
              7:      combo_SURROGATE_PAIR_PART_2(k);
              8:      combo_SURROGATE_PAIR_PART_3(k);
              9: m := combo_SURROGATE_PAIR_PART_4(k, m);
            else
              ;
            end;

          end;

      end; { if buffer[n] = H_0A_C then, else }

  end; { for n := beginning to buffer_size_1 do }

{ Counting lines }
  unknown_mode := True;
  for n := beginning to buffer_size_1 do
  begin
    k := Ord(buffer[n]);

    if buffer[n] = H_0A_C then
      begin
        Inc(count);

        combo[m] := H_0D_C;
        combo[ m + 1 ] := #0;
        Inc(m, SIZE_OF_CHAR);

        combo[m] := H_0A_C;
        combo[ m + 1 ] := #0;
        Inc(m, SIZE_OF_CHAR);
      end
    else
      begin
        if buffer[n] = H_0D_C then
          begin
            continue;
          end
        else
          begin
            if unknown_mode then
              begin
                group_part_mode := checkGroupPartMode(k);
              end;

            case group_part_mode of
              0: m := combo_WHOLE_ASCII(k, m);
              1:      combo_TWO_BYTE_CHARACTER_PART_1(k);
              2: m := combo_TWO_BYTE_CHARACTER_PART_2(k, m);
              3:      combo_KANJI_PART_1(k);
              4:      combo_KANJI_PART_2(k);
              5: m := combo_KANJI_PART_3(k, m);
              6:      combo_SURROGATE_PAIR_PART_1(k);
              7:      combo_SURROGATE_PAIR_PART_2(k);
              8:      combo_SURROGATE_PAIR_PART_3(k);
              9: m := combo_SURROGATE_PAIR_PART_4(k, m);
            else
              ;
            end;

          end;

      end; { if buffer[n] = H_0A_C then, else }

  end; { for n := beginning to buffer_size_1 do }

  Result := bufferingWriting(m);
end;

function CodePoint.reading_to_produce_UTF16_LF(nn: String): Integer;
var
  block_has_been_read: Integer;
  buffer_size_1: Integer;
  k: Integer;
  m: Integer;
  n: Integer;
  s: String;
  testFile: File;
begin
  buffer_size_1 := buffer_size - 1;
  carriage_return_appears := False;
  count := 0;
  new_name := nn;
  unknown_mode := True;

  for n := 0 to buffer_size_1 do
  begin
    buffer[n] := #0;
  end;

  try
{ Reaeding a file }
    AssignFile(testFile, file_name);
    Reset(testFile, SIZE_OF_ANSICHAR);

    m := 0;
    repeat
      y := CLEAR_ARRAY;
      BlockRead(testFile, y, Y_SIZE, block_has_been_read);

      for n:= 0 to Y_SIZE_1 do
      begin
        buffer[m] := y[n];
        Inc(m);
        if m > buffer_size_1 then
          break;
        end;

    until m > buffer_size_1;

    CloseFile(testFile);
  except
    s := 'Reset() error-182: ' + file_name;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    Halt(1);
  end;

{ Finding the first NEW_LINE }
  m := 0;
  combo[m] := H_FF_C;
  combo[ m + 1 ] := H_FE_C;
  Inc(m, KANJI_SIZE);
  unknown_mode := True;
  for n := beginning to buffer_size_1 do
  begin
    beginning := n + 1;
    k := Ord(buffer[n]);

    if buffer[n] = H_0A_C then
      begin
        count := 1;

        combo[m] := H_0A_C;
        combo[ m + 1 ] := #0;
        Inc(m, SIZE_OF_CHAR);

        break;
      end
    else
      begin
        if buffer[n] = H_0D_C then
          begin
            carriage_return_appears := True;
            NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
            NEW_LINE := EMPTY_ANSI + H_0D_C;
            NEW_LINE := NEW_LINE + H_0A_C;
          end
        else
          begin
            if unknown_mode then
              begin
                group_part_mode := checkGroupPartMode(k);
              end;

            case group_part_mode of
              0: m := combo_WHOLE_ASCII(k, m);
              1:      combo_TWO_BYTE_CHARACTER_PART_1(k);
              2: m := combo_TWO_BYTE_CHARACTER_PART_2(k, m);
              3:      combo_KANJI_PART_1(k);
              4:      combo_KANJI_PART_2(k);
              5: m := combo_KANJI_PART_3(k, m);
              6:      combo_SURROGATE_PAIR_PART_1(k);
              7:      combo_SURROGATE_PAIR_PART_2(k);
              8:      combo_SURROGATE_PAIR_PART_3(k);
              9: m := combo_SURROGATE_PAIR_PART_4(k, m);
            else
              ;
            end;

          end;

      end; { if buffer[n] = H_0A_C then, else }

  end; { for n := beginning to buffer_size_1 do }

{ Counting lines }
  unknown_mode := True;
  for n := beginning to buffer_size_1 do
  begin
    k := Ord(buffer[n]);

    if buffer[n] = H_0A_C then
      begin
        Inc(count);

        combo[m] := H_0A_C;
        combo[ m + 1 ] := #0;
        Inc(m, SIZE_OF_CHAR);
      end
    else
      begin
        if buffer[n] = H_0D_C then
          begin
            continue;
          end
        else
          begin
            if unknown_mode then
              begin
                group_part_mode := checkGroupPartMode(k);
              end;

            case group_part_mode of
              0: m := combo_WHOLE_ASCII(k, m);
              1:      combo_TWO_BYTE_CHARACTER_PART_1(k);
              2: m := combo_TWO_BYTE_CHARACTER_PART_2(k, m);
              3:      combo_KANJI_PART_1(k);
              4:      combo_KANJI_PART_2(k);
              5: m := combo_KANJI_PART_3(k, m);
              6:      combo_SURROGATE_PAIR_PART_1(k);
              7:      combo_SURROGATE_PAIR_PART_2(k);
              8:      combo_SURROGATE_PAIR_PART_3(k);
              9: m := combo_SURROGATE_PAIR_PART_4(k, m);
            else
              ;
            end;

          end;

      end; { if buffer[n] = H_0A_C then, else }

  end; { for n := beginning to buffer_size_1 do }

  Result := bufferingWriting(m);
end;

function CodePoint.reading_to_produce_UTF16_text(nn: String): Integer;
var
  block_has_been_read: Integer;
  buffer_size_1: Integer;
  k: Integer;
  m: Integer;
  n: Integer;
  s: String;
  testFile: File;
begin
  buffer_size_1 := buffer_size - 1;
  carriage_return_appears := False;
  count := 0;
  new_name := nn;
  unknown_mode := True;

  for n := 0 to buffer_size_1 do
  begin
    buffer[n] := #0;
  end;

  try
{ Reaeding a file }
    AssignFile(testFile, file_name);
    Reset(testFile, SIZE_OF_ANSICHAR);

    m := 0;
    repeat
      y := CLEAR_ARRAY;
      BlockRead(testFile, y, Y_SIZE, block_has_been_read);

      for n:= 0 to Y_SIZE_1 do
      begin
        buffer[m] := y[n];
        Inc(m);
        if m > buffer_size_1 then
          break;
        end;

    until m > buffer_size_1;

    CloseFile(testFile);
  except
    s := 'Reset() error-183: ' + file_name;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    Halt(1);
  end;

{ Finding the first NEW_LINE }
  m := 0;
  combo[m] := H_FF_C;
  combo[ m + 1 ] := H_FE_C;
  Inc(m, KANJI_SIZE);
  unknown_mode := True;
  for n := beginning to buffer_size_1 do
  begin
    beginning := n + 1;
    k := Ord(buffer[n]);

    if buffer[n] = H_0A_C then
      begin
        count := 1;
        combo[m] := H_0A_C;
        combo[ m + 1 ] := #0;
        Inc(m, SIZE_OF_CHAR);

        break;
      end
    else
      begin
        if buffer[n] = H_0D_C then
          begin
            combo[m] := H_0D_C;
            combo[ m + 1 ] := #0;
            Inc(m, SIZE_OF_CHAR);

            carriage_return_appears := True;
            NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
            NEW_LINE := EMPTY_ANSI + H_0D_C;
            NEW_LINE := NEW_LINE + H_0A_C;
          end
        else
          begin
            if unknown_mode then
              begin
                group_part_mode := checkGroupPartMode(k);
              end;

            case group_part_mode of
              0: m := combo_WHOLE_ASCII(k, m);
              1:      combo_TWO_BYTE_CHARACTER_PART_1(k);
              2: m := combo_TWO_BYTE_CHARACTER_PART_2(k, m);
              3:      combo_KANJI_PART_1(k);
              4:      combo_KANJI_PART_2(k);
              5: m := combo_KANJI_PART_3(k, m);
              6:      combo_SURROGATE_PAIR_PART_1(k);
              7:      combo_SURROGATE_PAIR_PART_2(k);
              8:      combo_SURROGATE_PAIR_PART_3(k);
              9: m := combo_SURROGATE_PAIR_PART_4(k, m);
            else
              ;
            end;

          end;

      end; { if buffer[n] = H_0A_C then, else }

  end; { for n := beginning to buffer_size_1 do }

{ Counting lines }
  unknown_mode := True;
  for n := beginning to buffer_size_1 do
  begin
    k := Ord(buffer[n]);

    if buffer[n] = H_0A_C then
      begin
        Inc(count);

        combo[m] := H_0A_C;
        combo[ m + 1 ] := #0;
        Inc(m, SIZE_OF_CHAR);
      end
    else
      begin
        if buffer[n] = H_0D_C then
          begin
            combo[m] := H_0D_C;
            combo[ m + 1 ] := #0;
            Inc(m, SIZE_OF_CHAR);
          end
        else
          begin
            if unknown_mode then
              begin
                group_part_mode := checkGroupPartMode(k);
              end;

            case group_part_mode of
              0: m := combo_WHOLE_ASCII(k, m);
              1:      combo_TWO_BYTE_CHARACTER_PART_1(k);
              2: m := combo_TWO_BYTE_CHARACTER_PART_2(k, m);
              3:      combo_KANJI_PART_1(k);
              4:      combo_KANJI_PART_2(k);
              5: m := combo_KANJI_PART_3(k, m);
              6:      combo_SURROGATE_PAIR_PART_1(k);
              7:      combo_SURROGATE_PAIR_PART_2(k);
              8:      combo_SURROGATE_PAIR_PART_3(k);
              9: m := combo_SURROGATE_PAIR_PART_4(k, m);
            else
              ;
            end;

          end;

      end; { if buffer[n] = H_0A_C then, else }

  end; { for n := beginning to buffer_size_1 do }

  Result := bufferingWriting(m);
end;

function CodePoint.setBufferSize: Integer;
var
  r: Integer;
  s: String;
begin
  r := buffer_size;

  try
    SetLength(buffer, buffer_size);
  except
    s := 'SetLength() error-184: ' + IntToStr(buffer_size);
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    r := BUFFER_ERROR;
  end;

  try
    SetLength(combo, combo_size);
  except
    s := 'SetLength() error-185: ' + IntToStr(combo_size);
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    r := COMBO_ERROR;
  end;

  Result := r;
end;

function CodePoint.renameBinaryFile(name: String; no_removing: Boolean): Integer;
var
  block_has_been_read: Integer;
  block_has_been_written: Integer;
  newFile: File;
  s: String;
  sum: Integer;
  testFile: File;
begin
  sum := 0;

  try
    AssignFile(testFile, TEMPORARY_BINARY);
    Reset(testFile, SIZE_OF_ANSICHAR);

    AssignFile(newFile, name);
    Rewrite(newFile, SIZE_OF_ANSICHAR);

    repeat
      a := CLEAR_ARRAY;
      BlockRead(testFile, a, A_SIZE, block_has_been_read);
      BlockWrite(newFile, a, block_has_been_read, block_has_been_written);
      sum := sum + block_has_been_written;
    until (block_has_been_read = 0) or (block_has_been_read <> block_has_been_written);

    CloseFile(newFile);
    CloseFile(testFile);

    if not no_removing then
      begin
        AssignFile(testFile, TEMPORARY_BINARY);
        Erase(testFile);
      end;

  except
    s := 'Reset() error-186: ' + name;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    Halt(1);
  end;

  Result := sum;
end;

procedure CodePoint.ifErrorOccurs(exit_code: Integer);
begin
  if buffer_size < 0 then
    Halt(exit_code);

end;

procedure CodePoint.setting(fn: String);
var
  block_has_been_read: Integer;
  i: Integer;
  q: array[0..2] of Byte;
  s: String;
  testFile: File;
begin
  buffer_size := -1;
  file_name := fn;

  count := -1;
  EMPTY_ANSI := '';

  PATTERN_LENGTH := 21; { type PatternArray = array[0..20] of Integer; }
  PATTERN_LENGTH_1 := PATTERN_LENGTH - 1;
  V_SIZE := SizeOf(v);
  V_SIZE_1 := V_SIZE - 1;
  Y_SIZE := SizeOf(y);
  Y_SIZE_1 := Y_SIZE - 1;

  for i := 0 to V_SIZE_1 do
  begin
    CLEAR_ARRAY[i] := #0;
  end;

  for i := 0 to PATTERN_LENGTH_1 do
  begin
    CLEAR_PATTERN[i] := 0;
  end;

  try
    AssignFile(testFile, file_name);
    Reset(testFile, SIZE_OF_ANSICHAR);
    BlockRead(testFile, v, V_SIZE, block_has_been_read);
    q[0] := Ord(v[0]);
    q[1] := Ord(v[1]);
    q[2] := Ord(v[2]);
    if (q[0] = H_EF) and (q[1] = H_BB) and (q[2] = H_BF) then
      beginning := BOM_SIZE
    else
      beginning := 0;

    CloseFile(testFile);

    buffer_size := file_size(file_name);
    combo_size := EXPANSION_FACTOR * buffer_size;
  except
    s := 'Reset() error-187: ' + file_name;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
  end;

  B_110110_S := B_110110 shl 10;
  B_110111_S := B_110111 shl 10;

  E_table;

{ Characters of AnsiChar type }
  H_FF_C := Chr(H_FF);
  H_0A_C := Chr(H_0A);
  H_0D_C := Chr(H_0D);
  H_FE_C := Chr(H_FE);
  CLEAR_BIT_C := Chr(CLEAR_BIT);
  SET_BIT_C := Chr(SET_BIT);

  EMPTY_ANSI := '';

  NEW_LINE := EMPTY_ANSI + H_0A_C;
  NEW_LINE_SIZE := 1;

  B_LENGTH := PATTERN_LENGTH;
  B_LENGTH_1 := B_LENGTH - 1;
  B[0] := 1;
  B[1] := 2;
  B[2] := 4;
  B[3] := 8;
  B[4] := 16;
  B[5] := 32;
  B[6] := 64;
  B[7] := 128;
  B[8] := 256;
  B[9] := 512;
  B[10] := 1024;
  B[11] := 2048;
  B[12] := 4096;
  B[13] := 8192;
  B[14] := 16384;
  B[15] := 32768;
  B[16] := 65536;
  B[17] := 131072;
  B[18] := 262144;
  B[19] := 524288;
  B[20] := 1048576;

  H[0] := 1;
  H[1] := 16;
  H[2] := 256;
  H[3] := 4096;
  H[4] := 65536;
  H[5] := 1048576;
  H[6] := 16777216;
  H[7] := 268435456;

  LOWER_PART_MASK := 1 + 2 + 4 + 8 + 16 + 32 + 64 + 128 + 256 + 512;

  setBufferSize;
end;

end.
