program Utf8to16;
{$APPTYPE CONSOLE}

{*
Utf8to16 FPC Version 1.02
Producing a UTF-16 text from the UTF-8 text
Copyright 2020 Sogaya

Delphi 4
Free Pascal Compiler 3.0.4

*}

uses
  SysUtils,
  ErrorText in 'ErrorText.pas',
  FinalEffect in 'FinalEffect.pas',
  Guide in 'Guide.pas',
  HyphenTerm in 'HyphenTerm.pas',
  MessagesText in 'MessagesText.pas',
  Numbered in 'Numbered.pas',
  ProduceUTF16 in 'ProduceUTF16.pas',
  TimeName in 'TimeName.pas';

{$R *.RES}

const
  BUFFER_ERROR = -1;
  C_ERR = 'err';
  REMOVE_ANCIENT_FILE = False;
  TEMPORARY_BINARY = 'binary.tmp';
  TEXT_FILE = '.txt';
  VERSION_NUMBER = '1.02';
  VERSION_WITH_COMMENTS = True;
  VERSION_WITHOUT_COMMENTS = False;

var
  c: CodePoint;
  carriage_return: Boolean;
  f: FinalEffects;
  file_name_length: Integer;
  g: Guiding;
  h: HyphenTerms;
  new_name: String;
  new_name_length: Integer;
  s: String;

begin
  s := ParamStr(1);
  if Length(s) = 0 then
    begin
      g := Guiding.Create;
      g.versionGuide(VERSION_NUMBER, VERSION_WITHOUT_COMMENTS);
      g.Free;

      Halt(1);
    end;

{ command parameters }
  h := HyphenTerms.Create;
  h.presetting;
  h.setting;

  if h.guideFile then
    begin
      g := Guiding.Create;
      g.versionGuide(VERSION_NUMBER, VERSION_WITH_COMMENTS);
      g.Free;

      h.Free;
      Halt(1);
    end
  else
    begin
      if h.versionNumber then
        begin
          g := Guiding.Create;
          g.versionGuide(VERSION_NUMBER, VERSION_WITHOUT_COMMENTS);
          g.Free;

          h.Free;
          Halt(1);
        end
      else
        begin
          new_name := h.listName(1);
          new_name_length := Length(new_name);
          if new_name_length = 0 then
            begin
              new_name := h.newName;
            end;

          if h.remove0Dh then
            begin
              if h.carriageReturn then
                begin
                  carriage_return := True;
                end
              else
                begin
                  carriage_return := False;
                end;

            end
          else
            begin
              carriage_return := True;
            end;

          f := FinalEffects.Create;
          f.setting(h.fileName, new_name, h.pathName,
            h.atPoint, 
            carriage_return, h.identicalTime, h.noRemoving, h.remove0Dh, h.produceTemporaryBinary);

{ main function }
  file_name_length := Length(f.fileName);
  if file_name_length = 0 then
    begin
      g := Guiding.Create;
      g.versionGuide(VERSION_NUMBER, VERSION_WITHOUT_COMMENTS);
      g.Free;

      h.Free;
      Halt(1);
    end;

  c := CodePoint.Create;
  c.setting(f.fileName);
  c.ifErrorOccurs(BUFFER_ERROR);

  if f.identicalTime then
    begin
      c.produceBinaryFile(f.fileName, REMOVE_ANCIENT_FILE);
      c.renameBinaryFile(f.fileName, f.noRemoving);
    end;

  if f.carriageReturn then
    begin
      if f.produceTemporaryBinary then
        begin
          c.reading_to_produce_UTF16_CRLF(TEMPORARY_BINARY);
          c.renameBinaryFile(f.newName, f.noRemoving);
        end
      else
        begin
          c.reading_to_produce_UTF16_CRLF(f.newName);
        end;

    end
  else
    begin
      if f.remove0Dh then
        begin
          if f.produceTemporaryBinary then
            begin
              c.reading_to_produce_UTF16_LF(TEMPORARY_BINARY);
              c.renameBinaryFile(f.newName, f.noRemoving);
            end
          else
            begin
              c.reading_to_produce_UTF16_LF(f.newName);
            end;

        end
      else
        begin
          if f.produceTemporaryBinary then
            begin
              c.reading_to_produce_UTF16_text(TEMPORARY_BINARY);
              c.renameBinaryFile(f.newName, f.noRemoving);
            end
          else
            begin
              c.reading_to_produce_UTF16_text(f.newName);
            end;

        end;

    end;

  c.Free;

{ end of the main function }
          f.Free; h.Free;
        end; { if h.versionNumber then, else }

    end; { if h.guideFile then, else }

end.
