unit SelectUTF16;

{$I+}

{*
Unarfij Version 1.02
Selecting a field from the UTF-16 file to produce a Shift_JIS text
Copyright 2020 Sogaya

Free Pascal Compiler 3.0.4

*}

interface

uses
  Windows, SysUtils, ErrorText, SurvivorUTF16, FinalEffect;

type AnsiCharArray = array[0..4095] of AnsiChar;
type ByteArray = array[0..223] of Byte;
type WideCharArray = array[0..4095] of WideChar;

type
  Selecting = class
  private
    ASCII_C: AnsiChar;
    beginning: Integer;
    BELL_ASCII_KATAKANA: ByteArray;
    buffer: array of WideChar;
    buffer_length: Integer;
    carriage_return_appears: Boolean;
    CLEAR_ARRAY: AnsiCharArray;
    CLEAR_Y_ARRAY: WideCharArray;
    combo: array of AnsiChar;
    combo_size: Integer;
    count: Integer;
    DOT_C: AnsiChar;
    EMPTY_ANSI: AnsiString;
    f: FinalEffects;
    file_name: String;
    H_0A_ANSI: AnsiChar;
    H_0A_C: WideChar;
    H_0D_ANSI: AnsiChar;
    H_0D_C: WideChar;
    KANJI_C: AnsiChar;
    KATAKANA_C: AnsiChar;
    label_byte: Byte;
    NEW_LINE: AnsiString;
    NEW_LINE_SIZE: Integer;
    new_name: String;
    NON_KANJI_C: AnsiChar;
    quote_byte: Byte;
    remove_separators: Boolean;
    field_to_be_selected: Integer;
    separator_byte: Byte;
    SEPARATOR_C: AnsiChar;
    SURROGATE_PAIR_C: AnsiChar;
    table: SurvivorsUTF16;
    v: AnsiCharArray;
    V_SIZE: Integer;
    V_SIZE_1: Integer;
    x: ErrorTexts;
    y: WideCharArray;
    Y_COUNT: Integer;
    Y_COUNT_1: Integer;
    ZERO_C: AnsiChar;
    function file_size(name: String): Integer;
    function setBufferSize: Integer;
  public
    destructor Destroy; override;
    function bufferingWriting(comboLength: Integer): Integer;
    function copyingArrayItem(index: Integer): AnsiChar;
    function displayDataText: Boolean;
    function extracting_decimal_to_produce_MS932_CRLF(nn: String): Integer;
    function extracting_decimal_to_produce_MS932_LF(nn: String): Integer;
    function extracting_decimal_to_produce_MS932_text(nn: String): Integer;
    function extracting_integer_to_produce_MS932_CRLF(nn: String): Integer;
    function extracting_integer_to_produce_MS932_LF(nn: String): Integer;
    function extracting_integer_to_produce_MS932_text(nn: String): Integer;
    function lineCount: Integer;
    function MS932_phrase_size: Integer;
    function newLine: AnsiString;
    function produce_MS932_phrase(s: PWideChar; s_length: Integer): AnsiCharArray;
    function produceBinaryFile(name: String; no_removing: Boolean): Integer;
    function reading_to_produce_MS932: AnsiCharArray;
    function reading_to_produce_MS932_CRLF(nn: String): Integer;
    function reading_to_produce_MS932_LF(nn: String): Integer;
    function reading_to_produce_MS932_text(nn: String): Integer;
    function renameBinaryFile(name: String; no_removing: Boolean): Integer;
    function skipping_control_to_produce_MS932_CRLF(nn: String): Integer;
    function skipping_control_to_produce_MS932_LF(nn: String): Integer;
    function skipping_control_to_produce_MS932_text(nn: String): Integer;
    function standard_output: Integer;
    function standard_output_of_decimals: Integer;
    function standard_output_of_integers: Integer;
    function standard_output_of_non_controllers: Integer;
    procedure ifErrorOccurs(exit_code: Integer);
    procedure presetting(final_effects: FinalEffects);
    procedure setting(final_effects: FinalEffects); overload;
    procedure setting(final_effects: FinalEffects; b: Integer); overload;
    procedure setting(final_effects: FinalEffects; fn: String); overload;
  end;

const
  BELL = 7;
  BOM_COUNT = 1;
  BUFFER_ERROR = -1;
  C_ERR = 'err';
  COMBO_ERROR = -3;
  DATA_TEXT = 'data.txt';
  EMPTY = '';
  EXPANSION_FACTOR = 2;
  FIRST_FIELD = 1;
  FILE_ERROR = -2;
  H_0A = 10; { LF }
  H_0D = 13; { CR }
  H_2D = 45; { '-' }
  H_2E = 46; { '.' }
  H_30 = 48; { '0' }
  H_3E = 62; { '>' }
  H_5F = 95; { '_' }
  H_7E = 126; { '~' }
  H_FE = 254;
  H_FF = 255;
  KANJI_SIZE = 2;
  SIZE_OF_ANSICHAR = 1;
  SIZE_OF_WIDECHAR = 2;
  TEMPORARY_BINARY = 'binary.tmp';
  TEXT_FILE = '.txt';

implementation

destructor Selecting.Destroy;
begin
  buffer := nil;
  combo := nil;
  table.Free;

  inherited Destroy;
end;

function Selecting.bufferingWriting(comboLength: Integer): Integer;
var
  block_has_been_written: Integer;
  exit_code: Integer;
  k: Integer;
  n: Integer;
  newFile: File;
  s: String;
begin
  exit_code := 0;
  n := 0;

{ Writing }
  try
    AssignFile(newFile, new_name);
    Rewrite(newFile, SIZE_OF_ANSICHAR);

    repeat
      v := CLEAR_ARRAY;

      for k := 0 to V_SIZE_1 do
      begin
        v[k] := combo[n];
        Inc(n);
        if n >= comboLength then
          begin
            V_SIZE := k + 1;
            V_SIZE_1 := V_SIZE - 1;
            break;
          end;

      end;

      BlockWrite(newFile, v, V_SIZE, block_has_been_written);
    until n >= comboLength;

    CloseFile(newFile);
    V_SIZE := SizeOf(v);
    V_SIZE_1 := V_SIZE - 1;
  except
    s := 'Rewrite() error-215: ' + new_name;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    exit_code := FILE_ERROR;
  end;

  Result := exit_code;
end;

function Selecting.copyingArrayItem(index: Integer): AnsiChar;
var
  r: AnsiChar;
begin
  if index < 0 then
    begin
      r := #0;
    end
  else
    begin
      if index > V_SIZE_1 then
        begin
          r := #0;
        end
      else
        begin
          r := v[index];
        end;

    end;

  Result := r;
end;

function Selecting.displayDataText: Boolean;
var
  b: Boolean;
  s: String;
  testFile: TextFile;
begin
  b := True;

  try
    AssignFile(testFile, DATA_TEXT);
    Reset(testFile);

    while not Eof(testFile) do
    begin
      ReadLn(testFile, s);
      WriteLn(s);
    end;

    CloseFile(testFile);
  except
    s := 'Reset() error-216: ' + DATA_TEXT;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    b := False;
  end;

  Result := b;
end;

function Selecting.extracting_decimal_to_produce_MS932_CRLF(nn: String): Integer;
var
  b: Byte;
  block_has_been_read: Integer;
  buffer_length_1: Integer;
  character_type: AnsiChar;
  dotted: Boolean;
  extracting: Boolean;
  field_number: Integer;
  m: Integer;
  n: Integer;
  no_splitting: Boolean;
  s: String;
  selected: Boolean;
  testFile: File;
  unformed: Boolean;
  w: Integer;
begin
  buffer_length_1 := buffer_length - 1;
  carriage_return_appears := False;
  count := 0;
  dotted := False;
  extracting := False;
  new_name := nn;

  for n := 0 to buffer_length_1 do
  begin
    buffer[n] := #0;
  end;

  try
{ Reading a file }
    AssignFile(testFile, file_name);
    Reset(testFile, SIZE_OF_WIDECHAR);

    m := 0;
    repeat
      y := CLEAR_Y_ARRAY;
      BlockRead(testFile, y, Y_COUNT, block_has_been_read);

      for n:= 0 to Y_COUNT_1 do
      begin
        buffer[m] := y[n];
        Inc(m);
        if m > buffer_length_1 then
          break;

      end;
    until m > buffer_length_1;

    CloseFile(testFile);
  except
    s := 'Reset() error-194: ' + file_name;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    Halt(1);
  end;

{ Finding the first NEW_LINE }
  if field_to_be_selected = FIRST_FIELD then
    begin
      field_number := 1;
      selected := True;
      BELL_ASCII_KATAKANA[H_2E] := H_2E; { '.' }
      BELL_ASCII_KATAKANA[H_2D] := H_2D; { '-' }
      dotted := False;
      extracting := False;
    end
  else
    begin
      field_number := 1;
      selected := False;
    end;

  m := 0;
  no_splitting := False;
  unformed := False;
  for n := beginning to buffer_length_1 do
  begin
    beginning := n + 1;
    w := Ord(buffer[n]);

    if no_splitting then
      begin
        if unformed then
          begin
            if selected then
              begin
                if extracting then
                  begin
                    selected := False;
                  end;

                continue;
              end;

            unformed := False;
            continue;
          end;

{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
        character_type := table.working(w, 0);
        if character_type = KANJI_C then
          begin
            if selected then
              begin
                if extracting then
                  begin
                    selected := False;
                  end;

                continue;
              end;

          end
        else
          begin
            if character_type = KATAKANA_C then
              begin
                if selected then
                  begin
                    if extracting then
                      begin
                        selected := False;
                      end;

                    continue;
                  end

              end
            else
              begin
                if character_type = SURROGATE_PAIR_C then
                  begin
                    unformed := True;
                    continue;
                  end
                else
                  begin
                    if w = H_0A then
                      begin
                        count := 1;

                        if not dotted then
                          begin
                            combo[m] := DOT_C;
                            Inc(m);

                            combo[m] := ZERO_C;
                            Inc(m);
                          end;

                        combo[m] := H_0D_ANSI;
                        Inc(m);

                        combo[m] := H_0A_ANSI;
                        Inc(m);

                        break;
                      end
                    else
                      begin
                        if w = H_0D then
                          begin
                            carriage_return_appears := True;
                            NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
                            NEW_LINE := EMPTY_ANSI + H_0D_ANSI;
                            NEW_LINE := NEW_LINE + H_0A_ANSI;
                          end
                        else
                          begin
                            if w = quote_byte then
                              begin
                                no_splitting := False;
                              end
                            else
                              begin
                                if w = separator_byte then
                                  begin
                                    if remove_separators then
                                      begin
                                        continue; { Commas disappear if placed in quotes. }
                                      end
                                    else
                                      begin
                                        if selected then
                                          begin
                                            if extracting then
                                              begin
                                                selected := False;
                                              end;

                                          end;

                                      end;

                                  end
                                else
                                  begin
                                    if selected then
                                      begin
                                        b := BELL_ASCII_KATAKANA[w];
                                        if b = BELL then
                                          begin
                                            if extracting then
                                              begin
                                                selected := False;
                                              end;

                                            continue;
                                          end
                                        else
                                          begin
                                            combo[m] := Chr(w);
                                            Inc(m);
                                            extracting := True;

                                            if w = H_2E then
                                              begin
                                                dotted := True;
                                                BELL_ASCII_KATAKANA[H_2E] := BELL;
                                              end;

                                            BELL_ASCII_KATAKANA[H_2D] := BELL;
                                          end;

                                      end;

                                  end;

                              end;

                          end;

                      end;

                  end; { if character_type = SURROGATE_PAIR_C then, else }

              end; { if character_type = KATAKANA_C then, else }

          end; { if c = KANJI_C then, else }

      end
    else
      begin
{ if no_splitting = False then }

        if unformed then
          begin
            if selected then
              begin
                if extracting then
                  begin
                    selected := False;
                  end;

                continue;
              end;

            unformed := False;
            continue;
          end;

{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
        character_type := table.working(w, 0);
        if character_type = KANJI_C then
          begin
            if selected then
              begin
                if extracting then
                  begin
                    selected := False;
                  end;

                continue;
              end;

          end
        else
          begin
            if character_type = KATAKANA_C then
              begin
                if selected then
                  begin
                    if extracting then
                      begin
                        selected := False;
                      end;

                    continue;
                  end

              end
            else
              begin
                if character_type = SURROGATE_PAIR_C then
                  begin
                    unformed := True;
                    continue;
                  end
                else
                  begin
                    if w = H_0A then
                      begin
                        count := 1;

                        if not dotted then
                          begin
                            combo[m] := DOT_C;
                            Inc(m);

                            combo[m] := ZERO_C;
                            Inc(m);
                          end;

                        combo[m] := H_0D_ANSI;
                        Inc(m);

                        combo[m] := H_0A_ANSI;
                        Inc(m);

                        break;
                      end
                    else
                      begin
                        if w = H_0D then
                          begin
                            carriage_return_appears := True;
                            NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
                            NEW_LINE := EMPTY_ANSI + H_0D_ANSI;
                            NEW_LINE := NEW_LINE + H_0A_ANSI;
                          end
                        else
                          begin
                            if w = quote_byte then
                              begin
                                no_splitting := True;
                              end
                            else
                              begin
                                if w = separator_byte then
                                  begin
                                    Inc(field_number);
                                    if field_number = field_to_be_selected then
                                      begin
                                        selected := True;
                                        BELL_ASCII_KATAKANA[H_2E] := H_2E; { '.' }
                                        BELL_ASCII_KATAKANA[H_2D] := H_2D; { '-' }
                                        dotted := False;
                                        extracting := False;
                                      end
                                    else
                                      begin
                                        selected := False;
                                      end;

                                  end
                                else
                                  begin
                                    if selected then
                                      begin
                                        b := BELL_ASCII_KATAKANA[w];
                                        if b = BELL then
                                          begin
                                            if extracting then
                                              begin
                                                selected := False;
                                              end;

                                            continue;
                                          end
                                        else
                                          begin
                                            combo[m] := Chr(w);
                                            Inc(m);
                                            extracting := True;

                                            if w = H_2E then
                                              begin
                                                dotted := True;
                                                BELL_ASCII_KATAKANA[H_2E] := BELL;
                                              end;

                                            BELL_ASCII_KATAKANA[H_2D] := BELL;
                                          end;

                                      end;

                                  end;

                              end;

                          end;

                      end;

                  end; { if character_type = SURROGATE_PAIR_C then, else }

              end; { if character_type = KATAKANA_C then, else }

          end; { if c = KANJI_C then, else }

      end;

  end;

{ Counting lines }
  if field_to_be_selected = FIRST_FIELD then
    begin
      field_number := 1;
      selected := True;
      BELL_ASCII_KATAKANA[H_2E] := H_2E; { '.' }
      BELL_ASCII_KATAKANA[H_2D] := H_2D; { '-' }
      dotted := False;
      extracting := False;
    end
  else
    begin
      field_number := 1;
      selected := False;
    end;

  no_splitting := False;
  unformed := False;
  for n := beginning to buffer_length_1 do
  begin
    w := Ord(buffer[n]);

    if no_splitting then
      begin
        if unformed then
          begin
            if selected then
              begin
                if extracting then
                  begin
                    selected := False;
                  end;

                continue;
              end;

            unformed := False;
            continue;
          end;

{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
        character_type := table.working(w, 0);
        if character_type = KANJI_C then
          begin
            if selected then
              begin
                if extracting then
                  begin
                    selected := False;
                  end;

                continue;
              end;

          end
        else
          begin
            if character_type = KATAKANA_C then
              begin
                if selected then
                  begin
                    if extracting then
                      begin
                        selected := False;
                      end;

                    continue;
                  end;

              end
            else
              begin
                if character_type = SURROGATE_PAIR_C then
                  begin
                    unformed := True;
                    continue;
                  end
                else
                  begin
                    if w = H_0A then
                      begin
                        Inc(count);

                        if not dotted then
                          begin
                            combo[m] := DOT_C;
                            Inc(m);

                            combo[m] := ZERO_C;
                            Inc(m);
                          end;

                        combo[m] := H_0D_ANSI;
                        Inc(m);

                        combo[m] := H_0A_ANSI;
                        Inc(m);

                        if field_to_be_selected = FIRST_FIELD then
                          begin
                            field_number := 1;
                            selected := True;
                            BELL_ASCII_KATAKANA[H_2E] := H_2E; { '.' }
                            BELL_ASCII_KATAKANA[H_2D] := H_2D; { '-' }
                            dotted := False;
                            extracting := False;
                          end
                        else
                          begin
                            field_number := 1;
                            selected := False;
                          end;

                      end
                    else
                      begin
                        if w = H_0D then
                          begin
                            continue;
                          end
                        else
                          begin
                            if w = quote_byte then
                              begin
                                no_splitting := False;
                              end
                            else
                              begin
                                if w = separator_byte then
                                  begin
                                    if remove_separators then
                                      begin
                                        continue; { Commas disappear if placed in quotes. }
                                      end
                                    else
                                      begin
                                        if selected then
                                          begin
                                            if extracting then
                                              begin
                                                selected := False;
                                              end;

                                          end;

                                      end;

                                  end
                                else
                                  begin
                                    if selected then
                                      begin
                                        b := BELL_ASCII_KATAKANA[w];
                                        if b = BELL then
                                          begin
                                            if extracting then
                                              begin
                                                selected := False;
                                              end;

                                            continue;
                                          end
                                        else
                                          begin
                                            combo[m] := Chr(w);
                                            Inc(m);
                                            extracting := True;

                                            if w = H_2E then
                                              begin
                                                dotted := True;
                                                BELL_ASCII_KATAKANA[H_2E] := BELL;
                                              end;

                                            BELL_ASCII_KATAKANA[H_2D] := BELL;
                                          end;

                                      end;

                                  end;

                              end;

                          end;

                      end;

                  end; { if character_type = SURROGATE_PAIR_C then }

              end; { if character_type = KATAKANA_C then, else }

          end; { if character_type = KANJI_C then }

      end
    else
      begin
{ if no_splitting = False then }

        if unformed then
          begin
            if selected then
              begin
                if extracting then
                  begin
                    selected := False;
                  end;

                continue;
              end;

            unformed := False;
            continue;
          end;

{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
        character_type := table.working(w, 0);
        if character_type = KANJI_C then
          begin
            if selected then
              begin
                if extracting then
                  begin
                    selected := False;
                  end;

                continue;
              end;

          end
        else
          begin
            if character_type = KATAKANA_C then
              begin
                if selected then
                  begin
                    if extracting then
                      begin
                        selected := False;
                      end;

                    continue;
                  end;

              end
            else
              begin
                if character_type = SURROGATE_PAIR_C then
                  begin
                    unformed := True;
                    continue;
                  end
                else
                  begin
                    if w = H_0A then
                      begin
                        Inc(count);

                        if not dotted then
                          begin
                            combo[m] := DOT_C;
                            Inc(m);

                            combo[m] := ZERO_C;
                            Inc(m);
                          end;

                        combo[m] := H_0D_ANSI;
                        Inc(m);

                        combo[m] := H_0A_ANSI;
                        Inc(m);

                        if field_to_be_selected = FIRST_FIELD then
                          begin
                            field_number := 1;
                            selected := True;
                            BELL_ASCII_KATAKANA[H_2E] := H_2E; { '.' }
                            BELL_ASCII_KATAKANA[H_2D] := H_2D; { '-' }
                            dotted := False;
                            extracting := False;
                          end
                        else
                          begin
                            field_number := 1;
                            selected := False;
                          end;

                      end
                    else
                      begin
                        if w = H_0D then
                          begin
                            continue;
                          end
                        else
                          begin
                            if w = quote_byte then
                              begin
                                no_splitting := True;
                              end
                            else
                              begin
                                if w = separator_byte then
                                  begin
                                    Inc(field_number);
                                    if field_number = field_to_be_selected then
                                      begin
                                        selected := True;
                                        BELL_ASCII_KATAKANA[H_2E] := H_2E; { '.' }
                                        BELL_ASCII_KATAKANA[H_2D] := H_2D; { '-' }
                                        dotted := False;
                                        extracting := False;
                                      end
                                    else
                                      begin
                                        selected := False;
                                      end;

                                  end
                                else
                                  begin
                                    if selected then
                                      begin
                                        b := BELL_ASCII_KATAKANA[w];
                                        if b = BELL then
                                          begin
                                            if extracting then
                                              begin
                                                selected := False;
                                              end;

                                            continue;
                                          end
                                        else
                                          begin
                                            combo[m] := Chr(w);
                                            Inc(m);
                                            extracting := True;

                                            if w = H_2E then
                                              begin
                                                dotted := True;
                                                BELL_ASCII_KATAKANA[H_2E] := BELL;
                                              end;

                                            BELL_ASCII_KATAKANA[H_2D] := BELL;
                                          end;

                                      end;

                                  end;

                              end;

                          end;

                      end;

                  end; { if character_type = SURROGATE_PAIR_C then }

              end; { if character_type = KATAKANA_C then, else }

          end; { if character_type = KANJI_C then }

      end;

  end;

  Result := bufferingWriting(m);
end;

function Selecting.extracting_decimal_to_produce_MS932_LF(nn: String): Integer;
var
  b: Byte;
  block_has_been_read: Integer;
  buffer_length_1: Integer;
  character_type: AnsiChar;
  dotted: Boolean;
  extracting: Boolean;
  field_number: Integer;
  m: Integer;
  n: Integer;
  no_splitting: Boolean;
  s: String;
  selected: Boolean;
  testFile: File;
  unformed: Boolean;
  w: Integer;
begin
  buffer_length_1 := buffer_length - 1;
  carriage_return_appears := False;
  count := 0;
  dotted := False;
  extracting := False;
  new_name := nn;

  for n := 0 to buffer_length_1 do
  begin
    buffer[n] := #0;
  end;

  try
{ Reading a file }
    AssignFile(testFile, file_name);
    Reset(testFile, SIZE_OF_WIDECHAR);

    m := 0;
    repeat
      y := CLEAR_Y_ARRAY;
      BlockRead(testFile, y, Y_COUNT, block_has_been_read);

      for n:= 0 to Y_COUNT_1 do
      begin
        buffer[m] := y[n];
        Inc(m);
        if m > buffer_length_1 then
          break;

      end;
    until m > buffer_length_1;

    CloseFile(testFile);
  except
    s := 'Reset() error-195: ' + file_name;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    Halt(1);
  end;

{ Finding the first NEW_LINE }
  if field_to_be_selected = FIRST_FIELD then
    begin
      field_number := 1;
      selected := True;
      BELL_ASCII_KATAKANA[H_2E] := H_2E; { '.' }
      BELL_ASCII_KATAKANA[H_2D] := H_2D; { '-' }
      dotted := False;
      extracting := False;
    end
  else
    begin
      field_number := 1;
      selected := False;
    end;

  m := 0;
  no_splitting := False;
  unformed := False;
  for n := beginning to buffer_length_1 do
  begin
    beginning := n + 1;
    w := Ord(buffer[n]);

    if no_splitting then
      begin
        if unformed then
          begin
            if selected then
              begin
                if extracting then
                  begin
                    selected := False;
                  end;

                continue;
              end;

            unformed := False;
            continue;
          end;

{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
        character_type := table.working(w, 0);
        if character_type = KANJI_C then
          begin
            if selected then
              begin
                if extracting then
                  begin
                    selected := False;
                  end;

                continue;
              end;

          end
        else
          begin
            if character_type = KATAKANA_C then
              begin
                if selected then
                  begin
                    if extracting then
                      begin
                        selected := False;
                      end;

                    continue;
                  end

              end
            else
              begin
                if character_type = SURROGATE_PAIR_C then
                  begin
                    unformed := True;
                    continue;
                  end
                else
                  begin
                    if w = H_0A then
                      begin
                        count := 1;

                        if not dotted then
                          begin
                            combo[m] := DOT_C;
                            Inc(m);

                            combo[m] := ZERO_C;
                            Inc(m);
                          end;

                        combo[m] := H_0A_ANSI;
                        Inc(m);

                        break;
                      end
                    else
                      begin
                        if w = H_0D then
                          begin
                            carriage_return_appears := True;
                            NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
                            NEW_LINE := EMPTY_ANSI + H_0D_ANSI;
                            NEW_LINE := NEW_LINE + H_0A_ANSI;
                          end
                        else
                          begin
                            if w = quote_byte then
                              begin
                                no_splitting := False;
                              end
                            else
                              begin
                                if w = separator_byte then
                                  begin
                                    if remove_separators then
                                      begin
                                        continue; { Commas disappear if placed in quotes. }
                                      end
                                    else
                                      begin
                                        if selected then
                                          begin
                                            if extracting then
                                              begin
                                                selected := False;
                                              end;

                                          end;

                                      end;

                                  end
                                else
                                  begin
                                    if selected then
                                      begin
                                        b := BELL_ASCII_KATAKANA[w];
                                        if b = BELL then
                                          begin
                                            if extracting then
                                              begin
                                                selected := False;
                                              end;

                                            continue;
                                          end
                                        else
                                          begin
                                            combo[m] := Chr(w);
                                            Inc(m);
                                            extracting := True;

                                            if w = H_2E then
                                              begin
                                                dotted := True;
                                                BELL_ASCII_KATAKANA[H_2E] := BELL;
                                              end;

                                            BELL_ASCII_KATAKANA[H_2D] := BELL;
                                          end;

                                      end;

                                  end;

                              end;

                          end;

                      end;

                  end; { if character_type = SURROGATE_PAIR_C then, else }

              end; { if character_type = KATAKANA_C then, else }

          end; { if c = KANJI_C then, else }

      end
    else
      begin
{ if no_splitting = False then }

        if unformed then
          begin
            if selected then
              begin
                if extracting then
                  begin
                    selected := False;
                  end;

                continue;
              end;

            unformed := False;
            continue;
          end;

{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
        character_type := table.working(w, 0);
        if character_type = KANJI_C then
          begin
            if selected then
              begin
                if extracting then
                  begin
                    selected := False;
                  end;

                continue;
              end;

          end
        else
          begin
            if character_type = KATAKANA_C then
              begin
                if selected then
                  begin
                    if extracting then
                      begin
                        selected := False;
                      end;

                    continue;
                  end

              end
            else
              begin
                if character_type = SURROGATE_PAIR_C then
                  begin
                    unformed := True;
                    continue;
                  end
                else
                  begin
                    if w = H_0A then
                      begin
                        count := 1;

                        if not dotted then
                          begin
                            combo[m] := DOT_C;
                            Inc(m);

                            combo[m] := ZERO_C;
                            Inc(m);
                          end;

                        combo[m] := H_0A_ANSI;
                        Inc(m);

                        break;
                      end
                    else
                      begin
                        if w = H_0D then
                          begin
                            carriage_return_appears := True;
                            NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
                            NEW_LINE := EMPTY_ANSI + H_0D_ANSI;
                            NEW_LINE := NEW_LINE + H_0A_ANSI;
                          end
                        else
                          begin
                            if w = quote_byte then
                              begin
                                no_splitting := True;
                              end
                            else
                              begin
                                if w = separator_byte then
                                  begin
                                    Inc(field_number);
                                    if field_number = field_to_be_selected then
                                      begin
                                        selected := True;
                                        BELL_ASCII_KATAKANA[H_2E] := H_2E; { '.' }
                                        BELL_ASCII_KATAKANA[H_2D] := H_2D; { '-' }
                                        dotted := False;
                                        extracting := False;
                                      end
                                    else
                                      begin
                                        selected := False;
                                      end;

                                  end
                                else
                                  begin
                                    if selected then
                                      begin
                                        b := BELL_ASCII_KATAKANA[w];
                                        if b = BELL then
                                          begin
                                            if extracting then
                                              begin
                                                selected := False;
                                              end;

                                            continue;
                                          end
                                        else
                                          begin
                                            combo[m] := Chr(w);
                                            Inc(m);
                                            extracting := True;

                                            if w = H_2E then
                                              begin
                                                dotted := True;
                                                BELL_ASCII_KATAKANA[H_2E] := BELL;
                                              end;

                                            BELL_ASCII_KATAKANA[H_2D] := BELL;
                                          end;

                                      end;

                                  end;

                              end;

                          end;

                      end;

                  end; { if character_type = SURROGATE_PAIR_C then, else }

              end; { if character_type = KATAKANA_C then, else }

          end; { if c = KANJI_C then, else }

      end;

  end;

{ Counting lines }
  if field_to_be_selected = FIRST_FIELD then
    begin
      field_number := 1;
      selected := True;
      BELL_ASCII_KATAKANA[H_2E] := H_2E; { '.' }
      BELL_ASCII_KATAKANA[H_2D] := H_2D; { '-' }
      dotted := False;
      extracting := False;
    end
  else
    begin
      field_number := 1;
      selected := False;
    end;

  no_splitting := False;
  unformed := False;
  for n := beginning to buffer_length_1 do
  begin
    w := Ord(buffer[n]);

    if no_splitting then
      begin
        if unformed then
          begin
            if selected then
              begin
                if extracting then
                  begin
                    selected := False;
                  end;

                continue;
              end;

            unformed := False;
            continue;
          end;

{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
        character_type := table.working(w, 0);
        if character_type = KANJI_C then
          begin
            if selected then
              begin
                if extracting then
                  begin
                    selected := False;
                  end;

                continue;
              end;

          end
        else
          begin
            if character_type = KATAKANA_C then
              begin
                if selected then
                  begin
                    if extracting then
                      begin
                        selected := False;
                      end;

                    continue;
                  end;

              end
            else
              begin
                if character_type = SURROGATE_PAIR_C then
                  begin
                    unformed := True;
                    continue;
                  end
                else
                  begin
                    if w = H_0A then
                      begin
                        Inc(count);

                        if not dotted then
                          begin
                            combo[m] := DOT_C;
                            Inc(m);

                            combo[m] := ZERO_C;
                            Inc(m);
                          end;

                        combo[m] := H_0A_ANSI;
                        Inc(m);

                        if field_to_be_selected = FIRST_FIELD then
                          begin
                            field_number := 1;
                            selected := True;
                            BELL_ASCII_KATAKANA[H_2E] := H_2E; { '.' }
                            BELL_ASCII_KATAKANA[H_2D] := H_2D; { '-' }
                            dotted := False;
                            extracting := False;
                          end
                        else
                          begin
                            field_number := 1;
                            selected := False;
                          end;

                      end
                    else
                      begin
                        if w = H_0D then
                          begin
                            continue;
                          end
                        else
                          begin
                            if w = quote_byte then
                              begin
                                no_splitting := False;
                              end
                            else
                              begin
                                if w = separator_byte then
                                  begin
                                    if remove_separators then
                                      begin
                                        continue; { Commas disappear if placed in quotes. }
                                      end
                                    else
                                      begin
                                        if selected then
                                          begin
                                            if extracting then
                                              begin
                                                selected := False;
                                              end;

                                          end;

                                      end;

                                  end
                                else
                                  begin
                                    if selected then
                                      begin
                                        b := BELL_ASCII_KATAKANA[w];
                                        if b = BELL then
                                          begin
                                            if extracting then
                                              begin
                                                selected := False;
                                              end;

                                            continue;
                                          end
                                        else
                                          begin
                                            combo[m] := Chr(w);
                                            Inc(m);
                                            extracting := True;

                                            if w = H_2E then
                                              begin
                                                dotted := True;
                                                BELL_ASCII_KATAKANA[H_2E] := BELL;
                                              end;

                                            BELL_ASCII_KATAKANA[H_2D] := BELL;
                                          end;

                                      end;

                                  end;

                              end;

                          end;

                      end;

                  end; { if character_type = SURROGATE_PAIR_C then }

              end; { if character_type = KATAKANA_C then, else }

          end; { if character_type = KANJI_C then }

      end
    else
      begin
{ if no_splitting = False then }

        if unformed then
          begin
            if selected then
              begin
                if extracting then
                  begin
                    selected := False;
                  end;

                continue;
              end;

            unformed := False;
            continue;
          end;

{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
        character_type := table.working(w, 0);
        if character_type = KANJI_C then
          begin
            if selected then
              begin
                if extracting then
                  begin
                    selected := False;
                  end;

                continue;
              end;

          end
        else
          begin
            if character_type = KATAKANA_C then
              begin
                if selected then
                  begin
                    if extracting then
                      begin
                        selected := False;
                      end;

                    continue;
                  end;

              end
            else
              begin
                if character_type = SURROGATE_PAIR_C then
                  begin
                    unformed := True;
                    continue;
                  end
                else
                  begin
                    if w = H_0A then
                      begin
                        Inc(count);

                        if not dotted then
                          begin
                            combo[m] := DOT_C;
                            Inc(m);

                            combo[m] := ZERO_C;
                            Inc(m);
                          end;

                        combo[m] := H_0A_ANSI;
                        Inc(m);

                        if field_to_be_selected = FIRST_FIELD then
                          begin
                            field_number := 1;
                            selected := True;
                            BELL_ASCII_KATAKANA[H_2E] := H_2E; { '.' }
                            BELL_ASCII_KATAKANA[H_2D] := H_2D; { '-' }
                            dotted := False;
                            extracting := False;
                          end
                        else
                          begin
                            field_number := 1;
                            selected := False;
                          end;

                      end
                    else
                      begin
                        if w = H_0D then
                          begin
                            continue;
                          end
                        else
                          begin
                            if w = quote_byte then
                              begin
                                no_splitting := True;
                              end
                            else
                              begin
                                if w = separator_byte then
                                  begin
                                    Inc(field_number);
                                    if field_number = field_to_be_selected then
                                      begin
                                        selected := True;
                                        BELL_ASCII_KATAKANA[H_2E] := H_2E; { '.' }
                                        BELL_ASCII_KATAKANA[H_2D] := H_2D; { '-' }
                                        dotted := False;
                                        extracting := False;
                                      end
                                    else
                                      begin
                                        selected := False;
                                      end;

                                  end
                                else
                                  begin
                                    if selected then
                                      begin
                                        b := BELL_ASCII_KATAKANA[w];
                                        if b = BELL then
                                          begin
                                            if extracting then
                                              begin
                                                selected := False;
                                              end;

                                            continue;
                                          end
                                        else
                                          begin
                                            combo[m] := Chr(w);
                                            Inc(m);
                                            extracting := True;

                                            if w = H_2E then
                                              begin
                                                dotted := True;
                                                BELL_ASCII_KATAKANA[H_2E] := BELL;
                                              end;

                                            BELL_ASCII_KATAKANA[H_2D] := BELL;
                                          end;

                                      end;

                                  end;

                              end;

                          end;

                      end;

                  end; { if character_type = SURROGATE_PAIR_C then }

              end; { if character_type = KATAKANA_C then, else }

          end; { if character_type = KANJI_C then }

      end;

  end;

  Result := bufferingWriting(m);
end;

function Selecting.extracting_decimal_to_produce_MS932_text(nn: String): Integer;
var
  b: Byte;
  block_has_been_read: Integer;
  buffer_length_1: Integer;
  character_type: AnsiChar;
  dotted: Boolean;
  extracting: Boolean;
  field_number: Integer;
  m: Integer;
  n: Integer;
  no_splitting: Boolean;
  s: String;
  selected: Boolean;
  testFile: File;
  unformed: Boolean;
  w: Integer;
begin
  buffer_length_1 := buffer_length - 1;
  carriage_return_appears := False;
  count := 0;
  dotted := False;
  extracting := False;
  new_name := nn;

  for n := 0 to buffer_length_1 do
  begin
    buffer[n] := #0;
  end;

  try
{ Reading a file }
    AssignFile(testFile, file_name);
    Reset(testFile, SIZE_OF_WIDECHAR);

    m := 0;
    repeat
      y := CLEAR_Y_ARRAY;
      BlockRead(testFile, y, Y_COUNT, block_has_been_read);

      for n:= 0 to Y_COUNT_1 do
      begin
        buffer[m] := y[n];
        Inc(m);
        if m > buffer_length_1 then
          break;

      end;
    until m > buffer_length_1;

    CloseFile(testFile);
  except
    s := 'Reset() error-196: ' + file_name;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    Halt(1);
  end;

{ Finding the first NEW_LINE }
  if field_to_be_selected = FIRST_FIELD then
    begin
      field_number := 1;
      selected := True;
      BELL_ASCII_KATAKANA[H_2E] := H_2E; { '.' }
      BELL_ASCII_KATAKANA[H_2D] := H_2D; { '-' }
      dotted := False;
      extracting := False;
    end
  else
    begin
      field_number := 1;
      selected := False;
    end;

  m := 0;
  no_splitting := False;
  unformed := False;
  for n := beginning to buffer_length_1 do
  begin
    beginning := n + 1;
    w := Ord(buffer[n]);

    if no_splitting then
      begin
        if unformed then
          begin
            if selected then
              begin
                if extracting then
                  begin
                    selected := False;
                  end;

                continue;
              end;

            unformed := False;
            continue;
          end;

{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
        character_type := table.working(w, 0);
        if character_type = KANJI_C then
          begin
            if selected then
              begin
                if extracting then
                  begin
                    selected := False;
                  end;

                continue;
              end;

          end
        else
          begin
            if character_type = KATAKANA_C then
              begin
                if selected then
                  begin
                    if extracting then
                      begin
                        selected := False;
                      end;

                    continue;
                  end

              end
            else
              begin
                if character_type = SURROGATE_PAIR_C then
                  begin
                    unformed := True;
                    continue;
                  end
                else
                  begin
                    if w = H_0A then
                      begin
                        count := 1;

                        if not dotted then
                          begin
                            combo[m] := DOT_C;
                            Inc(m);

                            combo[m] := ZERO_C;
                            Inc(m);
                          end;

                        combo[m] := H_0A_ANSI;
                        Inc(m);

                        break;
                      end
                    else
                      begin
                        if w = H_0D then
                          begin
                            if not dotted then
                              begin
                                combo[m] := DOT_C;
                                Inc(m);

                                combo[m] := ZERO_C;
                                Inc(m);

                                dotted := False;
                              end;

                            combo[m] := H_0D_ANSI;
                            Inc(m);

                            carriage_return_appears := True;
                            NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
                            NEW_LINE := EMPTY_ANSI + H_0D_ANSI;
                            NEW_LINE := NEW_LINE + H_0A_ANSI;
                          end
                        else
                          begin
                            if w = quote_byte then
                              begin
                                no_splitting := False;
                              end
                            else
                              begin
                                if w = separator_byte then
                                  begin
                                    if remove_separators then
                                      begin
                                        continue; { Commas disappear if placed in quotes. }
                                      end
                                    else
                                      begin
                                        if selected then
                                          begin
                                            if extracting then
                                              begin
                                                selected := False;
                                              end;

                                          end;

                                      end;

                                  end
                                else
                                  begin
                                    if selected then
                                      begin
                                        b := BELL_ASCII_KATAKANA[w];
                                        if b = BELL then
                                          begin
                                            if extracting then
                                              begin
                                                selected := False;
                                              end;

                                            continue;
                                          end
                                        else
                                          begin
                                            combo[m] := Chr(w);
                                            Inc(m);
                                            extracting := True;

                                            if w = H_2E then
                                              begin
                                                dotted := True;
                                                BELL_ASCII_KATAKANA[H_2E] := BELL;
                                              end;

                                            BELL_ASCII_KATAKANA[H_2D] := BELL;
                                          end;

                                      end;

                                  end;

                              end;

                          end;

                      end;

                  end; { if character_type = SURROGATE_PAIR_C then, else }

              end; { if character_type = KATAKANA_C then, else }

          end; { if c = KANJI_C then, else }

      end
    else
      begin
{ if no_splitting = False then }

        if unformed then
          begin
            if selected then
              begin
                if extracting then
                  begin
                    selected := False;
                  end;

                continue;
              end;

            unformed := False;
            continue;
          end;

{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
        character_type := table.working(w, 0);
        if character_type = KANJI_C then
          begin
            if selected then
              begin
                if extracting then
                  begin
                    selected := False;
                  end;

                continue;
              end;

          end
        else
          begin
            if character_type = KATAKANA_C then
              begin
                if selected then
                  begin
                    if extracting then
                      begin
                        selected := False;
                      end;

                    continue;
                  end

              end
            else
              begin
                if character_type = SURROGATE_PAIR_C then
                  begin
                    unformed := True;
                    continue;
                  end
                else
                  begin
                    if w = H_0A then
                      begin
                        count := 1;

                        if not dotted then
                          begin
                            combo[m] := DOT_C;
                            Inc(m);

                            combo[m] := ZERO_C;
                            Inc(m);
                          end;

                        combo[m] := H_0A_ANSI;
                        Inc(m);

                        break;
                      end
                    else
                      begin
                        if w = H_0D then
                          begin
                            if not dotted then
                              begin
                                combo[m] := DOT_C;
                                Inc(m);

                                combo[m] := ZERO_C;
                                Inc(m);

                                dotted := False;
                              end;

                            combo[m] := H_0D_ANSI;
                            Inc(m);

                            carriage_return_appears := True;
                            NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
                            NEW_LINE := EMPTY_ANSI + H_0D_ANSI;
                            NEW_LINE := NEW_LINE + H_0A_ANSI;
                          end
                        else
                          begin
                            if w = quote_byte then
                              begin
                                no_splitting := True;
                              end
                            else
                              begin
                                if w = separator_byte then
                                  begin
                                    Inc(field_number);
                                    if field_number = field_to_be_selected then
                                      begin
                                        selected := True;
                                        BELL_ASCII_KATAKANA[H_2E] := H_2E; { '.' }
                                        BELL_ASCII_KATAKANA[H_2D] := H_2D; { '-' }
                                        dotted := False;
                                        extracting := False;
                                      end
                                    else
                                      begin
                                        selected := False;
                                      end;

                                  end
                                else
                                  begin
                                    if selected then
                                      begin
                                        b := BELL_ASCII_KATAKANA[w];
                                        if b = BELL then
                                          begin
                                            if extracting then
                                              begin
                                                selected := False;
                                              end;

                                            continue;
                                          end
                                        else
                                          begin
                                            combo[m] := Chr(w);
                                            Inc(m);
                                            extracting := True;

                                            if w = H_2E then
                                              begin
                                                dotted := True;
                                                BELL_ASCII_KATAKANA[H_2E] := BELL;
                                              end;

                                            BELL_ASCII_KATAKANA[H_2D] := BELL;
                                          end;

                                      end;

                                  end;

                              end;

                          end;

                      end;

                  end; { if character_type = SURROGATE_PAIR_C then, else }

              end; { if character_type = KATAKANA_C then, else }

          end; { if c = KANJI_C then, else }

      end;

  end;

{ Counting lines }
  if field_to_be_selected = FIRST_FIELD then
    begin
      field_number := 1;
      selected := True;
      BELL_ASCII_KATAKANA[H_2E] := H_2E; { '.' }
      BELL_ASCII_KATAKANA[H_2D] := H_2D; { '-' }
      dotted := False;
      extracting := False;
    end
  else
    begin
      field_number := 1;
      selected := False;
    end;

  no_splitting := False;
  unformed := False;
  for n := beginning to buffer_length_1 do
  begin
    w := Ord(buffer[n]);

    if no_splitting then
      begin
        if unformed then
          begin
            if selected then
              begin
                if extracting then
                  begin
                    selected := False;
                  end;

                continue;
              end;

            unformed := False;
            continue;
          end;

{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
        character_type := table.working(w, 0);
        if character_type = KANJI_C then
          begin
            if selected then
              begin
                if extracting then
                  begin
                    selected := False;
                  end;

                continue;
              end;

          end
        else
          begin
            if character_type = KATAKANA_C then
              begin
                if selected then
                  begin
                    if extracting then
                      begin
                        selected := False;
                      end;

                    continue;
                  end;

              end
            else
              begin
                if character_type = SURROGATE_PAIR_C then
                  begin
                    unformed := True;
                    continue;
                  end
                else
                  begin
                    if w = H_0A then
                      begin
                        Inc(count);

                        if not dotted then
                          begin
                            combo[m] := DOT_C;
                            Inc(m);

                            combo[m] := ZERO_C;
                            Inc(m);
                          end;

                        combo[m] := H_0A_ANSI;
                        Inc(m);

                        if field_to_be_selected = FIRST_FIELD then
                          begin
                            field_number := 1;
                            selected := True;
                            BELL_ASCII_KATAKANA[H_2E] := H_2E; { '.' }
                            BELL_ASCII_KATAKANA[H_2D] := H_2D; { '-' }
                            dotted := False;
                            extracting := False;
                          end
                        else
                          begin
                            field_number := 1;
                            selected := False;
                          end;

                      end
                    else
                      begin
                        if w = H_0D then
                          begin
                            if not dotted then
                              begin
                                combo[m] := DOT_C;
                                Inc(m);

                                combo[m] := ZERO_C;
                                Inc(m);

                                dotted := False;
                              end;

                            combo[m] := H_0D_ANSI;
                            Inc(m);
                            continue;
                          end
                        else
                          begin
                            if w = quote_byte then
                              begin
                                no_splitting := False;
                              end
                            else
                              begin
                                if w = separator_byte then
                                  begin
                                    if remove_separators then
                                      begin
                                        continue; { Commas disappear if placed in quotes. }
                                      end
                                    else
                                      begin
                                        if selected then
                                          begin
                                            if extracting then
                                              begin
                                                selected := False;
                                              end;

                                          end;

                                      end;

                                  end
                                else
                                  begin
                                    if selected then
                                      begin
                                        b := BELL_ASCII_KATAKANA[w];
                                        if b = BELL then
                                          begin
                                            if extracting then
                                              begin
                                                selected := False;
                                              end;

                                            continue;
                                          end
                                        else
                                          begin
                                            combo[m] := Chr(w);
                                            Inc(m);
                                            extracting := True;

                                            if w = H_2E then
                                              begin
                                                dotted := True;
                                                BELL_ASCII_KATAKANA[H_2E] := BELL;
                                              end;

                                            BELL_ASCII_KATAKANA[H_2D] := BELL;
                                          end;

                                      end;

                                  end;

                              end;

                          end;

                      end;

                  end; { if character_type = SURROGATE_PAIR_C then }

              end; { if character_type = KATAKANA_C then, else }

          end; { if character_type = KANJI_C then }

      end
    else
      begin
{ if no_splitting = False then }

        if unformed then
          begin
            if selected then
              begin
                if extracting then
                  begin
                    selected := False;
                  end;

                continue;
              end;

            unformed := False;
            continue;
          end;

{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
        character_type := table.working(w, 0);
        if character_type = KANJI_C then
          begin
            if selected then
              begin
                if extracting then
                  begin
                    selected := False;
                  end;

                continue;
              end;

          end
        else
          begin
            if character_type = KATAKANA_C then
              begin
                if selected then
                  begin
                    if extracting then
                      begin
                        selected := False;
                      end;

                    continue;
                  end;

              end
            else
              begin
                if character_type = SURROGATE_PAIR_C then
                  begin
                    unformed := True;
                    continue;
                  end
                else
                  begin
                    if w = H_0A then
                      begin
                        Inc(count);

                        if not dotted then
                          begin
                            combo[m] := DOT_C;
                            Inc(m);

                            combo[m] := ZERO_C;
                            Inc(m);
                          end;

                        combo[m] := H_0A_ANSI;
                        Inc(m);

                        if field_to_be_selected = FIRST_FIELD then
                          begin
                            field_number := 1;
                            selected := True;
                            BELL_ASCII_KATAKANA[H_2E] := H_2E; { '.' }
                            BELL_ASCII_KATAKANA[H_2D] := H_2D; { '-' }
                            dotted := False;
                            extracting := False;
                          end
                        else
                          begin
                            field_number := 1;
                            selected := False;
                          end;

                      end
                    else
                      begin
                        if w = H_0D then
                          begin
                            if not dotted then
                              begin
                                combo[m] := DOT_C;
                                Inc(m);

                                combo[m] := ZERO_C;
                                Inc(m);

                                dotted := False;
                              end;

                            combo[m] := H_0D_ANSI;
                            Inc(m);
                            continue;
                          end
                        else
                          begin
                            if w = quote_byte then
                              begin
                                no_splitting := True;
                              end
                            else
                              begin
                                if w = separator_byte then
                                  begin
                                    Inc(field_number);
                                    if field_number = field_to_be_selected then
                                      begin
                                        selected := True;
                                        BELL_ASCII_KATAKANA[H_2E] := H_2E; { '.' }
                                        BELL_ASCII_KATAKANA[H_2D] := H_2D; { '-' }
                                        dotted := False;
                                        extracting := False;
                                      end
                                    else
                                      begin
                                        selected := False;
                                      end;

                                  end
                                else
                                  begin
                                    if selected then
                                      begin
                                        b := BELL_ASCII_KATAKANA[w];
                                        if b = BELL then
                                          begin
                                            if extracting then
                                              begin
                                                selected := False;
                                              end;

                                            continue;
                                          end
                                        else
                                          begin
                                            combo[m] := Chr(w);
                                            Inc(m);
                                            extracting := True;

                                            if w = H_2E then
                                              begin
                                                dotted := True;
                                                BELL_ASCII_KATAKANA[H_2E] := BELL;
                                              end;

                                            BELL_ASCII_KATAKANA[H_2D] := BELL;
                                          end;

                                      end;

                                  end;

                              end;

                          end;

                      end;

                  end; { if character_type = SURROGATE_PAIR_C then }

              end; { if character_type = KATAKANA_C then, else }

          end; { if character_type = KANJI_C then }

      end;

  end;

  Result := bufferingWriting(m);
end;

function Selecting.extracting_integer_to_produce_MS932_CRLF(nn: String): Integer;
var
  b: Byte;
  block_has_been_read: Integer;
  buffer_length_1: Integer;
  character_type: AnsiChar;
  extracting: Boolean;
  field_number: Integer;
  m: Integer;
  n: Integer;
  no_splitting: Boolean;
  s: String;
  selected: Boolean;
  testFile: File;
  unformed: Boolean;
  w: Integer;
begin
  buffer_length_1 := buffer_length - 1;
  carriage_return_appears := False;
  count := 0;
  extracting := False;
  new_name := nn;

  for n := 0 to buffer_length_1 do
  begin
    buffer[n] := #0;
  end;

  try
{ Reading a file }
    AssignFile(testFile, file_name);
    Reset(testFile, SIZE_OF_WIDECHAR);

    m := 0;
    repeat
      y := CLEAR_Y_ARRAY;
      BlockRead(testFile, y, Y_COUNT, block_has_been_read);

      for n:= 0 to Y_COUNT_1 do
      begin
        buffer[m] := y[n];
        Inc(m);
        if m > buffer_length_1 then
          break;

      end;
    until m > buffer_length_1;

    CloseFile(testFile);
  except
    s := 'Reset() error-197: ' + file_name;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    Halt(1);
  end;

{ Finding the first NEW_LINE }
  if field_to_be_selected = FIRST_FIELD then
    begin
      field_number := 1;
      selected := True;
      BELL_ASCII_KATAKANA[H_2D] := H_2D; { '-' }
      extracting := False;
    end
  else
    begin
      field_number := 1;
      selected := False;
    end;

  m := 0;
  no_splitting := False;
  unformed := False;
  for n := beginning to buffer_length_1 do
  begin
    beginning := n + 1;
    w := Ord(buffer[n]);

    if no_splitting then
      begin
        if unformed then
          begin
            if selected then
              begin
                if extracting then
                  begin
                    selected := False;
                  end;

                continue;
              end;

            unformed := False;
            continue;
          end;

{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
        character_type := table.working(w, 0);
        if character_type = KANJI_C then
          begin
            if selected then
              begin
                if extracting then
                  begin
                    selected := False;
                  end;

                continue;
              end;

          end
        else
          begin
            if character_type = KATAKANA_C then
              begin
                if selected then
                  begin
                    if extracting then
                      begin
                        selected := False;
                      end;

                    continue;
                  end

              end
            else
              begin
                if character_type = SURROGATE_PAIR_C then
                  begin
                    unformed := True;
                    continue;
                  end
                else
                  begin
                    if w = H_0A then
                      begin
                        count := 1;

                        combo[m] := H_0D_ANSI;
                        Inc(m);

                        combo[m] := H_0A_ANSI;
                        Inc(m);

                        break;
                      end
                    else
                      begin
                        if w = H_0D then
                          begin
                            carriage_return_appears := True;
                            NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
                            NEW_LINE := EMPTY_ANSI + H_0D_ANSI;
                            NEW_LINE := NEW_LINE + H_0A_ANSI;
                          end
                        else
                          begin
                            if w = quote_byte then
                              begin
                                no_splitting := False;
                              end
                            else
                              begin
                                if w = separator_byte then
                                  begin
                                    if remove_separators then
                                      begin
                                        continue; { Commas disappear if placed in quotes. }
                                      end
                                    else
                                      begin
                                        if selected then
                                          begin
                                            if extracting then
                                              begin
                                                selected := False;
                                              end;

                                          end;

                                      end;

                                  end
                                else
                                  begin
                                    if selected then
                                      begin
                                        b := BELL_ASCII_KATAKANA[w];
                                        if b = BELL then
                                          begin
                                            if extracting then
                                              begin
                                                selected := False;
                                              end;

                                            continue;
                                          end
                                        else
                                          begin
                                            combo[m] := Chr(w);
                                            Inc(m);
                                            extracting := True;
                                            BELL_ASCII_KATAKANA[H_2D] := BELL;
                                          end;

                                      end;

                                  end;

                              end;

                          end;

                      end;

                  end; { if character_type = SURROGATE_PAIR_C then, else }

              end; { if character_type = KATAKANA_C then, else }

          end; { if c = KANJI_C then, else }

      end
    else
      begin
{ if no_splitting = False then }

        if unformed then
          begin
            if selected then
              begin
                if extracting then
                  begin
                    selected := False;
                  end;

                continue;
              end;

            unformed := False;
            continue;
          end;

{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
        character_type := table.working(w, 0);
        if character_type = KANJI_C then
          begin
            if selected then
              begin
                if extracting then
                  begin
                    selected := False;
                  end;

                continue;
              end;

          end
        else
          begin
            if character_type = KATAKANA_C then
              begin
                if selected then
                  begin
                    if extracting then
                      begin
                        selected := False;
                      end;

                    continue;
                  end

              end
            else
              begin
                if character_type = SURROGATE_PAIR_C then
                  begin
                    unformed := True;
                    continue;
                  end
                else
                  begin
                    if w = H_0A then
                      begin
                        count := 1;

                        combo[m] := H_0D_ANSI;
                        Inc(m);

                        combo[m] := H_0A_ANSI;
                        Inc(m);

                        break;
                      end
                    else
                      begin
                        if w = H_0D then
                          begin
                            carriage_return_appears := True;
                            NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
                            NEW_LINE := EMPTY_ANSI + H_0D_ANSI;
                            NEW_LINE := NEW_LINE + H_0A_ANSI;
                          end
                        else
                          begin
                            if w = quote_byte then
                              begin
                                no_splitting := True;
                              end
                            else
                              begin
                                if w = separator_byte then
                                  begin
                                    Inc(field_number);
                                    if field_number = field_to_be_selected then
                                      begin
                                        selected := True;
                                        BELL_ASCII_KATAKANA[H_2D] := H_2D; { '-' }
                                        extracting := False;
                                      end
                                    else
                                      begin
                                        selected := False;
                                      end;

                                  end
                                else
                                  begin
                                    if selected then
                                      begin
                                        b := BELL_ASCII_KATAKANA[w];
                                        if b = BELL then
                                          begin
                                            if extracting then
                                              begin
                                                selected := False;
                                              end;

                                            continue;
                                          end
                                        else
                                          begin
                                            combo[m] := Chr(w);
                                            Inc(m);
                                            extracting := True;
                                            BELL_ASCII_KATAKANA[H_2D] := BELL;
                                          end;

                                      end;

                                  end;

                              end;

                          end;

                      end;

                  end; { if character_type = SURROGATE_PAIR_C then, else }

              end; { if character_type = KATAKANA_C then, else }

          end; { if c = KANJI_C then, else }

      end;

  end;

{ Counting lines }
  if field_to_be_selected = FIRST_FIELD then
    begin
      field_number := 1;
      selected := True;
      BELL_ASCII_KATAKANA[H_2D] := H_2D; { '-' }
      extracting := False;
    end
  else
    begin
      field_number := 1;
      selected := False;
    end;

  no_splitting := False;
  unformed := False;
  for n := beginning to buffer_length_1 do
  begin
    w := Ord(buffer[n]);

    if no_splitting then
      begin
        if unformed then
          begin
            if selected then
              begin
                if extracting then
                  begin
                    selected := False;
                  end;

                continue;
              end;

            unformed := False;
            continue;
          end;

{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
        character_type := table.working(w, 0);
        if character_type = KANJI_C then
          begin
            if selected then
              begin
                if extracting then
                  begin
                    selected := False;
                  end;

                continue;
              end;

          end
        else
          begin
            if character_type = KATAKANA_C then
              begin
                if selected then
                  begin
                    if extracting then
                      begin
                        selected := False;
                      end;

                    continue;
                  end;

              end
            else
              begin
                if character_type = SURROGATE_PAIR_C then
                  begin
                    unformed := True;
                    continue;
                  end
                else
                  begin
                    if w = H_0A then
                      begin
                        Inc(count);

                        combo[m] := H_0D_ANSI;
                        Inc(m);

                        combo[m] := H_0A_ANSI;
                        Inc(m);

                        if field_to_be_selected = FIRST_FIELD then
                          begin
                            field_number := 1;
                            selected := True;
                            BELL_ASCII_KATAKANA[H_2D] := H_2D; { '-' }
                            extracting := False;
                          end
                        else
                          begin
                            field_number := 1;
                            selected := False;
                          end;

                      end
                    else
                      begin
                        if w = H_0D then
                          begin
                            continue;
                          end
                        else
                          begin
                            if w = quote_byte then
                              begin
                                no_splitting := False;
                              end
                            else
                              begin
                                if w = separator_byte then
                                  begin
                                    if remove_separators then
                                      begin
                                        continue; { Commas disappear if placed in quotes. }
                                      end
                                    else
                                      begin
                                        if selected then
                                          begin
                                            if extracting then
                                              begin
                                                selected := False;
                                              end;

                                          end;

                                      end;

                                  end
                                else
                                  begin
                                    if selected then
                                      begin
                                        b := BELL_ASCII_KATAKANA[w];
                                        if b = BELL then
                                          begin
                                            if extracting then
                                              begin
                                                selected := False;
                                              end;

                                            continue;
                                          end
                                        else
                                          begin
                                            combo[m] := Chr(w);
                                            Inc(m);
                                            extracting := True;
                                            BELL_ASCII_KATAKANA[H_2D] := BELL;
                                          end;

                                      end;

                                  end;

                              end;

                          end;

                      end;

                  end; { if character_type = SURROGATE_PAIR_C then }

              end; { if character_type = KATAKANA_C then, else }

          end; { if character_type = KANJI_C then }

      end
    else
      begin
{ if no_splitting = False then }

        if unformed then
          begin
            if selected then
              begin
                if extracting then
                  begin
                    selected := False;
                  end;

                continue;
              end;

            unformed := False;
            continue;
          end;

{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
        character_type := table.working(w, 0);
        if character_type = KANJI_C then
          begin
            if selected then
              begin
                if extracting then
                  begin
                    selected := False;
                  end;

                continue;
              end;

          end
        else
          begin
            if character_type = KATAKANA_C then
              begin
                if selected then
                  begin
                    if extracting then
                      begin
                        selected := False;
                      end;

                    continue;
                  end;

              end
            else
              begin
                if character_type = SURROGATE_PAIR_C then
                  begin
                    unformed := True;
                    continue;
                  end
                else
                  begin
                    if w = H_0A then
                      begin
                        Inc(count);

                        combo[m] := H_0D_ANSI;
                        Inc(m);

                        combo[m] := H_0A_ANSI;
                        Inc(m);

                        if field_to_be_selected = FIRST_FIELD then
                          begin
                            field_number := 1;
                            selected := True;
                            BELL_ASCII_KATAKANA[H_2D] := H_2D; { '-' }
                            extracting := False;
                          end
                        else
                          begin
                            field_number := 1;
                            selected := False;
                          end;

                      end
                    else
                      begin
                        if w = H_0D then
                          begin
                            continue;
                          end
                        else
                          begin
                            if w = quote_byte then
                              begin
                                no_splitting := True;
                              end
                            else
                              begin
                                if w = separator_byte then
                                  begin
                                    Inc(field_number);
                                    if field_number = field_to_be_selected then
                                      begin
                                        selected := True;
                                        BELL_ASCII_KATAKANA[H_2D] := H_2D; { '-' }
                                        extracting := False;
                                      end
                                    else
                                      begin
                                        selected := False;
                                      end;

                                  end
                                else
                                  begin
                                    if selected then
                                      begin
                                        b := BELL_ASCII_KATAKANA[w];
                                        if b = BELL then
                                          begin
                                            if extracting then
                                              begin
                                                selected := False;
                                              end;

                                            continue;
                                          end
                                        else
                                          begin
                                            combo[m] := Chr(w);
                                            Inc(m);
                                            extracting := True;
                                            BELL_ASCII_KATAKANA[H_2D] := BELL;
                                          end;

                                      end;

                                  end;

                              end;

                          end;

                      end;

                  end; { if character_type = SURROGATE_PAIR_C then }

              end; { if character_type = KATAKANA_C then, else }

          end; { if character_type = KANJI_C then }

      end;

  end;

  Result := bufferingWriting(m);
end;

function Selecting.extracting_integer_to_produce_MS932_LF(nn: String): Integer;
var
  b: Byte;
  block_has_been_read: Integer;
  buffer_length_1: Integer;
  character_type: AnsiChar;
  extracting: Boolean;
  field_number: Integer;
  m: Integer;
  n: Integer;
  no_splitting: Boolean;
  s: String;
  selected: Boolean;
  testFile: File;
  unformed: Boolean;
  w: Integer;
begin
  buffer_length_1 := buffer_length - 1;
  carriage_return_appears := False;
  count := 0;
  extracting := False;
  new_name := nn;

  for n := 0 to buffer_length_1 do
  begin
    buffer[n] := #0;
  end;

  try
{ Reading a file }
    AssignFile(testFile, file_name);
    Reset(testFile, SIZE_OF_WIDECHAR);

    m := 0;
    repeat
      y := CLEAR_Y_ARRAY;
      BlockRead(testFile, y, Y_COUNT, block_has_been_read);

      for n:= 0 to Y_COUNT_1 do
      begin
        buffer[m] := y[n];
        Inc(m);
        if m > buffer_length_1 then
          break;

      end;
    until m > buffer_length_1;

    CloseFile(testFile);
  except
    s := 'Reset() error-198: ' + file_name;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    Halt(1);
  end;

{ Finding the first NEW_LINE }
  if field_to_be_selected = FIRST_FIELD then
    begin
      field_number := 1;
      selected := True;
      BELL_ASCII_KATAKANA[H_2D] := H_2D; { '-' }
      extracting := False;
    end
  else
    begin
      field_number := 1;
      selected := False;
    end;

  m := 0;
  no_splitting := False;
  unformed := False;
  for n := beginning to buffer_length_1 do
  begin
    beginning := n + 1;
    w := Ord(buffer[n]);

    if no_splitting then
      begin
        if unformed then
          begin
            if selected then
              begin
                if extracting then
                  begin
                    selected := False;
                  end;

                continue;
              end;

            unformed := False;
            continue;
          end;

{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
        character_type := table.working(w, 0);
        if character_type = KANJI_C then
          begin
            if selected then
              begin
                if extracting then
                  begin
                    selected := False;
                  end;

                continue;
              end;

          end
        else
          begin
            if character_type = KATAKANA_C then
              begin
                if selected then
                  begin
                    if extracting then
                      begin
                        selected := False;
                      end;

                    continue;
                  end

              end
            else
              begin
                if character_type = SURROGATE_PAIR_C then
                  begin
                    unformed := True;
                    continue;
                  end
                else
                  begin
                    if w = H_0A then
                      begin
                        count := 1;

                        combo[m] := H_0A_ANSI;
                        Inc(m);

                        break;
                      end
                    else
                      begin
                        if w = H_0D then
                          begin
                            carriage_return_appears := True;
                            NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
                            NEW_LINE := EMPTY_ANSI + H_0D_ANSI;
                            NEW_LINE := NEW_LINE + H_0A_ANSI;
                          end
                        else
                          begin
                            if w = quote_byte then
                              begin
                                no_splitting := False;
                              end
                            else
                              begin
                                if w = separator_byte then
                                  begin
                                    if remove_separators then
                                      begin
                                        continue; { Commas disappear if placed in quotes. }
                                      end
                                    else
                                      begin
                                        if selected then
                                          begin
                                            if extracting then
                                              begin
                                                selected := False;
                                              end;

                                          end;

                                      end;

                                  end
                                else
                                  begin
                                    if selected then
                                      begin
                                        b := BELL_ASCII_KATAKANA[w];
                                        if b = BELL then
                                          begin
                                            if extracting then
                                              begin
                                                selected := False;
                                              end;

                                            continue;
                                          end
                                        else
                                          begin
                                            combo[m] := Chr(w);
                                            Inc(m);
                                            extracting := True;
                                            BELL_ASCII_KATAKANA[H_2D] := BELL;
                                          end;

                                      end;

                                  end;

                              end;

                          end;

                      end;

                  end; { if character_type = SURROGATE_PAIR_C then, else }

              end; { if character_type = KATAKANA_C then, else }

          end; { if c = KANJI_C then, else }

      end
    else
      begin
{ if no_splitting = False then }

        if unformed then
          begin
            if selected then
              begin
                if extracting then
                  begin
                    selected := False;
                  end;

                continue;
              end;

            unformed := False;
            continue;
          end;

{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
        character_type := table.working(w, 0);
        if character_type = KANJI_C then
          begin
            if selected then
              begin
                if extracting then
                  begin
                    selected := False;
                  end;

                continue;
              end;

          end
        else
          begin
            if character_type = KATAKANA_C then
              begin
                if selected then
                  begin
                    if extracting then
                      begin
                        selected := False;
                      end;

                    continue;
                  end

              end
            else
              begin
                if character_type = SURROGATE_PAIR_C then
                  begin
                    unformed := True;
                    continue;
                  end
                else
                  begin
                    if w = H_0A then
                      begin
                        count := 1;

                        combo[m] := H_0A_ANSI;
                        Inc(m);

                        break;
                      end
                    else
                      begin
                        if w = H_0D then
                          begin
                            carriage_return_appears := True;
                            NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
                            NEW_LINE := EMPTY_ANSI + H_0D_ANSI;
                            NEW_LINE := NEW_LINE + H_0A_ANSI;
                          end
                        else
                          begin
                            if w = quote_byte then
                              begin
                                no_splitting := True;
                              end
                            else
                              begin
                                if w = separator_byte then
                                  begin
                                    Inc(field_number);
                                    if field_number = field_to_be_selected then
                                      begin
                                        selected := True;
                                        BELL_ASCII_KATAKANA[H_2D] := H_2D; { '-' }
                                        extracting := False;
                                      end
                                    else
                                      begin
                                        selected := False;
                                      end;

                                  end
                                else
                                  begin
                                    if selected then
                                      begin
                                        b := BELL_ASCII_KATAKANA[w];
                                        if b = BELL then
                                          begin
                                            if extracting then
                                              begin
                                                selected := False;
                                              end;

                                            continue;
                                          end
                                        else
                                          begin
                                            combo[m] := Chr(w);
                                            Inc(m);
                                            extracting := True;
                                            BELL_ASCII_KATAKANA[H_2D] := BELL;
                                          end;

                                      end;

                                  end;

                              end;

                          end;

                      end;

                  end; { if character_type = SURROGATE_PAIR_C then, else }

              end; { if character_type = KATAKANA_C then, else }

          end; { if c = KANJI_C then, else }

      end;

  end;

{ Counting lines }
  if field_to_be_selected = FIRST_FIELD then
    begin
      field_number := 1;
      selected := True;
      BELL_ASCII_KATAKANA[H_2D] := H_2D; { '-' }
      extracting := False;
    end
  else
    begin
      field_number := 1;
      selected := False;
    end;

  no_splitting := False;
  unformed := False;
  for n := beginning to buffer_length_1 do
  begin
    w := Ord(buffer[n]);

    if no_splitting then
      begin
        if unformed then
          begin
            if selected then
              begin
                if extracting then
                  begin
                    selected := False;
                  end;

                continue;
              end;

            unformed := False;
            continue;
          end;

{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
        character_type := table.working(w, 0);
        if character_type = KANJI_C then
          begin
            if selected then
              begin
                if extracting then
                  begin
                    selected := False;
                  end;

                continue;
              end;

          end
        else
          begin
            if character_type = KATAKANA_C then
              begin
                if selected then
                  begin
                    if extracting then
                      begin
                        selected := False;
                      end;

                    continue;
                  end;

              end
            else
              begin
                if character_type = SURROGATE_PAIR_C then
                  begin
                    unformed := True;
                    continue;
                  end
                else
                  begin
                    if w = H_0A then
                      begin
                        Inc(count);

                        combo[m] := H_0A_ANSI;
                        Inc(m);

                        if field_to_be_selected = FIRST_FIELD then
                          begin
                            field_number := 1;
                            selected := True;
                            BELL_ASCII_KATAKANA[H_2D] := H_2D; { '-' }
                            extracting := False;
                          end
                        else
                          begin
                            field_number := 1;
                            selected := False;
                          end;

                      end
                    else
                      begin
                        if w = H_0D then
                          begin
                            continue;
                          end
                        else
                          begin
                            if w = quote_byte then
                              begin
                                no_splitting := False;
                              end
                            else
                              begin
                                if w = separator_byte then
                                  begin
                                    if remove_separators then
                                      begin
                                        continue; { Commas disappear if placed in quotes. }
                                      end
                                    else
                                      begin
                                        if selected then
                                          begin
                                            if extracting then
                                              begin
                                                selected := False;
                                              end;

                                          end;

                                      end;

                                  end
                                else
                                  begin
                                    if selected then
                                      begin
                                        b := BELL_ASCII_KATAKANA[w];
                                        if b = BELL then
                                          begin
                                            if extracting then
                                              begin
                                                selected := False;
                                              end;

                                            continue;
                                          end
                                        else
                                          begin
                                            combo[m] := Chr(w);
                                            Inc(m);
                                            extracting := True;
                                            BELL_ASCII_KATAKANA[H_2D] := BELL;
                                          end;

                                      end;

                                  end;

                              end;

                          end;

                      end;

                  end; { if character_type = SURROGATE_PAIR_C then }

              end; { if character_type = KATAKANA_C then, else }

          end; { if character_type = KANJI_C then }

      end
    else
      begin
{ if no_splitting = False then }

        if unformed then
          begin
            if selected then
              begin
                if extracting then
                  begin
                    selected := False;
                  end;

                continue;
              end;

            unformed := False;
            continue;
          end;

{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
        character_type := table.working(w, 0);
        if character_type = KANJI_C then
          begin
            if selected then
              begin
                if extracting then
                  begin
                    selected := False;
                  end;

                continue;
              end;

          end
        else
          begin
            if character_type = KATAKANA_C then
              begin
                if selected then
                  begin
                    if extracting then
                      begin
                        selected := False;
                      end;

                    continue;
                  end;

              end
            else
              begin
                if character_type = SURROGATE_PAIR_C then
                  begin
                    unformed := True;
                    continue;
                  end
                else
                  begin
                    if w = H_0A then
                      begin
                        Inc(count);

                        combo[m] := H_0A_ANSI;
                        Inc(m);

                        if field_to_be_selected = FIRST_FIELD then
                          begin
                            field_number := 1;
                            selected := True;
                            BELL_ASCII_KATAKANA[H_2D] := H_2D; { '-' }
                            extracting := False;
                          end
                        else
                          begin
                            field_number := 1;
                            selected := False;
                          end;

                      end
                    else
                      begin
                        if w = H_0D then
                          begin
                            continue;
                          end
                        else
                          begin
                            if w = quote_byte then
                              begin
                                no_splitting := True;
                              end
                            else
                              begin
                                if w = separator_byte then
                                  begin
                                    Inc(field_number);
                                    if field_number = field_to_be_selected then
                                      begin
                                        selected := True;
                                        BELL_ASCII_KATAKANA[H_2D] := H_2D; { '-' }
                                        extracting := False;
                                      end
                                    else
                                      begin
                                        selected := False;
                                      end;

                                  end
                                else
                                  begin
                                    if selected then
                                      begin
                                        b := BELL_ASCII_KATAKANA[w];
                                        if b = BELL then
                                          begin
                                            if extracting then
                                              begin
                                                selected := False;
                                              end;

                                            continue;
                                          end
                                        else
                                          begin
                                            combo[m] := Chr(w);
                                            Inc(m);
                                            extracting := True;
                                            BELL_ASCII_KATAKANA[H_2D] := BELL;
                                          end;

                                      end;

                                  end;

                              end;

                          end;

                      end;

                  end; { if character_type = SURROGATE_PAIR_C then }

              end; { if character_type = KATAKANA_C then, else }

          end; { if character_type = KANJI_C then }

      end;

  end;

  Result := bufferingWriting(m);
end;

function Selecting.extracting_integer_to_produce_MS932_text(nn: String): Integer;
var
  b: Byte;
  block_has_been_read: Integer;
  buffer_length_1: Integer;
  character_type: AnsiChar;
  extracting: Boolean;
  field_number: Integer;
  m: Integer;
  n: Integer;
  no_splitting: Boolean;
  s: String;
  selected: Boolean;
  testFile: File;
  unformed: Boolean;
  w: Integer;
begin
  buffer_length_1 := buffer_length - 1;
  carriage_return_appears := False;
  count := 0;
  extracting := False;
  new_name := nn;

  for n := 0 to buffer_length_1 do
  begin
    buffer[n] := #0;
  end;

  try
{ Reading a file }
    AssignFile(testFile, file_name);
    Reset(testFile, SIZE_OF_WIDECHAR);

    m := 0;
    repeat
      y := CLEAR_Y_ARRAY;
      BlockRead(testFile, y, Y_COUNT, block_has_been_read);

      for n:= 0 to Y_COUNT_1 do
      begin
        buffer[m] := y[n];
        Inc(m);
        if m > buffer_length_1 then
          break;

      end;
    until m > buffer_length_1;

    CloseFile(testFile);
  except
    s := 'Reset() error-199: ' + file_name;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    Halt(1);
  end;

{ Finding the first NEW_LINE }
  if field_to_be_selected = FIRST_FIELD then
    begin
      field_number := 1;
      selected := True;
      BELL_ASCII_KATAKANA[H_2D] := H_2D; { '-' }
      extracting := False;
    end
  else
    begin
      field_number := 1;
      selected := False;
    end;

  m := 0;
  no_splitting := False;
  unformed := False;
  for n := beginning to buffer_length_1 do
  begin
    beginning := n + 1;
    w := Ord(buffer[n]);

    if no_splitting then
      begin
        if unformed then
          begin
            if selected then
              begin
                if extracting then
                  begin
                    selected := False;
                  end;

                continue;
              end;

            unformed := False;
            continue;
          end;

{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
        character_type := table.working(w, 0);
        if character_type = KANJI_C then
          begin
            if selected then
              begin
                if extracting then
                  begin
                    selected := False;
                  end;

                continue;
              end;

          end
        else
          begin
            if character_type = KATAKANA_C then
              begin
                if selected then
                  begin
                    if extracting then
                      begin
                        selected := False;
                      end;

                    continue;
                  end

              end
            else
              begin
                if character_type = SURROGATE_PAIR_C then
                  begin
                    unformed := True;
                    continue;
                  end
                else
                  begin
                    if w = H_0A then
                      begin
                        count := 1;

                        combo[m] := H_0A_ANSI;
                        Inc(m);

                        break;
                      end
                    else
                      begin
                        if w = H_0D then
                          begin
                            combo[m] := H_0D_ANSI;
                            Inc(m);

                            carriage_return_appears := True;
                            NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
                            NEW_LINE := EMPTY_ANSI + H_0D_ANSI;
                            NEW_LINE := NEW_LINE + H_0A_ANSI;
                          end
                        else
                          begin
                            if w = quote_byte then
                              begin
                                no_splitting := False;
                              end
                            else
                              begin
                                if w = separator_byte then
                                  begin
                                    if remove_separators then
                                      begin
                                        continue; { Commas disappear if placed in quotes. }
                                      end
                                    else
                                      begin
                                        if selected then
                                          begin
                                            if extracting then
                                              begin
                                                selected := False;
                                              end;

                                          end;

                                      end;

                                  end
                                else
                                  begin
                                    if selected then
                                      begin
                                        b := BELL_ASCII_KATAKANA[w];
                                        if b = BELL then
                                          begin
                                            if extracting then
                                              begin
                                                selected := False;
                                              end;

                                            continue;
                                          end
                                        else
                                          begin
                                            combo[m] := Chr(w);
                                            Inc(m);
                                            extracting := True;
                                            BELL_ASCII_KATAKANA[H_2D] := BELL;
                                          end;

                                      end;

                                  end;

                              end;

                          end;

                      end;

                  end; { if character_type = SURROGATE_PAIR_C then, else }

              end; { if character_type = KATAKANA_C then, else }

          end; { if c = KANJI_C then, else }

      end
    else
      begin
{ if no_splitting = False then }

        if unformed then
          begin
            if selected then
              begin
                if extracting then
                  begin
                    selected := False;
                  end;

                continue;
              end;

            unformed := False;
            continue;
          end;

{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
        character_type := table.working(w, 0);
        if character_type = KANJI_C then
          begin
            if selected then
              begin
                if extracting then
                  begin
                    selected := False;
                  end;

                continue;
              end;

          end
        else
          begin
            if character_type = KATAKANA_C then
              begin
                if selected then
                  begin
                    if extracting then
                      begin
                        selected := False;
                      end;

                    continue;
                  end

              end
            else
              begin
                if character_type = SURROGATE_PAIR_C then
                  begin
                    unformed := True;
                    continue;
                  end
                else
                  begin
                    if w = H_0A then
                      begin
                        count := 1;

                        combo[m] := H_0A_ANSI;
                        Inc(m);

                        break;
                      end
                    else
                      begin
                        if w = H_0D then
                          begin
                            combo[m] := H_0D_ANSI;
                            Inc(m);

                            carriage_return_appears := True;
                            NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
                            NEW_LINE := EMPTY_ANSI + H_0D_ANSI;
                            NEW_LINE := NEW_LINE + H_0A_ANSI;
                          end
                        else
                          begin
                            if w = quote_byte then
                              begin
                                no_splitting := True;
                              end
                            else
                              begin
                                if w = separator_byte then
                                  begin
                                    Inc(field_number);
                                    if field_number = field_to_be_selected then
                                      begin
                                        selected := True;
                                        BELL_ASCII_KATAKANA[H_2D] := H_2D; { '-' }
                                        extracting := False;
                                      end
                                    else
                                      begin
                                        selected := False;
                                      end;

                                  end
                                else
                                  begin
                                    if selected then
                                      begin
                                        b := BELL_ASCII_KATAKANA[w];
                                        if b = BELL then
                                          begin
                                            if extracting then
                                              begin
                                                selected := False;
                                              end;

                                            continue;
                                          end
                                        else
                                          begin
                                            combo[m] := Chr(w);
                                            Inc(m);
                                            extracting := True;
                                            BELL_ASCII_KATAKANA[H_2D] := BELL;
                                          end;

                                      end;

                                  end;

                              end;

                          end;

                      end;

                  end; { if character_type = SURROGATE_PAIR_C then, else }

              end; { if character_type = KATAKANA_C then, else }

          end; { if c = KANJI_C then, else }

      end;

  end;

{ Counting lines }
  if field_to_be_selected = FIRST_FIELD then
    begin
      field_number := 1;
      selected := True;
      BELL_ASCII_KATAKANA[H_2D] := H_2D; { '-' }
      extracting := False;
    end
  else
    begin
      field_number := 1;
      selected := False;
    end;

  no_splitting := False;
  unformed := False;
  for n := beginning to buffer_length_1 do
  begin
    w := Ord(buffer[n]);

    if no_splitting then
      begin
        if unformed then
          begin
            if selected then
              begin
                if extracting then
                  begin
                    selected := False;
                  end;

                continue;
              end;

            unformed := False;
            continue;
          end;

{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
        character_type := table.working(w, 0);
        if character_type = KANJI_C then
          begin
            if selected then
              begin
                if extracting then
                  begin
                    selected := False;
                  end;

                continue;
              end;

          end
        else
          begin
            if character_type = KATAKANA_C then
              begin
                if selected then
                  begin
                    if extracting then
                      begin
                        selected := False;
                      end;

                    continue;
                  end;

              end
            else
              begin
                if character_type = SURROGATE_PAIR_C then
                  begin
                    unformed := True;
                    continue;
                  end
                else
                  begin
                    if w = H_0A then
                      begin
                        Inc(count);

                        combo[m] := H_0A_ANSI;
                        Inc(m);

                        if field_to_be_selected = FIRST_FIELD then
                          begin
                            field_number := 1;
                            selected := True;
                            BELL_ASCII_KATAKANA[H_2D] := H_2D; { '-' }
                            extracting := False;
                          end
                        else
                          begin
                            field_number := 1;
                            selected := False;
                          end;

                      end
                    else
                      begin
                        if w = H_0D then
                          begin
                            combo[m] := H_0D_ANSI;
                            Inc(m);
                            continue;
                          end
                        else
                          begin
                            if w = quote_byte then
                              begin
                                no_splitting := False;
                              end
                            else
                              begin
                                if w = separator_byte then
                                  begin
                                    if remove_separators then
                                      begin
                                        continue; { Commas disappear if placed in quotes. }
                                      end
                                    else
                                      begin
                                        if selected then
                                          begin
                                            if extracting then
                                              begin
                                                selected := False;
                                              end;

                                          end;

                                      end;

                                  end
                                else
                                  begin
                                    if selected then
                                      begin
                                        b := BELL_ASCII_KATAKANA[w];
                                        if b = BELL then
                                          begin
                                            if extracting then
                                              begin
                                                selected := False;
                                              end;

                                            continue;
                                          end
                                        else
                                          begin
                                            combo[m] := Chr(w);
                                            Inc(m);
                                            extracting := True;
                                            BELL_ASCII_KATAKANA[H_2D] := BELL;
                                          end;

                                      end;

                                  end;

                              end;

                          end;

                      end;

                  end; { if character_type = SURROGATE_PAIR_C then }

              end; { if character_type = KATAKANA_C then, else }

          end; { if character_type = KANJI_C then }

      end
    else
      begin
{ if no_splitting = False then }

        if unformed then
          begin
            if selected then
              begin
                if extracting then
                  begin
                    selected := False;
                  end;

                continue;
              end;

            unformed := False;
            continue;
          end;

{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
        character_type := table.working(w, 0);
        if character_type = KANJI_C then
          begin
            if selected then
              begin
                if extracting then
                  begin
                    selected := False;
                  end;

                continue;
              end;

          end
        else
          begin
            if character_type = KATAKANA_C then
              begin
                if selected then
                  begin
                    if extracting then
                      begin
                        selected := False;
                      end;

                    continue;
                  end;

              end
            else
              begin
                if character_type = SURROGATE_PAIR_C then
                  begin
                    unformed := True;
                    continue;
                  end
                else
                  begin
                    if w = H_0A then
                      begin
                        Inc(count);

                        combo[m] := H_0A_ANSI;
                        Inc(m);

                        if field_to_be_selected = FIRST_FIELD then
                          begin
                            field_number := 1;
                            selected := True;
                            BELL_ASCII_KATAKANA[H_2D] := H_2D; { '-' }
                            extracting := False;
                          end
                        else
                          begin
                            field_number := 1;
                            selected := False;
                          end;

                      end
                    else
                      begin
                        if w = H_0D then
                          begin
                            combo[m] := H_0D_ANSI;
                            Inc(m);
                            continue;
                          end
                        else
                          begin
                            if w = quote_byte then
                              begin
                                no_splitting := True;
                              end
                            else
                              begin
                                if w = separator_byte then
                                  begin
                                    Inc(field_number);
                                    if field_number = field_to_be_selected then
                                      begin
                                        selected := True;
                                        BELL_ASCII_KATAKANA[H_2D] := H_2D; { '-' }
                                        extracting := False;
                                      end
                                    else
                                      begin
                                        selected := False;
                                      end;

                                  end
                                else
                                  begin
                                    if selected then
                                      begin
                                        b := BELL_ASCII_KATAKANA[w];
                                        if b = BELL then
                                          begin
                                            if extracting then
                                              begin
                                                selected := False;
                                              end;

                                            continue;
                                          end
                                        else
                                          begin
                                            combo[m] := Chr(w);
                                            Inc(m);
                                            extracting := True;
                                            BELL_ASCII_KATAKANA[H_2D] := BELL;
                                          end;

                                      end;

                                  end;

                              end;

                          end;

                      end;

                  end; { if character_type = SURROGATE_PAIR_C then }

              end; { if character_type = KATAKANA_C then, else }

          end; { if character_type = KANJI_C then }

      end;

  end;

  Result := bufferingWriting(m);
end;

function Selecting.file_size(name: String): Integer;
var
  t: Integer;
  size: Integer;
begin
  t := FileOpen(name, fmOpenRead or fmShareDenyNone);
  size := FileSeek(t, 0, 2); { EOF }
  FileClose(t);
  Result := size;
end;

function Selecting.lineCount: Integer;
begin
  Result := count;
end;

function Selecting.newLine: AnsiString;
begin
  Result := NEW_LINE;
end;

function Selecting.produce_MS932_phrase(s: PWideChar; s_length: Integer): AnsiCharArray;
var
  character_type: AnsiChar;
  field_number: Integer;
  high_part_of_kanji: Integer;
  low_part_of_kanji: Integer;
  m: Integer;
  n: Integer;
  no_splitting: Boolean;
  s_length_1: Integer;
  selected: Boolean;
  unformed: Boolean;
  w: Integer;
begin
  carriage_return_appears := False;
  s_length_1 := s_length - 1;
  v := CLEAR_ARRAY;
  V_SIZE := SizeOf(v);
  V_SIZE_1 := V_SIZE - 1;

{ Finding the first NEW_LINE }
  if field_to_be_selected = FIRST_FIELD then
    begin
      field_number := 1;
      selected := True;
    end
  else
    begin
      field_number := 1;
      selected := False;
    end;

  high_part_of_kanji := H_D840;
  m := 0;
  no_splitting := False;
  unformed := False;
  for n := beginning to s_length_1 do
  begin
    beginning := n + 1;
    w := Ord(s[n]);

    if no_splitting then
      begin
        if unformed then
          begin
            low_part_of_kanji := w;

            if selected then
              begin
                v[m] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 1);
                v[ m + 1 ] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 2);
                Inc(m, KANJI_SIZE);
              end;

            unformed := False;
            continue;
          end;

{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
        character_type := table.working(w, 0);
        if character_type = KANJI_C then
          begin
            if selected then
              begin
                v[m] := table.working(w, 1);
                v[ m + 1 ] := table.working(w, 2);
                Inc(m, KANJI_SIZE);
              end;

          end
        else
          begin
            if character_type = KATAKANA_C then
              begin
                if selected then
                  begin
                    v[m] := table.working(w, 1);
                    Inc(m);
                  end

              end
            else
              begin
                if character_type = SURROGATE_PAIR_C then
                  begin
                    high_part_of_kanji := w;
                    unformed := True;
                    continue;
                  end
                else
                  begin
                    if w = H_0A then
                      begin
                        count := 1;

                        v[m] := H_0D_ANSI;
                        Inc(m);

                        v[m] := H_0A_ANSI;
                        Inc(m);

                        beginning := n + 1;
                        break;
                      end
                    else
                      begin
                        if w = H_0D then
                          begin
                            carriage_return_appears := True;
                            NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
                            NEW_LINE := EMPTY_ANSI + H_0D_ANSI;
                            NEW_LINE := NEW_LINE + H_0A_ANSI;
                          end
                        else
                          begin
                            if w = quote_byte then
                              begin
                                no_splitting := False;
                              end
                            else
                              begin
                                if w = separator_byte then
                                  begin
                                    if remove_separators then
                                      begin
                                        continue; { Commas disappear if placed in quotes. }
                                      end
                                    else
                                      begin
                                        if selected then
                                          begin
                                            v[m] := Chr(w);
                                            Inc(m);
                                          end;

                                      end;

                                  end
                                else
                                  begin
                                    if selected then
                                      begin
                                        v[m] := Chr(w);
                                        Inc(m);
                                      end;

                                  end;

                              end;

                          end;

                      end;

                  end; { if character_type = SURROGATE_PAIR_C then, else }

              end; { if character_type = KATAKANA_C then, else }

          end; { if c = KANJI_C then, else }

      end
    else
      begin
{ if no_splitting = False then }

        if unformed then
          begin
            low_part_of_kanji := w;

            if selected then
              begin
                v[m] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 1);
                v[ m + 1 ] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 2);
                Inc(m, KANJI_SIZE);
              end;

            unformed := False;
            continue;
          end;

{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
        character_type := table.working(w, 0);
        if character_type = KANJI_C then
          begin
            if selected then
              begin
                v[m] := table.working(w, 1);
                v[ m + 1 ] := table.working(w, 2);
                Inc(m, KANJI_SIZE);
              end;

          end
        else
          begin
            if character_type = KATAKANA_C then
              begin
                if selected then
                  begin
                    v[m] := table.working(w, 1);
                    Inc(m);
                  end

              end
            else
              begin
                if character_type = SURROGATE_PAIR_C then
                  begin
                    high_part_of_kanji := w;
                    unformed := True;
                    continue;
                  end
                else
                  begin
                    if w = H_0A then
                      begin
                        count := 1;

                        v[m] := H_0D_ANSI;
                        Inc(m);

                        v[m] := H_0A_ANSI;
                        Inc(m);

                        break;
                      end
                    else
                      begin
                        if w = H_0D then
                          begin
                            carriage_return_appears := True;
                            NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
                            NEW_LINE := EMPTY_ANSI + H_0D_ANSI;
                            NEW_LINE := NEW_LINE + H_0A_ANSI;
                          end
                        else
                          begin
                            if w = quote_byte then
                              begin
                                no_splitting := True;
                              end
                            else
                              begin
                                if w = separator_byte then
                                  begin
                                    if selected then
                                      begin
                                        v[m] := Chr(w);
                                        Inc(m);
                                      end;

                                    Inc(field_number);
                                    if field_number = field_to_be_selected then
                                      begin
                                        selected := True;
                                      end
                                    else
                                      begin
                                        selected := False;
                                      end;

                                  end
                                else
                                  begin
                                    if selected then
                                      begin
                                        v[m] := Chr(w);
                                        Inc(m);
                                      end;

                                  end;

                              end;

                          end;

                      end;

                  end; { if character_type = SURROGATE_PAIR_C then, else }

              end; { if character_type = KATAKANA_C then, else }

          end; { if c = KANJI_C then, else }

      end;

  end;

  if beginning > 0 then
    begin
{ Counting lines }
      if field_to_be_selected = FIRST_FIELD then
        begin
          field_number := 1;
          selected := True;
        end
      else
        begin
          field_number := 1;
          selected := False;
        end;

      high_part_of_kanji := H_D840;
      no_splitting := False;
      unformed := False;
      for n := beginning to s_length_1 do
      begin
        w := Ord(s[n]);

        if no_splitting then
          begin
            if unformed then
              begin
                low_part_of_kanji := w;

                if selected then
                  begin
                    v[m] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 1);
                    v[ m + 1 ] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 2);
                    Inc(m, KANJI_SIZE);
                  end;

                unformed := False;
                continue;
              end;

{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
            character_type := table.working(w, 0);
            if character_type = KANJI_C then
              begin
                if selected then
                  begin
                    v[m] := table.working(w, 1);
                    v[ m + 1 ] := table.working(w, 2);
                    Inc(m, KANJI_SIZE);
                  end;

              end
            else
              begin
                if character_type = KATAKANA_C then
                  begin
                    if selected then
                      begin
                        v[m] := table.working(w, 1);
                        Inc(m);
                      end;

                  end
                else
                  begin
                    if character_type = SURROGATE_PAIR_C then
                      begin
                        high_part_of_kanji := w;
                        unformed := True;
                        continue;
                      end
                    else
                      begin
                        if w = H_0A then
                          begin
                            Inc(count);

                            v[m] := H_0D_ANSI;
                            Inc(m);

                            v[m] := H_0A_ANSI;
                            Inc(m);

                            if field_to_be_selected = FIRST_FIELD then
                              begin
                                field_number := 1;
                                selected := True;
                              end
                            else
                              begin
                                field_number := 1;
                                selected := False;
                              end;

                          end
                        else
                          begin
                            if w = H_0D then
                              begin
                                continue;
                              end
                            else
                              begin
                                if w = quote_byte then
                                  begin
                                    no_splitting := False;
                                  end
                                else
                                  begin
                                    if w = separator_byte then
                                      begin
                                        if remove_separators then
                                          begin
                                            continue; { Commas disappear if placed in quotes. }
                                          end
                                        else
                                          begin
                                            if selected then
                                              begin
                                                v[m] := Chr(w);
                                                Inc(m);
                                              end;

                                          end;

                                      end
                                    else
                                      begin
                                        if selected then
                                          begin
                                            v[m] := Chr(w);
                                            Inc(m);
                                          end;

                                      end;

                                  end;

                              end;

                          end;

                      end; { if character_type = SURROGATE_PAIR_C then }

                  end; { if character_type = KATAKANA_C then, else }

              end; { if character_type = KANJI_C then }

          end
        else
          begin
{ if no_splitting = False then }

            if unformed then
              begin
                low_part_of_kanji := w;

                if selected then
                  begin
                    v[m] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 1);
                    v[ m + 1 ] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 2);
                    Inc(m, KANJI_SIZE);
                  end;

                unformed := False;
                continue;
              end;

{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
            character_type := table.working(w, 0);
            if character_type = KANJI_C then
              begin
                if selected then
                  begin
                    v[m] := table.working(w, 1);
                    v[ m + 1 ] := table.working(w, 2);
                    Inc(m, KANJI_SIZE);
                  end;

              end
            else
              begin
                if character_type = KATAKANA_C then
                  begin
                    if selected then
                      begin
                        v[m] := table.working(w, 1);
                        Inc(m);
                      end;

                  end
                else
                  begin
                    if character_type = SURROGATE_PAIR_C then
                      begin
                        high_part_of_kanji := w;
                        unformed := True;
                        continue;
                      end
                    else
                      begin
                        if w = H_0A then
                          begin
                            Inc(count);

                            v[m] := H_0D_ANSI;
                            Inc(m);

                            v[m] := H_0A_ANSI;
                            Inc(m);

                            if field_to_be_selected = FIRST_FIELD then
                              begin
                                field_number := 1;
                                selected := True;
                              end
                            else
                              begin
                                field_number := 1;
                                selected := False;
                              end;

                          end
                        else
                          begin
                            if w = H_0D then
                              begin
                                continue;
                              end
                            else
                              begin
                                if w = quote_byte then
                                  begin
                                    no_splitting := True;
                                  end
                                else
                                  begin
                                    if w = separator_byte then
                                      begin
                                        if selected then
                                          begin
                                            v[m] := Chr(w);
                                            Inc(m);
                                          end;

                                        Inc(field_number);
                                        if field_number = field_to_be_selected then
                                          begin
                                            selected := True;
                                          end
                                        else
                                          begin
                                            selected := False;
                                          end;

                                      end
                                    else
                                      begin
                                        if selected then
                                          begin
                                            v[m] := Chr(w);
                                            Inc(m);
                                          end;

                                      end;

                                  end;

                              end;

                          end;

                      end; { if character_type = SURROGATE_PAIR_C then }

                  end; { if character_type = KATAKANA_C then, else }

              end; { if character_type = KANJI_C then }

          end;

      end;

    end; { if beginning > 0 then }

  V_SIZE := m;
  V_SIZE_1 := V_SIZE - 1;
  Result := v;
end;

function Selecting.produceBinaryFile(name: String; no_removing: Boolean): Integer;
var
  block_has_been_read: Integer;
  block_has_been_written: Integer;
  newFile: File;
  s: String;
  sum: Integer;
  testFile: File;
begin
  sum := 0;

  try
    AssignFile(testFile, name);
    Reset(testFile, SIZE_OF_ANSICHAR);

    AssignFile(newFile, TEMPORARY_BINARY);
    Rewrite(newFile, SIZE_OF_ANSICHAR);

    repeat
      v := CLEAR_ARRAY;
      BlockRead(testFile, v, V_SIZE, block_has_been_read);
      BlockWrite(newFile, v, block_has_been_read, block_has_been_written);
      sum := sum + block_has_been_written;
    until (block_has_been_read = 0) or (block_has_been_read <> block_has_been_written);

    CloseFile(newFile);
    CloseFile(testFile);

    if not no_removing then
      begin
        AssignFile(testFile, name);
        Erase(testFile);
      end;

  except
    s := 'Reset() error-193: ' + name;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    Halt(1);
  end;

  Result := sum;
end;

function Selecting.reading_to_produce_MS932: AnsiCharArray;
var
  block_has_been_read: Integer;
  buffer_length_1: Integer;
  character_type: AnsiChar;
  field_number: Integer;
  high_part_of_kanji: Integer;
  low_part_of_kanji: Integer;
  m: Integer;
  n: Integer;
  no_splitting: Boolean;
  s: String;
  selected: Boolean;
  testFile: File;
  unformed: Boolean;
  w: Integer;
begin
  buffer_length_1 := buffer_length - 1;
  carriage_return_appears := False;
  count := 0;
  v := CLEAR_ARRAY;
  V_SIZE := SizeOf(v);
  V_SIZE_1 := V_SIZE - 1;

  for n := 0 to buffer_length_1 do
  begin
    buffer[n] := #0;
  end;

  try
{ Reading a file }
    AssignFile(testFile, file_name);
    Reset(testFile, SIZE_OF_WIDECHAR);

    m := 0;
    repeat
      y := CLEAR_Y_ARRAY;
      BlockRead(testFile, y, Y_COUNT, block_has_been_read);

      for n:= 0 to Y_COUNT_1 do
      begin
        buffer[m] := y[n];
        Inc(m);
        if m > buffer_length_1 then
          break;

      end;
    until m > buffer_length_1;

    CloseFile(testFile);
  except
    s := 'Reset() error-217: ' + file_name;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    Halt(1);
  end;

{ Finding the first NEW_LINE }
  if field_to_be_selected = FIRST_FIELD then
    begin
      field_number := 1;
      selected := True;
    end
  else
    begin
      field_number := 1;
      selected := False;
    end;

  high_part_of_kanji := H_D840;
  m := 0;
  no_splitting := False;
  unformed := False;
  for n := beginning to buffer_length_1 do
  begin
    beginning := n + 1;
    w := Ord(buffer[n]);

    if no_splitting then
      begin
        if unformed then
          begin
            low_part_of_kanji := w;

            if selected then
              begin
                v[m] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 1);
                v[ m + 1 ] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 2);
                Inc(m, KANJI_SIZE);
              end;

            unformed := False;
            continue;
          end;

{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
        character_type := table.working(w, 0);
        if character_type = KANJI_C then
          begin
            if selected then
              begin
                v[m] := table.working(w, 1);
                v[ m + 1 ] := table.working(w, 2);
                Inc(m, KANJI_SIZE);
              end;

          end
        else
          begin
            if character_type = KATAKANA_C then
              begin
                if selected then
                  begin
                    v[m] := table.working(w, 1);
                    Inc(m);
                  end

              end
            else
              begin
                if character_type = SURROGATE_PAIR_C then
                  begin
                    high_part_of_kanji := w;
                    unformed := True;
                    continue;
                  end
                else
                  begin
                    if w = H_0A then
                      begin
                        count := 1;

                        v[m] := H_0D_ANSI;
                        Inc(m);

                        v[m] := H_0A_ANSI;
                        Inc(m);

                        break;
                      end
                    else
                      begin
                        if w = H_0D then
                          begin
                            carriage_return_appears := True;
                            NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
                            NEW_LINE := EMPTY_ANSI + H_0D_ANSI;
                            NEW_LINE := NEW_LINE + H_0A_ANSI;
                          end
                        else
                          begin
                            if w = quote_byte then
                              begin
                                no_splitting := False;
                              end
                            else
                              begin
                                if w = separator_byte then
                                  begin
                                    if remove_separators then
                                      begin
                                        continue; { Commas disappear if placed in quotes. }
                                      end
                                    else
                                      begin
                                        if selected then
                                          begin
                                            v[m] := Chr(w);
                                            Inc(m);
                                          end;

                                      end;

                                  end
                                else
                                  begin
                                    if selected then
                                      begin
                                        v[m] := Chr(w);
                                        Inc(m);
                                      end;

                                  end;

                              end;

                          end;

                      end;

                  end; { if character_type = SURROGATE_PAIR_C then, else }

              end; { if character_type = KATAKANA_C then, else }

          end; { if c = KANJI_C then, else }

      end
    else
      begin
{ if no_splitting = False then }

        if unformed then
          begin
            low_part_of_kanji := w;

            if selected then
              begin
                v[m] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 1);
                v[ m + 1 ] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 2);
                Inc(m, KANJI_SIZE);
              end;

            unformed := False;
            continue;
          end;

{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
        character_type := table.working(w, 0);
        if character_type = KANJI_C then
          begin
            if selected then
              begin
                v[m] := table.working(w, 1);
                v[ m + 1 ] := table.working(w, 2);
                Inc(m, KANJI_SIZE);
              end;

          end
        else
          begin
            if character_type = KATAKANA_C then
              begin
                if selected then
                  begin
                    v[m] := table.working(w, 1);
                    Inc(m);
                  end

              end
            else
              begin
                if character_type = SURROGATE_PAIR_C then
                  begin
                    high_part_of_kanji := w;
                    unformed := True;
                    continue;
                  end
                else
                  begin
                    if w = H_0A then
                      begin
                        count := 1;

                        v[m] := H_0D_ANSI;
                        Inc(m);

                        v[m] := H_0A_ANSI;
                        Inc(m);

                        break;
                      end
                    else
                      begin
                        if w = H_0D then
                          begin
                            carriage_return_appears := True;
                            NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
                            NEW_LINE := EMPTY_ANSI + H_0D_ANSI;
                            NEW_LINE := NEW_LINE + H_0A_ANSI;
                          end
                        else
                          begin
                            if w = quote_byte then
                              begin
                                no_splitting := True;
                              end
                            else
                              begin
                                if w = separator_byte then
                                  begin
                                    if selected then
                                      begin
                                        v[m] := Chr(w);
                                        Inc(m);
                                      end;

                                    Inc(field_number);
                                    if field_number = field_to_be_selected then
                                      begin
                                        selected := True;
                                      end
                                    else
                                      begin
                                        selected := False;
                                      end;

                                  end
                                else
                                  begin
                                    if selected then
                                      begin
                                        v[m] := Chr(w);
                                        Inc(m);
                                      end;

                                  end;

                              end;

                          end;

                      end;

                  end; { if character_type = SURROGATE_PAIR_C then, else }

              end; { if character_type = KATAKANA_C then, else }

          end; { if c = KANJI_C then, else }

      end;

  end;

{ Counting lines }
  if field_to_be_selected = FIRST_FIELD then
    begin
      field_number := 1;
      selected := True;
    end
  else
    begin
      field_number := 1;
      selected := False;
    end;

  high_part_of_kanji := H_D840;
  no_splitting := False;
  unformed := False;
  for n := beginning to buffer_length_1 do
  begin
    w := Ord(buffer[n]);

    if no_splitting then
      begin
        if unformed then
          begin
            low_part_of_kanji := w;

            if selected then
              begin
                v[m] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 1);
                v[ m + 1 ] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 2);
                Inc(m, KANJI_SIZE);
              end;

            unformed := False;
            continue;
          end;

{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
        character_type := table.working(w, 0);
        if character_type = KANJI_C then
          begin
            if selected then
              begin
                v[m] := table.working(w, 1);
                v[ m + 1 ] := table.working(w, 2);
                Inc(m, KANJI_SIZE);
              end;

          end
        else
          begin
            if character_type = KATAKANA_C then
              begin
                if selected then
                  begin
                    v[m] := table.working(w, 1);
                    Inc(m);
                  end;

              end
            else
              begin
                if character_type = SURROGATE_PAIR_C then
                  begin
                    high_part_of_kanji := w;
                    unformed := True;
                    continue;
                  end
                else
                  begin
                    if w = H_0A then
                      begin
                        Inc(count);

                        v[m] := H_0D_ANSI;
                        Inc(m);

                        v[m] := H_0A_ANSI;
                        Inc(m);

                        if field_to_be_selected = FIRST_FIELD then
                          begin
                            field_number := 1;
                            selected := True;
                          end
                        else
                          begin
                            field_number := 1;
                            selected := False;
                          end;

                      end
                    else
                      begin
                        if w = H_0D then
                          begin
                            continue;
                          end
                        else
                          begin
                            if w = quote_byte then
                              begin
                                no_splitting := False;
                              end
                            else
                              begin
                                if w = separator_byte then
                                  begin
                                    if remove_separators then
                                      begin
                                        continue; { Commas disappear if placed in quotes. }
                                      end
                                    else
                                      begin
                                        if selected then
                                          begin
                                            v[m] := Chr(w);
                                            Inc(m);
                                          end;

                                      end;

                                  end
                                else
                                  begin
                                    if selected then
                                      begin
                                        v[m] := Chr(w);
                                        Inc(m);
                                      end;

                                  end;

                              end;

                          end;

                      end;

                  end; { if character_type = SURROGATE_PAIR_C then }

              end; { if character_type = KATAKANA_C then, else }

          end; { if character_type = KANJI_C then }

      end
    else
      begin
{ if no_splitting = False then }

        if unformed then
          begin
            low_part_of_kanji := w;

            if selected then
              begin
                v[m] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 1);
                v[ m + 1 ] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 2);
                Inc(m, KANJI_SIZE);
              end;

            unformed := False;
            continue;
          end;

{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
        character_type := table.working(w, 0);
        if character_type = KANJI_C then
          begin
            if selected then
              begin
                v[m] := table.working(w, 1);
                v[ m + 1 ] := table.working(w, 2);
                Inc(m, KANJI_SIZE);
              end;

          end
        else
          begin
            if character_type = KATAKANA_C then
              begin
                if selected then
                  begin
                    v[m] := table.working(w, 1);
                    Inc(m);
                  end;

              end
            else
              begin
                if character_type = SURROGATE_PAIR_C then
                  begin
                    high_part_of_kanji := w;
                    unformed := True;
                    continue;
                  end
                else
                  begin
                    if w = H_0A then
                      begin
                        Inc(count);

                        v[m] := H_0D_ANSI;
                        Inc(m);

                        v[m] := H_0A_ANSI;
                        Inc(m);

                        if field_to_be_selected = FIRST_FIELD then
                          begin
                            field_number := 1;
                            selected := True;
                          end
                        else
                          begin
                            field_number := 1;
                            selected := False;
                          end;

                      end
                    else
                      begin
                        if w = H_0D then
                          begin
                            continue;
                          end
                        else
                          begin
                            if w = quote_byte then
                              begin
                                no_splitting := True;
                              end
                            else
                              begin
                                if w = separator_byte then
                                  begin
                                    if selected then
                                      begin
                                        v[m] := Chr(w);
                                        Inc(m);
                                      end;

                                    Inc(field_number);
                                    if field_number = field_to_be_selected then
                                      begin
                                        selected := True;
                                      end
                                    else
                                      begin
                                        selected := False;
                                      end;

                                  end
                                else
                                  begin
                                    if selected then
                                      begin
                                        v[m] := Chr(w);
                                        Inc(m);
                                      end;

                                  end;

                              end;

                          end;

                      end;

                  end; { if character_type = SURROGATE_PAIR_C then }

              end; { if character_type = KATAKANA_C then, else }

          end; { if character_type = KANJI_C then }

      end;

  end;

  V_SIZE := m;
  V_SIZE_1 := V_SIZE - 1;
  Result := v;
end;

function Selecting.reading_to_produce_MS932_CRLF(nn: String): Integer;
var
  b: Byte;
  block_has_been_read: Integer;
  buffer_length_1: Integer;
  c: AnsiChar;
  character_type: AnsiChar;
  field_number: Integer;
  high_part_of_kanji: Integer;
  low_part_of_kanji: Integer;
  m: Integer;
  n: Integer;
  no_splitting: Boolean;
  s: String;
  selected: Boolean;
  testFile: File;
  unformed: Boolean;
  w: Integer;
begin
  buffer_length_1 := buffer_length - 1;
  carriage_return_appears := False;
  count := 0;
  new_name := nn;

  for n := 0 to buffer_length_1 do
  begin
    buffer[n] := #0;
  end;

  try
{ Reading a file }
    AssignFile(testFile, file_name);
    Reset(testFile, SIZE_OF_WIDECHAR);

    m := 0;
    repeat
      y := CLEAR_Y_ARRAY;
      BlockRead(testFile, y, Y_COUNT, block_has_been_read);

      for n:= 0 to Y_COUNT_1 do
      begin
        buffer[m] := y[n];
        Inc(m);
        if m > buffer_length_1 then
          break;

      end;
    until m > buffer_length_1;

    CloseFile(testFile);
  except
    s := 'Reset() error-218: ' + file_name;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    Halt(1);
  end;

{ Finding the first NEW_LINE }
  if field_to_be_selected = FIRST_FIELD then
    begin
      field_number := 1;
      selected := True;
    end
  else
    begin
      field_number := 1;
      selected := False;
    end;

  high_part_of_kanji := H_D840;
  m := 0;
  no_splitting := False;
  unformed := False;
  for n := beginning to buffer_length_1 do
  begin
    beginning := n + 1;
    w := Ord(buffer[n]);

    if no_splitting then
      begin
        if unformed then
          begin
            low_part_of_kanji := w;

            if selected then
              begin
                combo[m] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 1);
                combo[ m + 1 ] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 2);
                Inc(m, KANJI_SIZE);
              end;

            unformed := False;
            continue;
          end;

{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
        character_type := table.working(w, 0);
        if character_type = KANJI_C then
          begin
            if selected then
              begin
                combo[m] := table.working(w, 1);
                combo[ m + 1 ] := table.working(w, 2);
                Inc(m, KANJI_SIZE);
              end;

          end
        else
          begin
            if character_type = KATAKANA_C then
              begin
                if selected then
                  begin
                    c := table.working(w, 1);
                    b := Ord(c);

                    b := BELL_ASCII_KATAKANA[b];
                    if b = BELL then
                      begin
                        continue;
                      end
                    else
                      begin
                        combo[m] := Chr(b);
                        Inc(m);
                      end;

                  end

              end
            else
              begin
                if character_type = SURROGATE_PAIR_C then
                  begin
                    high_part_of_kanji := w;
                    unformed := True;
                    continue;
                  end
                else
                  begin
                    if w = H_0A then
                      begin
                        count := 1;

                        combo[m] := H_0D_ANSI;
                        Inc(m);

                        combo[m] := H_0A_ANSI;
                        Inc(m);

                        break;
                      end
                    else
                      begin
                        if w = H_0D then
                          begin
                            carriage_return_appears := True;
                            NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
                            NEW_LINE := EMPTY_ANSI + H_0D_ANSI;
                            NEW_LINE := NEW_LINE + H_0A_ANSI;
                          end
                        else
                          begin
                            if w = quote_byte then
                              begin
                                no_splitting := False;
                              end
                            else
                              begin
                                if w = separator_byte then
                                  begin
                                    if remove_separators then
                                      begin
                                        continue; { Commas disappear if placed in quotes. }
                                      end
                                    else
                                      begin
                                        if selected then
                                          begin
                                            combo[m] := Chr(w);
                                            Inc(m);
                                          end;

                                      end;

                                  end
                                else
                                  begin
                                    if selected then
                                      begin
                                        combo[m] := Chr(w);
                                        Inc(m);
                                      end;

                                  end;

                              end;

                          end;

                      end;

                  end; { if character_type = SURROGATE_PAIR_C then, else }

              end; { if character_type = KATAKANA_C then, else }

          end; { if c = KANJI_C then, else }

      end
    else
      begin
{ if no_splitting = False then }

        if unformed then
          begin
            low_part_of_kanji := w;

            if selected then
              begin
                combo[m] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 1);
                combo[ m + 1 ] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 2);
                Inc(m, KANJI_SIZE);
              end;

            unformed := False;
            continue;
          end;

{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
        character_type := table.working(w, 0);
        if character_type = KANJI_C then
          begin
            if selected then
              begin
                combo[m] := table.working(w, 1);
                combo[ m + 1 ] := table.working(w, 2);
                Inc(m, KANJI_SIZE);
              end;

          end
        else
          begin
            if character_type = KATAKANA_C then
              begin
                if selected then
                  begin
                    c := table.working(w, 1);
                    b := Ord(c);

                    b := BELL_ASCII_KATAKANA[b];
                    if b = BELL then
                      begin
                        continue;
                      end
                    else
                      begin
                        combo[m] := Chr(b);
                        Inc(m);
                      end;

                  end

              end
            else
              begin
                if character_type = SURROGATE_PAIR_C then
                  begin
                    high_part_of_kanji := w;
                    unformed := True;
                    continue;
                  end
                else
                  begin
                    if w = H_0A then
                      begin
                        count := 1;

                        combo[m] := H_0D_ANSI;
                        Inc(m);

                        combo[m] := H_0A_ANSI;
                        Inc(m);

                        break;
                      end
                    else
                      begin
                        if w = H_0D then
                          begin
                            carriage_return_appears := True;
                            NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
                            NEW_LINE := EMPTY_ANSI + H_0D_ANSI;
                            NEW_LINE := NEW_LINE + H_0A_ANSI;
                          end
                        else
                          begin
                            if w = quote_byte then
                              begin
                                no_splitting := True;
                              end
                            else
                              begin
                                if w = separator_byte then
                                  begin
                                    if selected then
                                      begin
                                        combo[m] := Chr(w);
                                        Inc(m);
                                      end;

                                    Inc(field_number);
                                    if field_number = field_to_be_selected then
                                      begin
                                        selected := True;
                                      end
                                    else
                                      begin
                                        selected := False;
                                      end;

                                  end
                                else
                                  begin
                                    if selected then
                                      begin
                                        combo[m] := Chr(w);
                                        Inc(m);
                                      end;

                                  end;

                              end;

                          end;

                      end;

                  end; { if character_type = SURROGATE_PAIR_C then, else }

              end; { if character_type = KATAKANA_C then, else }

          end; { if c = KANJI_C then, else }

      end;

  end;

{ Counting lines }
  if field_to_be_selected = FIRST_FIELD then
    begin
      field_number := 1;
      selected := True;
    end
  else
    begin
      field_number := 1;
      selected := False;
    end;

  high_part_of_kanji := H_D840;
  no_splitting := False;
  unformed := False;
  for n := beginning to buffer_length_1 do
  begin
    w := Ord(buffer[n]);

    if no_splitting then
      begin
        if unformed then
          begin
            low_part_of_kanji := w;

            if selected then
              begin
                combo[m] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 1);
                combo[ m + 1 ] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 2);
                Inc(m, KANJI_SIZE);
              end;

            unformed := False;
            continue;
          end;

{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
        character_type := table.working(w, 0);
        if character_type = KANJI_C then
          begin
            if selected then
              begin
                combo[m] := table.working(w, 1);
                combo[ m + 1 ] := table.working(w, 2);
                Inc(m, KANJI_SIZE);
              end;

          end
        else
          begin
            if character_type = KATAKANA_C then
              begin
                if selected then
                  begin
                    c := table.working(w, 1);
                    b := Ord(c);

                    b := BELL_ASCII_KATAKANA[b];
                    if b = BELL then
                      begin
                        continue;
                      end
                    else
                      begin
                        combo[m] := Chr(b);
                        Inc(m);
                      end;

                  end;

              end
            else
              begin
                if character_type = SURROGATE_PAIR_C then
                  begin
                    high_part_of_kanji := w;
                    unformed := True;
                    continue;
                  end
                else
                  begin
                    if w = H_0A then
                      begin
                        Inc(count);

                        combo[m] := H_0D_ANSI;
                        Inc(m);

                        combo[m] := H_0A_ANSI;
                        Inc(m);

                        if field_to_be_selected = FIRST_FIELD then
                          begin
                            field_number := 1;
                            selected := True;
                          end
                        else
                          begin
                            field_number := 1;
                            selected := False;
                          end;

                      end
                    else
                      begin
                        if w = H_0D then
                          begin
                            continue;
                          end
                        else
                          begin
                            if w = quote_byte then
                              begin
                                no_splitting := False;
                              end
                            else
                              begin
                                if w = separator_byte then
                                  begin
                                    if remove_separators then
                                      begin
                                        continue; { Commas disappear if placed in quotes. }
                                      end
                                    else
                                      begin
                                        if selected then
                                          begin
                                            combo[m] := Chr(w);
                                            Inc(m);
                                          end;

                                      end;

                                  end
                                else
                                  begin
                                    if selected then
                                      begin
                                        combo[m] := Chr(w);
                                        Inc(m);
                                      end;

                                  end;

                              end;

                          end;

                      end;

                  end; { if character_type = SURROGATE_PAIR_C then }

              end; { if character_type = KATAKANA_C then, else }

          end; { if character_type = KANJI_C then }

      end
    else
      begin
{ if no_splitting = False then }

        if unformed then
          begin
            low_part_of_kanji := w;

            if selected then
              begin
                combo[m] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 1);
                combo[ m + 1 ] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 2);
                Inc(m, KANJI_SIZE);
              end;

            unformed := False;
            continue;
          end;

{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
        character_type := table.working(w, 0);
        if character_type = KANJI_C then
          begin
            if selected then
              begin
                combo[m] := table.working(w, 1);
                combo[ m + 1 ] := table.working(w, 2);
                Inc(m, KANJI_SIZE);
              end;

          end
        else
          begin
            if character_type = KATAKANA_C then
              begin
                if selected then
                  begin
                    c := table.working(w, 1);
                    b := Ord(c);

                    b := BELL_ASCII_KATAKANA[b];
                    if b = BELL then
                      begin
                        continue;
                      end
                    else
                      begin
                        combo[m] := Chr(b);
                        Inc(m);
                      end;

                  end;

              end
            else
              begin
                if character_type = SURROGATE_PAIR_C then
                  begin
                    high_part_of_kanji := w;
                    unformed := True;
                    continue;
                  end
                else
                  begin
                    if w = H_0A then
                      begin
                        Inc(count);

                        combo[m] := H_0D_ANSI;
                        Inc(m);

                        combo[m] := H_0A_ANSI;
                        Inc(m);

                        if field_to_be_selected = FIRST_FIELD then
                          begin
                            field_number := 1;
                            selected := True;
                          end
                        else
                          begin
                            field_number := 1;
                            selected := False;
                          end;

                      end
                    else
                      begin
                        if w = H_0D then
                          begin
                            continue;
                          end
                        else
                          begin
                            if w = quote_byte then
                              begin
                                no_splitting := True;
                              end
                            else
                              begin
                                if w = separator_byte then
                                  begin
                                    if selected then
                                      begin
                                        combo[m] := Chr(w);
                                        Inc(m);
                                      end;

                                    Inc(field_number);
                                    if field_number = field_to_be_selected then
                                      begin
                                        selected := True;
                                      end
                                    else
                                      begin
                                        selected := False;
                                      end;

                                  end
                                else
                                  begin
                                    if selected then
                                      begin
                                        combo[m] := Chr(w);
                                        Inc(m);
                                      end;

                                  end;

                              end;

                          end;

                      end;

                  end; { if character_type = SURROGATE_PAIR_C then }

              end; { if character_type = KATAKANA_C then, else }

          end; { if character_type = KANJI_C then }

      end;

  end;

  Result := bufferingWriting(m);
end;

function Selecting.reading_to_produce_MS932_LF(nn: String): Integer;
var
  b: Byte;
  block_has_been_read: Integer;
  buffer_length_1: Integer;
  c: AnsiChar;
  character_type: AnsiChar;
  field_number: Integer;
  high_part_of_kanji: Integer;
  low_part_of_kanji: Integer;
  m: Integer;
  n: Integer;
  no_splitting: Boolean;
  s: String;
  selected: Boolean;
  testFile: File;
  unformed: Boolean;
  w: Integer;
begin
  buffer_length_1 := buffer_length - 1;
  carriage_return_appears := False;
  count := 0;
  new_name := nn;

  for n := 0 to buffer_length_1 do
  begin
    buffer[n] := #0;
  end;

  try
{ Reading a file }
    AssignFile(testFile, file_name);
    Reset(testFile, SIZE_OF_WIDECHAR);

    m := 0;
    repeat
      y := CLEAR_Y_ARRAY;
      BlockRead(testFile, y, Y_COUNT, block_has_been_read);

      for n:= 0 to Y_COUNT_1 do
      begin
        buffer[m] := y[n];
        Inc(m);
        if m > buffer_length_1 then
          break;

      end;
    until m > buffer_length_1;

    CloseFile(testFile);
  except
    s := 'Reset() error-219: ' + file_name;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    Halt(1);
  end;

{ Finding the first NEW_LINE }
  if field_to_be_selected = FIRST_FIELD then
    begin
      field_number := 1;
      selected := True;
    end
  else
    begin
      field_number := 1;
      selected := False;
    end;

  high_part_of_kanji := H_D840;
  m := 0;
  no_splitting := False;
  unformed := False;
  for n := beginning to buffer_length_1 do
  begin
    beginning := n + 1;
    w := Ord(buffer[n]);

    if no_splitting then
      begin
        if unformed then
          begin
            low_part_of_kanji := w;

            if selected then
              begin
                combo[m] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 1);
                combo[ m + 1 ] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 2);
                Inc(m, KANJI_SIZE);
              end;

            unformed := False;
            continue;
          end;

{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
        character_type := table.working(w, 0);
        if character_type = KANJI_C then
          begin
            if selected then
              begin
                combo[m] := table.working(w, 1);
                combo[ m + 1 ] := table.working(w, 2);
                Inc(m, KANJI_SIZE);
              end;

          end
        else
          begin
            if character_type = KATAKANA_C then
              begin
                if selected then
                  begin
                    c := table.working(w, 1);
                    b := Ord(c);

                    b := BELL_ASCII_KATAKANA[b];
                    if b = BELL then
                      begin
                        continue;
                      end
                    else
                      begin
                        combo[m] := Chr(b);
                        Inc(m);
                      end;

                  end

              end
            else
              begin
                if character_type = SURROGATE_PAIR_C then
                  begin
                    high_part_of_kanji := w;
                    unformed := True;
                    continue;
                  end
                else
                  begin
                    if w = H_0A then
                      begin
                        count := 1;

                        combo[m] := H_0A_ANSI;
                        Inc(m);

                        break;
                      end
                    else
                      begin
                        if w = H_0D then
                          begin
                            carriage_return_appears := True;
                            NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
                            NEW_LINE := EMPTY_ANSI + H_0D_ANSI;
                            NEW_LINE := NEW_LINE + H_0A_ANSI;
                          end
                        else
                          begin
                            if w = quote_byte then
                              begin
                                no_splitting := False;
                              end
                            else
                              begin
                                if w = separator_byte then
                                  begin
                                    if remove_separators then
                                      begin
                                        continue; { Commas disappear if placed in quotes. }
                                      end
                                    else
                                      begin
                                        if selected then
                                          begin
                                            combo[m] := Chr(w);
                                            Inc(m);
                                          end;

                                      end;

                                  end
                                else
                                  begin
                                    if selected then
                                      begin
                                        combo[m] := Chr(w);
                                        Inc(m);
                                      end;

                                  end;

                              end;

                          end;

                      end;

                  end; { if character_type = SURROGATE_PAIR_C then, else }

              end; { if character_type = KATAKANA_C then, else }

          end; { if c = KANJI_C then, else }

      end
    else
      begin
{ if no_splitting = False then }

        if unformed then
          begin
            low_part_of_kanji := w;

            if selected then
              begin
                combo[m] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 1);
                combo[ m + 1 ] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 2);
                Inc(m, KANJI_SIZE);
              end;

            unformed := False;
            continue;
          end;

{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
        character_type := table.working(w, 0);
        if character_type = KANJI_C then
          begin
            if selected then
              begin
                combo[m] := table.working(w, 1);
                combo[ m + 1 ] := table.working(w, 2);
                Inc(m, KANJI_SIZE);
              end;

          end
        else
          begin
            if character_type = KATAKANA_C then
              begin
                if selected then
                  begin
                    c := table.working(w, 1);
                    b := Ord(c);

                    b := BELL_ASCII_KATAKANA[b];
                    if b = BELL then
                      begin
                        continue;
                      end
                    else
                      begin
                        combo[m] := Chr(b);
                        Inc(m);
                      end;

                  end

              end
            else
              begin
                if character_type = SURROGATE_PAIR_C then
                  begin
                    high_part_of_kanji := w;
                    unformed := True;
                    continue;
                  end
                else
                  begin
                    if w = H_0A then
                      begin
                        count := 1;

                        combo[m] := H_0A_ANSI;
                        Inc(m);

                        break;
                      end
                    else
                      begin
                        if w = H_0D then
                          begin
                            carriage_return_appears := True;
                            NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
                            NEW_LINE := EMPTY_ANSI + H_0D_ANSI;
                            NEW_LINE := NEW_LINE + H_0A_ANSI;
                          end
                        else
                          begin
                            if w = quote_byte then
                              begin
                                no_splitting := True;
                              end
                            else
                              begin
                                if w = separator_byte then
                                  begin
                                    if selected then
                                      begin
                                        combo[m] := Chr(w);
                                        Inc(m);
                                      end;

                                    Inc(field_number);
                                    if field_number = field_to_be_selected then
                                      begin
                                        selected := True;
                                      end
                                    else
                                      begin
                                        selected := False;
                                      end;

                                  end
                                else
                                  begin
                                    if selected then
                                      begin
                                        combo[m] := Chr(w);
                                        Inc(m);
                                      end;

                                  end;

                              end;

                          end;

                      end;

                  end; { if character_type = SURROGATE_PAIR_C then, else }

              end; { if character_type = KATAKANA_C then, else }

          end; { if c = KANJI_C then, else }

      end;

  end;

{ Counting lines }
  if field_to_be_selected = FIRST_FIELD then
    begin
      field_number := 1;
      selected := True;
    end
  else
    begin
      field_number := 1;
      selected := False;
    end;

  high_part_of_kanji := H_D840;
  no_splitting := False;
  unformed := False;
  for n := beginning to buffer_length_1 do
  begin
    w := Ord(buffer[n]);

    if no_splitting then
      begin
        if unformed then
          begin
            low_part_of_kanji := w;

            if selected then
              begin
                combo[m] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 1);
                combo[ m + 1 ] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 2);
                Inc(m, KANJI_SIZE);
              end;

            unformed := False;
            continue;
          end;

{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
        character_type := table.working(w, 0);
        if character_type = KANJI_C then
          begin
            if selected then
              begin
                combo[m] := table.working(w, 1);
                combo[ m + 1 ] := table.working(w, 2);
                Inc(m, KANJI_SIZE);
              end;

          end
        else
          begin
            if character_type = KATAKANA_C then
              begin
                if selected then
                  begin
                    c := table.working(w, 1);
                    b := Ord(c);

                    b := BELL_ASCII_KATAKANA[b];
                    if b = BELL then
                      begin
                        continue;
                      end
                    else
                      begin
                        combo[m] := Chr(b);
                        Inc(m);
                      end;

                  end;

              end
            else
              begin
                if character_type = SURROGATE_PAIR_C then
                  begin
                    high_part_of_kanji := w;
                    unformed := True;
                    continue;
                  end
                else
                  begin
                    if w = H_0A then
                      begin
                        Inc(count);

                        combo[m] := H_0A_ANSI;
                        Inc(m);

                        if field_to_be_selected = FIRST_FIELD then
                          begin
                            field_number := 1;
                            selected := True;
                          end
                        else
                          begin
                            field_number := 1;
                            selected := False;
                          end;

                      end
                    else
                      begin
                        if w = H_0D then
                          begin
                            continue;
                          end
                        else
                          begin
                            if w = quote_byte then
                              begin
                                no_splitting := False;
                              end
                            else
                              begin
                                if w = separator_byte then
                                  begin
                                    if remove_separators then
                                      begin
                                        continue; { Commas disappear if placed in quotes. }
                                      end
                                    else
                                      begin
                                        if selected then
                                          begin
                                            combo[m] := Chr(w);
                                            Inc(m);
                                          end;

                                      end;

                                  end
                                else
                                  begin
                                    if selected then
                                      begin
                                        combo[m] := Chr(w);
                                        Inc(m);
                                      end;

                                  end;

                              end;

                          end;

                      end;

                  end; { if character_type = SURROGATE_PAIR_C then }

              end; { if character_type = KATAKANA_C then, else }

          end; { if character_type = KANJI_C then }

      end
    else
      begin
{ if no_splitting = False then }

        if unformed then
          begin
            low_part_of_kanji := w;

            if selected then
              begin
                combo[m] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 1);
                combo[ m + 1 ] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 2);
                Inc(m, KANJI_SIZE);
              end;

            unformed := False;
            continue;
          end;

{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
        character_type := table.working(w, 0);
        if character_type = KANJI_C then
          begin
            if selected then
              begin
                combo[m] := table.working(w, 1);
                combo[ m + 1 ] := table.working(w, 2);
                Inc(m, KANJI_SIZE);
              end;

          end
        else
          begin
            if character_type = KATAKANA_C then
              begin
                if selected then
                  begin
                    c := table.working(w, 1);
                    b := Ord(c);

                    b := BELL_ASCII_KATAKANA[b];
                    if b = BELL then
                      begin
                        continue;
                      end
                    else
                      begin
                        combo[m] := Chr(b);
                        Inc(m);
                      end;

                  end;

              end
            else
              begin
                if character_type = SURROGATE_PAIR_C then
                  begin
                    high_part_of_kanji := w;
                    unformed := True;
                    continue;
                  end
                else
                  begin
                    if w = H_0A then
                      begin
                        Inc(count);

                        combo[m] := H_0A_ANSI;
                        Inc(m);

                        if field_to_be_selected = FIRST_FIELD then
                          begin
                            field_number := 1;
                            selected := True;
                          end
                        else
                          begin
                            field_number := 1;
                            selected := False;
                          end;

                      end
                    else
                      begin
                        if w = H_0D then
                          begin
                            continue;
                          end
                        else
                          begin
                            if w = quote_byte then
                              begin
                                no_splitting := True;
                              end
                            else
                              begin
                                if w = separator_byte then
                                  begin
                                    if selected then
                                      begin
                                        combo[m] := Chr(w);
                                        Inc(m);
                                      end;

                                    Inc(field_number);
                                    if field_number = field_to_be_selected then
                                      begin
                                        selected := True;
                                      end
                                    else
                                      begin
                                        selected := False;
                                      end;

                                  end
                                else
                                  begin
                                    if selected then
                                      begin
                                        combo[m] := Chr(w);
                                        Inc(m);
                                      end;

                                  end;

                              end;

                          end;

                      end;

                  end; { if character_type = SURROGATE_PAIR_C then }

              end; { if character_type = KATAKANA_C then, else }

          end; { if character_type = KANJI_C then }

      end;

  end;

  Result := bufferingWriting(m);
end;

function Selecting.reading_to_produce_MS932_text(nn: String): Integer;
var
  b: Byte;
  block_has_been_read: Integer;
  buffer_length_1: Integer;
  c: AnsiChar;
  character_type: AnsiChar;
  field_number: Integer;
  high_part_of_kanji: Integer;
  low_part_of_kanji: Integer;
  m: Integer;
  n: Integer;
  no_splitting: Boolean;
  s: String;
  selected: Boolean;
  testFile: File;
  unformed: Boolean;
  w: Integer;
begin
  buffer_length_1 := buffer_length - 1;
  carriage_return_appears := False;
  count := 0;
  new_name := nn;

  for n := 0 to buffer_length_1 do
  begin
    buffer[n] := #0;
  end;

  try
{ Reading a file }
    AssignFile(testFile, file_name);
    Reset(testFile, SIZE_OF_WIDECHAR);

    m := 0;
    repeat
      y := CLEAR_Y_ARRAY;
      BlockRead(testFile, y, Y_COUNT, block_has_been_read);

      for n:= 0 to Y_COUNT_1 do
      begin
        buffer[m] := y[n];
        Inc(m);
        if m > buffer_length_1 then
          break;

      end;
    until m > buffer_length_1;

    CloseFile(testFile);
  except
    s := 'Reset() error-220: ' + file_name;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    Halt(1);
  end;

{ Finding the first NEW_LINE }
  if field_to_be_selected = FIRST_FIELD then
    begin
      field_number := 1;
      selected := True;
    end
  else
    begin
      field_number := 1;
      selected := False;
    end;

  high_part_of_kanji := H_D840;
  m := 0;
  no_splitting := False;
  unformed := False;
  for n := beginning to buffer_length_1 do
  begin
    beginning := n + 1;
    w := Ord(buffer[n]);

    if no_splitting then
      begin
        if unformed then
          begin
            low_part_of_kanji := w;

            if selected then
              begin
                combo[m] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 1);
                combo[ m + 1 ] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 2);
                Inc(m, KANJI_SIZE);
              end;

            unformed := False;
            continue;
          end;

{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
        character_type := table.working(w, 0);
        if character_type = KANJI_C then
          begin
            if selected then
              begin
                combo[m] := table.working(w, 1);
                combo[ m + 1 ] := table.working(w, 2);
                Inc(m, KANJI_SIZE);
              end;

          end
        else
          begin
            if character_type = KATAKANA_C then
              begin
                if selected then
                  begin
                    c := table.working(w, 1);
                    b := Ord(c);

                    b := BELL_ASCII_KATAKANA[b];
                    if b = BELL then
                      begin
                        continue;
                      end
                    else
                      begin
                        combo[m] := Chr(b);
                        Inc(m);
                      end;

                  end

              end
            else
              begin
                if character_type = SURROGATE_PAIR_C then
                  begin
                    high_part_of_kanji := w;
                    unformed := True;
                    continue;
                  end
                else
                  begin
                    if w = H_0A then
                      begin
                        count := 1;

                        combo[m] := H_0A_ANSI;
                        Inc(m);

                        break;
                      end
                    else
                      begin
                        if w = H_0D then
                          begin
                            combo[m] := H_0D_ANSI;
                            Inc(m);

                            carriage_return_appears := True;
                            NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
                            NEW_LINE := EMPTY_ANSI + H_0D_ANSI;
                            NEW_LINE := NEW_LINE + H_0A_ANSI;
                          end
                        else
                          begin
                            if w = quote_byte then
                              begin
                                no_splitting := False;
                              end
                            else
                              begin
                                if w = separator_byte then
                                  begin
                                    if remove_separators then
                                      begin
                                        continue; { Commas disappear if placed in quotes. }
                                      end
                                    else
                                      begin
                                        if selected then
                                          begin
                                            combo[m] := Chr(w);
                                            Inc(m);
                                          end;

                                      end;

                                  end
                                else
                                  begin
                                    if selected then
                                      begin
                                        combo[m] := Chr(w);
                                        Inc(m);
                                      end;

                                  end;

                              end;

                          end;

                      end;

                  end; { if character_type = SURROGATE_PAIR_C then, else }

              end; { if character_type = KATAKANA_C then, else }

          end; { if c = KANJI_C then, else }

      end
    else
      begin
{ if no_splitting = False then }

        if unformed then
          begin
            low_part_of_kanji := w;

            if selected then
              begin
                combo[m] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 1);
                combo[ m + 1 ] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 2);
                Inc(m, KANJI_SIZE);
              end;

            unformed := False;
            continue;
          end;

{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
        character_type := table.working(w, 0);
        if character_type = KANJI_C then
          begin
            if selected then
              begin
                combo[m] := table.working(w, 1);
                combo[ m + 1 ] := table.working(w, 2);
                Inc(m, KANJI_SIZE);
              end;

          end
        else
          begin
            if character_type = KATAKANA_C then
              begin
                if selected then
                  begin
                    c := table.working(w, 1);
                    b := Ord(c);

                    b := BELL_ASCII_KATAKANA[b];
                    if b = BELL then
                      begin
                        continue;
                      end
                    else
                      begin
                        combo[m] := Chr(b);
                        Inc(m);
                      end;

                  end

              end
            else
              begin
                if character_type = SURROGATE_PAIR_C then
                  begin
                    high_part_of_kanji := w;
                    unformed := True;
                    continue;
                  end
                else
                  begin
                    if w = H_0A then
                      begin
                        count := 1;

                        combo[m] := H_0A_ANSI;
                        Inc(m);

                        break;
                      end
                    else
                      begin
                        if w = H_0D then
                          begin
                            combo[m] := H_0D_ANSI;
                            Inc(m);

                            carriage_return_appears := True;
                            NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
                            NEW_LINE := EMPTY_ANSI + H_0D_ANSI;
                            NEW_LINE := NEW_LINE + H_0A_ANSI;
                          end
                        else
                          begin
                            if w = quote_byte then
                              begin
                                no_splitting := True;
                              end
                            else
                              begin
                                if w = separator_byte then
                                  begin
                                    if selected then
                                      begin
                                        combo[m] := Chr(w);
                                        Inc(m);
                                      end;

                                    Inc(field_number);
                                    if field_number = field_to_be_selected then
                                      begin
                                        selected := True;
                                      end
                                    else
                                      begin
                                        selected := False;
                                      end;

                                  end
                                else
                                  begin
                                    if selected then
                                      begin
                                        combo[m] := Chr(w);
                                        Inc(m);
                                      end;

                                  end;

                              end;

                          end;

                      end;

                  end; { if character_type = SURROGATE_PAIR_C then, else }

              end; { if character_type = KATAKANA_C then, else }

          end; { if c = KANJI_C then, else }

      end;

  end;

{ Counting lines }
  if field_to_be_selected = FIRST_FIELD then
    begin
      field_number := 1;
      selected := True;
    end
  else
    begin
      field_number := 1;
      selected := False;
    end;

  high_part_of_kanji := H_D840;
  no_splitting := False;
  unformed := False;
  for n := beginning to buffer_length_1 do
  begin
    w := Ord(buffer[n]);

    if no_splitting then
      begin
        if unformed then
          begin
            low_part_of_kanji := w;

            if selected then
              begin
                combo[m] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 1);
                combo[ m + 1 ] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 2);
                Inc(m, KANJI_SIZE);
              end;

            unformed := False;
            continue;
          end;

{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
        character_type := table.working(w, 0);
        if character_type = KANJI_C then
          begin
            if selected then
              begin
                combo[m] := table.working(w, 1);
                combo[ m + 1 ] := table.working(w, 2);
                Inc(m, KANJI_SIZE);
              end;

          end
        else
          begin
            if character_type = KATAKANA_C then
              begin
                if selected then
                  begin
                    c := table.working(w, 1);
                    b := Ord(c);

                    b := BELL_ASCII_KATAKANA[b];
                    if b = BELL then
                      begin
                        continue;
                      end
                    else
                      begin
                        combo[m] := Chr(b);
                        Inc(m);
                      end;

                  end;

              end
            else
              begin
                if character_type = SURROGATE_PAIR_C then
                  begin
                    high_part_of_kanji := w;
                    unformed := True;
                    continue;
                  end
                else
                  begin
                    if w = H_0A then
                      begin
                        Inc(count);

                        combo[m] := H_0A_ANSI;
                        Inc(m);

                        if field_to_be_selected = FIRST_FIELD then
                          begin
                            field_number := 1;
                            selected := True;
                          end
                        else
                          begin
                            field_number := 1;
                            selected := False;
                          end;

                      end
                    else
                      begin
                        if w = H_0D then
                          begin
                            combo[m] := H_0D_ANSI;
                            Inc(m);
                            continue;
                          end
                        else
                          begin
                            if w = quote_byte then
                              begin
                                no_splitting := False;
                              end
                            else
                              begin
                                if w = separator_byte then
                                  begin
                                    if remove_separators then
                                      begin
                                        continue; { Commas disappear if placed in quotes. }
                                      end
                                    else
                                      begin
                                        if selected then
                                          begin
                                            combo[m] := Chr(w);
                                            Inc(m);
                                          end;

                                      end;

                                  end
                                else
                                  begin
                                    if selected then
                                      begin
                                        combo[m] := Chr(w);
                                        Inc(m);
                                      end;

                                  end;

                              end;

                          end;

                      end;

                  end; { if character_type = SURROGATE_PAIR_C then }

              end; { if character_type = KATAKANA_C then, else }

          end; { if character_type = KANJI_C then }

      end
    else
      begin
{ if no_splitting = False then }

        if unformed then
          begin
            low_part_of_kanji := w;

            if selected then
              begin
                combo[m] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 1);
                combo[ m + 1 ] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 2);
                Inc(m, KANJI_SIZE);
              end;

            unformed := False;
            continue;
          end;

{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
        character_type := table.working(w, 0);
        if character_type = KANJI_C then
          begin
            if selected then
              begin
                combo[m] := table.working(w, 1);
                combo[ m + 1 ] := table.working(w, 2);
                Inc(m, KANJI_SIZE);
              end;

          end
        else
          begin
            if character_type = KATAKANA_C then
              begin
                if selected then
                  begin
                    c := table.working(w, 1);
                    b := Ord(c);

                    b := BELL_ASCII_KATAKANA[b];
                    if b = BELL then
                      begin
                        continue;
                      end
                    else
                      begin
                        combo[m] := Chr(b);
                        Inc(m);
                      end;

                  end;

              end
            else
              begin
                if character_type = SURROGATE_PAIR_C then
                  begin
                    high_part_of_kanji := w;
                    unformed := True;
                    continue;
                  end
                else
                  begin
                    if w = H_0A then
                      begin
                        Inc(count);

                        combo[m] := H_0A_ANSI;
                        Inc(m);

                        if field_to_be_selected = FIRST_FIELD then
                          begin
                            field_number := 1;
                            selected := True;
                          end
                        else
                          begin
                            field_number := 1;
                            selected := False;
                          end;

                      end
                    else
                      begin
                        if w = H_0D then
                          begin
                            combo[m] := H_0D_ANSI;
                            Inc(m);
                            continue;
                          end
                        else
                          begin
                            if w = quote_byte then
                              begin
                                no_splitting := True;
                              end
                            else
                              begin
                                if w = separator_byte then
                                  begin
                                    if selected then
                                      begin
                                        combo[m] := Chr(w);
                                        Inc(m);
                                      end;

                                    Inc(field_number);
                                    if field_number = field_to_be_selected then
                                      begin
                                        selected := True;
                                      end
                                    else
                                      begin
                                        selected := False;
                                      end;

                                  end
                                else
                                  begin
                                    if selected then
                                      begin
                                        combo[m] := Chr(w);
                                        Inc(m);
                                      end;

                                  end;

                              end;

                          end;

                      end;

                  end; { if character_type = SURROGATE_PAIR_C then }

              end; { if character_type = KATAKANA_C then, else }

          end; { if character_type = KANJI_C then }

      end;

  end;

  Result := bufferingWriting(m);
end;

function Selecting.renameBinaryFile(name: String; no_removing: Boolean): Integer;
var
  block_has_been_read: Integer;
  block_has_been_written: Integer;
  newFile: File;
  s: String;
  sum: Integer;
  testFile: File;
begin
  sum := 0;

  try
    AssignFile(testFile, TEMPORARY_BINARY);
    Reset(testFile, SIZE_OF_ANSICHAR);

    AssignFile(newFile, name);
    Rewrite(newFile, SIZE_OF_ANSICHAR);

    repeat
      v := CLEAR_ARRAY;
      BlockRead(testFile, v, V_SIZE, block_has_been_read);
      BlockWrite(newFile, v, block_has_been_read, block_has_been_written);
      sum := sum + block_has_been_written;
    until (block_has_been_read = 0) or (block_has_been_read <> block_has_been_written);

    CloseFile(newFile);
    CloseFile(testFile);

    if not no_removing then
      begin
        AssignFile(testFile, TEMPORARY_BINARY);
        Erase(testFile);
      end;

  except
    s := 'Reset() error-221: ' + name;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    Halt(1);
  end;

  Result := sum;
end;

function Selecting.setBufferSize: Integer;
var
  r: Integer;
  s: String;
begin
  r := buffer_length;

  try
    SetLength(buffer, buffer_length);
  except
    s := 'SetLength() error-222: ' + IntToStr(buffer_length);
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    r := BUFFER_ERROR;
  end;

  try
    SetLength(combo, combo_size);
  except
    s := 'SetLength() error-223: ' + IntToStr(combo_size);
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    r := COMBO_ERROR;
  end;

  Result := r;
end;

function Selecting.skipping_control_to_produce_MS932_CRLF(nn: String): Integer;
var
  b: Byte;
  block_has_been_read: Integer;
  buffer_length_1: Integer;
  c: AnsiChar;
  character_type: AnsiChar;
  field_number: Integer;
  high_part_of_kanji: Integer;
  low_part_of_kanji: Integer;
  m: Integer;
  n: Integer;
  no_splitting: Boolean;
  s: String;
  selected: Boolean;
  testFile: File;
  unformed: Boolean;
  w: Integer;
begin
  buffer_length_1 := buffer_length - 1;
  carriage_return_appears := False;
  count := 0;
  new_name := nn;

  for n := 0 to buffer_length_1 do
  begin
    buffer[n] := #0;
  end;

  try
{ Reading a file }
    AssignFile(testFile, file_name);
    Reset(testFile, SIZE_OF_WIDECHAR);

    m := 0;
    repeat
      y := CLEAR_Y_ARRAY;
      BlockRead(testFile, y, Y_COUNT, block_has_been_read);

      for n:= 0 to Y_COUNT_1 do
      begin
        buffer[m] := y[n];
        Inc(m);
        if m > buffer_length_1 then
          break;

      end;
    until m > buffer_length_1;

    CloseFile(testFile);
  except
    s := 'Reset() error-224: ' + file_name;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    Halt(1);
  end;

{ Finding the first NEW_LINE }
  if field_to_be_selected = FIRST_FIELD then
    begin
      field_number := 1;
      selected := True;
    end
  else
    begin
      field_number := 1;
      selected := False;
    end;

  high_part_of_kanji := H_D840;
  m := 0;
  no_splitting := False;
  unformed := False;
  for n := beginning to buffer_length_1 do
  begin
    beginning := n + 1;
    w := Ord(buffer[n]);

    if no_splitting then
      begin
        if unformed then
          begin
            low_part_of_kanji := w;

            if selected then
              begin
                combo[m] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 1);
                combo[ m + 1 ] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 2);
                Inc(m, KANJI_SIZE);
              end;

            unformed := False;
            continue;
          end;

{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
        character_type := table.working(w, 0);
        if character_type = KANJI_C then
          begin
            if selected then
              begin
                combo[m] := table.working(w, 1);
                combo[ m + 1 ] := table.working(w, 2);
                Inc(m, KANJI_SIZE);
              end;

          end
        else
          begin
            if character_type = KATAKANA_C then
              begin
                if selected then
                  begin
                    c := table.working(w, 1);
                    b := Ord(c);

                    b := BELL_ASCII_KATAKANA[b];
                    if b = BELL then
                      begin
                        continue;
                      end
                    else
                      begin
                        combo[m] := Chr(b);
                        Inc(m);
                      end;

                  end

              end
            else
              begin
                if character_type = SURROGATE_PAIR_C then
                  begin
                    high_part_of_kanji := w;
                    unformed := True;
                    continue;
                  end
                else
                  begin
                    if w = H_0A then
                      begin
                        count := 1;

                        combo[m] := H_0D_ANSI;
                        Inc(m);

                        combo[m] := H_0A_ANSI;
                        Inc(m);

                        break;
                      end
                    else
                      begin
                        if w = H_0D then
                          begin
                            carriage_return_appears := True;
                            NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
                            NEW_LINE := EMPTY_ANSI + H_0D_ANSI;
                            NEW_LINE := NEW_LINE + H_0A_ANSI;
                          end
                        else
                          begin
                            if w = quote_byte then
                              begin
                                no_splitting := False;
                              end
                            else
                              begin
                                if w = separator_byte then
                                  begin
                                    if remove_separators then
                                      begin
                                        continue; { Commas disappear if placed in quotes. }
                                      end
                                    else
                                      begin
                                        if selected then
                                          begin
                                            b := BELL_ASCII_KATAKANA[w];
                                            if b = BELL then
                                              begin
                                                continue; { Removing control characters }
                                              end
                                            else
                                              begin
                                                combo[m] := Chr(w);
                                                Inc(m);
                                              end;

                                          end;

                                      end;

                                  end
                                else
                                  begin
                                    if selected then
                                      begin
                                        b := BELL_ASCII_KATAKANA[w];
                                        if b = BELL then
                                          begin
                                            continue; { Removing control characters }
                                          end
                                        else
                                          begin
                                            combo[m] := Chr(w);
                                            Inc(m);
                                          end;

                                      end;

                                  end;

                              end;

                          end;

                      end;

                  end; { if character_type = SURROGATE_PAIR_C then, else }

              end; { if character_type = KATAKANA_C then, else }

          end; { if c = KANJI_C then, else }

      end
    else
      begin
{ if no_splitting = False then }

        if unformed then
          begin
            low_part_of_kanji := w;

            if selected then
              begin
                combo[m] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 1);
                combo[ m + 1 ] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 2);
                Inc(m, KANJI_SIZE);
              end;

            unformed := False;
            continue;
          end;

{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
        character_type := table.working(w, 0);
        if character_type = KANJI_C then
          begin
            if selected then
              begin
                combo[m] := table.working(w, 1);
                combo[ m + 1 ] := table.working(w, 2);
                Inc(m, KANJI_SIZE);
              end;

          end
        else
          begin
            if character_type = KATAKANA_C then
              begin
                if selected then
                  begin
                    c := table.working(w, 1);
                    b := Ord(c);

                    b := BELL_ASCII_KATAKANA[b];
                    if b = BELL then
                      begin
                        continue;
                      end
                    else
                      begin
                        combo[m] := Chr(b);
                        Inc(m);
                      end;

                  end

              end
            else
              begin
                if character_type = SURROGATE_PAIR_C then
                  begin
                    high_part_of_kanji := w;
                    unformed := True;
                    continue;
                  end
                else
                  begin
                    if w = H_0A then
                      begin
                        count := 1;

                        combo[m] := H_0D_ANSI;
                        Inc(m);

                        combo[m] := H_0A_ANSI;
                        Inc(m);

                        break;
                      end
                    else
                      begin
                        if w = H_0D then
                          begin
                            carriage_return_appears := True;
                            NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
                            NEW_LINE := EMPTY_ANSI + H_0D_ANSI;
                            NEW_LINE := NEW_LINE + H_0A_ANSI;
                          end
                        else
                          begin
                            if w = quote_byte then
                              begin
                                no_splitting := True;
                              end
                            else
                              begin
                                if w = separator_byte then
                                  begin
                                    if selected then
                                      begin
                                        b := BELL_ASCII_KATAKANA[w];
                                        if b = BELL then
                                          begin
                                            continue; { Removing control characters }
                                          end
                                        else
                                          begin
                                            combo[m] := Chr(w);
                                            Inc(m);
                                          end;

                                      end;

                                    Inc(field_number);
                                    if field_number = field_to_be_selected then
                                      begin
                                        selected := True;
                                      end
                                    else
                                      begin
                                        selected := False;
                                      end;

                                  end
                                else
                                  begin
                                    if selected then
                                      begin
                                         b := BELL_ASCII_KATAKANA[w];
                                         if b = BELL then
                                           begin
                                             continue; { Removing control characters }
                                           end
                                         else
                                           begin
                                             combo[m] := Chr(w);
                                             Inc(m);
                                           end;

                                      end;

                                  end;

                              end;

                          end;

                      end;

                  end; { if character_type = SURROGATE_PAIR_C then, else }

              end; { if character_type = KATAKANA_C then, else }

          end; { if c = KANJI_C then, else }

      end;

  end;

{ Counting lines }
  if field_to_be_selected = FIRST_FIELD then
    begin
      field_number := 1;
      selected := True;
    end
  else
    begin
      field_number := 1;
      selected := False;
    end;

  high_part_of_kanji := H_D840;
  no_splitting := False;
  unformed := False;
  for n := beginning to buffer_length_1 do
  begin
    w := Ord(buffer[n]);

    if no_splitting then
      begin
        if unformed then
          begin
            low_part_of_kanji := w;

            if selected then
              begin
                combo[m] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 1);
                combo[ m + 1 ] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 2);
                Inc(m, KANJI_SIZE);
              end;

            unformed := False;
            continue;
          end;

{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
        character_type := table.working(w, 0);
        if character_type = KANJI_C then
          begin
            if selected then
              begin
                combo[m] := table.working(w, 1);
                combo[ m + 1 ] := table.working(w, 2);
                Inc(m, KANJI_SIZE);
              end;

          end
        else
          begin
            if character_type = KATAKANA_C then
              begin
                if selected then
                  begin
                    c := table.working(w, 1);
                    b := Ord(c);

                    b := BELL_ASCII_KATAKANA[b];
                    if b = BELL then
                      begin
                        continue;
                      end
                    else
                      begin
                        combo[m] := Chr(b);
                        Inc(m);
                      end;

                  end;

              end
            else
              begin
                if character_type = SURROGATE_PAIR_C then
                  begin
                    high_part_of_kanji := w;
                    unformed := True;
                    continue;
                  end
                else
                  begin
                    if w = H_0A then
                      begin
                        Inc(count);

                        combo[m] := H_0D_ANSI;
                        Inc(m);

                        combo[m] := H_0A_ANSI;
                        Inc(m);

                        if field_to_be_selected = FIRST_FIELD then
                          begin
                            field_number := 1;
                            selected := True;
                          end
                        else
                          begin
                            field_number := 1;
                            selected := False;
                          end;

                      end
                    else
                      begin
                        if w = H_0D then
                          begin
                            continue;
                          end
                        else
                          begin
                            if w = quote_byte then
                              begin
                                no_splitting := False;
                              end
                            else
                              begin
                                if w = separator_byte then
                                  begin
                                    if remove_separators then
                                      begin
                                        continue; { Commas disappear if placed in quotes. }
                                      end
                                    else
                                      begin
                                        if selected then
                                          begin
                                            b := BELL_ASCII_KATAKANA[w];
                                            if b = BELL then
                                              begin
                                                continue; { Removing control characters }
                                              end
                                            else
                                              begin
                                                combo[m] := Chr(w);
                                                Inc(m);
                                              end;

                                          end;

                                      end;

                                  end
                                else
                                  begin
                                    if selected then
                                      begin
                                        b := BELL_ASCII_KATAKANA[w];
                                        if b = BELL then
                                          begin
                                            continue; { Removing control characters }
                                          end
                                        else
                                          begin
                                            combo[m] := Chr(w);
                                            Inc(m);
                                          end;

                                      end;

                                  end;

                              end;

                          end;

                      end;

                  end; { if character_type = SURROGATE_PAIR_C then }

              end; { if character_type = KATAKANA_C then, else }

          end; { if character_type = KANJI_C then }

      end
    else
      begin
{ if no_splitting = False then }

        if unformed then
          begin
            low_part_of_kanji := w;

            if selected then
              begin
                combo[m] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 1);
                combo[ m + 1 ] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 2);
                Inc(m, KANJI_SIZE);
              end;

            unformed := False;
            continue;
          end;

{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
        character_type := table.working(w, 0);
        if character_type = KANJI_C then
          begin
            if selected then
              begin
                combo[m] := table.working(w, 1);
                combo[ m + 1 ] := table.working(w, 2);
                Inc(m, KANJI_SIZE);
              end;

          end
        else
          begin
            if character_type = KATAKANA_C then
              begin
                if selected then
                  begin
                    c := table.working(w, 1);
                    b := Ord(c);

                    b := BELL_ASCII_KATAKANA[b];
                    if b = BELL then
                      begin
                        continue;
                      end
                    else
                      begin
                        combo[m] := Chr(b);
                        Inc(m);
                      end;

                  end;

              end
            else
              begin
                if character_type = SURROGATE_PAIR_C then
                  begin
                    high_part_of_kanji := w;
                    unformed := True;
                    continue;
                  end
                else
                  begin
                    if w = H_0A then
                      begin
                        Inc(count);

                        combo[m] := H_0D_ANSI;
                        Inc(m);

                        combo[m] := H_0A_ANSI;
                        Inc(m);

                        if field_to_be_selected = FIRST_FIELD then
                          begin
                            field_number := 1;
                            selected := True;
                          end
                        else
                          begin
                            field_number := 1;
                            selected := False;
                          end;

                      end
                    else
                      begin
                        if w = H_0D then
                          begin
                            continue;
                          end
                        else
                          begin
                            if w = quote_byte then
                              begin
                                no_splitting := True;
                              end
                            else
                              begin
                                if w = separator_byte then
                                  begin
                                    if selected then
                                      begin
                                        b := BELL_ASCII_KATAKANA[w];
                                        if b = BELL then
                                          begin
                                            continue; { Removing control characters }
                                          end
                                        else
                                          begin
                                            combo[m] := Chr(w);
                                            Inc(m);
                                          end;

                                      end;

                                    Inc(field_number);
                                    if field_number = field_to_be_selected then
                                      begin
                                        selected := True;
                                      end
                                    else
                                      begin
                                        selected := False;
                                      end;

                                  end
                                else
                                  begin
                                    if selected then
                                      begin
                                        b := BELL_ASCII_KATAKANA[w];
                                        if b = BELL then
                                          begin
                                            continue; { Removing control characters }
                                          end
                                        else
                                          begin
                                            combo[m] := Chr(w);
                                            Inc(m);
                                          end;

                                      end;

                                  end;

                              end;

                          end;

                      end;

                  end; { if character_type = SURROGATE_PAIR_C then }

              end; { if character_type = KATAKANA_C then, else }

          end; { if character_type = KANJI_C then }

      end;

  end;

  Result := bufferingWriting(m);
end;

function Selecting.skipping_control_to_produce_MS932_LF(nn: String): Integer;
var
  b: Byte;
  block_has_been_read: Integer;
  buffer_length_1: Integer;
  c: AnsiChar;
  character_type: AnsiChar;
  field_number: Integer;
  high_part_of_kanji: Integer;
  low_part_of_kanji: Integer;
  m: Integer;
  n: Integer;
  no_splitting: Boolean;
  s: String;
  selected: Boolean;
  testFile: File;
  unformed: Boolean;
  w: Integer;
begin
  buffer_length_1 := buffer_length - 1;
  carriage_return_appears := False;
  count := 0;
  new_name := nn;

  for n := 0 to buffer_length_1 do
  begin
    buffer[n] := #0;
  end;

  try
{ Reading a file }
    AssignFile(testFile, file_name);
    Reset(testFile, SIZE_OF_WIDECHAR);

    m := 0;
    repeat
      y := CLEAR_Y_ARRAY;
      BlockRead(testFile, y, Y_COUNT, block_has_been_read);

      for n:= 0 to Y_COUNT_1 do
      begin
        buffer[m] := y[n];
        Inc(m);
        if m > buffer_length_1 then
          break;

      end;
    until m > buffer_length_1;

    CloseFile(testFile);
  except
    s := 'Reset() error-225: ' + file_name;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    Halt(1);
  end;

{ Finding the first NEW_LINE }
  if field_to_be_selected = FIRST_FIELD then
    begin
      field_number := 1;
      selected := True;
    end
  else
    begin
      field_number := 1;
      selected := False;
    end;

  high_part_of_kanji := H_D840;
  m := 0;
  no_splitting := False;
  unformed := False;
  for n := beginning to buffer_length_1 do
  begin
    beginning := n + 1;
    w := Ord(buffer[n]);

    if no_splitting then
      begin
        if unformed then
          begin
            low_part_of_kanji := w;

            if selected then
              begin
                combo[m] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 1);
                combo[ m + 1 ] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 2);
                Inc(m, KANJI_SIZE);
              end;

            unformed := False;
            continue;
          end;

{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
        character_type := table.working(w, 0);
        if character_type = KANJI_C then
          begin
            if selected then
              begin
                combo[m] := table.working(w, 1);
                combo[ m + 1 ] := table.working(w, 2);
                Inc(m, KANJI_SIZE);
              end;

          end
        else
          begin
            if character_type = KATAKANA_C then
              begin
                if selected then
                  begin
                    c := table.working(w, 1);
                    b := Ord(c);

                    b := BELL_ASCII_KATAKANA[b];
                    if b = BELL then
                      begin
                        continue;
                      end
                    else
                      begin
                        combo[m] := Chr(b);
                        Inc(m);
                      end;

                  end

              end
            else
              begin
                if character_type = SURROGATE_PAIR_C then
                  begin
                    high_part_of_kanji := w;
                    unformed := True;
                    continue;
                  end
                else
                  begin
                    if w = H_0A then
                      begin
                        count := 1;

                        combo[m] := H_0A_ANSI;
                        Inc(m);

                        break;
                      end
                    else
                      begin
                        if w = H_0D then
                          begin
                            carriage_return_appears := True;
                            NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
                            NEW_LINE := EMPTY_ANSI + H_0D_ANSI;
                            NEW_LINE := NEW_LINE + H_0A_ANSI;
                          end
                        else
                          begin
                            if w = quote_byte then
                              begin
                                no_splitting := False;
                              end
                            else
                              begin
                                if w = separator_byte then
                                  begin
                                    if remove_separators then
                                      begin
                                        continue; { Commas disappear if placed in quotes. }
                                      end
                                    else
                                      begin
                                        if selected then
                                          begin
                                            b := BELL_ASCII_KATAKANA[w];
                                            if b = BELL then
                                              begin
                                                continue; { Removing control characters }
                                              end
                                            else
                                              begin
                                                combo[m] := Chr(w);
                                                Inc(m);
                                              end;

                                          end;

                                      end;

                                  end
                                else
                                  begin
                                    if selected then
                                      begin
                                        b := BELL_ASCII_KATAKANA[w];
                                        if b = BELL then
                                          begin
                                            continue; { Removing control characters }
                                          end
                                        else
                                          begin
                                            combo[m] := Chr(w);
                                            Inc(m);
                                          end;

                                      end;

                                  end;

                              end;

                          end;

                      end;

                  end; { if character_type = SURROGATE_PAIR_C then, else }

              end; { if character_type = KATAKANA_C then, else }

          end; { if c = KANJI_C then, else }

      end
    else
      begin
{ if no_splitting = False then }

        if unformed then
          begin
            low_part_of_kanji := w;

            if selected then
              begin
                combo[m] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 1);
                combo[ m + 1 ] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 2);
                Inc(m, KANJI_SIZE);
              end;

            unformed := False;
            continue;
          end;

{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
        character_type := table.working(w, 0);
        if character_type = KANJI_C then
          begin
            if selected then
              begin
                combo[m] := table.working(w, 1);
                combo[ m + 1 ] := table.working(w, 2);
                Inc(m, KANJI_SIZE);
              end;

          end
        else
          begin
            if character_type = KATAKANA_C then
              begin
                if selected then
                  begin
                    c := table.working(w, 1);
                    b := Ord(c);

                    b := BELL_ASCII_KATAKANA[b];
                    if b = BELL then
                      begin
                        continue;
                      end
                    else
                      begin
                        combo[m] := Chr(b);
                        Inc(m);
                      end;

                  end

              end
            else
              begin
                if character_type = SURROGATE_PAIR_C then
                  begin
                    high_part_of_kanji := w;
                    unformed := True;
                    continue;
                  end
                else
                  begin
                    if w = H_0A then
                      begin
                        count := 1;

                        combo[m] := H_0A_ANSI;
                        Inc(m);

                        break;
                      end
                    else
                      begin
                        if w = H_0D then
                          begin
                            carriage_return_appears := True;
                            NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
                            NEW_LINE := EMPTY_ANSI + H_0D_ANSI;
                            NEW_LINE := NEW_LINE + H_0A_ANSI;
                          end
                        else
                          begin
                            if w = quote_byte then
                              begin
                                no_splitting := True;
                              end
                            else
                              begin
                                if w = separator_byte then
                                  begin
                                    if selected then
                                      begin
                                        b := BELL_ASCII_KATAKANA[w];
                                        if b = BELL then
                                          begin
                                            continue; { Removing control characters }
                                          end
                                        else
                                          begin
                                            combo[m] := Chr(w);
                                            Inc(m);
                                          end;

                                      end;

                                    Inc(field_number);
                                    if field_number = field_to_be_selected then
                                      begin
                                        selected := True;
                                      end
                                    else
                                      begin
                                        selected := False;
                                      end;

                                  end
                                else
                                  begin
                                    if selected then
                                      begin
                                         b := BELL_ASCII_KATAKANA[w];
                                         if b = BELL then
                                           begin
                                             continue; { Removing control characters }
                                           end
                                         else
                                           begin
                                             combo[m] := Chr(w);
                                             Inc(m);
                                           end;

                                      end;

                                  end;

                              end;

                          end;

                      end;

                  end; { if character_type = SURROGATE_PAIR_C then, else }

              end; { if character_type = KATAKANA_C then, else }

          end; { if c = KANJI_C then, else }

      end;

  end;

{ Counting lines }
  if field_to_be_selected = FIRST_FIELD then
    begin
      field_number := 1;
      selected := True;
    end
  else
    begin
      field_number := 1;
      selected := False;
    end;

  high_part_of_kanji := H_D840;
  no_splitting := False;
  unformed := False;
  for n := beginning to buffer_length_1 do
  begin
    w := Ord(buffer[n]);

    if no_splitting then
      begin
        if unformed then
          begin
            low_part_of_kanji := w;

            if selected then
              begin
                combo[m] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 1);
                combo[ m + 1 ] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 2);
                Inc(m, KANJI_SIZE);
              end;

            unformed := False;
            continue;
          end;

{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
        character_type := table.working(w, 0);
        if character_type = KANJI_C then
          begin
            if selected then
              begin
                combo[m] := table.working(w, 1);
                combo[ m + 1 ] := table.working(w, 2);
                Inc(m, KANJI_SIZE);
              end;

          end
        else
          begin
            if character_type = KATAKANA_C then
              begin
                if selected then
                  begin
                    c := table.working(w, 1);
                    b := Ord(c);

                    b := BELL_ASCII_KATAKANA[b];
                    if b = BELL then
                      begin
                        continue;
                      end
                    else
                      begin
                        combo[m] := Chr(b);
                        Inc(m);
                      end;

                  end;

              end
            else
              begin
                if character_type = SURROGATE_PAIR_C then
                  begin
                    high_part_of_kanji := w;
                    unformed := True;
                    continue;
                  end
                else
                  begin
                    if w = H_0A then
                      begin
                        Inc(count);

                        combo[m] := H_0A_ANSI;
                        Inc(m);

                        if field_to_be_selected = FIRST_FIELD then
                          begin
                            field_number := 1;
                            selected := True;
                          end
                        else
                          begin
                            field_number := 1;
                            selected := False;
                          end;

                      end
                    else
                      begin
                        if w = H_0D then
                          begin
                            continue;
                          end
                        else
                          begin
                            if w = quote_byte then
                              begin
                                no_splitting := False;
                              end
                            else
                              begin
                                if w = separator_byte then
                                  begin
                                    if remove_separators then
                                      begin
                                        continue; { Commas disappear if placed in quotes. }
                                      end
                                    else
                                      begin
                                        if selected then
                                          begin
                                            b := BELL_ASCII_KATAKANA[w];
                                            if b = BELL then
                                              begin
                                                continue; { Removing control characters }
                                              end
                                            else
                                              begin
                                                combo[m] := Chr(w);
                                                Inc(m);
                                              end;

                                          end;

                                      end;

                                  end
                                else
                                  begin
                                    if selected then
                                      begin
                                        b := BELL_ASCII_KATAKANA[w];
                                        if b = BELL then
                                          begin
                                            continue; { Removing control characters }
                                          end
                                        else
                                          begin
                                            combo[m] := Chr(w);
                                            Inc(m);
                                          end;

                                      end;

                                  end;

                              end;

                          end;

                      end;

                  end; { if character_type = SURROGATE_PAIR_C then }

              end; { if character_type = KATAKANA_C then, else }

          end; { if character_type = KANJI_C then }

      end
    else
      begin
{ if no_splitting = False then }

        if unformed then
          begin
            low_part_of_kanji := w;

            if selected then
              begin
                combo[m] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 1);
                combo[ m + 1 ] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 2);
                Inc(m, KANJI_SIZE);
              end;

            unformed := False;
            continue;
          end;

{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
        character_type := table.working(w, 0);
        if character_type = KANJI_C then
          begin
            if selected then
              begin
                combo[m] := table.working(w, 1);
                combo[ m + 1 ] := table.working(w, 2);
                Inc(m, KANJI_SIZE);
              end;

          end
        else
          begin
            if character_type = KATAKANA_C then
              begin
                if selected then
                  begin
                    c := table.working(w, 1);
                    b := Ord(c);

                    b := BELL_ASCII_KATAKANA[b];
                    if b = BELL then
                      begin
                        continue;
                      end
                    else
                      begin
                        combo[m] := Chr(b);
                        Inc(m);
                      end;

                  end;

              end
            else
              begin
                if character_type = SURROGATE_PAIR_C then
                  begin
                    high_part_of_kanji := w;
                    unformed := True;
                    continue;
                  end
                else
                  begin
                    if w = H_0A then
                      begin
                        Inc(count);

                        combo[m] := H_0A_ANSI;
                        Inc(m);

                        if field_to_be_selected = FIRST_FIELD then
                          begin
                            field_number := 1;
                            selected := True;
                          end
                        else
                          begin
                            field_number := 1;
                            selected := False;
                          end;

                      end
                    else
                      begin
                        if w = H_0D then
                          begin
                            continue;
                          end
                        else
                          begin
                            if w = quote_byte then
                              begin
                                no_splitting := True;
                              end
                            else
                              begin
                                if w = separator_byte then
                                  begin
                                    if selected then
                                      begin
                                        b := BELL_ASCII_KATAKANA[w];
                                        if b = BELL then
                                          begin
                                            continue; { Removing control characters }
                                          end
                                        else
                                          begin
                                            combo[m] := Chr(w);
                                            Inc(m);
                                          end;

                                      end;

                                    Inc(field_number);
                                    if field_number = field_to_be_selected then
                                      begin
                                        selected := True;
                                      end
                                    else
                                      begin
                                        selected := False;
                                      end;

                                  end
                                else
                                  begin
                                    if selected then
                                      begin
                                        b := BELL_ASCII_KATAKANA[w];
                                        if b = BELL then
                                          begin
                                            continue; { Removing control characters }
                                          end
                                        else
                                          begin
                                            combo[m] := Chr(w);
                                            Inc(m);
                                          end;

                                      end;

                                  end;

                              end;

                          end;

                      end;

                  end; { if character_type = SURROGATE_PAIR_C then }

              end; { if character_type = KATAKANA_C then, else }

          end; { if character_type = KANJI_C then }

      end;

  end;

  Result := bufferingWriting(m);
end;

function Selecting.skipping_control_to_produce_MS932_text(nn: String): Integer;
var
  b: Byte;
  block_has_been_read: Integer;
  buffer_length_1: Integer;
  c: AnsiChar;
  character_type: AnsiChar;
  field_number: Integer;
  high_part_of_kanji: Integer;
  low_part_of_kanji: Integer;
  m: Integer;
  n: Integer;
  no_splitting: Boolean;
  s: String;
  selected: Boolean;
  testFile: File;
  unformed: Boolean;
  w: Integer;
begin
  buffer_length_1 := buffer_length - 1;
  carriage_return_appears := False;
  count := 0;
  new_name := nn;

  for n := 0 to buffer_length_1 do
  begin
    buffer[n] := #0;
  end;

  try
{ Reading a file }
    AssignFile(testFile, file_name);
    Reset(testFile, SIZE_OF_WIDECHAR);

    m := 0;
    repeat
      y := CLEAR_Y_ARRAY;
      BlockRead(testFile, y, Y_COUNT, block_has_been_read);

      for n:= 0 to Y_COUNT_1 do
      begin
        buffer[m] := y[n];
        Inc(m);
        if m > buffer_length_1 then
          break;

      end;
    until m > buffer_length_1;

    CloseFile(testFile);
  except
    s := 'Reset() error-226: ' + file_name;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    Halt(1);
  end;

{ Finding the first NEW_LINE }
  if field_to_be_selected = FIRST_FIELD then
    begin
      field_number := 1;
      selected := True;
    end
  else
    begin
      field_number := 1;
      selected := False;
    end;

  high_part_of_kanji := H_D840;
  m := 0;
  no_splitting := False;
  unformed := False;
  for n := beginning to buffer_length_1 do
  begin
    beginning := n + 1;
    w := Ord(buffer[n]);

    if no_splitting then
      begin
        if unformed then
          begin
            low_part_of_kanji := w;

            if selected then
              begin
                combo[m] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 1);
                combo[ m + 1 ] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 2);
                Inc(m, KANJI_SIZE);
              end;

            unformed := False;
            continue;
          end;

{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
        character_type := table.working(w, 0);
        if character_type = KANJI_C then
          begin
            if selected then
              begin
                combo[m] := table.working(w, 1);
                combo[ m + 1 ] := table.working(w, 2);
                Inc(m, KANJI_SIZE);
              end;

          end
        else
          begin
            if character_type = KATAKANA_C then
              begin
                if selected then
                  begin
                    c := table.working(w, 1);
                    b := Ord(c);

                    b := BELL_ASCII_KATAKANA[b];
                    if b = BELL then
                      begin
                        continue;
                      end
                    else
                      begin
                        combo[m] := Chr(b);
                        Inc(m);
                      end;

                  end

              end
            else
              begin
                if character_type = SURROGATE_PAIR_C then
                  begin
                    high_part_of_kanji := w;
                    unformed := True;
                    continue;
                  end
                else
                  begin
                    if w = H_0A then
                      begin
                        count := 1;

                        combo[m] := H_0A_ANSI;
                        Inc(m);

                        break;
                      end
                    else
                      begin
                        if w = H_0D then
                          begin
                            combo[m] := H_0D_ANSI;
                            Inc(m);

                            carriage_return_appears := True;
                            NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
                            NEW_LINE := EMPTY_ANSI + H_0D_ANSI;
                            NEW_LINE := NEW_LINE + H_0A_ANSI;
                          end
                        else
                          begin
                            if w = quote_byte then
                              begin
                                no_splitting := False;
                              end
                            else
                              begin
                                if w = separator_byte then
                                  begin
                                    if remove_separators then
                                      begin
                                        continue; { Commas disappear if placed in quotes. }
                                      end
                                    else
                                      begin
                                        if selected then
                                          begin
                                            b := BELL_ASCII_KATAKANA[w];
                                            if b = BELL then
                                              begin
                                                continue; { Removing control characters }
                                              end
                                            else
                                              begin
                                                combo[m] := Chr(w);
                                                Inc(m);
                                              end;

                                          end;

                                      end;

                                  end
                                else
                                  begin
                                    if selected then
                                      begin
                                        b := BELL_ASCII_KATAKANA[w];
                                        if b = BELL then
                                          begin
                                            continue; { Removing control characters }
                                          end
                                        else
                                          begin
                                            combo[m] := Chr(w);
                                            Inc(m);
                                          end;

                                      end;

                                  end;

                              end;

                          end;

                      end;

                  end; { if character_type = SURROGATE_PAIR_C then, else }

              end; { if character_type = KATAKANA_C then, else }

          end; { if c = KANJI_C then, else }

      end
    else
      begin
{ if no_splitting = False then }

        if unformed then
          begin
            low_part_of_kanji := w;

            if selected then
              begin
                combo[m] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 1);
                combo[ m + 1 ] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 2);
                Inc(m, KANJI_SIZE);
              end;

            unformed := False;
            continue;
          end;

{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
        character_type := table.working(w, 0);
        if character_type = KANJI_C then
          begin
            if selected then
              begin
                combo[m] := table.working(w, 1);
                combo[ m + 1 ] := table.working(w, 2);
                Inc(m, KANJI_SIZE);
              end;

          end
        else
          begin
            if character_type = KATAKANA_C then
              begin
                if selected then
                  begin
                    c := table.working(w, 1);
                    b := Ord(c);

                    b := BELL_ASCII_KATAKANA[b];
                    if b = BELL then
                      begin
                        continue;
                      end
                    else
                      begin
                        combo[m] := Chr(b);
                        Inc(m);
                      end;

                  end

              end
            else
              begin
                if character_type = SURROGATE_PAIR_C then
                  begin
                    high_part_of_kanji := w;
                    unformed := True;
                    continue;
                  end
                else
                  begin
                    if w = H_0A then
                      begin
                        count := 1;

                        combo[m] := H_0A_ANSI;
                        Inc(m);

                        break;
                      end
                    else
                      begin
                        if w = H_0D then
                          begin
                            combo[m] := H_0D_ANSI;
                            Inc(m);

                            carriage_return_appears := True;
                            NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
                            NEW_LINE := EMPTY_ANSI + H_0D_ANSI;
                            NEW_LINE := NEW_LINE + H_0A_ANSI;
                          end
                        else
                          begin
                            if w = quote_byte then
                              begin
                                no_splitting := True;
                              end
                            else
                              begin
                                if w = separator_byte then
                                  begin
                                    if selected then
                                      begin
                                        b := BELL_ASCII_KATAKANA[w];
                                        if b = BELL then
                                          begin
                                            continue; { Removing control characters }
                                          end
                                        else
                                          begin
                                            combo[m] := Chr(w);
                                            Inc(m);
                                          end;

                                      end;

                                    Inc(field_number);
                                    if field_number = field_to_be_selected then
                                      begin
                                        selected := True;
                                      end
                                    else
                                      begin
                                        selected := False;
                                      end;

                                  end
                                else
                                  begin
                                    if selected then
                                      begin
                                         b := BELL_ASCII_KATAKANA[w];
                                         if b = BELL then
                                           begin
                                             continue; { Removing control characters }
                                           end
                                         else
                                           begin
                                             combo[m] := Chr(w);
                                             Inc(m);
                                           end;

                                      end;

                                  end;

                              end;

                          end;

                      end;

                  end; { if character_type = SURROGATE_PAIR_C then, else }

              end; { if character_type = KATAKANA_C then, else }

          end; { if c = KANJI_C then, else }

      end;

  end;

{ Counting lines }
  if field_to_be_selected = FIRST_FIELD then
    begin
      field_number := 1;
      selected := True;
    end
  else
    begin
      field_number := 1;
      selected := False;
    end;

  high_part_of_kanji := H_D840;
  no_splitting := False;
  unformed := False;
  for n := beginning to buffer_length_1 do
  begin
    w := Ord(buffer[n]);

    if no_splitting then
      begin
        if unformed then
          begin
            low_part_of_kanji := w;

            if selected then
              begin
                combo[m] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 1);
                combo[ m + 1 ] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 2);
                Inc(m, KANJI_SIZE);
              end;

            unformed := False;
            continue;
          end;

{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
        character_type := table.working(w, 0);
        if character_type = KANJI_C then
          begin
            if selected then
              begin
                combo[m] := table.working(w, 1);
                combo[ m + 1 ] := table.working(w, 2);
                Inc(m, KANJI_SIZE);
              end;

          end
        else
          begin
            if character_type = KATAKANA_C then
              begin
                if selected then
                  begin
                    c := table.working(w, 1);
                    b := Ord(c);

                    b := BELL_ASCII_KATAKANA[b];
                    if b = BELL then
                      begin
                        continue;
                      end
                    else
                      begin
                        combo[m] := Chr(b);
                        Inc(m);
                      end;

                  end;

              end
            else
              begin
                if character_type = SURROGATE_PAIR_C then
                  begin
                    high_part_of_kanji := w;
                    unformed := True;
                    continue;
                  end
                else
                  begin
                    if w = H_0A then
                      begin
                        Inc(count);

                        combo[m] := H_0A_ANSI;
                        Inc(m);

                        if field_to_be_selected = FIRST_FIELD then
                          begin
                            field_number := 1;
                            selected := True;
                          end
                        else
                          begin
                            field_number := 1;
                            selected := False;
                          end;

                      end
                    else
                      begin
                        if w = H_0D then
                          begin
                            combo[m] := H_0D_ANSI;
                            Inc(m);
                            continue;
                          end
                        else
                          begin
                            if w = quote_byte then
                              begin
                                no_splitting := False;
                              end
                            else
                              begin
                                if w = separator_byte then
                                  begin
                                    if remove_separators then
                                      begin
                                        continue; { Commas disappear if placed in quotes. }
                                      end
                                    else
                                      begin
                                        if selected then
                                          begin
                                            b := BELL_ASCII_KATAKANA[w];
                                            if b = BELL then
                                              begin
                                                continue; { Removing control characters }
                                              end
                                            else
                                              begin
                                                combo[m] := Chr(w);
                                                Inc(m);
                                              end;

                                          end;

                                      end;

                                  end
                                else
                                  begin
                                    if selected then
                                      begin
                                        b := BELL_ASCII_KATAKANA[w];
                                        if b = BELL then
                                          begin
                                            continue; { Removing control characters }
                                          end
                                        else
                                          begin
                                            combo[m] := Chr(w);
                                            Inc(m);
                                          end;

                                      end;

                                  end;

                              end;

                          end;

                      end;

                  end; { if character_type = SURROGATE_PAIR_C then }

              end; { if character_type = KATAKANA_C then, else }

          end; { if character_type = KANJI_C then }

      end
    else
      begin
{ if no_splitting = False then }

        if unformed then
          begin
            low_part_of_kanji := w;

            if selected then
              begin
                combo[m] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 1);
                combo[ m + 1 ] := table.workingPair(high_part_of_kanji, low_part_of_kanji, 2);
                Inc(m, KANJI_SIZE);
              end;

            unformed := False;
            continue;
          end;

{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
        character_type := table.working(w, 0);
        if character_type = KANJI_C then
          begin
            if selected then
              begin
                combo[m] := table.working(w, 1);
                combo[ m + 1 ] := table.working(w, 2);
                Inc(m, KANJI_SIZE);
              end;

          end
        else
          begin
            if character_type = KATAKANA_C then
              begin
                if selected then
                  begin
                    c := table.working(w, 1);
                    b := Ord(c);

                    b := BELL_ASCII_KATAKANA[b];
                    if b = BELL then
                      begin
                        continue;
                      end
                    else
                      begin
                        combo[m] := Chr(b);
                        Inc(m);
                      end;

                  end;

              end
            else
              begin
                if character_type = SURROGATE_PAIR_C then
                  begin
                    high_part_of_kanji := w;
                    unformed := True;
                    continue;
                  end
                else
                  begin
                    if w = H_0A then
                      begin
                        Inc(count);

                        combo[m] := H_0A_ANSI;
                        Inc(m);

                        if field_to_be_selected = FIRST_FIELD then
                          begin
                            field_number := 1;
                            selected := True;
                          end
                        else
                          begin
                            field_number := 1;
                            selected := False;
                          end;

                      end
                    else
                      begin
                        if w = H_0D then
                          begin
                            combo[m] := H_0D_ANSI;
                            Inc(m);
                            continue;
                          end
                        else
                          begin
                            if w = quote_byte then
                              begin
                                no_splitting := True;
                              end
                            else
                              begin
                                if w = separator_byte then
                                  begin
                                    if selected then
                                      begin
                                        b := BELL_ASCII_KATAKANA[w];
                                        if b = BELL then
                                          begin
                                            continue; { Removing control characters }
                                          end
                                        else
                                          begin
                                            combo[m] := Chr(w);
                                            Inc(m);
                                          end;

                                      end;

                                    Inc(field_number);
                                    if field_number = field_to_be_selected then
                                      begin
                                        selected := True;
                                      end
                                    else
                                      begin
                                        selected := False;
                                      end;

                                  end
                                else
                                  begin
                                    if selected then
                                      begin
                                        b := BELL_ASCII_KATAKANA[w];
                                        if b = BELL then
                                          begin
                                            continue; { Removing control characters }
                                          end
                                        else
                                          begin
                                            combo[m] := Chr(w);
                                            Inc(m);
                                          end;

                                      end;

                                  end;

                              end;

                          end;

                      end;

                  end; { if character_type = SURROGATE_PAIR_C then }

              end; { if character_type = KATAKANA_C then, else }

          end; { if character_type = KANJI_C then }

      end;

  end;

  Result := bufferingWriting(m);
end;

function Selecting.MS932_phrase_size: Integer;
begin
  Result := V_SIZE;
end;

function Selecting.standard_output: Integer;
var
  b: Byte;
  block_has_been_read: Integer;
  buffer_length_1: Integer;
  c: AnsiChar;
  character_type: AnsiChar;
  field_number: Integer;
  high_part_of_kanji: Integer;
  low_part_of_kanji: Integer;
  m: Integer;
  n: Integer;
  no_splitting: Boolean;
  s: String;
  selected: Boolean;
  testFile: File;
  unformed: Boolean;
  w: Integer;
begin
  buffer_length_1 := buffer_length - 1;
  carriage_return_appears := False;
  count := 0;

  for n := 0 to buffer_length_1 do
  begin
    buffer[n] := #0;
  end;

  try
{ Reading a file }
    AssignFile(testFile, file_name);
    Reset(testFile, SIZE_OF_WIDECHAR);

    m := 0;
    repeat
      y := CLEAR_Y_ARRAY;
      BlockRead(testFile, y, Y_COUNT, block_has_been_read);

      for n:= 0 to Y_COUNT_1 do
      begin
        buffer[m] := y[n];
        Inc(m);
        if m > buffer_length_1 then
          break;

      end;
    until m > buffer_length_1;

    CloseFile(testFile);
  except
    s := 'Reset() error-227: ' + file_name;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    Halt(1);
  end;

{ Finding the first NEW_LINE }
  if field_to_be_selected = FIRST_FIELD then
    begin
      field_number := 1;
      selected := True;
    end
  else
    begin
      field_number := 1;
      selected := False;
    end;

  high_part_of_kanji := H_D840;
  no_splitting := False;
  unformed := False;
  for n := beginning to buffer_length_1 do
  begin
    beginning := n + 1;
    w := Ord(buffer[n]);

    if no_splitting then
      begin
        if unformed then
          begin
            low_part_of_kanji := w;

            if selected then
              begin
                Write( table.workingPair(high_part_of_kanji, low_part_of_kanji, 1) );
                Write( table.workingPair(high_part_of_kanji, low_part_of_kanji, 2) );
              end;

            unformed := False;
            continue;
          end;

{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
        character_type := table.working(w, 0);
        if character_type = KANJI_C then
          begin
            if selected then
              begin
                Write( table.working(w, 1) );
                Write( table.working(w, 2) );
              end;

          end
        else
          begin
            if character_type = KATAKANA_C then
              begin
                if selected then
                  begin
                    c := table.working(w, 1);
                    b := Ord(c);

                    b := BELL_ASCII_KATAKANA[b];
                    if b = BELL then
                      begin
                        continue;
                      end
                    else
                      begin
                        Write( Chr(b) );
                      end;

                  end

              end
            else
              begin
                if character_type = SURROGATE_PAIR_C then
                  begin
                    high_part_of_kanji := w;
                    unformed := True;
                    continue;
                  end
                else
                  begin
                    if w = H_0A then
                      begin
                        count := 1;

                        Write( H_0D_ANSI );
                        Write( H_0A_ANSI );

                        break;
                      end
                    else
                      begin
                        if w = H_0D then
                          begin
                            carriage_return_appears := True;
                            NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
                            NEW_LINE := EMPTY_ANSI + H_0D_ANSI;
                            NEW_LINE := NEW_LINE + H_0A_ANSI;
                          end
                        else
                          begin
                            if w = quote_byte then
                              begin
                                no_splitting := False;
                              end
                            else
                              begin
                                if w = separator_byte then
                                  begin
                                    if remove_separators then
                                      begin
                                        continue; { Commas disappear if placed in quotes. }
                                      end
                                    else
                                      begin
                                        if selected then
                                          begin
                                            Write( Chr(w) );
                                          end;

                                      end;

                                  end
                                else
                                  begin
                                    if selected then
                                      begin
                                        Write( Chr(w) );
                                      end;

                                  end;

                              end;

                          end;

                      end;

                  end; { if character_type = SURROGATE_PAIR_C then, else }

              end; { if character_type = KATAKANA_C then, else }

          end; { if c = KANJI_C then, else }

      end
    else
      begin
{ if no_splitting = False then }

        if unformed then
          begin
            low_part_of_kanji := w;

            if selected then
              begin
                Write( table.workingPair(high_part_of_kanji, low_part_of_kanji, 1) );
                Write( table.workingPair(high_part_of_kanji, low_part_of_kanji, 2) );
              end;

            unformed := False;
            continue;
          end;

{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
        character_type := table.working(w, 0);
        if character_type = KANJI_C then
          begin
            if selected then
              begin
                Write( table.working(w, 1) );
                Write( table.working(w, 2) );
              end;

          end
        else
          begin
            if character_type = KATAKANA_C then
              begin
                if selected then
                  begin
                    c := table.working(w, 1);
                    b := Ord(c);

                    b := BELL_ASCII_KATAKANA[b];
                    if b = BELL then
                      begin
                        continue;
                      end
                    else
                      begin
                        Write( Chr(b) );
                      end;

                  end

              end
            else
              begin
                if character_type = SURROGATE_PAIR_C then
                  begin
                    high_part_of_kanji := w;
                    unformed := True;
                    continue;
                  end
                else
                  begin
                    if w = H_0A then
                      begin
                        count := 1;

                        Write( H_0D_ANSI );
                        Write( H_0A_ANSI );

                        break;
                      end
                    else
                      begin
                        if w = H_0D then
                          begin
                            carriage_return_appears := True;
                            NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
                            NEW_LINE := EMPTY_ANSI + H_0D_ANSI;
                            NEW_LINE := NEW_LINE + H_0A_ANSI;
                          end
                        else
                          begin
                            if w = quote_byte then
                              begin
                                no_splitting := True;
                              end
                            else
                              begin
                                if w = separator_byte then
                                  begin
                                    if selected then
                                      begin
                                        Write( Chr(w) );
                                      end;

                                    Inc(field_number);
                                    if field_number = field_to_be_selected then
                                      begin
                                        selected := True;
                                      end
                                    else
                                      begin
                                        selected := False;
                                      end;

                                  end
                                else
                                  begin
                                    if selected then
                                      begin
                                         Write( Chr(w) );
                                      end;

                                  end;

                              end;

                          end;

                      end;

                  end; { if character_type = SURROGATE_PAIR_C then, else }

              end; { if character_type = KATAKANA_C then, else }

          end; { if c = KANJI_C then, else }

      end;

  end;

{ Counting lines }
  if field_to_be_selected = FIRST_FIELD then
    begin
      field_number := 1;
      selected := True;
    end
  else
    begin
      field_number := 1;
      selected := False;
    end;

  high_part_of_kanji := H_D840;
  no_splitting := False;
  unformed := False;
  for n := beginning to buffer_length_1 do
  begin
    w := Ord(buffer[n]);

    if no_splitting then
      begin
        if unformed then
          begin
            low_part_of_kanji := w;

            if selected then
              begin
                Write( table.workingPair(high_part_of_kanji, low_part_of_kanji, 1) );
                Write( table.workingPair(high_part_of_kanji, low_part_of_kanji, 2) );
              end;

            unformed := False;
            continue;
          end;

{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
        character_type := table.working(w, 0);
        if character_type = KANJI_C then
          begin
            if selected then
              begin
                Write( table.working(w, 1) );
                Write( table.working(w, 2) );
              end;

          end
        else
          begin
            if character_type = KATAKANA_C then
              begin
                if selected then
                  begin
                    c := table.working(w, 1);
                    b := Ord(c);

                    b := BELL_ASCII_KATAKANA[b];
                    if b = BELL then
                      begin
                        continue;
                      end
                    else
                      begin
                        Write( Chr(b) );
                      end;

                  end;

              end
            else
              begin
                if character_type = SURROGATE_PAIR_C then
                  begin
                    high_part_of_kanji := w;
                    unformed := True;
                    continue;
                  end
                else
                  begin
                    if w = H_0A then
                      begin
                        Inc(count);

                        Write( H_0D_ANSI );
                        Write( H_0A_ANSI );

                        if field_to_be_selected = FIRST_FIELD then
                          begin
                            field_number := 1;
                            selected := True;
                          end
                        else
                          begin
                            field_number := 1;
                            selected := False;
                          end;

                      end
                    else
                      begin
                        if w = H_0D then
                          begin
                            continue;
                          end
                        else
                          begin
                            if w = quote_byte then
                              begin
                                no_splitting := False;
                              end
                            else
                              begin
                                if w = separator_byte then
                                  begin
                                    if remove_separators then
                                      begin
                                        continue; { Commas disappear if placed in quotes. }
                                      end
                                    else
                                      begin
                                        if selected then
                                          begin
                                            Write( Chr(w) );
                                          end;

                                      end;

                                  end
                                else
                                  begin
                                    if selected then
                                      begin
                                        Write( Chr(w) );
                                      end;

                                  end;

                              end;

                          end;

                      end;

                  end; { if character_type = SURROGATE_PAIR_C then }

              end; { if character_type = KATAKANA_C then, else }

          end; { if character_type = KANJI_C then }

      end
    else
      begin
{ if no_splitting = False then }

        if unformed then
          begin
            low_part_of_kanji := w;

            if selected then
              begin
                Write( table.workingPair(high_part_of_kanji, low_part_of_kanji, 1) );
                Write( table.workingPair(high_part_of_kanji, low_part_of_kanji, 2) );
              end;

            unformed := False;
            continue;
          end;

{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
        character_type := table.working(w, 0);
        if character_type = KANJI_C then
          begin
            if selected then
              begin
                Write( table.working(w, 1) );
                Write( table.working(w, 2) );
              end;

          end
        else
          begin
            if character_type = KATAKANA_C then
              begin
                if selected then
                  begin
                    c := table.working(w, 1);
                    b := Ord(c);

                    b := BELL_ASCII_KATAKANA[b];
                    if b = BELL then
                      begin
                        continue;
                      end
                    else
                      begin
                        Write( Chr(b) );
                      end;

                  end;

              end
            else
              begin
                if character_type = SURROGATE_PAIR_C then
                  begin
                    high_part_of_kanji := w;
                    unformed := True;
                    continue;
                  end
                else
                  begin
                    if w = H_0A then
                      begin
                        Inc(count);

                        Write( H_0D_ANSI );
                        Write( H_0A_ANSI );

                        if field_to_be_selected = FIRST_FIELD then
                          begin
                            field_number := 1;
                            selected := True;
                          end
                        else
                          begin
                            field_number := 1;
                            selected := False;
                          end;

                      end
                    else
                      begin
                        if w = H_0D then
                          begin
                            continue;
                          end
                        else
                          begin
                            if w = quote_byte then
                              begin
                                no_splitting := True;
                              end
                            else
                              begin
                                if w = separator_byte then
                                  begin
                                    if selected then
                                      begin
                                            Write( Chr(w) );
                                      end;

                                    Inc(field_number);
                                    if field_number = field_to_be_selected then
                                      begin
                                        selected := True;
                                      end
                                    else
                                      begin
                                        selected := False;
                                      end;

                                  end
                                else
                                  begin
                                    if selected then
                                      begin
                                        Write( Chr(w) );
                                      end;

                                  end;

                              end;

                          end;

                      end;

                  end; { if character_type = SURROGATE_PAIR_C then }

              end; { if character_type = KATAKANA_C then, else }

          end; { if character_type = KANJI_C then }

      end;

  end;

  Result := count;
end;

function Selecting.standard_output_of_decimals: Integer;
var
  b: Byte;
  block_has_been_read: Integer;
  buffer_length_1: Integer;
  character_type: AnsiChar;
  dotted: Boolean;
  extracting: Boolean;
  field_number: Integer;
  m: Integer;
  n: Integer;
  no_splitting: Boolean;
  s: String;
  selected: Boolean;
  testFile: File;
  unformed: Boolean;
  w: Integer;
begin
  buffer_length_1 := buffer_length - 1;
  carriage_return_appears := False;
  count := 0;
  dotted := False;
  extracting := False;

  for n := 0 to buffer_length_1 do
  begin
    buffer[n] := #0;
  end;

  try
{ Reading a file }
    AssignFile(testFile, file_name);
    Reset(testFile, SIZE_OF_WIDECHAR);

    m := 0;
    repeat
      y := CLEAR_Y_ARRAY;
      BlockRead(testFile, y, Y_COUNT, block_has_been_read);

      for n:= 0 to Y_COUNT_1 do
      begin
        buffer[m] := y[n];
        Inc(m);
        if m > buffer_length_1 then
          break;

      end;
    until m > buffer_length_1;

    CloseFile(testFile);
  except
    s := 'Reset() error-200: ' + file_name;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    Halt(1);
  end;

{ Finding the first NEW_LINE }
  if field_to_be_selected = FIRST_FIELD then
    begin
      field_number := 1;
      selected := True;
      BELL_ASCII_KATAKANA[H_2E] := H_2E; { '.' }
      BELL_ASCII_KATAKANA[H_2D] := H_2D; { '-' }
      dotted := False;
      extracting := False;
    end
  else
    begin
      field_number := 1;
      selected := False;
    end;

  m := 0;
  no_splitting := False;
  unformed := False;
  for n := beginning to buffer_length_1 do
  begin
    beginning := n + 1;
    w := Ord(buffer[n]);

    if no_splitting then
      begin
        if unformed then
          begin
            if selected then
              begin
                if extracting then
                  begin
                    selected := False;
                  end;

                continue;
              end;

            unformed := False;
            continue;
          end;

{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
        character_type := table.working(w, 0);
        if character_type = KANJI_C then
          begin
            if selected then
              begin
                if extracting then
                  begin
                    selected := False;
                  end;

                continue;
              end;

          end
        else
          begin
            if character_type = KATAKANA_C then
              begin
                if selected then
                  begin
                    if extracting then
                      begin
                        selected := False;
                      end;

                    continue;
                  end

              end
            else
              begin
                if character_type = SURROGATE_PAIR_C then
                  begin
                    unformed := True;
                    continue;
                  end
                else
                  begin
                    if w = H_0A then
                      begin
                        count := 1;

                        if not dotted then
                          begin
                            Write( DOT_C );
                            Write( ZERO_C );
                          end;

                        Write( H_0D_ANSI );
                        Write( H_0A_ANSI );

                        break;
                      end
                    else
                      begin
                        if w = H_0D then
                          begin
                            carriage_return_appears := True;
                            NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
                            NEW_LINE := EMPTY_ANSI + H_0D_ANSI;
                            NEW_LINE := NEW_LINE + H_0A_ANSI;
                          end
                        else
                          begin
                            if w = quote_byte then
                              begin
                                no_splitting := False;
                              end
                            else
                              begin
                                if w = separator_byte then
                                  begin
                                    if remove_separators then
                                      begin
                                        continue; { Commas disappear if placed in quotes. }
                                      end
                                    else
                                      begin
                                        if selected then
                                          begin
                                            if extracting then
                                              begin
                                                selected := False;
                                              end;

                                          end;

                                      end;

                                  end
                                else
                                  begin
                                    if selected then
                                      begin
                                        b := BELL_ASCII_KATAKANA[w];
                                        if b = BELL then
                                          begin
                                            if extracting then
                                              begin
                                                selected := False;
                                              end;

                                            continue;
                                          end
                                        else
                                          begin
                                            Write( Chr(w) );
                                            extracting := True;

                                            if w = H_2E then
                                              begin
                                                dotted := True;
                                                BELL_ASCII_KATAKANA[H_2E] := BELL;
                                              end;

                                            BELL_ASCII_KATAKANA[H_2D] := BELL;
                                          end;

                                      end;

                                  end;

                              end;

                          end;

                      end;

                  end; { if character_type = SURROGATE_PAIR_C then, else }

              end; { if character_type = KATAKANA_C then, else }

          end; { if c = KANJI_C then, else }

      end
    else
      begin
{ if no_splitting = False then }

        if unformed then
          begin
            if selected then
              begin
                if extracting then
                  begin
                    selected := False;
                  end;

                continue;
              end;

            unformed := False;
            continue;
          end;

{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
        character_type := table.working(w, 0);
        if character_type = KANJI_C then
          begin
            if selected then
              begin
                if extracting then
                  begin
                    selected := False;
                  end;

                continue;
              end;

          end
        else
          begin
            if character_type = KATAKANA_C then
              begin
                if selected then
                  begin
                    if extracting then
                      begin
                        selected := False;
                      end;

                    continue;
                  end

              end
            else
              begin
                if character_type = SURROGATE_PAIR_C then
                  begin
                    unformed := True;
                    continue;
                  end
                else
                  begin
                    if w = H_0A then
                      begin
                        count := 1;

                        if not dotted then
                          begin
                            Write( DOT_C );
                            Write( ZERO_C );
                          end;

                        Write( H_0D_ANSI );
                        Write( H_0A_ANSI );

                        break;
                      end
                    else
                      begin
                        if w = H_0D then
                          begin
                            carriage_return_appears := True;
                            NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
                            NEW_LINE := EMPTY_ANSI + H_0D_ANSI;
                            NEW_LINE := NEW_LINE + H_0A_ANSI;
                          end
                        else
                          begin
                            if w = quote_byte then
                              begin
                                no_splitting := True;
                              end
                            else
                              begin
                                if w = separator_byte then
                                  begin
                                    Inc(field_number);
                                    if field_number = field_to_be_selected then
                                      begin
                                        selected := True;
                                        BELL_ASCII_KATAKANA[H_2E] := H_2E; { '.' }
                                        BELL_ASCII_KATAKANA[H_2D] := H_2D; { '-' }
                                        dotted := False;
                                        extracting := False;
                                      end
                                    else
                                      begin
                                        selected := False;
                                      end;

                                  end
                                else
                                  begin
                                    if selected then
                                      begin
                                        b := BELL_ASCII_KATAKANA[w];
                                        if b = BELL then
                                          begin
                                            if extracting then
                                              begin
                                                selected := False;
                                              end;

                                            continue;
                                          end
                                        else
                                          begin
                                            Write( Chr(w) );
                                            extracting := True;

                                            if w = H_2E then
                                              begin
                                                dotted := True;
                                                BELL_ASCII_KATAKANA[H_2E] := BELL;
                                              end;

                                            BELL_ASCII_KATAKANA[H_2D] := BELL;
                                          end;

                                      end;

                                  end;

                              end;

                          end;

                      end;

                  end; { if character_type = SURROGATE_PAIR_C then, else }

              end; { if character_type = KATAKANA_C then, else }

          end; { if c = KANJI_C then, else }

      end;

  end;

{ Counting lines }
  if field_to_be_selected = FIRST_FIELD then
    begin
      field_number := 1;
      selected := True;
    end
  else
    begin
      field_number := 1;
      selected := False;
    end;

  no_splitting := False;
  unformed := False;
  for n := beginning to buffer_length_1 do
  begin
    w := Ord(buffer[n]);

    if no_splitting then
      begin
        if unformed then
          begin
            if selected then
              begin
                if extracting then
                  begin
                    selected := False;
                  end;

                continue;
              end;

            unformed := False;
            continue;
          end;

{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
        character_type := table.working(w, 0);
        if character_type = KANJI_C then
          begin
            if selected then
              begin
                if extracting then
                  begin
                    selected := False;
                  end;

                continue;
              end;

          end
        else
          begin
            if character_type = KATAKANA_C then
              begin
                if selected then
                  begin
                    if extracting then
                      begin
                        selected := False;
                      end;

                    continue;
                  end;

              end
            else
              begin
                if character_type = SURROGATE_PAIR_C then
                  begin
                    unformed := True;
                    continue;
                  end
                else
                  begin
                    if w = H_0A then
                      begin
                        Inc(count);

                        if not dotted then
                          begin
                            Write( DOT_C );
                            Write( ZERO_C );
                          end;

                        Write( H_0D_ANSI );
                        Write( H_0A_ANSI );

                        if field_to_be_selected = FIRST_FIELD then
                          begin
                            field_number := 1;
                            selected := True;
                          end
                        else
                          begin
                            field_number := 1;
                            selected := False;
                          end;

                      end
                    else
                      begin
                        if w = H_0D then
                          begin
                            continue;
                          end
                        else
                          begin
                            if w = quote_byte then
                              begin
                                no_splitting := False;
                              end
                            else
                              begin
                                if w = separator_byte then
                                  begin
                                    if remove_separators then
                                      begin
                                        continue; { Commas disappear if placed in quotes. }
                                      end
                                    else
                                      begin
                                        if selected then
                                          begin
                                            if extracting then
                                              begin
                                                selected := False;
                                              end;

                                          end;

                                      end;

                                  end
                                else
                                  begin
                                    if selected then
                                      begin
                                        b := BELL_ASCII_KATAKANA[w];
                                        if b = BELL then
                                          begin
                                            if extracting then
                                              begin
                                                selected := False;
                                              end;

                                            continue;
                                          end
                                        else
                                          begin
                                            Write( Chr(w) );
                                            extracting := True;

                                            if w = H_2E then
                                              begin
                                                dotted := True;
                                                BELL_ASCII_KATAKANA[H_2E] := BELL;
                                              end;

                                            BELL_ASCII_KATAKANA[H_2D] := BELL;
                                          end;

                                      end;

                                  end;

                              end;

                          end;

                      end;

                  end; { if character_type = SURROGATE_PAIR_C then }

              end; { if character_type = KATAKANA_C then, else }

          end; { if character_type = KANJI_C then }

      end
    else
      begin
{ if no_splitting = False then }

        if unformed then
          begin
            if selected then
              begin
                if extracting then
                  begin
                    selected := False;
                  end;

                continue;
              end;

            unformed := False;
            continue;
          end;

{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
        character_type := table.working(w, 0);
        if character_type = KANJI_C then
          begin
            if selected then
              begin
                if extracting then
                  begin
                    selected := False;
                  end;

                continue;
              end;

          end
        else
          begin
            if character_type = KATAKANA_C then
              begin
                if selected then
                  begin
                    if extracting then
                      begin
                        selected := False;
                      end;

                    continue;
                  end;

              end
            else
              begin
                if character_type = SURROGATE_PAIR_C then
                  begin
                    unformed := True;
                    continue;
                  end
                else
                  begin
                    if w = H_0A then
                      begin
                        Inc(count);

                        if not dotted then
                          begin
                            Write( DOT_C );
                            Write( ZERO_C );
                          end;

                        Write( H_0D_ANSI );
                        Write( H_0A_ANSI );

                        if field_to_be_selected = FIRST_FIELD then
                          begin
                            field_number := 1;
                            selected := True;
                          end
                        else
                          begin
                            field_number := 1;
                            selected := False;
                          end;

                      end
                    else
                      begin
                        if w = H_0D then
                          begin
                            continue;
                          end
                        else
                          begin
                            if w = quote_byte then
                              begin
                                no_splitting := True;
                              end
                            else
                              begin
                                if w = separator_byte then
                                  begin
                                    Inc(field_number);
                                    if field_number = field_to_be_selected then
                                      begin
                                        selected := True;
                                        BELL_ASCII_KATAKANA[H_2E] := H_2E; { '.' }
                                        BELL_ASCII_KATAKANA[H_2D] := H_2D; { '-' }
                                        dotted := False;
                                        extracting := False;
                                      end
                                    else
                                      begin
                                        selected := False;
                                      end;

                                  end
                                else
                                  begin
                                    if selected then
                                      begin
                                        b := BELL_ASCII_KATAKANA[w];
                                        if b = BELL then
                                          begin
                                            if extracting then
                                              begin
                                                selected := False;
                                              end;

                                            continue;
                                          end
                                        else
                                          begin
                                            Write( Chr(w) );
                                            extracting := True;

                                            if w = H_2E then
                                              begin
                                                dotted := True;
                                                BELL_ASCII_KATAKANA[H_2E] := BELL;
                                              end;

                                            BELL_ASCII_KATAKANA[H_2D] := BELL;
                                          end;

                                      end;

                                  end;

                              end;

                          end;

                      end;

                  end; { if character_type = SURROGATE_PAIR_C then }

              end; { if character_type = KATAKANA_C then, else }

          end; { if character_type = KANJI_C then }

      end;

  end;

  Result := bufferingWriting(m);
end;

function Selecting.standard_output_of_integers: Integer;
var
  b: Byte;
  block_has_been_read: Integer;
  buffer_length_1: Integer;
  character_type: AnsiChar;
  extracting: Boolean;
  field_number: Integer;
  m: Integer;
  n: Integer;
  no_splitting: Boolean;
  s: String;
  selected: Boolean;
  testFile: File;
  unformed: Boolean;
  w: Integer;
begin
  buffer_length_1 := buffer_length - 1;
  carriage_return_appears := False;
  count := 0;
  extracting := False;

  for n := 0 to buffer_length_1 do
  begin
    buffer[n] := #0;
  end;

  try
{ Reading a file }
    AssignFile(testFile, file_name);
    Reset(testFile, SIZE_OF_WIDECHAR);

    m := 0;
    repeat
      y := CLEAR_Y_ARRAY;
      BlockRead(testFile, y, Y_COUNT, block_has_been_read);

      for n:= 0 to Y_COUNT_1 do
      begin
        buffer[m] := y[n];
        Inc(m);
        if m > buffer_length_1 then
          break;

      end;
    until m > buffer_length_1;

    CloseFile(testFile);
  except
    s := 'Reset() error-201: ' + file_name;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    Halt(1);
  end;

{ Finding the first NEW_LINE }
  if field_to_be_selected = FIRST_FIELD then
    begin
      field_number := 1;
      selected := True;
      BELL_ASCII_KATAKANA[H_2D] := H_2D; { '-' }
      extracting := False;
    end
  else
    begin
      field_number := 1;
      selected := False;
    end;

  m := 0;
  no_splitting := False;
  unformed := False;
  for n := beginning to buffer_length_1 do
  begin
    beginning := n + 1;
    w := Ord(buffer[n]);

    if no_splitting then
      begin
        if unformed then
          begin
            if selected then
              begin
                if extracting then
                  begin
                    selected := False;
                  end;

                continue;
              end;

            unformed := False;
            continue;
          end;

{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
        character_type := table.working(w, 0);
        if character_type = KANJI_C then
          begin
            if selected then
              begin
                if extracting then
                  begin
                    selected := False;
                  end;

                continue;
              end;

          end
        else
          begin
            if character_type = KATAKANA_C then
              begin
                if selected then
                  begin
                    if extracting then
                      begin
                        selected := False;
                      end;

                    continue;
                  end

              end
            else
              begin
                if character_type = SURROGATE_PAIR_C then
                  begin
                    unformed := True;
                    continue;
                  end
                else
                  begin
                    if w = H_0A then
                      begin
                        count := 1;

                        Write( H_0D_ANSI );
                        Write( H_0A_ANSI );

                        break;
                      end
                    else
                      begin
                        if w = H_0D then
                          begin
                            carriage_return_appears := True;
                            NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
                            NEW_LINE := EMPTY_ANSI + H_0D_ANSI;
                            NEW_LINE := NEW_LINE + H_0A_ANSI;
                          end
                        else
                          begin
                            if w = quote_byte then
                              begin
                                no_splitting := False;
                              end
                            else
                              begin
                                if w = separator_byte then
                                  begin
                                    if remove_separators then
                                      begin
                                        continue; { Commas disappear if placed in quotes. }
                                      end
                                    else
                                      begin
                                        if selected then
                                          begin
                                            if extracting then
                                              begin
                                                selected := False;
                                              end;

                                          end;

                                      end;

                                  end
                                else
                                  begin
                                    if selected then
                                      begin
                                        b := BELL_ASCII_KATAKANA[w];
                                        if b = BELL then
                                          begin
                                            if extracting then
                                              begin
                                                selected := False;
                                              end;

                                            continue;
                                          end
                                        else
                                          begin
                                            Write( Chr(w) );
                                            extracting := True;
                                            BELL_ASCII_KATAKANA[H_2D] := BELL;
                                          end;

                                      end;

                                  end;

                              end;

                          end;

                      end;

                  end; { if character_type = SURROGATE_PAIR_C then, else }

              end; { if character_type = KATAKANA_C then, else }

          end; { if c = KANJI_C then, else }

      end
    else
      begin
{ if no_splitting = False then }

        if unformed then
          begin
            if selected then
              begin
                if extracting then
                  begin
                    selected := False;
                  end;

                continue;
              end;

            unformed := False;
            continue;
          end;

{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
        character_type := table.working(w, 0);
        if character_type = KANJI_C then
          begin
            if selected then
              begin
                if extracting then
                  begin
                    selected := False;
                  end;

                continue;
              end;

          end
        else
          begin
            if character_type = KATAKANA_C then
              begin
                if selected then
                  begin
                    if extracting then
                      begin
                        selected := False;
                      end;

                    continue;
                  end

              end
            else
              begin
                if character_type = SURROGATE_PAIR_C then
                  begin
                    unformed := True;
                    continue;
                  end
                else
                  begin
                    if w = H_0A then
                      begin
                        count := 1;

                        Write( H_0D_ANSI );
                        Write( H_0A_ANSI );

                        break;
                      end
                    else
                      begin
                        if w = H_0D then
                          begin
                            carriage_return_appears := True;
                            NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
                            NEW_LINE := EMPTY_ANSI + H_0D_ANSI;
                            NEW_LINE := NEW_LINE + H_0A_ANSI;
                          end
                        else
                          begin
                            if w = quote_byte then
                              begin
                                no_splitting := True;
                              end
                            else
                              begin
                                if w = separator_byte then
                                  begin
                                    Inc(field_number);
                                    if field_number = field_to_be_selected then
                                      begin
                                        selected := True;
                                        BELL_ASCII_KATAKANA[H_2D] := H_2D; { '-' }
                                        extracting := False;
                                      end
                                    else
                                      begin
                                        selected := False;
                                      end;

                                  end
                                else
                                  begin
                                    if selected then
                                      begin
                                        b := BELL_ASCII_KATAKANA[w];
                                        if b = BELL then
                                          begin
                                            if extracting then
                                              begin
                                                selected := False;
                                              end;

                                            continue;
                                          end
                                        else
                                          begin
                                            Write( Chr(w) );
                                            extracting := True;
                                            BELL_ASCII_KATAKANA[H_2D] := BELL;
                                          end;

                                      end;

                                  end;

                              end;

                          end;

                      end;

                  end; { if character_type = SURROGATE_PAIR_C then, else }

              end; { if character_type = KATAKANA_C then, else }

          end; { if c = KANJI_C then, else }

      end;

  end;

{ Counting lines }
  if field_to_be_selected = FIRST_FIELD then
    begin
      field_number := 1;
      selected := True;
    end
  else
    begin
      field_number := 1;
      selected := False;
    end;

  no_splitting := False;
  unformed := False;
  for n := beginning to buffer_length_1 do
  begin
    w := Ord(buffer[n]);

    if no_splitting then
      begin
        if unformed then
          begin
            if selected then
              begin
                if extracting then
                  begin
                    selected := False;
                  end;

                continue;
              end;

            unformed := False;
            continue;
          end;

{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
        character_type := table.working(w, 0);
        if character_type = KANJI_C then
          begin
            if selected then
              begin
                if extracting then
                  begin
                    selected := False;
                  end;

                continue;
              end;

          end
        else
          begin
            if character_type = KATAKANA_C then
              begin
                if selected then
                  begin
                    if extracting then
                      begin
                        selected := False;
                      end;

                    continue;
                  end;

              end
            else
              begin
                if character_type = SURROGATE_PAIR_C then
                  begin
                    unformed := True;
                    continue;
                  end
                else
                  begin
                    if w = H_0A then
                      begin
                        Inc(count);

                        Write( H_0D_ANSI );
                        Write( H_0A_ANSI );

                        if field_to_be_selected = FIRST_FIELD then
                          begin
                            field_number := 1;
                            selected := True;
                          end
                        else
                          begin
                            field_number := 1;
                            selected := False;
                          end;

                      end
                    else
                      begin
                        if w = H_0D then
                          begin
                            continue;
                          end
                        else
                          begin
                            if w = quote_byte then
                              begin
                                no_splitting := False;
                              end
                            else
                              begin
                                if w = separator_byte then
                                  begin
                                    if remove_separators then
                                      begin
                                        continue; { Commas disappear if placed in quotes. }
                                      end
                                    else
                                      begin
                                        if selected then
                                          begin
                                            if extracting then
                                              begin
                                                selected := False;
                                              end;

                                          end;

                                      end;

                                  end
                                else
                                  begin
                                    if selected then
                                      begin
                                        b := BELL_ASCII_KATAKANA[w];
                                        if b = BELL then
                                          begin
                                            if extracting then
                                              begin
                                                selected := False;
                                              end;

                                            continue;
                                          end
                                        else
                                          begin
                                            Write( Chr(w) );
                                            extracting := True;
                                            BELL_ASCII_KATAKANA[H_2D] := BELL;
                                          end;

                                      end;

                                  end;

                              end;

                          end;

                      end;

                  end; { if character_type = SURROGATE_PAIR_C then }

              end; { if character_type = KATAKANA_C then, else }

          end; { if character_type = KANJI_C then }

      end
    else
      begin
{ if no_splitting = False then }

        if unformed then
          begin
            if selected then
              begin
                if extracting then
                  begin
                    selected := False;
                  end;

                continue;
              end;

            unformed := False;
            continue;
          end;

{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
        character_type := table.working(w, 0);
        if character_type = KANJI_C then
          begin
            if selected then
              begin
                if extracting then
                  begin
                    selected := False;
                  end;

                continue;
              end;

          end
        else
          begin
            if character_type = KATAKANA_C then
              begin
                if selected then
                  begin
                    if extracting then
                      begin
                        selected := False;
                      end;

                    continue;
                  end;

              end
            else
              begin
                if character_type = SURROGATE_PAIR_C then
                  begin
                    unformed := True;
                    continue;
                  end
                else
                  begin
                    if w = H_0A then
                      begin
                        Inc(count);

                        Write( H_0D_ANSI );
                        Write( H_0A_ANSI );

                        if field_to_be_selected = FIRST_FIELD then
                          begin
                            field_number := 1;
                            selected := True;
                          end
                        else
                          begin
                            field_number := 1;
                            selected := False;
                          end;

                      end
                    else
                      begin
                        if w = H_0D then
                          begin
                            continue;
                          end
                        else
                          begin
                            if w = quote_byte then
                              begin
                                no_splitting := True;
                              end
                            else
                              begin
                                if w = separator_byte then
                                  begin
                                    Inc(field_number);
                                    if field_number = field_to_be_selected then
                                      begin
                                        selected := True;
                                        BELL_ASCII_KATAKANA[H_2D] := H_2D; { '-' }
                                        extracting := False;
                                      end
                                    else
                                      begin
                                        selected := False;
                                      end;

                                  end
                                else
                                  begin
                                    if selected then
                                      begin
                                        b := BELL_ASCII_KATAKANA[w];
                                        if b = BELL then
                                          begin
                                            if extracting then
                                              begin
                                                selected := False;
                                              end;

                                            continue;
                                          end
                                        else
                                          begin
                                            Write( Chr(w) );
                                            extracting := True;
                                            BELL_ASCII_KATAKANA[H_2D] := BELL;
                                          end;

                                      end;

                                  end;

                              end;

                          end;

                      end;

                  end; { if character_type = SURROGATE_PAIR_C then }

              end; { if character_type = KATAKANA_C then, else }

          end; { if character_type = KANJI_C then }

      end;

  end;

  Result := bufferingWriting(m);
end;

function Selecting.standard_output_of_non_controllers: Integer;
var
  b: Byte;
  block_has_been_read: Integer;
  buffer_length_1: Integer;
  c: AnsiChar;
  character_type: AnsiChar;
  field_number: Integer;
  high_part_of_kanji: Integer;
  low_part_of_kanji: Integer;
  m: Integer;
  n: Integer;
  no_splitting: Boolean;
  s: String;
  selected: Boolean;
  testFile: File;
  unformed: Boolean;
  w: Integer;
begin
  buffer_length_1 := buffer_length - 1;
  carriage_return_appears := False;
  count := 0;

  for n := 0 to buffer_length_1 do
  begin
    buffer[n] := #0;
  end;

  try
{ Reading a file }
    AssignFile(testFile, file_name);
    Reset(testFile, SIZE_OF_WIDECHAR);

    m := 0;
    repeat
      y := CLEAR_Y_ARRAY;
      BlockRead(testFile, y, Y_COUNT, block_has_been_read);

      for n:= 0 to Y_COUNT_1 do
      begin
        buffer[m] := y[n];
        Inc(m);
        if m > buffer_length_1 then
          break;

      end;
    until m > buffer_length_1;

    CloseFile(testFile);
  except
    s := 'Reset() error-228: ' + file_name;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    Halt(1);
  end;

{ Finding the first NEW_LINE }
  if field_to_be_selected = FIRST_FIELD then
    begin
      field_number := 1;
      selected := True;
    end
  else
    begin
      field_number := 1;
      selected := False;
    end;

  high_part_of_kanji := H_D840;
  no_splitting := False;
  unformed := False;
  for n := beginning to buffer_length_1 do
  begin
    beginning := n + 1;
    w := Ord(buffer[n]);

    if no_splitting then
      begin
        if unformed then
          begin
            low_part_of_kanji := w;

            if selected then
              begin
                Write( table.workingPair(high_part_of_kanji, low_part_of_kanji, 1) );
                Write( table.workingPair(high_part_of_kanji, low_part_of_kanji, 2) );
              end;

            unformed := False;
            continue;
          end;

{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
        character_type := table.working(w, 0);
        if character_type = KANJI_C then
          begin
            if selected then
              begin
                Write( table.working(w, 1) );
                Write( table.working(w, 2) );
              end;

          end
        else
          begin
            if character_type = KATAKANA_C then
              begin
                if selected then
                  begin
                    c := table.working(w, 1);
                    b := Ord(c);

                    b := BELL_ASCII_KATAKANA[b];
                    if b = BELL then
                      begin
                        continue;
                      end
                    else
                      begin
                        Write( Chr(b) );
                      end;

                  end

              end
            else
              begin
                if character_type = SURROGATE_PAIR_C then
                  begin
                    high_part_of_kanji := w;
                    unformed := True;
                    continue;
                  end
                else
                  begin
                    if w = H_0A then
                      begin
                        count := 1;

                        Write( H_0D_ANSI );
                        Write( H_0A_ANSI );

                        break;
                      end
                    else
                      begin
                        if w = H_0D then
                          begin
                            carriage_return_appears := True;
                            NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
                            NEW_LINE := EMPTY_ANSI + H_0D_ANSI;
                            NEW_LINE := NEW_LINE + H_0A_ANSI;
                          end
                        else
                          begin
                            if w = quote_byte then
                              begin
                                no_splitting := False;
                              end
                            else
                              begin
                                if w = separator_byte then
                                  begin
                                    if remove_separators then
                                      begin
                                        continue; { Commas disappear if placed in quotes. }
                                      end
                                    else
                                      begin
                                        if selected then
                                          begin
                                            b := BELL_ASCII_KATAKANA[w];
                                            if b = BELL then
                                              begin
                                                continue; { Removing control characters }
                                              end
                                            else
                                              begin
                                                Write( Chr(w) );
                                              end;

                                          end;

                                      end;

                                  end
                                else
                                  begin
                                    if selected then
                                      begin
                                        b := BELL_ASCII_KATAKANA[w];
                                        if b = BELL then
                                          begin
                                            continue; { Removing control characters }
                                          end
                                        else
                                          begin
                                            Write( Chr(w) );
                                          end;

                                      end;

                                  end;

                              end;

                          end;

                      end;

                  end; { if character_type = SURROGATE_PAIR_C then, else }

              end; { if character_type = KATAKANA_C then, else }

          end; { if c = KANJI_C then, else }

      end
    else
      begin
{ if no_splitting = False then }

        if unformed then
          begin
            low_part_of_kanji := w;

            if selected then
              begin
                Write( table.workingPair(high_part_of_kanji, low_part_of_kanji, 1) );
                Write( table.workingPair(high_part_of_kanji, low_part_of_kanji, 2) );
              end;

            unformed := False;
            continue;
          end;

{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
        character_type := table.working(w, 0);
        if character_type = KANJI_C then
          begin
            if selected then
              begin
                Write( table.working(w, 1) );
                Write( table.working(w, 2) );
              end;

          end
        else
          begin
            if character_type = KATAKANA_C then
              begin
                if selected then
                  begin
                    c := table.working(w, 1);
                    b := Ord(c);

                    b := BELL_ASCII_KATAKANA[b];
                    if b = BELL then
                      begin
                        continue;
                      end
                    else
                      begin
                        Write( Chr(b) );
                      end;

                  end

              end
            else
              begin
                if character_type = SURROGATE_PAIR_C then
                  begin
                    high_part_of_kanji := w;
                    unformed := True;
                    continue;
                  end
                else
                  begin
                    if w = H_0A then
                      begin
                        count := 1;

                        Write( H_0D_ANSI );
                        Write( H_0A_ANSI );

                        break;
                      end
                    else
                      begin
                        if w = H_0D then
                          begin
                            carriage_return_appears := True;
                            NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
                            NEW_LINE := EMPTY_ANSI + H_0D_ANSI;
                            NEW_LINE := NEW_LINE + H_0A_ANSI;
                          end
                        else
                          begin
                            if w = quote_byte then
                              begin
                                no_splitting := True;
                              end
                            else
                              begin
                                if w = separator_byte then
                                  begin
                                    if selected then
                                      begin
                                        b := BELL_ASCII_KATAKANA[w];
                                        if b = BELL then
                                          begin
                                            continue; { Removing control characters }
                                          end
                                        else
                                          begin
                                            Write( Chr(w) );
                                          end;

                                      end;

                                    Inc(field_number);
                                    if field_number = field_to_be_selected then
                                      begin
                                        selected := True;
                                      end
                                    else
                                      begin
                                        selected := False;
                                      end;

                                  end
                                else
                                  begin
                                    if selected then
                                      begin
                                         b := BELL_ASCII_KATAKANA[w];
                                         if b = BELL then
                                           begin
                                             continue; { Removing control characters }
                                           end
                                         else
                                           begin
                                             Write( Chr(w) );
                                           end;

                                      end;

                                  end;

                              end;

                          end;

                      end;

                  end; { if character_type = SURROGATE_PAIR_C then, else }

              end; { if character_type = KATAKANA_C then, else }

          end; { if c = KANJI_C then, else }

      end;

  end;

{ Counting lines }
  if field_to_be_selected = FIRST_FIELD then
    begin
      field_number := 1;
      selected := True;
    end
  else
    begin
      field_number := 1;
      selected := False;
    end;

  high_part_of_kanji := H_D840;
  no_splitting := False;
  unformed := False;
  for n := beginning to buffer_length_1 do
  begin
    w := Ord(buffer[n]);

    if no_splitting then
      begin
        if unformed then
          begin
            low_part_of_kanji := w;

            if selected then
              begin
                Write( table.workingPair(high_part_of_kanji, low_part_of_kanji, 1) );
                Write( table.workingPair(high_part_of_kanji, low_part_of_kanji, 2) );
              end;

            unformed := False;
            continue;
          end;

{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
        character_type := table.working(w, 0);
        if character_type = KANJI_C then
          begin
            if selected then
              begin
                Write( table.working(w, 1) );
                Write( table.working(w, 2) );
              end;

          end
        else
          begin
            if character_type = KATAKANA_C then
              begin
                if selected then
                  begin
                    c := table.working(w, 1);
                    b := Ord(c);

                    b := BELL_ASCII_KATAKANA[b];
                    if b = BELL then
                      begin
                        continue;
                      end
                    else
                      begin
                        Write( Chr(b) );
                      end;

                  end;

              end
            else
              begin
                if character_type = SURROGATE_PAIR_C then
                  begin
                    high_part_of_kanji := w;
                    unformed := True;
                    continue;
                  end
                else
                  begin
                    if w = H_0A then
                      begin
                        Inc(count);

                        Write( H_0D_ANSI );
                        Write( H_0A_ANSI );

                        if field_to_be_selected = FIRST_FIELD then
                          begin
                            field_number := 1;
                            selected := True;
                          end
                        else
                          begin
                            field_number := 1;
                            selected := False;
                          end;

                      end
                    else
                      begin
                        if w = H_0D then
                          begin
                            continue;
                          end
                        else
                          begin
                            if w = quote_byte then
                              begin
                                no_splitting := False;
                              end
                            else
                              begin
                                if w = separator_byte then
                                  begin
                                    if remove_separators then
                                      begin
                                        continue; { Commas disappear if placed in quotes. }
                                      end
                                    else
                                      begin
                                        if selected then
                                          begin
                                            b := BELL_ASCII_KATAKANA[w];
                                            if b = BELL then
                                              begin
                                                continue; { Removing control characters }
                                              end
                                            else
                                              begin
                                                Write( Chr(w) );
                                              end;

                                          end;

                                      end;

                                  end
                                else
                                  begin
                                    if selected then
                                      begin
                                        b := BELL_ASCII_KATAKANA[w];
                                        if b = BELL then
                                          begin
                                            continue; { Removing control characters }
                                          end
                                        else
                                          begin
                                            Write( Chr(w) );
                                          end;

                                      end;

                                  end;

                              end;

                          end;

                      end;

                  end; { if character_type = SURROGATE_PAIR_C then }

              end; { if character_type = KATAKANA_C then, else }

          end; { if character_type = KANJI_C then }

      end
    else
      begin
{ if no_splitting = False then }

        if unformed then
          begin
            low_part_of_kanji := w;

            if selected then
              begin
                Write( table.workingPair(high_part_of_kanji, low_part_of_kanji, 1) );
                Write( table.workingPair(high_part_of_kanji, low_part_of_kanji, 2) );
              end;

            unformed := False;
            continue;
          end;

{*
place = 0: KANJI_C or ASCII_C or KATAKANA_C or SURROGATE_PAIR_C
place = 1: lower-byte (little-endian architecture)
place = 2: higher-byte
*}
        character_type := table.working(w, 0);
        if character_type = KANJI_C then
          begin
            if selected then
              begin
                Write( table.working(w, 1) );
                Write( table.working(w, 2) );
              end;

          end
        else
          begin
            if character_type = KATAKANA_C then
              begin
                if selected then
                  begin
                    c := table.working(w, 1);
                    b := Ord(c);

                    b := BELL_ASCII_KATAKANA[b];
                    if b = BELL then
                      begin
                        continue;
                      end
                    else
                      begin
                        Write( Chr(b) );
                      end;

                  end;

              end
            else
              begin
                if character_type = SURROGATE_PAIR_C then
                  begin
                    high_part_of_kanji := w;
                    unformed := True;
                    continue;
                  end
                else
                  begin
                    if w = H_0A then
                      begin
                        Inc(count);

                        Write( H_0D_ANSI );
                        Write( H_0A_ANSI );

                        if field_to_be_selected = FIRST_FIELD then
                          begin
                            field_number := 1;
                            selected := True;
                          end
                        else
                          begin
                            field_number := 1;
                            selected := False;
                          end;

                      end
                    else
                      begin
                        if w = H_0D then
                          begin
                            continue;
                          end
                        else
                          begin
                            if w = quote_byte then
                              begin
                                no_splitting := True;
                              end
                            else
                              begin
                                if w = separator_byte then
                                  begin
                                    if selected then
                                      begin
                                        b := BELL_ASCII_KATAKANA[w];
                                        if b = BELL then
                                          begin
                                            continue; { Removing control characters }
                                          end
                                        else
                                          begin
                                            Write( Chr(w) );
                                          end;

                                      end;

                                    Inc(field_number);
                                    if field_number = field_to_be_selected then
                                      begin
                                        selected := True;
                                      end
                                    else
                                      begin
                                        selected := False;
                                      end;

                                  end
                                else
                                  begin
                                    if selected then
                                      begin
                                        b := BELL_ASCII_KATAKANA[w];
                                        if b = BELL then
                                          begin
                                            continue; { Removing control characters }
                                          end
                                        else
                                          begin
                                            Write( Chr(w) );
                                          end;

                                      end;

                                  end;

                              end;

                          end;

                      end;

                  end; { if character_type = SURROGATE_PAIR_C then }

              end; { if character_type = KATAKANA_C then, else }

          end; { if character_type = KANJI_C then }

      end;

  end;

  Result := count;
end;

procedure Selecting.ifErrorOccurs(exit_code: Integer);
begin
  if buffer_length < 0 then
    Halt(exit_code);

end;

procedure Selecting.presetting(final_effects: FinalEffects);
var
  k: Integer;
begin
  f := final_effects;

  if f.extractDecimal then
    begin
      BELL_ASCII_KATAKANA[0] := 7; BELL_ASCII_KATAKANA[1] := 7; BELL_ASCII_KATAKANA[2] := 7; BELL_ASCII_KATAKANA[3] := 7; BELL_ASCII_KATAKANA[4] := 7; BELL_ASCII_KATAKANA[5] := 7;
      BELL_ASCII_KATAKANA[6] := 7; BELL_ASCII_KATAKANA[7] := 7; BELL_ASCII_KATAKANA[8] := 7; BELL_ASCII_KATAKANA[9] := 7; BELL_ASCII_KATAKANA[10] := 7; BELL_ASCII_KATAKANA[11] := 7;
      BELL_ASCII_KATAKANA[12] := 7; BELL_ASCII_KATAKANA[13] := 7; BELL_ASCII_KATAKANA[14] := 7; BELL_ASCII_KATAKANA[15] := 7; BELL_ASCII_KATAKANA[16] := 7; BELL_ASCII_KATAKANA[17] := 7;
      BELL_ASCII_KATAKANA[18] := 7; BELL_ASCII_KATAKANA[19] := 7; BELL_ASCII_KATAKANA[20] := 7; BELL_ASCII_KATAKANA[21] := 7; BELL_ASCII_KATAKANA[22] := 7; BELL_ASCII_KATAKANA[23] := 7;
      BELL_ASCII_KATAKANA[24] := 7; BELL_ASCII_KATAKANA[25] := 7; BELL_ASCII_KATAKANA[26] := 7; BELL_ASCII_KATAKANA[27] := 7; BELL_ASCII_KATAKANA[28] := 7; BELL_ASCII_KATAKANA[29] := 7;
      BELL_ASCII_KATAKANA[30] := 7; BELL_ASCII_KATAKANA[31] := 7; BELL_ASCII_KATAKANA[32] := 7; BELL_ASCII_KATAKANA[33] := 7; BELL_ASCII_KATAKANA[34] := 7; BELL_ASCII_KATAKANA[35] := 7;
      BELL_ASCII_KATAKANA[36] := 7; BELL_ASCII_KATAKANA[37] := 7; BELL_ASCII_KATAKANA[38] := 7; BELL_ASCII_KATAKANA[39] := 7; BELL_ASCII_KATAKANA[40] := 7; BELL_ASCII_KATAKANA[41] := 7;
      BELL_ASCII_KATAKANA[42] := 7; BELL_ASCII_KATAKANA[43] := 7; BELL_ASCII_KATAKANA[44] := 7; BELL_ASCII_KATAKANA[45] := 45; BELL_ASCII_KATAKANA[46] := 46; BELL_ASCII_KATAKANA[47] := 7;
      BELL_ASCII_KATAKANA[48] := 48; BELL_ASCII_KATAKANA[49] := 49; BELL_ASCII_KATAKANA[50] := 50; BELL_ASCII_KATAKANA[51] := 51; BELL_ASCII_KATAKANA[52] := 52; BELL_ASCII_KATAKANA[53] := 53;
      BELL_ASCII_KATAKANA[54] := 54; BELL_ASCII_KATAKANA[55] := 55; BELL_ASCII_KATAKANA[56] := 56; BELL_ASCII_KATAKANA[57] := 57; BELL_ASCII_KATAKANA[58] := 7; BELL_ASCII_KATAKANA[59] := 7;
      BELL_ASCII_KATAKANA[60] := 7; BELL_ASCII_KATAKANA[61] := 7; BELL_ASCII_KATAKANA[62] := 7; BELL_ASCII_KATAKANA[63] := 7; BELL_ASCII_KATAKANA[64] := 7; BELL_ASCII_KATAKANA[65] := 7;
      BELL_ASCII_KATAKANA[66] := 7; BELL_ASCII_KATAKANA[67] := 7; BELL_ASCII_KATAKANA[68] := 7; BELL_ASCII_KATAKANA[69] := 7; BELL_ASCII_KATAKANA[70] := 7; BELL_ASCII_KATAKANA[71] := 7;
      BELL_ASCII_KATAKANA[72] := 7; BELL_ASCII_KATAKANA[73] := 7; BELL_ASCII_KATAKANA[74] := 7; BELL_ASCII_KATAKANA[75] := 7; BELL_ASCII_KATAKANA[76] := 7; BELL_ASCII_KATAKANA[77] := 7;
      BELL_ASCII_KATAKANA[78] := 7; BELL_ASCII_KATAKANA[79] := 7; BELL_ASCII_KATAKANA[80] := 7; BELL_ASCII_KATAKANA[81] := 7; BELL_ASCII_KATAKANA[82] := 7; BELL_ASCII_KATAKANA[83] := 7;
      BELL_ASCII_KATAKANA[84] := 7; BELL_ASCII_KATAKANA[85] := 7; BELL_ASCII_KATAKANA[86] := 7; BELL_ASCII_KATAKANA[87] := 7; BELL_ASCII_KATAKANA[88] := 7; BELL_ASCII_KATAKANA[89] := 7;
      BELL_ASCII_KATAKANA[90] := 7; BELL_ASCII_KATAKANA[91] := 7; BELL_ASCII_KATAKANA[92] := 7; BELL_ASCII_KATAKANA[93] := 7; BELL_ASCII_KATAKANA[94] := 7; BELL_ASCII_KATAKANA[95] := 7;
      BELL_ASCII_KATAKANA[96] := 7; BELL_ASCII_KATAKANA[97] := 7; BELL_ASCII_KATAKANA[98] := 7; BELL_ASCII_KATAKANA[99] := 7; BELL_ASCII_KATAKANA[100] := 7; BELL_ASCII_KATAKANA[101] := 7;
      BELL_ASCII_KATAKANA[102] := 7; BELL_ASCII_KATAKANA[103] := 7; BELL_ASCII_KATAKANA[104] := 7; BELL_ASCII_KATAKANA[105] := 7; BELL_ASCII_KATAKANA[106] := 7; BELL_ASCII_KATAKANA[107] := 7;
      BELL_ASCII_KATAKANA[108] := 7; BELL_ASCII_KATAKANA[109] := 7; BELL_ASCII_KATAKANA[110] := 7; BELL_ASCII_KATAKANA[111] := 7; BELL_ASCII_KATAKANA[112] := 7; BELL_ASCII_KATAKANA[113] := 7;
      BELL_ASCII_KATAKANA[114] := 7; BELL_ASCII_KATAKANA[115] := 7; BELL_ASCII_KATAKANA[116] := 7; BELL_ASCII_KATAKANA[117] := 7; BELL_ASCII_KATAKANA[118] := 7; BELL_ASCII_KATAKANA[119] := 7;
      BELL_ASCII_KATAKANA[120] := 7; BELL_ASCII_KATAKANA[121] := 7; BELL_ASCII_KATAKANA[122] := 7; BELL_ASCII_KATAKANA[123] := 7; BELL_ASCII_KATAKANA[124] := 7; BELL_ASCII_KATAKANA[125] := 7;
      BELL_ASCII_KATAKANA[126] := 7; BELL_ASCII_KATAKANA[127] := 7; BELL_ASCII_KATAKANA[128] := 7; BELL_ASCII_KATAKANA[129] := 7; BELL_ASCII_KATAKANA[130] := 7; BELL_ASCII_KATAKANA[131] := 7;
      BELL_ASCII_KATAKANA[132] := 7; BELL_ASCII_KATAKANA[133] := 7; BELL_ASCII_KATAKANA[134] := 7; BELL_ASCII_KATAKANA[135] := 7; BELL_ASCII_KATAKANA[136] := 7; BELL_ASCII_KATAKANA[137] := 7;
      BELL_ASCII_KATAKANA[138] := 7; BELL_ASCII_KATAKANA[139] := 7; BELL_ASCII_KATAKANA[140] := 7; BELL_ASCII_KATAKANA[141] := 7; BELL_ASCII_KATAKANA[142] := 7; BELL_ASCII_KATAKANA[143] := 7;
      BELL_ASCII_KATAKANA[144] := 7; BELL_ASCII_KATAKANA[145] := 7; BELL_ASCII_KATAKANA[146] := 7; BELL_ASCII_KATAKANA[147] := 7; BELL_ASCII_KATAKANA[148] := 7; BELL_ASCII_KATAKANA[149] := 7;
      BELL_ASCII_KATAKANA[150] := 7; BELL_ASCII_KATAKANA[151] := 7; BELL_ASCII_KATAKANA[152] := 7; BELL_ASCII_KATAKANA[153] := 7; BELL_ASCII_KATAKANA[154] := 7; BELL_ASCII_KATAKANA[155] := 7;
      BELL_ASCII_KATAKANA[156] := 7; BELL_ASCII_KATAKANA[157] := 7; BELL_ASCII_KATAKANA[158] := 7; BELL_ASCII_KATAKANA[159] := 7; BELL_ASCII_KATAKANA[160] := 7;
      DOT_C := Chr(H_2E); { '.' }
      ZERO_C := Chr( f.jointByte ); { '0' for example }
    end
  else
    begin
      if f.extractInteger then
        begin
          BELL_ASCII_KATAKANA[0] := 7; BELL_ASCII_KATAKANA[1] := 7; BELL_ASCII_KATAKANA[2] := 7; BELL_ASCII_KATAKANA[3] := 7; BELL_ASCII_KATAKANA[4] := 7; BELL_ASCII_KATAKANA[5] := 7;
          BELL_ASCII_KATAKANA[6] := 7; BELL_ASCII_KATAKANA[7] := 7; BELL_ASCII_KATAKANA[8] := 7; BELL_ASCII_KATAKANA[9] := 7; BELL_ASCII_KATAKANA[10] := 7; BELL_ASCII_KATAKANA[11] := 7;
          BELL_ASCII_KATAKANA[12] := 7; BELL_ASCII_KATAKANA[13] := 7; BELL_ASCII_KATAKANA[14] := 7; BELL_ASCII_KATAKANA[15] := 7; BELL_ASCII_KATAKANA[16] := 7; BELL_ASCII_KATAKANA[17] := 7;
          BELL_ASCII_KATAKANA[18] := 7; BELL_ASCII_KATAKANA[19] := 7; BELL_ASCII_KATAKANA[20] := 7; BELL_ASCII_KATAKANA[21] := 7; BELL_ASCII_KATAKANA[22] := 7; BELL_ASCII_KATAKANA[23] := 7;
          BELL_ASCII_KATAKANA[24] := 7; BELL_ASCII_KATAKANA[25] := 7; BELL_ASCII_KATAKANA[26] := 7; BELL_ASCII_KATAKANA[27] := 7; BELL_ASCII_KATAKANA[28] := 7; BELL_ASCII_KATAKANA[29] := 7;
          BELL_ASCII_KATAKANA[30] := 7; BELL_ASCII_KATAKANA[31] := 7; BELL_ASCII_KATAKANA[32] := 7; BELL_ASCII_KATAKANA[33] := 7; BELL_ASCII_KATAKANA[34] := 7; BELL_ASCII_KATAKANA[35] := 7;
          BELL_ASCII_KATAKANA[36] := 7; BELL_ASCII_KATAKANA[37] := 7; BELL_ASCII_KATAKANA[38] := 7; BELL_ASCII_KATAKANA[39] := 7; BELL_ASCII_KATAKANA[40] := 7; BELL_ASCII_KATAKANA[41] := 7;
          BELL_ASCII_KATAKANA[42] := 7; BELL_ASCII_KATAKANA[43] := 7; BELL_ASCII_KATAKANA[44] := 7; BELL_ASCII_KATAKANA[45] := 45; BELL_ASCII_KATAKANA[46] := 7; BELL_ASCII_KATAKANA[47] := 7;
          BELL_ASCII_KATAKANA[48] := 48; BELL_ASCII_KATAKANA[49] := 49; BELL_ASCII_KATAKANA[50] := 50; BELL_ASCII_KATAKANA[51] := 51; BELL_ASCII_KATAKANA[52] := 52; BELL_ASCII_KATAKANA[53] := 53;
          BELL_ASCII_KATAKANA[54] := 54; BELL_ASCII_KATAKANA[55] := 55; BELL_ASCII_KATAKANA[56] := 56; BELL_ASCII_KATAKANA[57] := 57; BELL_ASCII_KATAKANA[58] := 7; BELL_ASCII_KATAKANA[59] := 7;
          BELL_ASCII_KATAKANA[60] := 7; BELL_ASCII_KATAKANA[61] := 7; BELL_ASCII_KATAKANA[62] := 7; BELL_ASCII_KATAKANA[63] := 7; BELL_ASCII_KATAKANA[64] := 7; BELL_ASCII_KATAKANA[65] := 7;
          BELL_ASCII_KATAKANA[66] := 7; BELL_ASCII_KATAKANA[67] := 7; BELL_ASCII_KATAKANA[68] := 7; BELL_ASCII_KATAKANA[69] := 7; BELL_ASCII_KATAKANA[70] := 7; BELL_ASCII_KATAKANA[71] := 7;
          BELL_ASCII_KATAKANA[72] := 7; BELL_ASCII_KATAKANA[73] := 7; BELL_ASCII_KATAKANA[74] := 7; BELL_ASCII_KATAKANA[75] := 7; BELL_ASCII_KATAKANA[76] := 7; BELL_ASCII_KATAKANA[77] := 7;
          BELL_ASCII_KATAKANA[78] := 7; BELL_ASCII_KATAKANA[79] := 7; BELL_ASCII_KATAKANA[80] := 7; BELL_ASCII_KATAKANA[81] := 7; BELL_ASCII_KATAKANA[82] := 7; BELL_ASCII_KATAKANA[83] := 7;
          BELL_ASCII_KATAKANA[84] := 7; BELL_ASCII_KATAKANA[85] := 7; BELL_ASCII_KATAKANA[86] := 7; BELL_ASCII_KATAKANA[87] := 7; BELL_ASCII_KATAKANA[88] := 7; BELL_ASCII_KATAKANA[89] := 7;
          BELL_ASCII_KATAKANA[90] := 7; BELL_ASCII_KATAKANA[91] := 7; BELL_ASCII_KATAKANA[92] := 7; BELL_ASCII_KATAKANA[93] := 7; BELL_ASCII_KATAKANA[94] := 7; BELL_ASCII_KATAKANA[95] := 7;
          BELL_ASCII_KATAKANA[96] := 7; BELL_ASCII_KATAKANA[97] := 7; BELL_ASCII_KATAKANA[98] := 7; BELL_ASCII_KATAKANA[99] := 7; BELL_ASCII_KATAKANA[100] := 7; BELL_ASCII_KATAKANA[101] := 7;
          BELL_ASCII_KATAKANA[102] := 7; BELL_ASCII_KATAKANA[103] := 7; BELL_ASCII_KATAKANA[104] := 7; BELL_ASCII_KATAKANA[105] := 7; BELL_ASCII_KATAKANA[106] := 7; BELL_ASCII_KATAKANA[107] := 7;
          BELL_ASCII_KATAKANA[108] := 7; BELL_ASCII_KATAKANA[109] := 7; BELL_ASCII_KATAKANA[110] := 7; BELL_ASCII_KATAKANA[111] := 7; BELL_ASCII_KATAKANA[112] := 7; BELL_ASCII_KATAKANA[113] := 7;
          BELL_ASCII_KATAKANA[114] := 7; BELL_ASCII_KATAKANA[115] := 7; BELL_ASCII_KATAKANA[116] := 7; BELL_ASCII_KATAKANA[117] := 7; BELL_ASCII_KATAKANA[118] := 7; BELL_ASCII_KATAKANA[119] := 7;
          BELL_ASCII_KATAKANA[120] := 7; BELL_ASCII_KATAKANA[121] := 7; BELL_ASCII_KATAKANA[122] := 7; BELL_ASCII_KATAKANA[123] := 7; BELL_ASCII_KATAKANA[124] := 7; BELL_ASCII_KATAKANA[125] := 7;
          BELL_ASCII_KATAKANA[126] := 7; BELL_ASCII_KATAKANA[127] := 7; BELL_ASCII_KATAKANA[128] := 7; BELL_ASCII_KATAKANA[129] := 7; BELL_ASCII_KATAKANA[130] := 7; BELL_ASCII_KATAKANA[131] := 7;
          BELL_ASCII_KATAKANA[132] := 7; BELL_ASCII_KATAKANA[133] := 7; BELL_ASCII_KATAKANA[134] := 7; BELL_ASCII_KATAKANA[135] := 7; BELL_ASCII_KATAKANA[136] := 7; BELL_ASCII_KATAKANA[137] := 7;
          BELL_ASCII_KATAKANA[138] := 7; BELL_ASCII_KATAKANA[139] := 7; BELL_ASCII_KATAKANA[140] := 7; BELL_ASCII_KATAKANA[141] := 7; BELL_ASCII_KATAKANA[142] := 7; BELL_ASCII_KATAKANA[143] := 7;
          BELL_ASCII_KATAKANA[144] := 7; BELL_ASCII_KATAKANA[145] := 7; BELL_ASCII_KATAKANA[146] := 7; BELL_ASCII_KATAKANA[147] := 7; BELL_ASCII_KATAKANA[148] := 7; BELL_ASCII_KATAKANA[149] := 7;
          BELL_ASCII_KATAKANA[150] := 7; BELL_ASCII_KATAKANA[151] := 7; BELL_ASCII_KATAKANA[152] := 7; BELL_ASCII_KATAKANA[153] := 7; BELL_ASCII_KATAKANA[154] := 7; BELL_ASCII_KATAKANA[155] := 7;
          BELL_ASCII_KATAKANA[156] := 7; BELL_ASCII_KATAKANA[157] := 7; BELL_ASCII_KATAKANA[158] := 7; BELL_ASCII_KATAKANA[159] := 7; BELL_ASCII_KATAKANA[160] := 7;
        end
      else
        begin
          if f.removeControllers then
            begin
              BELL_ASCII_KATAKANA[0] := 7; BELL_ASCII_KATAKANA[1] := 7; BELL_ASCII_KATAKANA[2] := 7; BELL_ASCII_KATAKANA[3] := 7; BELL_ASCII_KATAKANA[4] := 7; BELL_ASCII_KATAKANA[5] := 7;
              BELL_ASCII_KATAKANA[6] := 7; BELL_ASCII_KATAKANA[7] := 7; BELL_ASCII_KATAKANA[8] := 7; BELL_ASCII_KATAKANA[9] := 7; BELL_ASCII_KATAKANA[10] := 7; BELL_ASCII_KATAKANA[11] := 7;
              BELL_ASCII_KATAKANA[12] := 7; BELL_ASCII_KATAKANA[13] := 7; BELL_ASCII_KATAKANA[14] := 7; BELL_ASCII_KATAKANA[15] := 7; BELL_ASCII_KATAKANA[16] := 7; BELL_ASCII_KATAKANA[17] := 7;
              BELL_ASCII_KATAKANA[18] := 7; BELL_ASCII_KATAKANA[19] := 7; BELL_ASCII_KATAKANA[20] := 7; BELL_ASCII_KATAKANA[21] := 7; BELL_ASCII_KATAKANA[22] := 7; BELL_ASCII_KATAKANA[23] := 7;
              BELL_ASCII_KATAKANA[24] := 7; BELL_ASCII_KATAKANA[25] := 7; BELL_ASCII_KATAKANA[26] := 7; BELL_ASCII_KATAKANA[27] := 7; BELL_ASCII_KATAKANA[28] := 7; BELL_ASCII_KATAKANA[29] := 7;
              BELL_ASCII_KATAKANA[30] := 7; BELL_ASCII_KATAKANA[31] := 7; BELL_ASCII_KATAKANA[32] := 32; BELL_ASCII_KATAKANA[33] := 33; BELL_ASCII_KATAKANA[34] := 34; BELL_ASCII_KATAKANA[35] := 35;
              BELL_ASCII_KATAKANA[36] := 36; BELL_ASCII_KATAKANA[37] := 37; BELL_ASCII_KATAKANA[38] := 38; BELL_ASCII_KATAKANA[39] := 39; BELL_ASCII_KATAKANA[40] := 40; BELL_ASCII_KATAKANA[41] := 41;
              BELL_ASCII_KATAKANA[42] := 42; BELL_ASCII_KATAKANA[43] := 43; BELL_ASCII_KATAKANA[44] := 44; BELL_ASCII_KATAKANA[45] := 45; BELL_ASCII_KATAKANA[46] := 46; BELL_ASCII_KATAKANA[47] := 47;
              BELL_ASCII_KATAKANA[48] := 48; BELL_ASCII_KATAKANA[49] := 49; BELL_ASCII_KATAKANA[50] := 50; BELL_ASCII_KATAKANA[51] := 51; BELL_ASCII_KATAKANA[52] := 52; BELL_ASCII_KATAKANA[53] := 53;
              BELL_ASCII_KATAKANA[54] := 54; BELL_ASCII_KATAKANA[55] := 55; BELL_ASCII_KATAKANA[56] := 56; BELL_ASCII_KATAKANA[57] := 57; BELL_ASCII_KATAKANA[58] := 58; BELL_ASCII_KATAKANA[59] := 59;
              BELL_ASCII_KATAKANA[60] := 60; BELL_ASCII_KATAKANA[61] := 61; BELL_ASCII_KATAKANA[62] := 62; BELL_ASCII_KATAKANA[63] := 63; BELL_ASCII_KATAKANA[64] := 64; BELL_ASCII_KATAKANA[65] := 65;
              BELL_ASCII_KATAKANA[66] := 66; BELL_ASCII_KATAKANA[67] := 67; BELL_ASCII_KATAKANA[68] := 68; BELL_ASCII_KATAKANA[69] := 69; BELL_ASCII_KATAKANA[70] := 70; BELL_ASCII_KATAKANA[71] := 71;
              BELL_ASCII_KATAKANA[72] := 72; BELL_ASCII_KATAKANA[73] := 73; BELL_ASCII_KATAKANA[74] := 74; BELL_ASCII_KATAKANA[75] := 75; BELL_ASCII_KATAKANA[76] := 76; BELL_ASCII_KATAKANA[77] := 77;
              BELL_ASCII_KATAKANA[78] := 78; BELL_ASCII_KATAKANA[79] := 79; BELL_ASCII_KATAKANA[80] := 80; BELL_ASCII_KATAKANA[81] := 81; BELL_ASCII_KATAKANA[82] := 82; BELL_ASCII_KATAKANA[83] := 83;
              BELL_ASCII_KATAKANA[84] := 84; BELL_ASCII_KATAKANA[85] := 85; BELL_ASCII_KATAKANA[86] := 86; BELL_ASCII_KATAKANA[87] := 87; BELL_ASCII_KATAKANA[88] := 88; BELL_ASCII_KATAKANA[89] := 89;
              BELL_ASCII_KATAKANA[90] := 90; BELL_ASCII_KATAKANA[91] := 91; BELL_ASCII_KATAKANA[92] := 92; BELL_ASCII_KATAKANA[93] := 93; BELL_ASCII_KATAKANA[94] := 94; BELL_ASCII_KATAKANA[95] := 95;
              BELL_ASCII_KATAKANA[96] := 96; BELL_ASCII_KATAKANA[97] := 97; BELL_ASCII_KATAKANA[98] := 98; BELL_ASCII_KATAKANA[99] := 99; BELL_ASCII_KATAKANA[100] := 100; BELL_ASCII_KATAKANA[101] := 101;
              BELL_ASCII_KATAKANA[102] := 102; BELL_ASCII_KATAKANA[103] := 103; BELL_ASCII_KATAKANA[104] := 104; BELL_ASCII_KATAKANA[105] := 105; BELL_ASCII_KATAKANA[106] := 106; BELL_ASCII_KATAKANA[107] := 107;
              BELL_ASCII_KATAKANA[108] := 108; BELL_ASCII_KATAKANA[109] := 109; BELL_ASCII_KATAKANA[110] := 110; BELL_ASCII_KATAKANA[111] := 111; BELL_ASCII_KATAKANA[112] := 112; BELL_ASCII_KATAKANA[113] := 113;
              BELL_ASCII_KATAKANA[114] := 114; BELL_ASCII_KATAKANA[115] := 115; BELL_ASCII_KATAKANA[116] := 116; BELL_ASCII_KATAKANA[117] := 117; BELL_ASCII_KATAKANA[118] := 118; BELL_ASCII_KATAKANA[119] := 119;
              BELL_ASCII_KATAKANA[120] := 120; BELL_ASCII_KATAKANA[121] := 121; BELL_ASCII_KATAKANA[122] := 122; BELL_ASCII_KATAKANA[123] := 123; BELL_ASCII_KATAKANA[124] := 124; BELL_ASCII_KATAKANA[125] := 125;
              BELL_ASCII_KATAKANA[126] := 126; BELL_ASCII_KATAKANA[127] := 7; BELL_ASCII_KATAKANA[128] := 7; BELL_ASCII_KATAKANA[129] := 7; BELL_ASCII_KATAKANA[130] := 7; BELL_ASCII_KATAKANA[131] := 7;
              BELL_ASCII_KATAKANA[132] := 7; BELL_ASCII_KATAKANA[133] := 7; BELL_ASCII_KATAKANA[134] := 7; BELL_ASCII_KATAKANA[135] := 7; BELL_ASCII_KATAKANA[136] := 7; BELL_ASCII_KATAKANA[137] := 7;
              BELL_ASCII_KATAKANA[138] := 7; BELL_ASCII_KATAKANA[139] := 7; BELL_ASCII_KATAKANA[140] := 7; BELL_ASCII_KATAKANA[141] := 7; BELL_ASCII_KATAKANA[142] := 7; BELL_ASCII_KATAKANA[143] := 7;
              BELL_ASCII_KATAKANA[144] := 7; BELL_ASCII_KATAKANA[145] := 7; BELL_ASCII_KATAKANA[146] := 7; BELL_ASCII_KATAKANA[147] := 7; BELL_ASCII_KATAKANA[148] := 7; BELL_ASCII_KATAKANA[149] := 7;
              BELL_ASCII_KATAKANA[150] := 7; BELL_ASCII_KATAKANA[151] := 7; BELL_ASCII_KATAKANA[152] := 7; BELL_ASCII_KATAKANA[153] := 7; BELL_ASCII_KATAKANA[154] := 7; BELL_ASCII_KATAKANA[155] := 7;
              BELL_ASCII_KATAKANA[156] := 7; BELL_ASCII_KATAKANA[157] := 7; BELL_ASCII_KATAKANA[158] := 7; BELL_ASCII_KATAKANA[159] := 7; BELL_ASCII_KATAKANA[160] := 7; 
            end
          else
            begin
              for k := 0 to 160 do
              begin
                BELL_ASCII_KATAKANA[k] := k;
              end; { for k := 0 to 160 do }

            end; { if f.removeControllers then, else }

        end; { if f.extractInteger then, else }

    end; { if f.extractDecimal then, else }

  if f.omit_H_and_I then
    begin
      BELL_ASCII_KATAKANA[72] := 7; { 48h, 'H' }
      BELL_ASCII_KATAKANA[73] := 7; { 49h, 'I' }
    end;

  label_byte := f.labelByte;
  if f.replaceKana then
    begin
      BELL_ASCII_KATAKANA[161] := label_byte; { A1h }
      BELL_ASCII_KATAKANA[162] := label_byte; BELL_ASCII_KATAKANA[163] := label_byte; BELL_ASCII_KATAKANA[164] := label_byte; BELL_ASCII_KATAKANA[165] := label_byte; BELL_ASCII_KATAKANA[166] := label_byte; BELL_ASCII_KATAKANA[167] := label_byte;
      BELL_ASCII_KATAKANA[168] := label_byte; BELL_ASCII_KATAKANA[169] := label_byte; BELL_ASCII_KATAKANA[170] := label_byte; BELL_ASCII_KATAKANA[171] := label_byte; BELL_ASCII_KATAKANA[172] := label_byte; BELL_ASCII_KATAKANA[173] := label_byte;
      BELL_ASCII_KATAKANA[174] := label_byte; BELL_ASCII_KATAKANA[175] := label_byte; BELL_ASCII_KATAKANA[176] := label_byte; BELL_ASCII_KATAKANA[177] := label_byte; BELL_ASCII_KATAKANA[178] := label_byte; BELL_ASCII_KATAKANA[179] := label_byte;
      BELL_ASCII_KATAKANA[180] := label_byte; BELL_ASCII_KATAKANA[181] := label_byte; BELL_ASCII_KATAKANA[182] := label_byte; BELL_ASCII_KATAKANA[183] := label_byte; BELL_ASCII_KATAKANA[184] := label_byte; BELL_ASCII_KATAKANA[185] := label_byte;
      BELL_ASCII_KATAKANA[186] := label_byte; BELL_ASCII_KATAKANA[187] := label_byte; BELL_ASCII_KATAKANA[188] := label_byte; BELL_ASCII_KATAKANA[189] := label_byte; BELL_ASCII_KATAKANA[190] := label_byte; BELL_ASCII_KATAKANA[191] := label_byte;
      BELL_ASCII_KATAKANA[192] := label_byte; BELL_ASCII_KATAKANA[193] := label_byte; BELL_ASCII_KATAKANA[194] := label_byte; BELL_ASCII_KATAKANA[195] := label_byte; BELL_ASCII_KATAKANA[196] := label_byte; BELL_ASCII_KATAKANA[197] := label_byte;
      BELL_ASCII_KATAKANA[198] := label_byte; BELL_ASCII_KATAKANA[199] := label_byte; BELL_ASCII_KATAKANA[200] := label_byte; BELL_ASCII_KATAKANA[201] := label_byte; BELL_ASCII_KATAKANA[202] := label_byte; BELL_ASCII_KATAKANA[203] := label_byte;
      BELL_ASCII_KATAKANA[204] := label_byte; BELL_ASCII_KATAKANA[205] := label_byte; BELL_ASCII_KATAKANA[206] := label_byte; BELL_ASCII_KATAKANA[207] := label_byte; BELL_ASCII_KATAKANA[208] := label_byte; BELL_ASCII_KATAKANA[209] := label_byte;
      BELL_ASCII_KATAKANA[210] := label_byte; BELL_ASCII_KATAKANA[211] := label_byte; BELL_ASCII_KATAKANA[212] := label_byte; BELL_ASCII_KATAKANA[213] := label_byte; BELL_ASCII_KATAKANA[214] := label_byte; BELL_ASCII_KATAKANA[215] := label_byte;
      BELL_ASCII_KATAKANA[216] := label_byte; BELL_ASCII_KATAKANA[217] := label_byte; BELL_ASCII_KATAKANA[218] := label_byte; BELL_ASCII_KATAKANA[219] := label_byte; BELL_ASCII_KATAKANA[220] := label_byte; BELL_ASCII_KATAKANA[221] := label_byte;
      BELL_ASCII_KATAKANA[222] := label_byte; BELL_ASCII_KATAKANA[223] := label_byte;
    end
  else
    begin
      if f.removeDakuten then
        begin
          BELL_ASCII_KATAKANA[161] := 161; { A1h }
          BELL_ASCII_KATAKANA[162] := 162; BELL_ASCII_KATAKANA[163] := 163; BELL_ASCII_KATAKANA[164] := 164; BELL_ASCII_KATAKANA[165] := 165; BELL_ASCII_KATAKANA[166] := 166; BELL_ASCII_KATAKANA[167] := 167;
          BELL_ASCII_KATAKANA[168] := 168; BELL_ASCII_KATAKANA[169] := 169; BELL_ASCII_KATAKANA[170] := 170; BELL_ASCII_KATAKANA[171] := 171; BELL_ASCII_KATAKANA[172] := 172; BELL_ASCII_KATAKANA[173] := 173;
          BELL_ASCII_KATAKANA[174] := 174; BELL_ASCII_KATAKANA[175] := 175; BELL_ASCII_KATAKANA[176] := 176; BELL_ASCII_KATAKANA[177] := 177; BELL_ASCII_KATAKANA[178] := 178; BELL_ASCII_KATAKANA[179] := 179;
          BELL_ASCII_KATAKANA[180] := 180; BELL_ASCII_KATAKANA[181] := 181; BELL_ASCII_KATAKANA[182] := 182; BELL_ASCII_KATAKANA[183] := 183; BELL_ASCII_KATAKANA[184] := 184; BELL_ASCII_KATAKANA[185] := 185;
          BELL_ASCII_KATAKANA[186] := 186; BELL_ASCII_KATAKANA[187] := 187; BELL_ASCII_KATAKANA[188] := 188; BELL_ASCII_KATAKANA[189] := 189; BELL_ASCII_KATAKANA[190] := 190; BELL_ASCII_KATAKANA[191] := 191;
          BELL_ASCII_KATAKANA[192] := 192; BELL_ASCII_KATAKANA[193] := 193; BELL_ASCII_KATAKANA[194] := 194; BELL_ASCII_KATAKANA[195] := 195; BELL_ASCII_KATAKANA[196] := 196; BELL_ASCII_KATAKANA[197] := 197;
          BELL_ASCII_KATAKANA[198] := 198; BELL_ASCII_KATAKANA[199] := 199; BELL_ASCII_KATAKANA[200] := 200; BELL_ASCII_KATAKANA[201] := 201; BELL_ASCII_KATAKANA[202] := 202; BELL_ASCII_KATAKANA[203] := 203;
          BELL_ASCII_KATAKANA[204] := 204; BELL_ASCII_KATAKANA[205] := 205; BELL_ASCII_KATAKANA[206] := 206; BELL_ASCII_KATAKANA[207] := 207; BELL_ASCII_KATAKANA[208] := 208; BELL_ASCII_KATAKANA[209] := 209;
          BELL_ASCII_KATAKANA[210] := 210; BELL_ASCII_KATAKANA[211] := 211; BELL_ASCII_KATAKANA[212] := 212; BELL_ASCII_KATAKANA[213] := 213; BELL_ASCII_KATAKANA[214] := 214; BELL_ASCII_KATAKANA[215] := 215;
          BELL_ASCII_KATAKANA[216] := 216; BELL_ASCII_KATAKANA[217] := 217; BELL_ASCII_KATAKANA[218] := 218; BELL_ASCII_KATAKANA[219] := 219; BELL_ASCII_KATAKANA[220] := 220; BELL_ASCII_KATAKANA[221] := 221;
          BELL_ASCII_KATAKANA[222] := 7; BELL_ASCII_KATAKANA[223] := 7;
        end
      else
        begin
          for k := 161 to 223 do
          begin
            BELL_ASCII_KATAKANA[k] := k;
          end;

        end;

    end;

end;

procedure Selecting.setting(final_effects: FinalEffects);
var
  b: array[0..13] of WideChar;
  i: Integer;
  m: Integer;
  w: WideChar;
begin
  f := final_effects;

{ Setting for converting phrases with no extra memory space }
  buffer_length := 0;
  combo_size := 0;

  beginning := 0;
  file_name := '';
  field_to_be_selected := f.fieldSelected;
  new_name := '';
  quote_byte := f.quoteByte;
  remove_separators := f.removeSeparators;
  separator_byte := f.separatorByte;

  ASCII_C := #0;
  count := -1;
  EMPTY_ANSI := '';
  KANJI_C := #1;
  KATAKANA_C := #2;
  NON_KANJI_C := #0;
  SEPARATOR_C := Chr(separator_byte);
  SURROGATE_PAIR_C := #3;

  V_SIZE := SizeOf(v);
  V_SIZE_1 := V_SIZE - 1;
  Y_COUNT := SizeOf(y) div SIZE_OF_WIDECHAR;
  Y_COUNT_1 := Y_COUNT - 1;

  for i := 0 to V_SIZE_1 do
  begin
    CLEAR_ARRAY[i] := #0;
  end;

  for m := 0 to Y_COUNT_1 do
  begin
    CLEAR_Y_ARRAY[m] := #0;
  end;

{ Characters of WideChar type }
  w := WideChar(#0);
  b[0] := w;
  for i := 1 to H_0D do
  begin
    Inc(w);
    b[i] := w;
  end;

  H_0A_C := b[H_0A];
  H_0D_C := b[H_0D];
  H_0A_ANSI := Chr(H_0A);
  H_0D_ANSI := Chr(H_0D);

  EMPTY_ANSI := '';

  NEW_LINE := EMPTY_ANSI + H_0A_ANSI;
  NEW_LINE_SIZE := 1;

{ Producing a conversion table in the main memory }
  table := SurvivorsUTF16.Create;
  table.setting(f);

  if f.surrogatePairToUnderline then
    begin
      table.surrogatePairUnderline;
    end
  else
    begin
      table.surrogatePairSetting;
    end;

end;

procedure Selecting.setting(final_effects: FinalEffects; b: Integer);
var
  p: array[0..13] of WideChar;
  i: Integer;
  m: Integer;
  w: WideChar;
begin
  buffer_length := b;
  combo_size := EXPANSION_FACTOR * buffer_length;
  f := final_effects;

  beginning := 0;
  file_name := '';
  field_to_be_selected := f.fieldSelected;
  new_name := '';
  quote_byte := f.quoteByte;
  remove_separators := f.removeSeparators;
  separator_byte := f.separatorByte;

  ASCII_C := #0;
  count := -1;
  EMPTY_ANSI := '';
  KANJI_C := #1;
  KATAKANA_C := #2;
  NON_KANJI_C := #0;
  SEPARATOR_C := Chr(separator_byte);
  SURROGATE_PAIR_C := #3;

  V_SIZE := SizeOf(v);
  V_SIZE_1 := V_SIZE - 1;
  Y_COUNT := SizeOf(y) div SIZE_OF_WIDECHAR;
  Y_COUNT_1 := Y_COUNT - 1;

  for i := 0 to V_SIZE_1 do
  begin
    CLEAR_ARRAY[i] := #0;
  end;

  for m := 0 to Y_COUNT_1 do
  begin
    CLEAR_Y_ARRAY[m] := #0;
  end;

{ Characters of WideChar type }
  w := WideChar(#0);
  p[0] := w;
  for i := 1 to H_0D do
  begin
    Inc(w);
    p[i] := w;
  end;

  H_0A_C := p[H_0A];
  H_0D_C := p[H_0D];
  H_0A_ANSI := Chr(H_0A);
  H_0D_ANSI := Chr(H_0D);

  EMPTY_ANSI := '';

  NEW_LINE := EMPTY_ANSI + H_0A_ANSI;
  NEW_LINE_SIZE := 1;

  setBufferSize;

{ Producing a conversion table in the main memory }
  table := SurvivorsUTF16.Create;
  table.setting(f);

  if f.surrogatePairToUnderline then
    begin
      table.surrogatePairUnderline;
    end
  else
    begin
      table.surrogatePairSetting;
    end;

end;

procedure Selecting.setting(final_effects: FinalEffects; fn: String);
var
  b: array[0..13] of WideChar;
  block_has_been_read: Integer;
  i: Integer;
  m: Integer;
  q: array[0..1] of Byte;
  s: String;
  testFile: File;
  w: WideChar;
begin
  buffer_length := -1;
  f := final_effects;
  file_name := fn;
  field_to_be_selected := f.fieldSelected;
  new_name := '';
  quote_byte := f.quoteByte;
  remove_separators := f.removeSeparators;
  separator_byte := f.separatorByte;

  ASCII_C := #0;
  count := -1;
  EMPTY_ANSI := '';
  KANJI_C := #1;
  KATAKANA_C := #2;
  NON_KANJI_C := #0;
  SEPARATOR_C := Chr(separator_byte);
  SURROGATE_PAIR_C := #3;

  V_SIZE := SizeOf(v);
  V_SIZE_1 := V_SIZE - 1;
  Y_COUNT := SizeOf(y) div SIZE_OF_WIDECHAR;
  Y_COUNT_1 := Y_COUNT - 1;

  for i := 0 to V_SIZE_1 do
  begin
    CLEAR_ARRAY[i] := #0;
  end;

  for m := 0 to Y_COUNT_1 do
  begin
    CLEAR_Y_ARRAY[m] := #0;
  end;

  try
    AssignFile(testFile, file_name);
    Reset(testFile, SIZE_OF_ANSICHAR);
    BlockRead(testFile, v, V_SIZE, block_has_been_read);
    q[0] := Ord(v[0]);
    q[1] := Ord(v[1]);
    if (q[0] = H_FF) and (q[1] = H_FE) then
      beginning := BOM_COUNT
    else
      beginning := 0;

    CloseFile(testFile);

    buffer_length := file_size(file_name) div SIZE_OF_WIDECHAR;
    combo_size := EXPANSION_FACTOR * buffer_length;
  except
    s := 'Reset() error-229: ' + file_name;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
  end;

{ Characters of WideChar type }
  w := WideChar(#0);
  b[0] := w;
  for i := 1 to H_0D do
  begin
    Inc(w);
    b[i] := w;
  end;

  H_0A_C := b[H_0A];
  H_0D_C := b[H_0D];
  H_0A_ANSI := Chr(H_0A);
  H_0D_ANSI := Chr(H_0D);

  EMPTY_ANSI := '';

  NEW_LINE := EMPTY_ANSI + H_0A_ANSI;
  NEW_LINE_SIZE := 1;

  setBufferSize;

{ Producing a conversion table in the main memory }
  table := SurvivorsUTF16.Create;
  table.setting(f);

  if f.surrogatePairToUnderline then
    begin
      table.surrogatePairUnderline;
    end
  else
    begin
      table.surrogatePairSetting;
    end;

end;

end.
